function gNameLst=get_file(num, filter, prompt, newwdir, res)
% function gNameLst=get_file(num, filter, prompt, newwdir)
% ScAnVp Browser to Get the names of a set of files
%
% Displays a user window for interactive file selection.
% For single file selection the program utilizes the MATLAB uigetfile
% tool directly. For multiple file selection the user can separately 
% pick files from one or more directories, add to the list of selected 
% files or edit the list of previously selected files.
%
%*Note: The program sorts file names in alphabetical order for each ADD to List
% operation so that if a particular order has to be maintained one may have
% to do several repetitions of the ADD files operation. Use the REVIEW
% option to check that the list is ordered properly.
% The entire list can be resorted alphabetically.
% The list can be cleared or reset to undo a previous clear.
%
% Input arguments specify the number of files to be selected, an optional
% filter for image names or types, a user prompt and an initial directory 
% to overide the default working directory (HOME). This may be
% changed to select files from other directories but is restored to the
% original on exit unless the user checks the save option before exiting.
%
% The user can pick or key in directory names or select the directory 
% from a list of sub directories or previously selected directories.
% On subsequent calls the program defaults to the previously active
% directory even if it has not been saved as a new working directory. 
% This version is not dependent on spm modules for its functionality
% and has a limited resemblence (other than output format) to previous
% versions of get_file that were based on the spm99 browser routine,
% (In the spm99 version the selection of files from multiple directories
%  was performed in one operation.)
% 
% For higher versions of MATLAB, the program activates the MATLAB function 
% uigetfile for file selection. However, for versions of MATLAB such as 6.1
% that do not support directory navigation in file selection, some directory 
% options are provided that may not be necessary for users in higher versions.
% For the earlier versions, the user must first separately select each directory.
%
%________________________________________________________________________
% Input:
% num      -    Number of files to be returned, Inf allows any number
% filter   -    Filename filter ('*' is prepended), Default *.*
% prompt   -    prompt string    
% newwdir  -    New working directory. (Default is working directory)
% res      -    Reserved field
%
% Output:
% gNameLst -    Returned list of selected full path file names or null.
%__________________________________________________________________________
% get_file.m	ScAnVP ver. 6.1 	P. Spetsieris  Jan. 25, 2011
%		revised ver. 6.2	Apr. 26, 2011, Oct. 27, 2011
%		revised ver. 6.3	Dec. 18, 2013

global gFNames gFName gNameLst gPrevLst 
global gFileDir gPrevDirs FMenu
global gFilter gPrompt gHomeDir
global HelpMode Ecolor Bcolor Fcolor 
global MenuWid MenuHgt

if HelpMode, vp_help(mfilename), return ; end;


getfilesgui=findobj('Name','Get Files');
if ~isempty(getfilesgui), figure(getfilesgui);
vp_note('ACTIVE BROWSER: GET FILES - PRESS DONE or CANCEL',4); return; end;


gNameLst=[];
NewFIds=[];
gHomeDir=pwd;
MenuWid=325;
%MenuHgt=215;
MenuHgt=250;

if nargin < 1, num=Inf; end;

if nargin > 1, gFilter=['*',filter];
 elseif isempty(gFilter) gFilter = '*.*';  
end;

if nargin > 2, gPrompt=prompt;
else gPrompt = 'Please Select Files';  
end;


nds=size(gPrevDirs,1);
if nds==0,
    gFileDir =pwd;
else
    if ~exist(gPrevDirs(nds,:)), 
      gFileDir =pwd;
    else
      gFileDir=gPrevDirs(nds,:);
    end;
end;

if nargin > 3, 
 newwdir=deblank(newwdir);
 if ~isempty(newwdir)
   gFileDir=newwdir;
   if isempty(findstr(gFileDir,'\')), gFileDir=strcat(gFileDir,'\'), end;
 end;
end;

cd(gFileDir) ;

gFName=' ';
gPrevLst={};
gFNames={};

Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];
%Fcolor = [.82,.78, 1.0];
Fcolor = [.95, 1.0, .95];


% Initialize Previous Directories List. Truncate Initial List to 1st 3 and last 3 

cbPreDirUpdate=[' in=0; nds=size(gPrevDirs,1);',...
  'if nds==0,',... 
  '  vp=which(''scanvp'');',...
  '  gPrevDirs=strvcat(gPrevDirs,vp(1:max(strfind(vp,''\''))-5));',...
  '  if (exist(''spm.m'')==2), gPrevDirs=strvcat(gPrevDirs,spm(''Dir'')); end;',...
  'elseif (nds > 10), pre3=gPrevDirs(1:3,:);',...
  ' post7=gPrevDirs(nds-7:nds,:); gPrevDirs=strvcat(pre3,post7);',...
  'end;',...
  'nds=size(gPrevDirs,1);',...
  'for i=1:nds, if or( strcmp(deblank(gPrevDirs(i,:)),deblank(gFileDir)), ',...
  '  strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ), in=1; ',...
  '  if i~=nds,in=in+1, for j=i:nds-1,gPrevDirs(j,:)=gPrevDirs(j+1,:); end; end;',...
  'end; end; ',...
  'if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir),',...
  'elseif in > 1, gPrevDirs=strvcat(gPrevDirs(1:nds-in+1,:), gFileDir), end; '];

%For Only One File, Invoke MATLAB function
if num==1, [gFName, pathdir]=uigetfile(gFilter,gPrompt); 
 if ~isequal(gFName,0), gNameLst=strcat(pathdir, char(gFName)), 
  gFileDir=pathdir; eval(cbPreDirUpdate); end; %For use with multiple selection
  cd(gHomeDir);
return; end;


getfilesgui = figure('Color',Fcolor,'Name','Get Files',...
'NumberTitle','off','Position',[100,200,MenuWid,MenuHgt],...
'Units','Pixels');
	
	
uicontrol(getfilesgui,'string', 'HELP',...
	'Units','Normalized',...
	'position',[255/MenuWid 162/MenuHgt 60/MenuWid 20/MenuHgt],'BackgroundColor',Ecolor,...
	'callback',['vp_help(''get_file'')']);
	

fsize=14; if length(gPrompt) > 25, fsize=12; end;
if length(gPrompt) > 35, fsize=10; end; if length(gPrompt) > 45, fsize=8; end;

cbTitle= ['text(5/MenuWid, MenuHgt/MenuHgt,gPrompt,''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',fsize,''FontUnits'',''Normalized'',''FontAngle'',''Italic'',',...
	'''Color'',[.7 .2 .2],''HorizontalAlignment'',''left'');'] ;

cbWarning= ['text(0/MenuWid, 220/MenuHgt,''WARNING: Review Order (May Not Be As Picked)'',',...
	'''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''FontUnits'',''Normalized'',''FontAngle'',''Italic'',',...
	'''Color'',[.8 .1 .1],''HorizontalAlignment'',''left'');'] ;

%cbTitle= ['text(5, 175,gPrompt,''Units'',''pixels'',''Fontweight'',''bold'',',...
%	'''FontName'',''Times'',''FontSize'',fsize,''FontAngle'',''Italic'',',...
%	'''Color'',[.7 .2 .2],''HorizontalAlignment'',''left'');'] ;
%

%cbgFiltertxt= ['text(5,148,''Filter'',''Units'',''pixels'',''Fontweight'',''bold'',',...
%	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;
%

cbgFiltertxt= ['text(5/MenuWid,178/MenuHgt,''Filter'',''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''FontUnits'',''Normalized'',''HorizontalAlignment'',''center'');'] ;

%cbPreDirtxt= ['text(5,127,''Previous'',''Units'',''pixels'',''Fontweight'',''bold'',',...
%	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;

cbPreDirtxt= ['text(5/MenuWid,157/MenuHgt,''Previous'',''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''FontUnits'',''Normalized'',''HorizontalAlignment'',''center'');'] ;

%cbDirectory1= ['text(5,57,''Current'',''Units'',''pixels'',''Fontweight'',''bold'',',...
%	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;

cbDirectory1= ['text(5/MenuWid,65/MenuHgt,''Current'',''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''FontUnits'',''Normalized'',''HorizontalAlignment'',''center'');'] ;

%cbDirectory2= ['text(1,42,''Directory'',''Units'',''pixels'',''Fontweight'',''bold'',',...
%	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;

cbDirectory2= ['text(1/MenuWid,50/MenuHgt,''Directory'',''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''FontUnits'',''Normalized'',''HorizontalAlignment'',''center'');'] ;

eval(cbTitle);
eval(cbWarning);
eval(cbgFiltertxt);
eval(cbPreDirtxt);
eval(cbDirectory1);
eval(cbDirectory2);


cbPreDirUpList=[' set(findobj(''Tag'',''PrevDirMenu''),''string'',',...
	    ' strvcat(''Previous Directories ...'',  gPrevDirs))'];

eval(cbPreDirUpdate);

hgFilter=uicontrol(getfilesgui,'style','edit','string', gFilter,...
	'TooltipString','Specifiy Filter ',...
	'Units','Normalized',...
	'position',[77/MenuWid 163/MenuHgt 60/MenuWid 18/MenuHgt],'BackgroundColor',Kcolor,'callback',...
        ['gFilter=get(gco,''string''); ']);

	
uicontrol(getfilesgui,'String','ReSORT',...
	'Units','Normalized',...
	'Position',[110/MenuWid 85/MenuHgt 48/MenuWid 20/MenuHgt],...
	'BackgroundColor',Bcolor,...
	'TooltipString','RE-SORT Previous File Selection Alphabetically. ',...
	'CallBack',['strnms=cellstr(gFNames);',...
	' if ~isempty(gFNames), strnms=sort(strnms); gFNames=strnms, ',...
	' disp(''NAME LIST RE-SORTED''); gNameLst=char(gFNames),',...
	' else,  vp_note([''NO FILES FOUND''],3);  end;']);
	
	
uicontrol(getfilesgui,'String','RESET',...
	'Units','Normalized',...
	'Position',[61/MenuWid 85/MenuHgt 48/MenuWid 20/MenuHgt],...
	'BackgroundColor',Bcolor,...
	'TooltipString','RESET Previous File Selection ',...
	'CallBack',['gFNames=gPrevLst; disp(''NAME LIST RESET''); gNameLst=char(gFNames),']);
	
uicontrol(getfilesgui,'String','CLEAR',...
	'Units','Normalized',...
	'Position',[15/MenuWid 85/MenuHgt 45/MenuWid 20/MenuHgt],...
	'BackgroundColor',Bcolor,...
	'TooltipString','Clear Previous File Selection ',...
	'CallBack',['gFNames={}; disp(''NAME LIST CLEARED''); gNameLst=[],']);
	
uicontrol(getfilesgui,'String','HOME',...
	'Units','Normalized',...
	'Position',[255/MenuWid 40/MenuHgt 60/MenuWid 15/MenuHgt],...
	'TooltipString','Set File Directory to Home Directory ',...
	'CallBack',['gFileDir=gHomeDir; cd(gFileDir);',...
	  'set(findobj(''Tag'',''KeyDir''),''string'',gFileDir);']);

axis off;

pcbWDirUpdate=['gFileDir=pwd;',...
	  '  set(findobj(''Tag'',''KeyDir''),''string'',gFileDir);',...
	  '' ];

pcbGetNewFiles=[' gFileDir=get(findobj(''Tag'',''KeyDir''),''string''); gFileDir=deblank(gFileDir);',...
	' if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	' newstrnms=[]; numids=0;',...
	' try, [strnms, pathname]=uigetfile(gFilter,''Select Files'',''MultiSelect'',''on''),',...
	'   if ~isequal(strnms, 0),  if iscellstr(strnms), newstrnms = strnms'', ',...
	'    else,  newstrnms=cellstr(strnms), end;',...
	'    newstrnms=sort(newstrnms),',...
	'    gFileDir=deblank(pathname); numids = size(newstrnms,1),',...
	'    set(findobj(''Tag'',''KeyDir''),''string'',gFileDir); cd(gFileDir);',...
	'    nds=size(gPrevDirs,1); in=0;',...
	'    for i=1:nds, if or( strcmp(deblank(gPrevDirs(i,:)),deblank(gFileDir)), ',...
	'      strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ), in=1; ',...
	'      if i~=nds,in=in+1, for j=i:nds-1,gPrevDirs(j,:)=gPrevDirs(j+1,:); end; end;',...
	'    end; end; ',...
	'    if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir),',...
	'    elseif in > 1, gPrevDirs=strvcat(gPrevDirs(1:nds-in+1,:), gFileDir); end;',... 
	'    set(findobj(''Tag'',''PrevDirMenu''),''string'',',...
	'    strvcat(''Previous Directories ...'',  gPrevDirs));',...
	'   end; ',... 	
	' catch ',...
	'   cd(gFileDir); numids=0; dfls=dir(gFilter); strnms={dfls.name};',...
	'   strnms=char(strnms); n=size(strnms,1); dstrnms=[];',...
	'   for i=1:n, if(~isdir(deblank(strnms(i,:)))), dstrnms=strvcat(dstrnms,strnms(i,:)); end; end;',...
	'   if ~isempty(dstrnms), dfls=cellstr(dstrnms); else dfls={}; end; ',...
	'   if isempty(dfls), vp_note([''NO FILES In Directory''],3); ',...
	'   else [NewFIds, v]=listdlg(''PromptString'', [''SELECT FILES IN '',gFileDir],',...
	'   ''SelectionMode'',''multiple'',''ListString'',dfls,''ListSize'',[350 300]);',...
	'      if ~isempty(NewFIds), numids=size(NewFIds,2); else, numids=0; end;',...
	'      if (numids ~=0), newstrnms=dfls(NewFIds); sort(newstrnms); gFileDir=deblank(gFileDir); end;',...
	' end; end;',...
	' if ~isempty(newstrnms),',...
	'    nch=size(gFileDir,2);',...
	'    if (gFileDir(nch) == ''\''), names=strcat(gFileDir,char(newstrnms)),',...
	'    else, names=strcat(gFileDir,''\'',char(newstrnms)), end;',...
	'    addstrnms=strvcat(char(gFNames), names); gFNames=cellstr(addstrnms); ',...
	'    gPrevLst=cellstr(gFNames);',...
	'    vp_note([num2str(numids),'' File(s)s Selected. Press DONE to Exit or ADD/EDIT files.''], 2);',...
	' end;'];
	

       
pcbEditFileList=['strnms=cellstr(gFNames);',...
	' if ~isempty(gFNames), [NewFIds, v]=listdlg(''PromptString'', [''SELECT ALL or EDIT FILES IN LIST ''],',...
	' ''SelectionMode'',''multiple'',''ListString'',strnms,''ListSize'',[350 300]);',...
	'    if ~isempty(NewFIds), numids=size(NewFIds,2), else, numids=0; end;',...
	'    if (numids ~=0), newstrnms=strnms(NewFIds), gFNames=newstrnms, ',...
	'    vp_note([num2str(numids),'' File(s)s Selected. Press DONE to Exit or ADD/EDIT files.''], 2);',...
	'    else, gFNames={}; vp_note(''NAME LIST CLEARED'', 3); end;',...
	' else,  vp_note([''NO FILES FOUND''],3);  end;'];
	
       
pcbGetParent=['gFileDir = get(findobj(''Tag'',''KeyDir''),''string''); gFileDir=deblank(gFileDir);',...
	'     nch=size(gFileDir,2);',...
	'     if (gFileDir(nch) == ''\''), gFileDir=gFileDir(1:nch-1); end;',...
	'     ndx=strfind(gFileDir,''\''); if ~isempty(ndx),',...
	'     nd=ndx(size(ndx,2)); gFileDir=gFileDir(1:nd-1); end;',...
	'     if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	'     set(findobj(''Tag'',''KeyDir''),''string'',gFileDir); cd(gFileDir);',...
	'    nds=size(gPrevDirs,1); in=0;',...
	'    for i=1:nds, if or( strcmp(deblank(gPrevDirs(i,:)),deblank(gFileDir)), ',...
	'      strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ), in=1; ',...
	'      if i~=nds,in=in+1, for j=i:nds-1,gPrevDirs(j,:)=gPrevDirs(j+1,:); end; end;',...
	'    end; end; ',...
	'    if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir),',...
	'    elseif in > 1, gPrevDirs=strvcat(gPrevDirs(1:nds-in+1,:), gFileDir); end;',... 
	'    set(findobj(''Tag'',''PrevDirMenu''),''string'',',...
	'    strvcat(''Previous Directories ...'',  gPrevDirs));',...
	'    '];
	
	
pcbGetSubDir=['gFileDir = get(findobj(''Tag'',''KeyDir''),''string''); gFileDir=deblank(gFileDir);',...
	' if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	' cd(gFileDir); try  gFileDir=uigetdir(gFileDir); deblank(gFileDir); ',...
	' catch ',...
	'  dfls=dir(''*.*''); strnms={dfls.name}; strnms=char(strnms); n=size(strnms,1); dstrnms=[];',...
	'  for i=1:n, if(isdir(deblank(strnms(i,:)))), dstrnms=strvcat(dstrnms,strnms(i,:)); end; end;',...
	'  dfls=cellstr(dstrnms); if ~isempty(dfls), [NewFIds, v]=listdlg(''PromptString'', [''DIR '', gFileDir],',...
	'  ''SelectionMode'',''single'',''ListString'',dfls,''ListSize'',[350 200]);',...
	'    if ~isempty(NewFIds), numids=size(NewFIds,2); else, numids=0; end;',...
	'    if (numids ~=0), newdir=deblank(char(dfls(NewFIds))); gFileDir=deblank(gFileDir);',...
	'     nch=size(gFileDir,2);',...
	'     if (gFileDir(nch) == ''\''), gFileDir=gFileDir(1:nch-1); end;',...
	'     if strcmp(newdir,''..''),',...
	'      ndx=strfind(gFileDir,''\''); if ~isempty(ndx),',...
	'      nd=ndx(size(ndx,2)); gFileDir=gFileDir(1:nd-1); end;',...
	'     elseif ~strcmp(newdir,''.''),',...
	'        gFileDir=[deblank(gFileDir), ''\'', newdir], end; ',...
	' end; end; end;',...
	' if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	' set(findobj(''Tag'',''KeyDir''),''string'',gFileDir); cd(gFileDir);',...
	' nds=size(gPrevDirs,1); in=0;',...
	' for i=1:nds, if or( strcmp(deblank(gPrevDirs(i,:)),deblank(gFileDir)), ',...
	'      strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ), in=1; ',...
	'      if i~=nds,in=in+1, for j=i:nds-1,gPrevDirs(j,:)=gPrevDirs(j+1,:); end; end;',...
	' end; end; ',...
	' if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir),',...
	'    elseif in > 1, gPrevDirs=strvcat(gPrevDirs(1:nds-in+1,:), gFileDir); end;',... 
	'    set(findobj(''Tag'',''PrevDirMenu''),''string'',',...
	'    strvcat(''Previous Directories ...'',  gPrevDirs));',...
	'    '];
	

uicontrol(getfilesgui,'Style','Check','String','SAVE',...
 	'TooltipString','Save as New Current Directory on Exit ON/OFF',...
	'Tag','SaveCD',...
	'Units','Normalized',...
 	'Value',0,'Position',[15/MenuWid 40/MenuHgt 60/MenuWid 15/MenuHgt],...
 	'BackgroundColor',Bcolor,...
 	'CallBack',[ 'val=get(gco,''value''), if val==0,',...
	' disp(''Restore Home Dir on Exit''), else disp(''Set as New Working Dir on Exit''); end;']);
 	

uicontrol(getfilesgui,'string', ' <<--',...
	'Units','Normalized',...
	'TooltipString','Select the parent directory ..',...
	'position',[77/MenuWid 40/MenuHgt 33/MenuWid 15/MenuHgt],'BackgroundColor',Bcolor,'callback',pcbGetParent);
	
uicontrol(getfilesgui,'string', '(SUB) DIRECTORY',...
	'TooltipString','Select a (sub)directory',...
	'Units','Normalized',...
	'position',[110/MenuWid 40/MenuHgt 145/MenuWid 15/MenuHgt],'BackgroundColor',Bcolor,'callback',pcbGetSubDir);
	
	
uicontrol(getfilesgui,'string', 'SELECT FILES OR ADD FILES TO LIST',...
	'Units','Normalized',...
	'TooltipString','Get (Additional) Files, Press DONE When Complete','ForegroundColor',[.5 .1 .2],'FontWeight','bold',...
	'FontSize',10,'position',[15/MenuWid 108/MenuHgt 300/MenuWid 25/MenuHgt],'BackgroundColor',Bcolor,'callback',pcbGetNewFiles);
	
uicontrol(getfilesgui,'string', 'REVIEW/EDIT/CANCEL File List',...
	'Units','Normalized',...
	'TooltipString','Review cancel/edit/confirm selected files','ForegroundColor',[.1 .5 .2],'FontWeight','bold',...
	'position',[159/MenuWid 85/MenuHgt 156/MenuWid 20/MenuHgt],'BackgroundColor',Bcolor,'callback',pcbEditFileList);
	
cbFDir= [' gFileDir=get(gco,''string''); gFileDir=deblank(gFileDir); ',...
	' if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	' cd(gFileDir); nds=size(gPrevDirs,1); in=0;',...
	' for i=1:nds, if or( strcmp(deblank(gPrevDirs(i,:)),deblank(gFileDir)), ',...
	'      strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ), in=1; ',...
	'      if i~=nds,in=in+1, for j=i:nds-1,gPrevDirs(j,:)=gPrevDirs(j+1,:); end; end;',...
	' end; end; ',...
	' if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir),',...
	'    elseif in > 1, gPrevDirs=strvcat(gPrevDirs(1:nds-in+1,:), gFileDir); end;',... 
	'    set(findobj(''Tag'',''PrevDirMenu''),''string'',',...
	'    strvcat(''Previous Directories ...'',  gPrevDirs));',...
	'    '];

	
uicontrol(getfilesgui,'style','edit','string',gFileDir,...
	'Units','Normalized',...
	'Tag','KeyDir',...
	'TooltipString','Specify Directory Name',...
	'position',[75/MenuWid 60/MenuHgt 240/MenuWid 25/MenuHgt],'BackgroundColor',Kcolor,'callback', cbFDir); 


cbPrevDirMenu = ['val=get(gco,''Value'');',...
	'if val > 1,  prevdir=get(gco,''string'');',...
	'  if ~exist(prevdir(val,:)), vp_note(''Directory does not exist''); return; ',...
	'  else, gFileDir=prevdir(val,:); ',...
	'    if isempty(findstr(gFileDir,''\'')), gFileDir=strcat(gFileDir,''\''), end; ',...
	'    cd(gFileDir); set(findobj(''Tag'',''KeyDir''),''string'',gFileDir); end;',...
	'end; '];

hprevdir=uicontrol(getfilesgui,'Style','popup','Visible','on',...
	'Units','Normalized',...
	'position',[75/MenuWid 140/MenuHgt 242/MenuWid 20/MenuHgt],'BackgroundColor',Bcolor,...
	'TooltipString','Select Previous Directory',...
	'String',...
	  strvcat('Previous Directories ...',  gPrevDirs),...
	'Tag','PrevDirMenu',...
    	'callback',cbPrevDirMenu);


uicontrol(getfilesgui,'String','DONE - PRESS TO COMPLETE SELECTION',...
	'TooltipString','EXIT - FILE SELECTION COMPLETE',...
	'Units','Normalized',...
	'Position',[77/MenuWid 10/MenuHgt 238/MenuWid 25/MenuHgt],...
	'ForegroundColor',[.5 .1 .2],...
	'FontWeight','Bold',...
	'UserData',num,...
	'CallBack',[ 'if isempty(gFNames), vp_note([''ERR: FILE SET Not Specified''],4); return; end;',...
	'num=get(gco,''UserData''); if isfinite(num), disp(''No of Files = ''); size(gFNames,1), if size(gFNames,1) ~= num,',...
	' vp_note([''No. Of Files Selected Must Equal '', num2str(num)],4); return; end; end;',...
	' if get(findobj(''tag'',''SaveCD''), ''value'')==1, gHomeDir=gFileDir; end;',...
	' cd(gHomeDir); gNameLst=char(gFNames),',...
	' delete(findobj(''name'',''Get Files''));','set(FMenu,''Pointer'', ''Arrow'');']);
	
uicontrol(getfilesgui,'String','CANCEL',...
	'Units','Normalized',...
	'Position',[15/MenuWid 10/MenuHgt 60/MenuWid 25/MenuHgt],...
	'CallBack',[' cd(gHomeDir); gFNames={};,gNameLst=[], delete(findobj(''name'',''Get Files''));',...
	'set(FMenu,''Pointer'', ''Arrow'');']);
	
set(findobj('Name','ScAnVP'),'Pointer', 'Watch');
waitfor(findobj('name','Get Files'));     

