% scanvp.m  Copyright (C) 2003-2016 Center for Neurosciences,
% Feinstein Institute for Medical Research (FIMR),
% Northwell Health (Formerly, North Shore LIJ Health System) 
% ScAnVP (Scan Analysis and Visualization Processor)
%
%                 Version  7.0w - Feb. 1, 2016
%
% ScAnVP is a set of MATLAB routines developed to aid research in   
% VOXEL or ROI based image processing, statistics, visualization and
% analysis of Analyze or Nifti format Positron Emission Tomography (PET)
% or other single volume brain images. ScAnVP integrates several 
% methodological approaches to multi-modal functional image data
% visualization and analysis.
% The software can be installed on a Windows based personal computers or
% other processors that support MATLAB. The latest version was tested in 
% the Microsoft Windows 7 operating systems using MATLAB 7.3 but is not
% limited to this windows version. A "Windows Classic" desktop display may
% be required.
%
% The ScAnVP package was developed in-house at the Feinstein institute
% in collaboration with members of the PET Functional Brain Imaging Laboratory.
% It is designed to be used as a stand alone set of routines or in conjunction
% with other research pachages such as SPM. One of the primary modules of
% ScAnVP incorporates an implementation of the scaled subprofile model of
% principal component analysis (SSM-PCA) for regional and voxel based spatial
% covariance image analysis applied to derive characteristic imaging disease
% pattern network biomarkers used to study neurodegenerative disorders.
%
% ScAnVP is available as copyright software that is free under the terms of the
% GNU General Public License version 2 as published by the Free Software Foundation 
% (or any later version at your option). License terms can be read by invoking
% 'show license' or by visiting http://www.gnu.org/copyleft/. ScAnVP comes with
% ABSOLUTELY NO WARRANTY and is not intended for commercial use.
% SPM software can be obtained by visiting http://www.fil.ion.ucl.ac.uk/spm.
% License to use MATLAB must be obtained from MathWorks, Inc..
%
% For general information and help see the ScAnVP menu help utilities.
% For help on menu items, set the 'Help' button ON and select individual
% routines. Run time help buttons and tooltip strings are also provided.
%
%			Phoebe Spetsieris, Ph.D.
%			Research Associate Investigator
%			Functional Brain Imaging Laboratory	
%			Center for Neurosciences
%			Feinstein Institute for Medical Research
%			Northwell Health
%			350 Community Drive
%			Manhasset, New York 11030
%
%			Send comments to:
%			pspetsie@northwell.edu
%_______________________________________________________________________

% Previous ScAnVP version release dates
%		C.0-    May 1998,        1st PC Version, 
%		4.7.8-  Nov. 08, 2002,   First public release
%		5.0-    Feb. 11, 2003,   First GNU version
%       5.7.w-  Nov. 13, 2006,   Second public release
%       5.8.w-  Apr. 03, 2007,   Third public release
%       5.8.2.w-July 19, 2007,   Fourth public release
%       5.9.1.w-May 14,  2008,   Fifth public release
%		6.2w-   Feb. 26, 2013,   Sixth public release   
%		6.2.1w- Apr. 29, 2013,   Seventh public release
%		6.3.w-  Feb. 18, 2014,   Eighth public release
%		7.0w-   Feb. 01, 2016,   Ninth public release   
%
%-Clear the Command window & delete all figures
%-----------------------------------------------------------------------
clc
% close all

%-----------------------------------------------------------------------
%-Open startup window, set window defaults
%-----------------------------------------------------------------------

global HelpMode hrev FMenu
global Ecolor Mcolor HDRTYPE NIFTITYPE 
global Bcolor Fcolor
global BkgColor

global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN EXPTIM DURATION
global MATYPE DATAMIN DATAMAX VAL_OFFSET Message GCM HelpMsg UFI
global Keyin NameLst  
global SSize MenuWid MenuHgt
global gPrevDirs gFileDir
global Scattering
global InputOrient DisplayOrient ActiveOps DefAtlasNm

ScAnVPc= ['Copyright (C) 2016'];

FMenu=findobj('Tag','ScAnVP MENU');

if (HelpMode & FMenu), vp_help(mfilename), return ; end;

if ~isempty(FMenu)

	if(strcmp(get(FMenu,'Visible'),'on'))
		figure(FMenu);
	end
	return
end

Message=['Welcome to ScAnVP '];

HelpMsg='See MATLAB Help Window';
	
HelpMode=0;

Ecolor = [.9,.8,.9];
HEcolor = [.9,.5,.8];
HEBcolor = [.9,.8,.9];
Mcolor = [.7, .6, 1.0];
%BkgColor= [.2 .4 .6]; % Gray Blue
%BkgColor= [.0 .0 .0]; % Black Image Screen Background Color
%BkgColor= [.2 .3 .4]; % Dark Blue
BkgColor= [.3 .3 .4]; % Greyish purple
%Wcolor = [.72, .70, .78]; % MENU background color
Wcolor = [.65, .75, .75]; % MENU background color (Catalina Blue)

Bcolor = [.9, .9, .9];
Tcolor = [.0, .1, .1,];
Txcolor = [0.3, 0.0, 0.1,]; % MENU text color

SSize = get(0,'ScreenSize');
%%MenuX=10;
MenuX=SSize(3)/10;
MenuY=0.5*SSize(4);
%MenuWid=365;
MenuWid=373;
%MenuHgt=355;
MenuHgt=375;
%MenuWid=SSize(3)/6;
%MenuHgt=SSize(4)/4;
Scattering=0;

%To invert the background screen color and text on printout
%set the following figure property to 'on' and restart:
%(It can also be set in utilities.)
set(0, 'DefaultFigureInvertHardCopy', 'off');

%Set Global Defaults
InputOrient=1; %Radiological Default
DisplayOrient=2; %Neurological Display Only
ActiveOps=1; %Active control options
DefAtlasNm='ROIAtlasC';
vp_defaults;

HDRTYPE='ANALYZE'; % ANALYZE or NIFTI
NIFTITYPE=0;  % 0; .img, .hdr, % 1: .nii

vp_help('vp_helpbutton');

FMenu = figure('Color',Mcolor,...
	'Name','ScAnVP',...
	'NumberTitle','off',...
	'Position',[MenuX,MenuY,MenuWid,MenuHgt],...
	'Resize','on',...
	'Tag','ScAnVP MENU',...
	'Pointer','Watch',...
	'Visible','off');

	w=which('scanvp');
	cf=[w(1:(max(findstr(w,'\'))-4)),'col\jetbk64'];
	load(cf); colormap(CM); 

	savdir=pwd;
        %wdir=vp_nm('dir',which('scanvp'));
	wdir=[w(1:(max(findstr(w,'\'))-4)),'work'];
	try cd(wdir); load workparams; cd(pwdlast); , catch if exist(wdir) ~=7, mkdir(wdir); end; cd(savdir); , end;
	%gPrevDirs % test
  
GCM=colormap; % Set Global colormap

%-Frames and text
%-----------------------------------------------------------------------
AxM=axes('Position',[0 0 1 1],'Visible','Off');

backimg=ones(128,128); for i=1:27, backimg(i,:)=i+17; end;
for i=28:71, backimg(i,:)= 72-i ; end;
image(backimg); axis off; clear backimg;
colormap jet;
 
AxM=axes('Position',[0 0 1 1],'Visible','Off');
text(0.44,.95,'ScAnVP',...
	'FontName','Times','FontSize',22,...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Center',...
	'ButtonDownFcn','vp_help(''scanvp'')',...
	'Color',Txcolor);

text(0.62,.93,'version 7.0w',...
	'FontName','Times','FontSize',12,...
	'FontWeight','Bold',...
	'HorizontalAlignment','Left',...
	'ButtonDownFcn','vp_help(''scanvp'')',...
	'Color',Txcolor);

axes('Position',[.15 .73 .8 .16],'Visible','off')

text(0.42,0.87,'Scan Analysis and Visualization',...
	'FontName','Times','FontSize',15,'FontAngle','Italic',...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Center',...
	'FontWeight','Bold',...
	'Color',[0 0 0]*.7);

text(0.43,0.55,' Center for Neurosciences, Feinstein Inst. for Med. Research',...
	'FontName','Times','FontSize',10,'FontAngle','Italic',...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Center',...
	'FontWeight','Bold',...
	'Color',[.5 0 1]*.7);


cbhelp = ['HelpOpt=get(hhelpmenu,''Value'');',...
	'w=which(''scanvp'');',...
	'scanvpdir=[w(1:(max(findstr(w,''\''))-5))];',...
	'if HelpOpt==2,',...
	'vp_help([scanvpdir,''\vpm\Contents'']);',...
	'elseif HelpOpt==3, vp_basicop;',...
	'elseif HelpOpt==4, vp_general;',...
    'elseif HelpOpt==5, vp_help(''vp_ssminfo'');',...
	'elseif HelpOpt==6, docfile=get_file(1,',...
        '  ''.*'',''SELECT FILE'',[scanvpdir,''\documentation'']);',...
        'if ~isempty(docfile), dos(docfile); end;',...
	'elseif HelpOpt==7, vp_revisions;',...
	'elseif HelpOpt==8, vp_naming;',...
	'elseif HelpOpt==9, vp_displayinfo;',...
    'elseif HelpOpt==10, vp_anahdr;',...
	'elseif HelpOpt==11, vp_fixhdr;',...
	'elseif HelpOpt==12, vp_fixtimes;',...
	'elseif HelpOpt==13, vp_fixorigins;',...
	'elseif HelpOpt==14, vp_errata;',...
	'elseif HelpOpt==15, vp_helpbutton;',...
	'elseif HelpOpt==16, vp_help(''scanvp'');',...
    'elseif HelpOpt==17, vp_help(''vp_License'');',...
	'else ; end;'];
	
cbproc1 = ['ProcOpt=get(hprocmenu1,''Value''); if ProcOpt == 2 ,',...
	'vp_append;',...
	'elseif (ProcOpt>= 3 & ProcOpt <=8), vp_cslices;',...
	'elseif ProcOpt== 9, vp_cvtype;',...
	'elseif ProcOpt== 10, vp_flipim;',...
	'elseif ProcOpt== 11, vp_interp;',...
	'elseif ProcOpt== 12, vp_marker;',...
	'elseif ProcOpt== 13, vp_zeronegs;',...
	'elseif ProcOpt== 14, vp_hemimage;',...
	'elseif ProcOpt== 15, vp_groupmask;',...
	'elseif ProcOpt== 16, vp_note(''For CROP can use SLICE Ops. or use IMVIEW for Rotate and Crop Operations'');',...
	'else ; end;'];
	
cbfun = ['FunOpt=get(hfunmenu,''Value''); if FunOpt == 2 ,',...
	'vp_imfun;',...
	'elseif FunOpt== 3, vp_getroiparams;',...
	'elseif FunOpt== 4, vp_getvoxparams;',...
	'else ; end;'];

% Check Previous Directories List. If new directory is in the list make it the last entry.
% Otherwisejust add it to the end of the list.
 
cbPreDirUpdate=[' in=0; gFileDir=pwd,  nds=size(gPrevDirs,1);',...
  'if nds==0,',... 
  '  vp=which(''scanvp'');',...
  '  gPrevDirs=strvcat(gPrevDirs,vp(1:max(strfind(vp,''\''))-5));',...
  '  if (exist(''spm.m'')==2), gPrevDirs=strvcat(gPrevDirs,spm(''Dir'')); end;',...
  'elseif (nds > 10), pre3=gPrevDirs(1:3,:);',...
  ' post7=gPrevDirs(nds-7:nds,:); gPrevDirs=strvcat(pre3,post7); end;',...
  'nds=size(gPrevDirs,1);',...
  'for i=1:nds,',... 
  '  if or( strcmp(deblank(gPrevDirs(i,:)), deblank(gFileDir)), ',...
  '  strcmp([deblank(gPrevDirs(i,:)), ''\''], deblank(gFileDir)) ),',...
  '  in=i; end; end;',...
  'if in==0, gPrevDirs=strvcat(gPrevDirs, gFileDir); ',...
  'elseif in~=nds,  ',...
  '  swapdir = gPrevDirs(nds,:);  ',...
  '  gPrevDirs(nds,:)=gPrevDirs(in,:); ',...
  '  gPrevDirs(in,:)=swapdir;  ',...
  'end;',...
  'nds=size(gPrevDirs,1); j=1;',...
  'for i=1:nds,',... 
  ' if (exist(gPrevDirs(i,:)) ==7),',... 
  '  gPrevDirs(j,:)=gPrevDirs(i,:); j=j+1;',...
  ' end;',...
  'end; if j ~= nds+1, gPrevDirs(j:nds,:)=''''; end;',...
  ''];

%%cbSaveParam=['try cd(wdir); save pwdlast pwdlast gPrevDirs; cd(pwdlast);, catch cd(pwdlast); end;'];

hframe=uicontrol(FMenu,'Style','Frame','Units','Normalized',...
'BackgroundColor',Wcolor,'Position',[15/MenuWid 10/MenuHgt 340/MenuWid 283/MenuHgt ]);

uicontrol(FMenu,'Style','Text','String','Help',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[210/MenuWid 270/MenuHgt 100/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)

uicontrol(FMenu,'Style','Text','String','Conversion',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'Units','Normalized',...
	'BackgroundColor',Wcolor,...
	'Position',[30/MenuWid 270/MenuHgt 100/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)

cbconvert = ['ConvOpt=get(hconvmenu,''Value''); if ConvOpt == 2 , vp_convertge;',...
	'elseif ConvOpt==3, vp_genmri; elseif ConvOpt==4, vp_cvtunix;',...
	'elseif ConvOpt==5, vp_cvtNIF2ANA; elseif ConvOpt==6, vp_cvtype;',...
	'elseif ConvOpt==7, vp_compress; end;'];
	
hconvmenu=uicontrol(FMenu,'Style','PopUp','String',...
	'Convert... | GE (PET)->ANA | GE (MRI)->ANA | UNIX<->DOS | NIFTI -> ANA | ANA DATA TYPE | GZIP',...
	'TooltipString',['menu: 1: PET GE (iee) to ANALYZE | 2: MRI GENESIS to ANALYZE',...
	' | 3: UNIX<-->DOS ANALYZE | 4: NIFTI -> ANALYZE | 5: CHANGE ANA-DATA TYPE ',...
	' | 6: GZIP un/compress'],...
	'Units','Normalized',...
	'Position',[25/MenuWid 241/MenuHgt 100/MenuWid 030/MenuHgt],...
	'CallBack',cbconvert,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
cbhelpon=['help vp_helpbutton; hch=get(gcf,''children'');',...
	' vp_note(''Help Mode ON. Pick task for help. Then turn OFF.''); HelpMode=1;',...
	' for i=1:length(hch), ',...
	' if ~strcmp(get(hch(i),''type''),''axes''),  ',...
	' if strcmp(get(hch(i),''style''),''text''),',...
	' set(hch(i),''Backgroundcolor'',HEBcolor); end; end; end;'];

cbhelpoff=['hch=get(gcf,''children''); for i=1:length(hch), ',...
	' if ~strcmp(get(hch(i),''type''),''axes''),  ',...
	' if strcmp(get(hch(i),''style''),''text''),',...
	' set(hch(i),''Backgroundcolor'',Wcolor); end; end; end;',...
	' vp_note(''Help Mode OFF'',2);'];

hhelpon=uicontrol(FMenu,'String','Help Mode','Style','Checkbox',...
	'FontWeight','Normal',...
	'TooltipString','ON/OFF Switch-  (OFF: Tasks Active | ON: Display Help)',...
	'Units','Normalized',...
	'Position',[137/MenuWid 248/MenuHgt 80/MenuWid 019/MenuHgt],...
	'CallBack',['if HelpMode ==0, ',...
	'set(FMenu,''Color'',HEcolor); set(gco,''BackgroundColor'',HEcolor);',...
	'set(gco,''FontWeight'',''Bold'');',...
	'set(hframe,''BackgroundColor'',HEBcolor);',...
	'eval(cbhelpon);',...
	'else HelpMode =0; ',...
	'set(FMenu,''Color'',Mcolor); set(gco,''BackgroundColor'',Bcolor);',...
	'set(hframe,''BackgroundColor'',Wcolor);',...
	'set(gco,''FontWeight'',''Normal'');',...
	'eval(cbhelpoff);',...
	' end; '],...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
hhelpmenu=uicontrol(FMenu,'Style','PopUp','String',['Help Info./Utilities ',...
        '| CONTENTS | BASIC OPERATIONS | GENERAL OVERVIEW | SSM-PCA OVERVIEW| DOCUMENTATION | REVISIONS ',...
        '| NAMING INFO | DISPLAY INFO | ANALYZE HDR | FIX HDR | FIX TIMES | FIX ORIGINS ',...
	'| ERRATA| Help Mode Switch| About ScAnVP | Show License'],...
	'TooltipString',['menu:  1: Contents | 2: Basic Tasks | 3: Task Overview | 4: SSM-PCA Info| 5: Documentation ',...
        '| 6:  Revisions | 7: File Naming | 8: Display Information | 9: Analyze headers |',...
	' 10: Fix Header | 11: Fix Time Header | 12: Fix Origins ',...
        '| 13: Error List | 14: About Help Mode Switch | 15: About ScAnVP | 16: License Info'],...
	'Units','Normalized',...
	'Position',[225/MenuWid 241/MenuHgt 120/MenuWid 030/MenuHgt],...
	'CallBack',cbhelp,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
		

cbinputorient = ['if HelpMode ==0,',...,
    'InputOrient=get(hinputorient,''Value'');',...
	'if InputOrient== 1, vp_note(''Input Radiological, Right is Left (RL), e.g. spm5 normalized (sw...)'',3);',...
	'elseif InputOrient== 2, vp_note(''Input Neurological, Right is Right (LR), e.g. spm99 normalized (sn...)'',3);',...
	'end;',...
    'else, set(hinputorient, ''Value'', InputOrient);',...
    'vp_note(''Allows user to specify the input orientation of image files'',3);',...
    'end;' ];
	
cbdisplayorient = ['DisplayHelp=get(hdisplorient,''Value'');',...
	'if DisplayHelp==1, vp_note(''See Display Orientation Help'',3); vp_displayinfo; set(hdisplorient,''value'',DisplayOrient);',...
	'elseif DisplayHelp==2, vp_note(''Neurological Display (LR): Anatomical Right is Displayed Right'',3);',...
	'end;'];
	
hdisplorient=uicontrol(FMenu,'Style','PopUp','String',[' Help | Display Neurological (LR) '],...
	'TooltipString',[' Neurological Display (LR): Anatomical Right is Displayed Right ',...
	' for Correctly Specified Input Orientation'],...
	'Units','Normalized',...
	'Position',[25/MenuWid 229/MenuHgt 138/MenuWid 010/MenuHgt],...
    'FontName','Times','FontSize',8,...
	'Value', DisplayOrient,...
	'CallBack', cbdisplayorient,...
	'BackgroundColor',[0 .4 .6],...
	'ForegroundColor',[1.0 1.0 1.0]);
	
hinputorient=uicontrol(FMenu,'Style','PopUp','String',[' Input Radiological Files (RL, sw)| Input Neurological Files (LR, sn)'],...
	'TooltipString',['Radiological: Right is Left (e.g. spm5 normalized (swname.img)) | ',...
	'Neurological: Right is Right (e.g. spm99 normalized (snname.img))'],...
	'Units','Normalized',...
	'Position',[163/MenuWid 229/MenuHgt 185/MenuWid 010/MenuHgt],...
    'FontName','Times','FontSize',8,...
	'Value', InputOrient,...
	'CallBack', cbinputorient,...
	'BackgroundColor',[0 .4 .6],...
	'ForegroundColor',[1.0 1.0 1.0]);
	
uicontrol(FMenu,'String','IMAGE VIEW',...
	'TooltipString','Display analyze image planes in 3 orthogonal views',...
	'Units','Normalized',...
	'Position',[32/MenuWid 189/MenuHgt 95/MenuWid 020/MenuHgt],...
	'CallBack','vp_imview',...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String','TRANSP',...
	'TooltipString','Display Transparent PET/MRI Superimposition',...
	'Units','Normalized',...
	'Position',[138/MenuWid 189/MenuHgt 68/MenuWid 020/MenuHgt],...
	'CallBack','vp_transp',...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String','Utilities...',...
	'TooltipString','Image Display Utilities',...
	'Units','Normalized',...
	'Position',[220/MenuWid 191/MenuHgt 90/MenuWid 019/MenuHgt],...
	'FontName','Times','FontSize',11,'FontAngle','Italic',...
	'CallBack','vp_imutil',...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);

   
uicontrol(FMenu,'Style','Text','String','Functionalization',...
	'TooltipString','Convert Analyze image to FDG glucose data.',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[25/MenuWid 158/MenuHgt 130/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	

uicontrol(FMenu,'Style','Text','String','Image Processing',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[200/MenuWid 173/MenuHgt 127/MenuWid 016/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
hfunmenu=uicontrol(FMenu,'Style','PopUp','String',['Image Fun... |FDG,FDOPA-Blood File |',...
	'ROI Params and PLOT |VOX Params -ROI File '],...
	'TooltipString',['1: VOX FDG, PATLAK blood input|',...
	'2: ROI PARAM & PLOT|',...
	'3: VOXEL PARAM - ROI input file'],...
	'Units','Normalized',...
	'Position',[25/MenuWid 130/MenuHgt 130/MenuWid 030/MenuHgt],...
	'CallBack', cbfun,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
hprocmenu1=uicontrol(FMenu,'Style','PopUp','String',['Supl. Image Ops ... | Append Images|',...
	' Slice Add | Slice Avg | Slice Crop | Slice Extract | Slice Pad | Slice Shift | Convert Type| Flip Axis/View |',...
	' Interpolate | Marker | Zero Negs. | Hemi-Image R/L |',...
	' Create/Mask Data |',...
	' Crop, Rotate '],...
	'TooltipString',['1: Append |',...
	'2: Slice AVG |',...
	'3: Slice ADD |',...
	'4: Slice EXTRACT |',...
	'5: Slice CROP |',...
	'6: Slice PAD |',...
	'7: Slice SHIFT |',...
	'8: ConvType |',...
	'9: Flip |',...
	'10: Interp.|',...
	'11: Marker |',...
	'12: Zero Negs. |',...
	'13: Hemi-Image|',...
	'14: Create/Mask Group Data|',...
	'15: CROP, ROTATE'],...
	'Units','Normalized',...
	'Position',[200/MenuWid 120/MenuHgt 137/MenuWid 030/MenuHgt],...
	'CallBack', cbproc1,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
uicontrol(FMenu,'String','Sl/Gp',...
	'TooltipString',['Individual Image Operations| Scalar Arithmetic| ',...
	'Thresh Low | Exclusive Thresh Range | Exclusive Mask | Log | ',...
	'Smooth Box Convolution | Global Normalize| Vector Normalize| ',...
	'Z-score image| Image Mean| Mask Image'],...
	'BackgroundColor',Bcolor,...
	'HorizontalAlignment','Left',...
	'Units','Normalized',...
	'CallBack', 'vp_imageop;',...
	'Position',[200/MenuWid 148/MenuHgt 45/MenuWid 023/MenuHgt],...
	'ForegroundColor',Tcolor);
	
uicontrol(FMenu,'String','Multi',...
	'TooltipString',['Multi Image Operations | ADD | ',...
	'AVG | DIV | MLT  '],...
	'BackgroundColor',Bcolor,...
	'HorizontalAlignment','Left',...
	'Units','Normalized',...
	'CallBack', 'vp_multi_imageop;',...
	'Position',[245/MenuWid 148/MenuHgt 45/MenuWid 023/MenuHgt],...
	'ForegroundColor',Tcolor);
	
uicontrol(FMenu,'String','Pair',...
	'TooltipString',['Pair-Wise Operations | ADD | ',...
	'AVG | DIV | MLT | EXC  '],...
	'BackgroundColor',Bcolor,...
	'HorizontalAlignment','Left',...
	'Units','Normalized',...
	'CallBack', 'vp_pair_imageop;',...
	'Position',[290/MenuWid 148/MenuHgt 45/MenuWid 023/MenuHgt],...
	'ForegroundColor',Tcolor);
	

uicontrol(FMenu,'Style','Text','String','Rois',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[20/MenuWid 106/MenuHgt 90/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
cbrois = ['RoisOpt=get(hroismenu,''Value'');',...
	'if RoisOpt== 2, vp_roi;',...
	'elseif RoisOpt== 3, vp_grouproi;',...
	'elseif RoisOpt== 4, vp_note(''For Thresholded Regions Use the GIS Map Routine'',3); vp_mask;',...
	'elseif RoisOpt== 5, vp_GISimg;',...
	'elseif RoisOpt== 6, vp_GISAimg;',...
	'elseif RoisOpt== 7, vp_VOIvalues;',...
	'end;'];
	
hroismenu=uicontrol(FMenu,'Style','PopUp','String',['Roi Routines... | ROI (Template) |',...
	' GROUP-ROI | ROI MASK | GIS ROI MAP | GIS ATLAS MAP | VOI-Plot/Save '],...
	'TooltipString',['1: ROI (Template Definition) |',...
	'2: Group ROI (Replay and Report) |',...
	'3: Mask definition (0|1) |',...
	'4: GIS ROI MAP 5: GIS ATLAS MAP |',...
	'6: VOI Plot/Save'],...
	'Units','Normalized',...
	'Position',[025/MenuWid 080/MenuHgt 85/MenuWid 030/MenuHgt],...
	'CallBack', cbrois,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
uicontrol(FMenu,'Style','Text','String','Stat. Maps',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[130/MenuWid 106/MenuHgt 100/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	

cbmap = ['MapOpt=get(hmapmenu,''Value'');',...
	'if MapOpt== 2, vp_statmap;',...
	'elseif MapOpt== 3, vp_spmlike;',...
	'elseif MapOpt== 4, vp_GISimg;',...
    'elseif MapOpt== 5, vp_GISAimg;',...
	'elseif MapOpt== 6, vp_groupmask;',...
	'elseif MapOpt== 7, vp_ratiomap;',...
	'end;'];
	
hmapmenu=uicontrol(FMenu,'Style','PopUp','String',['Maps... | Statistical Map |',...
	' SPM-like Results | GIS ROI Map/Mask | GIS ATLAS Map | Create/Mask Data | Ratio Maps'],...
	'TooltipString',['1: Statistical Map |',...
	'2: SPM-like Results Pixel Map |',...
	'3: GIS ROI Template Map/Mask |',...
    '4: GIS ATLAS Map |',...
	'5: Make Group Lower Threshold Voxel Mask or Mask Data |',...
	'6: Make Group Ratio Maps '],...
	'Units','Normalized',...
	'Position',[125/MenuWid 079/MenuHgt 115/MenuWid 030/MenuHgt],...
	'CallBack', cbmap,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	
uicontrol(FMenu,'Style','Text','String','Statistics',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[230/MenuWid 106/MenuHgt 100/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
	
cbstat = ['StatOpt=get(hstatmenu,''Value''); if StatOpt == 2 ,',...
	'if HelpMode ==0, if exist(''spm.m''),spm, else,',...
	'vp_message(''Error: SPM not installed''); end;',...
	'else vp_note(''Invokes SPM menu'',2); end;',...
	'elseif StatOpt== 3, vp_SSM;',...
	'else ; end;'];
	

hstatmenu=uicontrol(FMenu,'Style','PopUp','String',['Statistics... | SPM | SSM '],...
	'TooltipString',['1: Statistical Parametric Mapping |',...
	'2: Scaled Subprofile Model '],...
	'Units','Normalized',...
	'Position',[240/MenuWid 080/MenuHgt 85/MenuWid 030/MenuHgt],...
	'CallBack', cbstat,...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);
	

uicontrol(FMenu,'Style','Text','String','Control',...
	'FontName','Times','FontSize',10,...
	'FontWeight','Bold',...
	'BackgroundColor',Wcolor,...
	'Units','Normalized',...
	'Position',[060/MenuWid 56/MenuHgt 100/MenuWid 020/MenuHgt],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	

uicontrol(FMenu,'String','RESTART',...
	'TooltipString','Clear all, close all and Restart',...
	'Units','Normalized',...
	'Position',[040/MenuWid 040/MenuHgt 70/MenuWid 020/MenuHgt],...
	'CallBack',['if HelpMode ==0,',...
	' pwdlast=pwd; w=which(''scanvp''); wdir=[w(1:(max(findstr(w,''\''))-4)),''work''];',...
	' try cd(wdir); save workparams pwdlast gPrevDirs InputOrient; cd(pwdlast);, catch cd(pwdlast); end;',...
	' close all, clear all, clc, scanvp; figure(FMenu);',...
	' else vp_note([''RESTART: Aborts tasks, Clears Screen',...
	' and Restarts ScAnVP -- Set HELP Mode OFF'']); end;'],...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String','QUIT',...
	'TooltipString','Clear all, close all and EXIT',...
	'Units','Normalized',...
	'Position',[120/MenuWid 040/MenuHgt 70/MenuWid 020/MenuHgt],...
	'CallBack',['if HelpMode ==0,',...
	' pwdlast=pwd; w=which(''scanvp''); wdir=[w(1:(max(findstr(w,''\''))-4)),''work''];',...
	' try cd(wdir); save pwdlast pwdlast gPrevDirs; cd(pwdlast);, catch cd(pwdlast); end;',...
	' close all, clear all, clc, fprintf(''Bye...\n\n>> '');',...
	' else vp_note([''QUIT: Aborts Tasks, Clears Screen and',...
	' Exits ScAnVP -- Set HELP mode OFF'']); end;'],...
	'BackgroundColor',Bcolor,...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String','MESSAGE',...
	'TooltipString','Display status messages',...
	'Units','Normalized',...
	'Position',[240/MenuWid 045/MenuHgt 80/MenuWid 015/MenuHgt],...
	'FontName','Times','FontSize',8,'FontAngle','Italic',...
	'CallBack', 'vp_message(Message);',...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String','Browser/Current Dir',...
	'TooltipString','Set Browser to MATLAB Current Directory',...
	'Units','Normalized',...
	'Position',[193/MenuWid 020/MenuHgt 157/MenuWid 020/MenuHgt],...
	'FontName','Times','FontSize',9,'FontAngle','Italic',...
	'CallBack', ['gFileDir=pwd; try  gf=uigetdir(gFileDir);',...
	' if(gf ~=0) gFileDir=gf; cd(gFileDir); end; ',... 
	' catch, filebrowser; end; eval(cbPreDirUpdate);'],...
	'ForegroundColor',Tcolor);

uicontrol(FMenu,'String',ScAnVPc,'ToolTipString',ScAnVPc,...
	'Units','Normalized',...
	'ForegroundColor','w',...
	'BackgroundColor',Wcolor,...
	'FontName','Times','FontSize',09,'FontAngle','Italic',...
	'HorizontalAlignment','left',...
	'ButtonDownFcn','vp_help(''vp_License'')',...
	'Callback','vp_help(''vp_License'')',...
	'Position',[20/MenuWid 20/MenuHgt 165/MenuWid 015/MenuHgt])
	
	
pos=get(FMenu,'position');
%pos=[pos(1) pos(2) pos(3)*1.05 pos(4)*.90];
pos=[pos(1) pos(2) pos(3)*.90 pos(4)*.85];
set(FMenu,'position',pos);

%dos('set path = %path%;c:\scanvp\bin;'),

vp_note(['Set MATLAB Current Directory. Default: < ',cd,' >'],2);
filebrowser;

set(FMenu,'Pointer','Arrow','Visible','on')

