function [h,p,ci,stats] = tutest2(x,y,alpha,tail)
%TUEST2 Hypothesis test: Compares the averages of two samples.
%   [H,P,CI,STATS] = TUTEST2(X,Y,ALPHA,TAIL) performs a t-test to
%   determine whether two samples from a normal distribution (with
%   unknown but significantly unequal variances) could have the same mean.
%
%   The null hypothesis is: "means are equal".
%   For TAIL =  0  the alternative hypothesis is: "means are not equal."
%   For TAIL =  1, alternative: "mean of X is greater than mean of Y."
%   For TAIL = -1, alternative: "mean of X is less than mean of Y."
%   TAIL = 0 by default.
%
%   ALPHA is desired significance level (ALPHA = 0.05 by default). 
%   P is the p-value, or the probability of observing the given result
%     by chance given that the null hypothesis is true. Small values
%     of P cast doubt on the validity of the null hypothesis.
%   CI is a confidence interval for the true difference in means.
%   STATS is a structure with two elements named 'tstat' (the value
%     of the t statistic) and 'df' (its degrees of freedom).
%
%   H=0 => "Do not reject null hypothesis at significance level of alpha."
%   H=1 => "Reject null hypothesis at significance level of alpha."
%
%   Based on ttest2 for equal variance (MAYTLAB stats toolbox 6.5.0)
%   Copyright 1993-2002 The MathWorks, Inc. 
%   $Revision: 2.14 $  $Date: 2002/05/08 18:43:57 $
%
%   Modifications for unequal variance:
%	ScAnVP ver. 5.8 by P. Spetsieris  12/14/06
%	Ref: "Numerical Recipes, The Art of Scientific Computing"
%	Cambridge University Press, 1986, pg. 467 

if nargin < 2, 
    error('Requires at least two input arguments'); 
end

[m1 n1] = size(x);
[m2 n2] = size(y);
if (m1 ~= 1 & n1 ~= 1) | (m2 ~= 1 & n2 ~= 1)
    error('Requires vector first and second inputs.');
end
x = x(~isnan(x));
y = y(~isnan(y));
 
if nargin < 4, 
    tail = 0; 
end 

if nargin < 3, 
    alpha = 0.05; 
end 

if (prod(size(alpha))>1), error('ALPHA must be a scalar.'); end
if (alpha<=0 | alpha>=1), error('ALPHA must be between 0 and 1.'); end

nx=length(x);
ny=length(y);
dfx = nx - 1;
dfy = ny - 1;
varx=var(x); vary=var(y);
dfe  = (varx/nx + vary/ny)^2/( ((varx/nx)^2/dfx) + ((vary/ny)^2/dfy) );

difference = mean(x) - mean(y);
pooleds    = sqrt( (varx/nx  + vary/ny) );

ratio = difference / pooleds;
if (nargout>3), stats = struct('tstat', ratio, 'df', dfe); end

% Find the p-value for the tail = 1 test.
p  = 1 - tcdf(ratio,dfe);

% Adjust the p-value for other null hypotheses.
if (tail == 0)
    p = 2 * min(p, 1-p);
    if (nargout>2)
        spread = tinv(1 - alpha / 2,dfe) * pooleds;
        ci = [(difference - spread) (difference + spread)];
    end
else
    if (tail == 1)
       if (nargout>2)
           spread = tinv(1 - alpha,dfe) * pooleds;
           ci = [(difference - spread), Inf];
       end
    else
       p = 1 - p;
       if (nargout>2)
           spread = tinv(1 - alpha,dfe) * pooleds;
           ci = [-Inf, (difference + spread)];
       end
    end
end

% Determine if the actual significance exceeds the desired significance
h = 0;
if p <= alpha, 
    h = 1; 
end 

if isnan(p), 
    h = NaN; 
end
