% function [cn, lscores, dev, OutImg, vafn, pcombo] =vp_GIS_coeff(gisfnames, method)
% Use logistic or linear regression to determine best coefficients for GIS addition.
%
% This function reads in a list of GIS fnames and locates the corresponding
% score files. It then reads in the score values for patients and controls
% and formats values into a predictor matrix X (scores) and Y a binary vector
% (1: patient, 0: control) that are input into the MATLAB function 'glmfit' 
% to determine the best coefficients to fit a generalized linear model 
% for a binomial or normal distribution. The object is to logistically
% or linearly combine these scores into a resulting score set that 
% that can best differentiate between patients and controls.
% The resulting score set is output as a temporary combined Score File.
% The program also reads the %Vaf (variance accounted for) for each GIS
% and calculates the total %Vaf for the linear GIS combination by adding 
% the component Vaf values modified by the square of their coefficients.
% This calculation assumes orthogonal GISs from the same data set.
%
% OUTPUT:
%
% cn (nGISs): A normalized horizontal vector of numerical coefficient estimates.
% lscores:    A logistic linear combination of the input scores
% dev:        The deviance of the fit
% OutImg:     The global OutImg default is set for the GIS addition file name
% vafn :      A horizontal vector of component GIS %Vaf values and the combined value.
% pcombo :    The combined p value of the 2-sample t-test. (Global value)
%
% An Combined SSF Score file is output in the user directory
%
%________________________________________________________________________
% function vp_GIS_coeff  ScAnVP 6.1	P. Spetsieris 04/16/09, 2/18/10
%                           Rev.6.2     P. Spetsieris 09/26/11, 4/16/12 
%
function [cn, lscores, dev, OutImg, vafn, pcombo] =vp_GIS_coeff(gisfnames, method)

global NameLst P2 OutImg

figsave=gcf; % Save Calling Figure Number; 

ngiss=size(gisfnames, 1);

scoreset=[]; subid=[];
P2=-1; 

gischs=[];

lscores=[]; OutImg=[]; pcombo=[];
cn=[]; dev=[]; stats=[]; 
Tvaf=0; vafn=[];

for i=1:ngiss

  gisfile=gisfnames(i,:),
  [nm, dnm, dir]=vp_nm('nm',gisfile),

  [ngisno, nmbase]=vp_num3str(nm);
  ln=length(nmbase); if ln > 3, nmbase=nmbase(1:ln-3); end;

  ssffile=[dir, nmbase, 'VScores', num2str(ngisno), '.txt'],
  gischs=[gischs num2str(ngisno)];
  if exist(ssffile,'file')
    [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2, vafn(i)] =vp_ssm_scores(-1,ssffile);
    if norm1==0, vp_note('ERROR: EACH SCORE FILE MUST INCLUDE CONTROL SUBJECTS'); return; end;
    nsubi=size(ssfscores,1), 
    if (ngis ~= 1), vp_note('WARNING: SHOULD BE SINGLE GIS SCORE FILES', 3); end;
    if i==1,
	nsubs=nsubi; nn1=norm1; nn2=norm2;
	subid=ones(nsubs,1); for j=nn1:nn2, subid(j)=0; end;
	scoreset=zeros(nsubs,ngiss);
	scoreset(:,1)=ssfscores(:,1); 
    else
    	if (nsubi ~= nsubs), vp_note('FATAL: UNEQUAL NO OF SUBJECTS IN GIS FILES'); return; end;
    	if (nn2 ~= norm2), vp_note('FATAL: UNEQUAL CONTROLS IN GIS FILES'); return; end;
	scoreset(:,i)=ssfscores(:,1);
    end
  else, vp_note([ssffile, ' DOES NOT EXIST']); return;
  end

end % for no subs

% Fit using normal distribution


if (strcmp(method,'linear')),
  [cn, dev, stats]=glmfit(scoreset,subid,'normal'),
  scn=length(cn); cn=cn(2:scn,:),
  normcn=cn/sqrt(sum(cn.*cn)); % vector normalize output
  % export horizontal vector 
  cn=normcn',
  lscores=scoreset*normcn;

elseif (strcmp(method,'logis')),

  [cn, dev, stats]=glmfit(scoreset,subid,'binomial','link','logit'),
  scn=length(cn); cn=cn(2:scn,:),
  normcn=cn/sqrt(sum(cn.*cn)); % vector normalize output
  % export horizontal vector of coefficients and vertical score vector 
  cn=normcn',
  lscores=scoreset*normcn;
end

% Calculate Total Vaf Assuming Orthogonal GISs from the same PCA analysis:
% Tvaf= sq(Coef1)xVaf1 + sq(Coef2)xVaf2 +... sq(Coefn)xVafn.

  Tvaf= (cn.*cn) * vafn',
  vafn(ngiss+1)=Tvaf,


%write Temporary Score file in user directory

  gissno=length(gischs); if gissno < 6, gissno=str2num(gischs); end;
  dir=pwd;
  outfile=[dir, '\', nm(1:ln-4), 'ADD_VScores', num2str(gissno), '.txt'],

  vp_write_ssmfile(outfile,gissno,Tvaf,NameLst,lscores,4,norm1,norm2);
  [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2] =vp_ssm_scores(-1,outfile);
  outfile=[dir, '\', nm(1:ln-4), 'ADD_ZScores', num2str(gissno), '.txt'],
  vp_write_ssmfile(outfile,gissno,Tvaf,NameLst,Zssfs,5,norm1,norm2);

%Set the global default for the outimg name
  OutImg=[dir, '\', nm(1:ln-4), 'ADD_gis', num2str(gissno), '.img'],

  figure(figsave);

  pcombo=P2;
  vp_note(['Note Combined P2 value =', num2str(P2), ', %vaf = ', num2str(Tvaf)], 10);
