% vp_GISimg.m
% Creates a GIS ROI Statistical Image Map or ROI weighted mask.
%___________________________________________________________________________
% 
% This routine creates qualitative visual representations (PET/MRI) of GIS
% regional weight statistical output of SSM ROI analysis. It also produces 
% a pixelated output image file that can be input into the 'Statistical Map'
% or 'SPM-like Results' routines or can be used as a ROI based voxel mask
% once GIS weight values are converted to 0/1 (see below).
%
% The routine reads 1) a GIS file (namegis.txt) listing 
% region names and their corresponding weights, 2) region of interest 
% templates defining the original geometric specifications and thresholds
% of the regions and 3) a normal subject or average image file.
%
% The GIS file has one of the following formats: 
% a) The file may be a formatted output file from a previous ROI pca analysis.
% b) The file may be formatted as follows:
% The initial line should read 'NRois:' followed by the number of 
% regions. The routine will read that number of text lines. 
% Each line is preceded by an optional letter (L or R) representing
% left or right orientation followed by the region name and weight.
% The region name and orientation must be identical to that in the template file.
%
% The method used for regional thresholding can be set to correspond to
% that of the original 'grouproi' analysis for a closer simulation of
% significant pixels. Options include fixed and variable 2D and 3D thresholding.
%
% The program outputs an ANALYZE format image file 'namegisroi.img'.
% The output volume has all pixels outside of regional and threshold
% limits set to zero. Internal regional values can be set to a constant
% corresponding to the region weight or 'Max Tapered' so that the maximum
% corresponds to the region weight and other pixels are to some degree modulated
% by the foreground pixel variation (to better distinguish brain topology and
% the relative contribution of regional pixels).
%
% VOXEL MASK
% For a 0/1 output mask, the output image can be converted to 0/1 values using
% Image Processing (IP) options or the input "GIS" file weights can be previously
% set to 1. If only one region is entered, the "Max Tapered" fill option should be
% used and the output image can be conveted to 0/1 values using IP options. 
%
% The program operates in both automatic and interactive modes.
% In interactive mode, the regional shape and positions can be adjusted
% as in 'grouproi'. The 'GIS PROFILE MAP' option creates axial planar 
% visuals of regions specified by the GIS file over a user specified
% background image (usually MRI). These visuals can be printed or saved 
% as Matlab figures and in other formats.
%
% The complete gis weighted pixel volume 'namegisroi.img' (without the background)
% is saved as an ANALYZE format image and can be input into the 'statmap'
% routine and remapped onto a background (MRI) image to create a 'statmap.img' 
% representing the GIS. This volume can then be viewed with axial, sagittal
% or coronal multiplanar viewing options. In this representation
% the upper half of the colormap is assigned to the statistical map values
% and the lower half to background and can be manipulated separately using
% the 'semirange' routine. See also (grouproi.m, statmap.m & semirange.m).
%
% Pre and Post smoothing options are provided. Pre smoothing is a slower
% process but eliminates artifacts. Post smoothing in 'interactive' mode
% may create regional outline artifacts. The regional outlines are
% automatically fitted to the new dimensions.
%___________________________________________________________________________
% vp_GISimg.m	C.4.6 	P. Spetsieris	December 6, 2000
%

global HelpMode GisFName Message
global FI FNames FName gisgui hgiserr Automatic PlaneNo0 PlaneNo
global RoiFNames TemplImg CompImg PlaneImg
global ThreshMode FixedThresh FixedThMin
global htemp Fitting Flipping Ecolor
global GisOri GisLab GisWeight
global BgrndFName GMax3 GMin3 RangeMin RangeMax RI 
global Smoothing PreSmooth FILLType TaperFactor herror


if HelpMode, vp_help('vp_GISimg'), return ; end;

%vp_message('Sorry this routine is not currently operational.');

%                (1 - Thresh/100)*Max <= Pix Value <= Max,

delete(findobj('Name','GROUP ROIgui'));
delete(findobj('Name','grprogui'));
delete(findobj('Name','ROIgui'));
delete(findobj('Name','gROIgui'));
delete(findobj('Name','gisprogui'));

delete(findobj('Name','GIS gui'));

%gisgui=findobj('Name','GIS gui');
%if ~isempty(gisgui)
%	if(strcmp(get(gisgui,'Visible'),'on'))
%		figure(gisgui);
%	end
%	return
%end

vp_imframe;

FName=[];
RoiFNames=[' ',' '];

GisFName=[];
IdentGIS=0;

IdentBgrnd=0;
BgrndFName=[];

WndNo=0;
WndScl=4;
PlaneNo=1;
Message0=['............................................ '];
Message=Message0;
Automatic=0;
IdentSubj=0;
DefTemplate=0;
TemplImg=0;
PlaneImg=0;
CompImg=1;
% Note FILLType ==1, is not working well so use Type==2 with 0%
% Note made some changes, should check it again

FILLType=2;     % 1=CONSTANT; 2=MAX TAPERED
TaperFactor=0;  % Percent Tapering for FILLType 2
Smoothing=0;    % 0=OFF; 1=ON
PreSmooth=0;    % 0=OFF; 1=ON
ThreshMode=1;
FixedThresh=100;
FixedThMin=0;
Fitting=1;
Flipping=0;

Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];
Fcolor = [.8,.8,.9];

gisgui = figure('Color',Fcolor,'Name','GIS gui',...
'NumberTitle','off','Position',[100,200,250,285],...
'Units','Pixels');
	

cbtext= ['text(28,70,''Region Fill Type'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

cbtext2= ['text(10,95,''Threshold'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

cbtext3= ['text(25,46,''Pre-Smoothing'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

eval(cbtext);

eval(cbtext2);

eval(cbtext3);

axis off;

uicontrol(gisgui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[141 265 40 17],'callback','vp_help(''vp_GISimg'');');

%	'eval([''type '' GisFName]); [IdentGIS, Message]=vp_gisfun(''readgis'',GisFName);',...
%	'if IdentGIS, vp_dispfile(GisFName), end;',...
%	'set(hgiserr,''string'',Message); pause(1);',...
%
pcbgis=['GisFName=get_file(1,''*gis*.txt'',''please select GIS file.'',[]); ',...
	'if ~isempty(GisFName),',...
	'IdentGIS=vp_readgis(1,GisFName);',...
	'if IdentGIS==2, vp_note(''ERROR: GIS FILE MUST BE FORMATTED''); end;',...
	'set(hgiserr,''string'',Message); end;'];
	
	
pcbgrnd=['BgrndFName=get_file(1,''.img'',''please select background image'',[]), ',...
	'if ~isempty(BgrndFName),',...
	'FName=BgrndFName; vp_fimage;',...
	'[GMax3 GMin3]=vp_roifuns(''gminmax'');',...
	'if isempty(Message), IdentBgrnd=1; PlaneNo=max(PlaneNo-1,1); Message=[''BGRND :  '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), IdentBgrnd=1; end;',...
	'set(hgiserr,''string'',['' '',Message]); end;'];
	

pcbsubj=['FNames=get_file(1,''.img'',''please select subject image or average'',[]), ',...
	'if ~isempty(FNames),',...
	'FName=FNames(1,:); vp_fimage;',...
	'figure(gisgui); eval(cbtext); axis off;',...
	'if isempty(Message), IdentSubj=1; PlaneNo=max(PlaneNo-1,1); Message=FName; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), IdentSubj=1; end;',...
	'set(hgiserr,''string'',['' '',Message]); end;'];
	
uicontrol(gisgui,'string', 'GIS FILE',...
	'TooltipString','Select SSM GIS File (Region Names vs. Weight)',...
	'position',[10 238 80 25],'BackgroundColor',Bcolor,'callback',pcbgis);
	

uicontrol(gisgui,'string', 'BGRND IMAGE',...
	'TooltipString','Pick Background Image file (MRI).',...
	'position',[140 238 100 25],'BackgroundColor',Bcolor,'callback',pcbgrnd);
	
uicontrol(gisgui,'string', 'SUBJECT IMAGE',...
	'TooltipString','Select subject, average image',...
	'position',[140 213 100 25],'BackgroundColor',Bcolor,'callback',pcbsubj);
	
cbRoiFName='';
	
	
%	'[nam,sfx]=strtok(RoiFNames(1,:),''.''); [pln,sfx]=strtok(sfx,''.'');',...
%	'if isempty(sfx), q=max([find(isletter(nam)==1) 0]); ',...
%	'sz=size(nam); PlaneNo=str2num(nam(q+1:sz(2))) ; nam=nam(1:q);',...
%	'else, PlaneNo=str2num(pln); end;',...
%


cbPRoiName=['if get(htemp,''value'')== 1, RoiFNames=get_file(2,''.roi'',''Please Select Roi Template File Limits (2 files)'',[]); ',...
	'else  RoiFNames=get_file(1,''.roi'',''Please Select Roi Template File'',[]); end; ',...
	'if ~isempty(RoiFNames),',...
	'[dnm,dnm,dir,nm,sfx]=vp_nm(''dnm'',RoiFNames(1,:)), ',...
	'[nam,pln]=strtok(nm,''.''), ',...
	'if isempty(pln), q=max([find(isletter(nam)==1) 0]), ',...
	'sz=size(nam); PlaneNo=str2num(nam(q+1:sz(2))) , nam=nam(1:q),',...
	'else, PlaneNo=str2num(pln(2:length(pln))); end;',...
	'nam=[dir,nam], PlaneNo,',...
	'set(hname,''string'',nam); ','Message=Message0 ; vp_fimage;',...
	'set(hgiserr,''string'',['' '',Message]);',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=FName;',...
	'elseif ~strcmp(Message,''Maximum Plane Reached''), end;',...
	'figure(gisgui); eval(cbtext); axis off;',...
	'set(hplane,''string'',PlaneNo); DefTemplate=1;',...
	'set(hgiserr,''string'',['' '',Message]); end;'] ;
	
	
%uicontrol(gisgui,'string', 'TEMPLATE',...
%	'position',[10 163 75 25],'BackgroundColor',Bcolor,...
%	'callback',cbPRoiName);
	
htemp=uicontrol(gisgui,'Style','Popup','string', 'TEMPLATE Limits | TEMPLATE Single Plane',...
	'TooltipString','Select Planar Limit Templates or Single Plane Template',...,...
	'position',[10 185 140 25],'BackgroundColor',Bcolor,...
	'callback',cbPRoiName);
	
%	'callback',['if get(htemp,''value'')==1, eval(cbPRoiName), else eval(cbP1RoiName), end;']);
	
hname=uicontrol(gisgui,'style','text','string',RoiFNames(1,:),...
	'position',[10 163 230 25],'BackgroundColor',Fcolor,'callback',cbRoiFName);

	
uicontrol(gisgui,'Style','Check','String','FIT',...
	'TooltipString','Resize Template to Fit Image Dimensions',...
	'Value',Fitting,'Position',[150 190 45 20],...
	'CallBack',[ 'if ~Fitting, Fitting=1; else Fitting=0, end;',...
	'set(hgiserr,''string'',''     '');' ]);
	
uicontrol(gisgui,'Style','Check','String','FLIP',...
	'TooltipString','Flip Correction for Y',...
	'Value',Flipping,'Position',[195 190 45 20],...
	'CallBack',[ 'if ~Flipping, Flipping=1; else Flipping=0, end;',...
	'set(hgiserr,''string'',''     '');' ]);
	

pcbplanedn=['Message=Message0; PlaneNo=PlaneNo-1; vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hgiserr,''string'',[''MSG:'',Message]);'];

pcbplaneup=['Message=Message0; PlaneNo=PlaneNo+1; vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hgiserr,''string'',[''MSG:'',Message]);'];

%	'set(hgiserr,''string'',[''MSG:'',Message]);'];

uicontrol(gisgui,'string', '<<VIEW',...
	'TooltipString','Decrease Plane No. for viewing only.',...
	'position',[106 142 50 20],'BackgroundColor',Bcolor,'callback',pcbplanedn);

uicontrol(gisgui,'string', 'PLANE>>',...
	'TooltipString','Increment Plane for viewing only.',...
	'position',[155 142 50 20],'BackgroundColor',Bcolor,'callback',pcbplaneup);

uicontrol(gisgui,'Style','popup','Visible','on',...
	'TooltipString','Select INTERACTIVE to modify ROIS and see GISMAP. ',...
	'String','INTERACTIVE | AUTOMATIC ',...
	'position',[10 137 95 25],'BackgroundColor',Bcolor,'callback',...
	['val=get(gco,''value''); if val==1, Automatic=0; else,',...
	'Automatic=1; end;']);

%uicontrol(gisgui,'string', 'AUTOMATIC',...
%	'TooltipString','Select INTERACTIVE to modify region positions or outline. ',...
%	'position',[10 137 75 25],'BackgroundColor',Bcolor,'callback',...
%	['if Automatic==1, Automatic=0; set(gco,''string'',''INTERACTIVE''); else,',...
%	'Automatic=1; set(gco,''string'',''AUTOMATIC''); end;']);


cbplane=['PlaneNo=str2num(get(hplane,''string'')); Message=Message0;',...
	'vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hgiserr,''string'',[''MSG:'',Message]);'];

hplane=uicontrol(gisgui,'style','edit','string',PlaneNo,...
	'TooltipString','Key in Plane for viewing only.',...
	'position',[205 142 35 20],'BackgroundColor',Kcolor,'callback',cbplane);


% Note FILLType ==1, is not working well so use Type==2 with 0%
cbFillMenu = ['FILLType=get(hFillMenu,''Value''); if FILLType==2, vp_tapergui;',...
		'else, FILLType=2; TaperFactor=0;  end;'];

cbThrMenu = ['ThreshMode=get(hthrmenu,''Value''); if ThreshMode > 4, vp_thmin;',...
	'elseif ThreshMode > 2, vp_thresh;',...
	'set(hgiserr,''string'',Message); end;'];

	
hthrmenu=uicontrol(gisgui,'Style','popup','Visible','on',...
	'TooltipString','Select method for selecting threshold minimum',...
	'String',...
	['VARIABLE %of 2D REG MAX|VARIABLE %of 3D REG MAX|FIXED %of 2D REG MAX|',...
	'FIXED %of 3D REG MAX|FIXED MINIMUM THRESHOLD'],'Tag','ThrMenu',...
    	'Position',[76 107 164 25],'callback',cbThrMenu);
	
hFillMenu=uicontrol(gisgui,'Style','popup','Visible','on',...
	'TooltipString','Select Region Fill Method',...
	'String',...
	'CONSTANT|MAX TAPERED','Tag','FillMenu',...
    	'Position',[120 82 120 25],'callback',cbFillMenu);
	
	
%uicontrol(gisgui,'Style','Check','String','TEMPLAT',...
%	'TooltipString',['Full Image Template Style Format-',...
%	'Warning Ovewrites input template if subject the same name'],...
%	'Value',TemplImg,'Position',[10 62 72 23],...
%	'CallBack',[ 'if ~TemplImg, TemplImg=1, else TemplImg=0, end; ',...
%	'set(hgiserr,''string'',''    '');' ]);
	
%uicontrol(gisgui,'Style','Check','String','Pre-SMOOTH',...
%	'TooltipString','Interpolate each plane before processing',...
%	'Value',PreSmooth,'Position',[10 62 110 22],...
%	'CallBack',[ 'if ~PreSmooth, PreSmooth=1; else PreSmooth=0, end;',...
%	'set(hgiserr,''string'',''     '');' ]);
	
uicontrol(gisgui,'Style','popup','Visible','on',...
	'TooltipString','Interpolate each plane Ntimes before processing',...
	'String',...
	'NO Pre-Smoothing|Pre_Smooth Once|Pre_Smooth Twice','Tag','PreSmoothMenu',...
    	'Position',[120 58 120 25],'callback',...
	['PreSmooth=(get(gco,''Value'') -1);  '] );
	

uicontrol(gisgui,'String','PROCESS Image',...
	'TooltipString','Initiate Processing.',...
	'Position',[10 35 110 25],...
	'CallBack',[ 'if ~IdentSubj, set(hgiserr,''string'',''ERR: SUBJECT Not Defined''); return; end; ',...
	'if ~DefTemplate, set(hgiserr,''string'',''ERR: Templates Not Defined''); return; end; ',...
	'if ~IdentGIS, set(hgiserr,''string'',''ERR: GIS Not Defined''); return; end; ',...
	'if ~IdentBgrnd, set(hgiserr,''string'',''ERR: Background Not Defined''); return; end; ',...
	'if ~Automatic & (ThreshMode==2 | ThreshMode==4), set(hgiserr,''string'',''ERR: Interactive 3D Not Operative''); return; end; ',...
	'if ~TemplImg &~PlaneImg & ~CompImg, set(hgiserr,''string'',''ERR: Image Type(s) Not Specified'');',...
	'return; end; ',...
	'if isempty(RoiFNames(1,:)), Message=''Template Not Defined'',',...
	'set(hgiserr,''string'',[''Error: '',Message]); return; end;',...
	'FName=FNames(1,:); vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1);',...
	'set(hplane,''string'',PlaneNo); PlaneNo0=PlaneNo; ',...
	'set(gisgui,''Visible'',''off''); drawnow; vp_GISpro;',...
	'else set(hgiserr,''string'',[''MSG: '',Message]);',...
	'if PlaneNo==DIM(3), vp_GISpro, set(gisgui,''Visible'',''off'');, end;',...
	' end;'] );
	
%	'CallBack',['set(gisgui,''UserData'',1)']);

	
uicontrol(gisgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Position',[180 35 60 25],...
	'CallBack',['delete(gisgui);','delete(FI);']);
%	'CallBack',['set(gisgui,''UserData'',1); ', 'delete(gisgui)']);

hgiserr = uicontrol(gisgui,'string', ['MSG : ',Message],...
	'TooltipString','Image error or message field',...
	'position',[10 10 230 25],...
	'CallBack',['set(hgiserr,''string'',[''MSG: '',Message]);'],...
	'BackgroundColor',Ecolor);

