% vp_SSM.m  Copyright (C) 2003-2016 Center for Neurosciences, FIMR. 
% Options for SSM (Scaled Subprofile Model) Statistical Analysis
%
% These routines implement a version of the SSM Scaled Subprofile Model
% first described by J.R.Moeller, et al (1987,1991,1994).
%
% For theoretical information see the main ScAnVP menu Help in SSM-PCA INFO.
%
% Basic routines were written by John D. Van Horn Ph.D. (1996) and modified
% by Marc Mentis M.D. (1998) who also implemented an initial MATLAB version
% and interface. The routines were substantially modified and adapted for the
% ScAnVP interface by P. Spetsieris Ph.D.. They include some contributions for 
% roi based SSM-PCA by Yilong Ma Ph.D. (2003) and voxel based orthogonal PCA by
% Chris Habeck Ph.D. (2002). Many new interactive and automatic options
% have since been added in the ScAnVP implementation by P. Spetsieris.
% GIS images resulting from the analysis can be visualized using
% ScAnVP statistical mapping routines which include SPM-like results.
%
% Operations are performed on common whole brain gray matter space determined
% as the group cross section of individual subject space and any external
% mask. It is assumed that excessive atrophy or hemi-image inclusion is
% limited to similar images to avoid zero or highly reduced common voxel space.
%
% Options:
%
% A)  PCA (SSM Principal Component Analysis)
%               Evaluation of GISs (Group Invariant Subprofiles)
%               and SSFs (Subject Scaling Factors).
% B)  TPR (Topographic Profile Rating)
%               SSF Score Evaluation for new subjects.
%               Highly Automated and Interactive options are available.
% C)  O-PCA (Principal Component Analysis in Orthogonal Space)
%               Evaluate and optionally remove selected component 
%               covariance from data set and perform PCA in
%               remaining orthogonal space.
% D)  SCORES    SSM analysis output files are read and bar graphs of
%               SSF subject scores can be displayed for selected GISs.
%
%
% 1) VOXEL BASED SSM:
%	Analysis is performed on Talairach normalized Analyze formatted image files.
% 
% 2) ROI BASED SSM:
%	Analysis is performed on ROI data of Talairach normalized, Analyze formatted
%	image files as tabulated in ScAnVP Group ROI Summary reports.
% 
% Additional options are under construction.
% Invoke specific task Help routines for more detailed help. 
%
% vp_SSM Interface    ScAnVP 4.7.8  P. Spetsieris  10/01/2002
%                            5.1                    7/15/2003
%                            5.2                    2/04/2004
%                            5.3                    8/06/2004
%                            5.4                    11/10/2004
%                            5.6                    11/07/2005
%                            5.7.w                  11/03/2006
%                            5.8.2                  7/12/2007
%                            5.9                    4/16/2008
%                            6.3                    1/10/2014
%_______________________________________________________________________
% REFERENCES
% 
%  J.R. Moeller, S.C. Strother, J.J. Sidtis and D.A. Rottenberg
%  "Scaled Subprofile Model: A statistical approach to the analysis
%  of functional patterns in positron emission tomographic data,"
%  J Cereb Blood Flow Metbol., 1987; vol.7, no.5, pp.649-658.
% 
%  J.R. Moeller, S.C. Strother, "A regional covariance approach
%  to the analysis of functional patterns in positron emission
%  tomographic data," J Cereb Blood Flow Metbol., 1991; vol.11,
%  suppl.1, pp.A121-A135.
% 
%  G.E. Alexander and J.R. Moeller, "Application of the scaled
%  subprofile model to functional imaging in neuropsychiatric
%  disorders: a principal component approach to modeling brain
%  function in disease," Hum. Brain Mapp., 1994, vol. 2, pp.79-94
% 
%  D. Eidelberg, J.R. Moeller, V. Dhawan, P.  Spetsieris,
%  S. Takikawa, T. Ishikawa, T. et al., "The metabolic topography
%  of parkinsonism," J Cereb Blood Flow Metab. 1994, vol. 14,
%  pp.783-801.
%
%  P.G. Spetsieris, Y. Ma, V. Dhawan, J. R. Moeller and D. Eidelberg
% "Highly automated computer-aided diagnosis of neurological disorders
%  using functional brain imaging," In: Reinhardt JM, Pluim JPW, eds.
%  Med. Imaging 2006: Image Processing, Proc. of SPIE, vol. 6144, pp.61445M1-12.
%
%  Ma Y, Tang C, Spetsieris P, Dhawan V, Eidelberg D. Abnormal metabolic network
%  activity in Parkinsons disease: Test-retest reproducibility. J Cereb Blood Flow
%  Metab 2007; 27(3):597-605. [NIHMS: 57714]
%
%  D. Eidelberg, Metabolic brain networks in neurodegenerative disorders: a functional
%  imaging approach, Trends in Neurosciences, vol. 32, no. 10, pp. 548-57, 2009.
%
%  Spetsieris P, Eidelberg D. Scaled subprofile modeling of resting state imaging data
%  in Parkinson's disease: Methodological issues, NeuroImage; 54(4):2899-914, 2011.
% 
%  Peng S, Ma Y, Spetsieris PG, Mattis P, Feigin A, Dhawan V, Eidelberg D.
%  Characterization of disease-related covariance topographies with SSMPCA toolbox:
%  effects of spatial normalization and PET scanners. Hum Brain Mapp, 2013; May 14
%  doi: 10.1002/hbm.22295. www.ncbi.nlm.nih.gov/pubmed/23671030
%
%  Spetsieris P, Ma Y, Peng S, Ko JH, Dhawan V, Tang CC, Eidelberg D. Identification
%  of disease-related spatial covariance patterns using neuroimaging data.
%  J Vis Exp, 2013; 76: e50319  Video: www.jove.com/video/50319/ Jun 25, 2013 - 14 min
%___________________________________________________________________________
%

global HelpMode
global POpt

if HelpMode, vp_help(mfilename); return ; end;

POpt=vp_opt('X','Select Type of SSM Analysis','VOXEL BASED','Voxel Based SSM','ROI BASED','ROI BASED SSM','EXIT','','HELP','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'VOXEL BASED')),
  vp_voxssm;
return; end;
if(strcmp(POpt,'ROI BASED')),
  vp_roissm; return; end;
%if(strcmp(POpt,'SCORES')),
% vp_ssm_scores; return; end;
if(strcmp(POpt,'HELP')),
  vp_help('vp_ssm');  
  vp_note('See HELP window. For additional help, invoke specific task HELP routines.');
  vp_ssm;
end
