% vp_VOIvalues.m
% Creates a plot or text file of VOI values at given points for a set of images
%___________________________________________________________________________
% 
% This routine creates a text file of values for single points or 3D
% spherical or cubic volumes centered at given points in real coordinate
% space for a user selected set of images.
%
% There are three options for reading in point coordinates. The user can
% either 1) Key in a text file, 2) Enter points on an image or 3) Key in points. 
% Text files should list the real X Y Z mm. coordinates of the points.
% The user must also select the image files. For groups of subjects, image
% files should be normalized to a common coordinate space such as MNI
% or refer to a common origin.
% Disk orientation of the input files is assumed to be that specified on
% the main menu. Display coordinates refer to Neurological orientation
% (positive x values correspond to anatomical right side of the brain in
% Axial and Coronal views). The VIEW display option will flip radiological
% disk input files. For the specified files, the entered coordinates will
% process the corresponding neurological space by obtaining the value of
% the right side of the brain for positive x for either input orientation 
% assuming input orientation is correctly specified on the menu.
%
% If more than one text file is entered or new points are keyed in or
% pointed to, the new points are added to the previous list.
% Subjects can be added as well assuming constant file orientation.
% The same subject or text file should not be entered twice.
% Options are provided to review or reset entries. 
%
% The point coordinate text file should have the following format: 
% Initial text is ignored until the keyword 'NPnts:' is found followed by
% the number of points. The routine will read that number of text lines. 
% Each line must contain the X,Y,Z coordinates of interest refering
% to distances in millimeters from the origin set in the header.
%
% Point values can be evaluated as that of the Nearest Point or the 
% average of all voxels within a Cubic or Sperical Volume of Interest
% centered at the given point for a given keyed in radius.
% To obtain 3D values, the routine evaluates adjacent image planes.
% Only the non-zero values are included in averages. However, all
% voxel coordinates, values and the min, max range of values within
% each evaluated volume are listed in the command window.
%
% Output values can be listed, plotted or saved. Plots can be values
% against file number (only 1 point/file) or point number (only 1 file).
% Output can be saved as a text file 'Name_VAL.txt' using the name of
% the first file. The output has the following format:
%   File#  Point#  Value
%   File Names
%   Point Normal Coordinates (mm) (No. xmm ymm zmm)
%
% Values of 0 usually refers to a voxels outside the masked image area
% and are not included in regional averages.
%___________________________________________________________________________
% vp_VOIvalues.m   C.4.7.7   P. Spetsieris   Feb. 27, 2002
%                 rev. 6.3   P. Spetsieris   Jan. 2, 2014
%                 rev. 6.3.1 P. Spetsieris   Dec. 2, 2014
%                 rev. 7.0   P. Spetsieris   Jan. 2, 2016

global HelpMode 
global ValFNames PNTFName 
global VOIgui hPNTerr 
global Ecolor FI
global PNTxyzmm PNTVals POpt RegType RegName RegRadius
global rPNTxyzmm InputOrient DisplayOrient

%global FName hrname htype hform hplane0 hmaxpln Message0


if HelpMode, vp_help('vp_VOIvalues'), return ; end;

delete(findobj('Name','VOI gui'));

ValFNames=[];

PNTFName=[];
IdentPNT=0;
PNTxyzmm=[];
rPNTxyzmm=[];
PNTVals=[];
RegType=0;
RegName='Point';
RegRadius=0;

PlaneNo=1;
Message0=['............................................ '];
Message=Message0;
IdentSubj=0;

Bcolor = [.8,.8,.8];
BPcolor = [.8,.9,.8];
Kcolor = [.8,1,1];
Fcolor = [.6,.8,.8];

VOIgui = figure('Color',Fcolor,'Name','VOI gui',...
'NumberTitle','off','Position',[50,200,250,250],...
'Units','Pixels');
	
axis off;

% ImgOrient=InputOrient; Defined on the main menu

pcbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

uicontrol(VOIgui,'string', ' Check RL/LR Input Disk File Orientation',...
	'TooltipString',['Check: Radiological (Right is Left) or Neurological (Right is Right)',...
    ' Input File Orientation.'],...
	'position',[10 223 185 20],...
	'BackgroundColor',Ecolor,'callback',pcbImgOrient);

uicontrol(VOIgui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[200 223 40 20],'callback','vp_help(''vp_VOIvalues'');');

pcbTXT=['PNTFName=get_file(1,''.txt'',''please select Point Coordinate file.'',[]); ',...
	'if ~isempty(PNTFName),',...
	'eval([''type '' PNTFName]); [out1, out2, Message]=vp_VOIfun(''readPNT'',PNTFName),',...
	'set(hPNTerr,''string'',Message); ',...
	'if out1>0, vp_dispfile(PNTFName),',...
	'Message=[''File read '',num2str(size(out2,1)),'' Points'']; ',...
	'PNTxyzmm=[PNTxyzmm ; out2]; IdentPNT=1; end;',...
	'set(hPNTerr,''string'',Message); end;'];
	
pcbsubj=['vfnames=get_file(inf,''.img'',''please select subject image(s)'',[]), ',...
	'if ~isempty(vfnames), ',...
	'IdentSubj=1; Message=vfnames(1,:);',...
	'ValFNames=strvcat(ValFNames , vfnames);',...
	'set(hPNTerr,''string'',['' '',Message]); end;'];
	
uicontrol(VOIgui,'string', 'TXT FILE  X Y Z mm',...
	'TooltipString','Select Point Coordinate Text File (Normalized X Y Z mm)',...
	'position',[10 195 110 20],'BackgroundColor',Bcolor,'callback',pcbTXT);
	
uicontrol(VOIgui,'string', 'SUBJECT IMAGES',...
	'TooltipString','Select subject images',...
	'position',[130 195 115 20],'BackgroundColor',Bcolor,'callback',pcbsubj);

%cbpnt=  ['if ~IdentSubj, vp_note([''Identify Subject''],1); else,'
cbpnt=  [' ',...
	' [out1 val xmm ymm zmm]=vp_impnt; if out1 >=0,',...
	' pntxyz=[num2str(xmm),'' '',num2str(ymm),'' '',num2str(zmm)]; ',...
	' pntxyz=[''XYZmm= '',pntxyz, ''; Pnt Value: '',num2str(val)], ',...
	' set(hPNTerr,''string'',pntxyz); ',...
	' vp_note(pntxyz); ',...
	' POpt=vp_opt(''Reject'',''Accept Coordinates?'',''Accept'','''',''Reject'','''');',...
	' if strcmp(POpt,''Accept''), vp_note([''Point Coordinates Accepted''],1);',...
	 ' IdentPNT=1; npnts= size(PNTxyzmm,1)+1;',...
	 ' PNTxyzmm(npnts,:)= [xmm ymm zmm]; ',... 
	' else vp_note([''Point Coordinates Rejected''],1); end;',...
	' set(hpnted,''string'',['' '']); ',...
	' else vp_note([''Invalid Point''],1); end;',...
	' set(hPNTerr,''string'',[''MSG: '',Message]);'];

cbimview = [' ',...
	'vp_note([''NOTE: Radiological images are flipped in display.''],3);',...
	'vp_imview;'];

uicontrol(VOIgui,'string','VIEW',...
	'TooltipString','Display Image',...
	'position',[10 164 45 20],'BackgroundColor',BPcolor,'callback', cbimview);

uicontrol(VOIgui,'string',' POINT TO  X Y Z ',...
	'TooltipString','Click on Point on Previously Displayed Image',...
	'position',[60 164 95 20],'BackgroundColor',BPcolor,'callback',cbpnt);

uicontrol(VOIgui,'string',['PICK FIG  (', num2str(FI), ')'],...
	'TooltipString','Pick An Image for Interactive Input - Use VIEW First',...
	'position',[160 164 82 20],'BackgroundColor',BPcolor,...
	'callback',['waitforbuttonpress; FI=gcf;',...
	'hpstr=[''PICK FIG  ('', num2str(FI), '')''];',...
	'set(gcbo,''string'',hpstr); ']);
	
cbtype=['RegType=str2num(get(htype,''string''));',...
	' if (RegType < 0 | RegType > 2), ',...
	'   Message =[''Invalid Type '', num2str(RegType) ];',...
	'   RegType=0; RegName=''Point'';',...
	'   set(htype,''string'',num2str(RegType)); ',...
	' else, if RegType == 0, RegName=''Point'', elseif RegType==1,',...
	'   RegName=''Cube'', elseif RegType==2, RegName = ''Sphere'', end;',...
	'   Message=[''Region Type '', RegName]; ',...
	' end; set(hPNTerr,''string'',Message); '];
	
uicontrol(VOIgui,'string', 'Type  0=Point',...
	'TooltipString','Single Point',...
	'position',[10 139 72 20],'callback',...
	['RegType=0; RegName=''Point'';',...
	' set(hPNTerr,''string'',[''RegType: '',RegName]);',...
	'set(htype,''string'',''0'');']);
	
uicontrol(VOIgui,'string', '1=CUBE',...
	'TooltipString','Type Cubic Volume',...
	'position',[82 139 60 20],'callback',...
	['RegType=1; RegName=''Cube'';',...
	' set(hPNTerr,''string'',[''RegType: '',RegName]);',...
	' set(htype,''string'',''1'');']);
	
uicontrol(VOIgui,'string', '2=SPHERE',...
	'TooltipString','Type Spherical Volume',...
	'position',[142 139 63 20],'callback',...
	['RegType=2; RegName=''Sphere'';',...
	' set(hPNTerr,''string'',[''RegType: '',RegName]);',...
	'  set(htype,''string'',''2'');']);
	
htype=uicontrol(VOIgui,'style','edit','string',num2str(RegType),...
	'TooltipString','Enter Region Type 0,1 or 2',...
	'position',[208 139 30 20],'BackgroundColor',Kcolor,'callback',cbtype);

       
cbradius=['radstr=get(hradius,''string''); ',...
	'[RegRadius,cnt,Message]=sscanf(radstr,''%f %f %f''),',...
	'if cnt == 1,  ',... 
	'vp_note([''Keyed Radius Accepted''],1); ',...
	'else vp_note([''Invalid Entry''],2); end;',...
	'if RegType == 0,  ',... 
	'vp_note([''Radius Insignificant for Region Type: Point''],1); end; ',...
	'set(hPNTerr,''string'',[''MSG:'',Message]);'];

hradius=uicontrol(VOIgui,'style','edit','string','0',...
	'TooltipString','Key in Radius in mm',...
	'position',[135 119 105 17],'BackgroundColor',Kcolor,'callback',cbradius);
	
uicontrol(VOIgui,'style','text','string','  KEY IN RADIUS (mm.) :',...
	'TooltipString','Key in Radius in mm.',...
	'position',[10 110 120 20],'BackgroundColor',Fcolor,'callback',cbradius);
	
cbkeypnt=['pntstr=get(hpnted,''string''); ',...
	'[A,cnt,Message]=sscanf(pntstr,''%f %f %f''),',...
	'if cnt == 3,  IdentPNT=1; npnts= size(PNTxyzmm,1)+1; PNTxyzmm(npnts,:)=A''; ',... 
	'vp_note([''Keyed Coordinates Accepted''],1); ',...
	' set(hpnted,''string'',['' '']); ',...
	'else vp_note([''Invalid Entry''],1); end;',...
	'set(hPNTerr,''string'',[''MSG:'',Message]);'];

hpnted=uicontrol(VOIgui,'style','edit','string',' ',...
	'TooltipString','Key in X Y Z mm Normalized coordinates',...
	'position',[125 89 115 17],'BackgroundColor',Kcolor,'callback',cbkeypnt);
	
uicontrol(VOIgui,'style','text','string','  KEY IN  X Y Z (mm.) :',...
	'TooltipString','Key in X Y Z mm Normalized coordinates',...
	'position',[10 85 110 20],'BackgroundColor',Fcolor,'callback',cbkeypnt);
	
uicontrol(VOIgui,'String','GET VALUES',...
	'TooltipString','Initiate Processing.',...
	'Position',[10 60 70 25],...
	'CallBack',[ 'GoOn=1; if ~IdentSubj, vp_note(''ERR: SUBJECT(s) Not Defined'',1); GoOn=0; end; ',...
	'if GoOn, if ~IdentPNT, vp_note(''ERR: Point(s) Not Defined'',1); GoOn=0; end; end; ',...
	'if GoOn, if ( (RegType>0) & (RegRadius<=0) ), vp_note(''ERR: Invalid Radius'',1); GoOn=0; end; end; ',...
	'if GoOn, rPNTxyzmm=PNTxyzmm; if (InputOrient==1), npnts=size(PNTxyzmm,1); ',...
	'  for i=1:npnts, rPNTxyzmm(i,1) =0-PNTxyzmm(i,1); end; end; InputOrient, rPNTxyzmm, end; ',...
	'if GoOn, [out1, PNTVals, Message]=vp_VOIfun(''PNTvalues'',rPNTxyzmm,ValFNames,RegType,RegRadius);',...
	'disp(''File#   Point#   Value'');',...
	'PNTVals, vp_note(Message);  end;',...
	'set(hPNTerr,''string'',[''MSG:'',Message]);'] );
	
uicontrol(VOIgui,'String','List Points',...
	'TooltipString','Show Coordinate Array in Command Window',...
	'Position',[80 60 55 25],...
	'CallBack',['vp_note(''Note coordinates in Command Window'',1); PNTxyzmm, ']);

uicontrol(VOIgui,'String','RST',...
	'TooltipString','Clear Points',...
	'Position',[135 60 27 25],...
	'CallBack',['IdentPNT=0; PNTxyzmm=[]; PNTVals=[];',...
	'vp_note(''Points Cleared'',1); ',...
	'set(hPNTerr,''string'',Message); ']);

uicontrol(VOIgui,'String','List Files',...
	'TooltipString','Show File Names in Command Window',...
	'Position',[162 60 55 25],...
	'CallBack',['vp_note(''Note names in Command Window'',1); ValFNames, ']);

uicontrol(VOIgui,'String','RST',...
	'TooltipString','Clear Files',...
	'Position',[217 60 27 25],...
	'CallBack',['IdentSubj=0; ValFNames=[]; PNTVals=[];',...
	'vp_note(''Subject Names Cleared'',1); ',...
	'set(hPNTerr,''string'',Message); ']);

	%' plot(PNTVals(:,2),PNTVals(:,3));',
cbplotmenu = ['val=get(gcbo,''Value'');',... 
	'npts=size(PNTxyzmm,1); ',...
	'if val == 2, ',...
	'if ~isempty(PNTVals), if npts> 1, ',...
	'vp_note(''Only works for 1 point/file'',2), else, figure;',...
	' plot(PNTVals(:,1),PNTVals(:,3));',...
	'xlabel(''File No.''); ylabel(''VALUE''); end;',...
	'else, vp_note(''Get values first'',1); ',...
	'set(hPNTerr,''string'',[''MSG:'',Message]); end;',...
	'elseif val == 3,',...
	'if ~isempty(PNTVals), nfls=size(ValFNames,1); if nfls> 1, ',...
	'vp_note(''Only works for single file 1'',2), end; figure;',...
	' plot(PNTVals(1:npts,2,1),PNTVals(1:npts,3,1));',...
	'xlabel(''Point No.''); ylabel(''VALUE''); ',...
	'else, vp_note(''Get values first'',1); ',...
	'set(hPNTerr,''string'',[''MSG:'',Message]); end;',...
	'end;'];

hplotmenu=uicontrol(VOIgui,'Style','popup','Visible','on',...
	'position',[10 33 93 25],'BackgroundColor',Bcolor,...
	'TooltipString','Select Plot Type',...
	'String',...
	['Plot Type...|Values vs. File#|Values vs. Point#'],...
	'Tag','PlotMenu',...
    	'callback',cbplotmenu);

%	'CallBack',['vp_note(''Note Values in Command Window'',1); PNTVals, ']);
%	
uicontrol(VOIgui,'String','List Values',...
	'TooltipString','Show Value Output in Command Window',...
	'Position',[103 35 60 25],...
        'CallBack',[' if isempty(PNTVals), vp_note(''Get Values first''); else,',...
	' disp(''''); vp_note(''Note Values in Command Window'',1);',...
	' disp(''   File#      Pnt#      Value'');',...
	'  PNTVals, end; ']);
	
uicontrol(VOIgui,'String','SAVE',...
	'TooltipString','SAVE Values in File',...
	'Position',[164 35 40 25],...
	'CallBack',...
	['title=['' POINT VALUE TABLE '']; if RegType > 0, title=[title,'' NON 0 Vox. AVG '',  RegName, '' Radius='',num2str(RegRadius),''mm.'']; end;',...
	'[out1,out2,Message]=vp_VOIfun(''savePNTS'',PNTVals,ValFNames,PNTxyzmm,title);',...
	'vp_note([Message,'' -Values Saved'']); ',...
	'set(hPNTerr,''string'',Message); ']);

uicontrol(VOIgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Position',[204 35 40 25],...
	'CallBack',['delete(VOIgui);']);

hPNTerr = uicontrol(VOIgui,'string', ['MSG : ',Message],...
	'TooltipString','Image error or message field',...
	'position',[10 10 234 25],...
	'CallBack',['set(hPNTerr,''string'',[''MSG: '',Message]);'],...
	'BackgroundColor',Ecolor);

