% ScAnVP (Scan Analysis and Visualization Processor) Version 7.0- (C) 2016
% Center for Neurosciences, Feinstein Institute for Medical Research
%
%                                  BASIC OPERATION
%
% The following notes refer to items that are of interest in the basic
% operation of ScAnVP.
%
%   STARTUP
%		ScAnVP is initiated by typing 'scanvp' in the MATLAB command window.
%		The path to the directory ...\scanvp\vpm must be previously setup:
%		In the MATLAB window go to File\Set Path\Add Folder and type in
%		the full vpm\ directory name on disk and then Save.
%
%   DIRECTORIES 
%		The scanvp directory contains the following subdirectory structure:
%		The main operation is controled by Matlab .m file scripts that are
%		in the scanvp\vpm directory. Various other subdirectories of scanvp\
%		contain files required for normal operation such as color files that
%		are stored in scanvp\col. Executable C (.exe) files are contained in
%		the scanvp\bin directory and are intitiated by scanvp\Matlab routines.
%		The system path enviromental variable does not need to include the
%		scanvp\bin directory for their operation within scanvp.
%
%   BROWSER	
%		The ScAnVP file browser 'get_file.m' is based on in-house code that
%		interfaces with Matlab functions. 
%		
%   INTERFACE
%		Routines are initiated from the ScAnVP main menu and usually
%		provide a GUI application task menu.
%		Interface features include:
%		 .Help and tooltip strings provided on most of the application menus.
%		 .A waitbar for many of the lengthly operations.
%		 .Pop up messages to indicate processing information.
%		 .Image Controls. Image windows include a bar at the top of the image
%		  with various functions including color selection, printing, text,
%		  and point value interogation. Some can also be invoked from the MENU
%		  utilities menu for images lacking controls.
%		 .Menus. The units for control buttons on the main ScAnVP menu as well
%		  as some submenus have been normalized to accomodate resizing.
%
%   CONVERSION
%		GE(PET)->ANALYZE (.iee conversion) routine converts GE PET .iee
%		images to ANALYZE format. The actual Frame Start Time (Clock Time)
%		is stored in the header and printed in the program output. Time
%		and other header parameters can be displayed by the 'fix header'
%		and 'ANALYZE hdr' routines.
%
%		NIFTI--> ANALYZE  Converts nifti format image header ((.img, .hdr)
%		and .nii type) to ScAnVP ANALYZE format.
%
%		UNIX<--> DOS conversion routine converts ANALYZE image headers
%		reversibly. If pixels have greater than 8bits per byte, the
%		corresponding image data files are also converted (bytes swapped).
%
%   HELP	
%		Help for ScAnVP individual tasks is displayed in the desktop
%		HELP window. Specific help routines include information, revision
%		and error listings and various header info and editing routines
%		such as "FIX HDR" and "FIX ORIGIN". 
%
%   FIX HDR
%		Using the "FIX HDR" routine, the user can edit a large number of
%		variables in an existing or new header. Defaults are provided.
%
%   Fix Origin
%		Allows the X Y Z voxel coordinates of the origin of an image or a
%		set of coregistered images to be keyed in by the user.
%
%   DISPLAY
%
%     PARAMETERS
%		Using convertGE, PET images are read as short values 128x128/plane
%		from the scanner's data base. Negative values are set to zero if
%		they are stored as 8 bit ANALYZE images.
%		Images with other dimensions or having negative values result from
%		normalization or other image processing operations. MRI images are
%		usually 256x256/plane and are saved as either 8 or 16bit values.
%
%     COORDINATES
%		Scanvp display routines such as 'IMAGE VIEW' (imview) calculate 
%		physical coordinates based on the voxel coordinates of the origin
%		and the physical dimensions of each voxel. These values are read
%		from the ANALYZE header and are stored in the MATLAB variables
%		ORIGIN and VOX. If the images are successfully normalized using
%		SPM99, the values are ORIGIN=[40 57 26] and VOX=[2 2 2]mm and the
%		image has dimensions 79x95x68. In spm5 dimensions are 79x95x69.
%		The physical position of the origin should correspond to that
%		defined by the normalized space of the SPM template which is
%		Talairach based in SPM99 and corresponds to MNI space in spm5.
%		When a normalized image is displayed by 'IMAGE VIEW', the cross in 
%		the three orthogonal views should initially point to the Talairach
%		origin. Coordinates of other points are measured mm from the origin
%		and should correspond to Talairach/MNI values for normalized images.
%		PET and MRI images are saved in the Radiological 'RL' orientation
%		(patient anatomical right is image left and left is right) but
%		they are converted to neurological format 'LR' (left is left)
%		by ScAnVP in this version only for display and coordinate evaluation.
%
%     IMAGE VIEW (imview)
%		This routine displays Analyze images of any sized voxels (Uchar,
%		Floating, Double, Int32 & Negative values). Orthogonal sections
%		and sequential slices are labeled numerically and/or physically (mm).
%		The Left<=>Right Orientation of the display is detailed under
%		"DISPLAY INFO" help.
%		Corrections are made to modify the aspect ratio of images
%		with unequal voxel dimensions so that displays reflect
%		physical dimensions rather than voxel dimensions.
%		Orthogonal view controls allow interactive repositioning of the
%		point through which orthogonal slices are displayed and display
%		the value and coordinates of the point as well as perform immediate
%		update following keyed coordinate specification.
%		Multiple slice options allow interactive selection of window size,
%		starting and ending slices and slice skip factors. Options are included
%		for surface and line plotting as well as 2D and 3D cluster removal.
%		Another option displays rotated views and can be used to create movies.
%
%     TRANSPARENCY
%		This option is used to view transparent superimposed sections of
%		a PET image over an MRI image with variable levels of transparency. 
%
%     UTILITIES
%		Various utilities can be invoked from the main menu or from 
%		a control bar on top of image figures. They include operations for
%		smoothing, reading a value of a point on an image, text placement,
%		colormap selection and colorbar placement, colormap manipulation,
%		thresholding and normalization of multiple image figures to a common
%		Min/Max range, deletion and movement of component elements, line and
%		surface plot creation, saving MATLAB figures in various other
%		standard image formats and displaying different format images as well
%		as previously created movies in a MATLAB figure.  
%
%       Range
%		A slider is provided to modify the center of the colormap 
%		range in addition to the upper and lower thresholds. The real data
%		global and threshold limits are displayed as well.
%		A "SEMI" range option is provided to enable the user to manipulate
%		separately the upper and lower segments of the colormap for merged
%		PET/MRI displays.
%		A "NORM" option is provided to allow the user to "normalize" all
%		displayed image figures to a common color scale.
%		The user can reset the global data range DATAMAX DATAMIN in both the
%		"NORM" utility and in the "FULL" range tool and redraw colorbars 
%		using the provided buttons.
%
%       Value
%		This function allows the user to point to an image voxel and get its
%		value, coordinates and physical label in Neurological orientation. 
%
%       Colors
%		Various pre-defined colormaps with one or more scales can be selected
%		on the color utility menu or from a set of .mat files in the
%		scanvp\col directory. Options are provided for color inversion,
%		background screen color manipulation and saving of custom
%		colormaps.
%
%       LinePlot/SurfPlot
%		These 'IMAGE VIEW' and 'Utilities' options create 2D and 3D plots of the
%		intensities of linear segment or planar surface of a planar area.
%		The plane may be in any orthogonal view. Coordinates are in mm from
%		the image origin specified in the file header. 
%
%       Movies
%		A GUI interface allows previously created MATLAB movie
%		frames to be played. Movies of rotated sectional views
%		can be automatically created using options on the orthogonal
%		view displays of the 'IMAGE VIEW' and 'Statistical Map' routines.
%
%
%   FUNCTIONALIZATION
%
%	  FDG Glucose:  Blood File Input
%			Converts raw PET images to regional Cerebral Metabolic Glucose
%			rates rCMRG (mg/100g/min).
%
%	  FDOPA Uptake: Blood File Input
%			Converts raw count Dynamic Analyze Images to Uptake 
%			units (micro liters/g/min). 
%
%	  ROI Parameters: Roi Input File, Varous Tracers
%			Performs ROI based graphic analysis- Produces SOR, Ki
%			and DVR plots and an output text file.
%
%	  VOX Parameters: Roi Input File, Varous Tracers
%			Creates voxel based Logan plots and Patlak maps saved as 
%			ANALYZE DVR and Ki parameter images.
%
%
%   ROI/GROUP ROI  Region of Interest Analysis
%
%       Using the ROI TEMPLATE creation routine users can create multiplanar
%       templates of irregular, elliptical and rectangular regions saved as
%       separtate files for each plane of interest. Regions can be drawn on
%       a PET image on a coregistered MRI image. Various options are provided
%       to manipulate the regions,delete or change their size and orientation,
%       label with standard and non-standard names, display statistics with
%       and without thresholding,import other templates, and display selected
%       (thresholded) voxels. Results are saved for each plane in (Name.Pln.roi)
%       files, which can be modified at any time, using the same program.
%       These files contain statistical data as well as geometric information
%       which can be used to recreate the regions for group replay.
%       The GROUP ROI routine uses .roi templates to generate various 2D
%       and 3D statistical reports. GROUP ROI can also use an ATLAS image
%       with predefined indexed regions based on the MNI standard template
%       (see GROUP ROI help for further help.) GROUP SUMMARY COMPOSITE
%       reports can be input into SSM ROI PCA routines for further analysis.
%       
%       MASK -  Analyze format volume image files are used to create 0/1
%		value masks by interactively defining regional non-zero areas.
%
%       VOI PLOT/SAVE - POINT, SPHERICAL and CUBIC Region Reports
%		This routine creates a text file of values at point coordinates given 
%		in physical units for a sequence of normalized images. Center
%		point coordinates can be input from a text file or interactively.
%		An average value of pixels within a specified radius can be evaluated 
%		for each point. Pixels can be averaged within a cubic or spherical
%		volume or can simply be the nearest point value. Basic plot options
%		are provided.
%
%
%   IMAGE PROCESSING
%       
%       ScAnVP includes numerous image processing routines that can act on
%       sets of multiple images or pair-wise to create a single output image
%       or act individually. These operations include CONVERT from any
%       TYPE to another, ADD, SUBTRACT & DIVIDE, MULT, EXTRACT or APPEND
%       SLICES, FLIP AXIS/VIEW, INTERPOLATE, EXCLUDE VOXELS, MASK, THRESHOLD,
%       SMOOTH, CROP, PAD, SHIFT, CREATE a HEMI-RIGHT or LEFT image, as well
%       as various normalization procedures, log transformation, Global Mean
%       tabulation, etc.. See individual task help for further details.  
%
%   STATISTICS
%
%	SPM
%		A Link is provided to invoke SPM assuming that it has been pre-installed.
%
%	SSM	(See SSM HELP and DOCUMENTATION)
%		Options include voxel and regional based SSM-PCA analysis and routines
%		for prospective scoring (TPR) with options for removing covariance 
%		components (orthogonal PCs) from the data. 
%
%   STATISTICAL MAPS
%
%      STATISTICAL MAP
%       This routine works for any type of ANALYZE format input image. The
%       default colormap used is a bi-level colormap with a multiple color
%       scale in the upper half assigned to statistical results and a graded
%       gray scale in the lower half assigned to background. The routine
%       accepts both lower and upper threshold limits as well as any 0/(NOT 0)
%       volume mask for filtering statistical data that is then mapped onto
%       a background image. An exclusive option is included for data
%       thresholding and masking. Other options include cluster removal and
%       surface plotting. Both thresholded images as well as composite images
%       with background are saved.
%
%
%      GIS ROI TEMPLATE MAP
% 
%       This routine creates visual map representations (PET/MRI) of GIS
%       regional weight PC vectors output from SSM ROI analysis of GROUP ROI
%       template data. The weight value of each region is mapped to all
%       voxels within the region defined by the template subject to user-
%       selected smoothing and threshold options. An output ANALYZE file 
%       without background can be input into the 'Statistical Map' or 'SPM-
%       like Results' routines or can be converted to a ROI based voxel mask.
%
%      GIS ATLAS MAP
%
%       This routine creates an ANALYZE image map of GIS ROI weight output
%       from SSM ATLAS GROUP ROI analysis by mapping the weight values to
%       the atlas defined rois. The resulting voxel image can be input into
%       the 'Statistical Map' or 'SPM-like Results' routines to create PET/MRI
%       representations or can be converted to a ROI based voxel mask.
% 
%      SPM-LIKE RESULTS
%       This option provides an interactive gui for Analyze format statistical
%       images such as SSM voxel analysis results and GIS ROI voxel maps to
%       create SPM-like representations using SPM based routines. All of the
%       SPM operations are not functional but 3-slice, sectional and volumetric
%       rendering are enabled. An extension was added to enable multi-slice
%       section views. 
%
%________________________________________________________________________________________
% vp_basicop.m  7.0              Phoebe Spetsieris, Ph.D.   Jan. 22, 2016

	vp_help('vp_basicop');
	
