% vp_cslices.m
%
% This routine allows the user to extract, combine, crop, pad or shift
% planar slices within a single Analyze or Nifti image to create a new
% Analyze image. The same operation can be performed on groups of image
% files processed individually.
% Extraction of multiple planes (slices) can be performed to limit the
% planar output range or addition or averaging of planes can be performed
% to combine two or more slices within an image into one output slice. 
% For cropping, a 3-D volume is extracted by specifying a rectangular 
% area to be included withn Z plane limits. 
% The corresponding Operations are denoted as: AVG, ADD, EXT, or CRP, 
% or SHF. For SHF operations the origin remains at the same coordinates.
% A default name is assigned, e.g., name.img -> nameEXT.img for each.
% The planes processed are indicated in the Analyze header.
% Planar X,Y limits are specified as voxel coordinates within the native
% disk volume. Thus Left/Right anatomical orientation is not considered.
% If this is important the user can re-evaluate a required X coordinate as
% (DIMX -X), where DIMX is the Axial Voxel Width of the image.
%
% For additional documentation on padding and cropping spatially
% normalized images of different dimensions see also the documentation
% file 'Crop_Pad_Normal.doc' invoked under Help --> Special Topics.
%
% vp_cslices calls vp_cslicespro to perform the slice operations:
%
% imop='AVG':	Average Slices
% imop='ADD':	Add Slices
% imop='EXT':   Extract Slices
% imop='CRP':   Crop and Extract Slices
% imop='PAD':   PAD image with zero values before or after outer
%		limit voxels by the specified +/- DX, DY, DZ
% imop='SHF':   Shift image voxels by DX, DY, DZ but not origin
%		The original volume size and origin is maintained
%		but the volume within the plane limits is shifted
%		by the specified +/- DX, DY, DZ
%___________________________________________________________________________
% vp_cslices.m	C.4.5	P. Spetsieris  Feb. 22, 2000
% 	     Rev. 5.8	P. Spetsieris  March 7, 2007
% 	     Rev. 6.1	P. Spetsieris  March 11,2011
% 	     Rev. 7.0	P. Spetsieris  Jan. 14,2016


global Plane0 MaxPlane MaxDim FName Message
global DIM VOX SCALE TYPE OFFSET ORIGIN GCM SliceDelta FI hrname
global HelpMode OP Ecolor

if HelpMode, vp_help(mfilename), return ; end;

FName=[];
FNames=[];

%Set image user interface controls
%vp_imui;

frgcolor = [.8,.8,1];
bkgcolor = [.8,.9,1];
Message0=['....'];
Message=Message0;

SliceDelta=1 ;
Plane0=1;
MaxPlane = 1 ;
VoxX1=1;
VoxY1=1;
VoxZ1=0;
VoxX2=0;
VoxY2=0;

GCM=colormap;

delete(findobj('Tag','cslicesgui'));

OP='AVG';

cslicesgui = figure('Color',[.6,.6,1],'Name','Extract Slices',...
'Tag','cslicesgui','NumberTitle','off','Position',[100,100,260,145],...
'Units','Pixels');
	
uicontrol(cslicesgui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[210 118 40 20],'callback','vp_help(''vp_cslices''); ');

	
cbplane0=['Plane0=str2num(get(hplane0,''string''));'];
cbmaxpln=['MaxPlane=str2num(get(hmaxpln,''string''));'];


cbPName=['FNames=[]; FNames=get_file(inf,''.img'',''please select images'',[]); ',...
	'if ~isempty(FNames),',...
	'FName=deblank(FNames(1,:));',...
  	'P=vp_nm(''dnm'',FName);',...
	'vp_hread([P ''.hdr'']);',...
	'Plane0=1; MaxPlane=DIM(3);',...
	'set(hrname,''string'',FName); ',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'if isempty(Message), Message=Message0; end; end;',...
	];
	
uicontrol(cslicesgui,'string', 'PICK IMAGE(S)',...
	'position',[10 90 85 25],'BackgroundColor',frgcolor,...
	'callback',cbPName);
	
cbFName=['FName=get(hrname,''string'');','Message=Message0 ;',...
	'FNames=[]; FNames(1,:)=FName;',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'if isempty(Message), Message=Message0; end;',...
	];
	
hrname=uicontrol(cslicesgui,'style','edit','string',FName,...
	'position',[95 90 155 25],'BackgroundColor',bkgcolor,'callback',cbFName);

cbSliMenu=['val=get(hslimenu,''Value''); ',...
	' if val== 1, OP='' ''; return; ',...
	'elseif val==2, OP=''AVG''; set(hop,''string'',''                '');',... 
	'elseif val==3, OP=''ADD''; set(hop,''string'',''                '');',... 
	'elseif val==4, OP=''EXT''; set(hop,''string'',''                '');',... 
	'elseif val==5, OP=''CRP''; set(hop,''string'',''  ''); eval(cbcrop);',... 
	'elseif val==6, OP=''PAD''; set(hop,''string'',''  ''); eval(cbpad);',... 
	'elseif val==7, OP=''SHF''; set(hop,''string'',''  ''); eval(cbshift);',... 
	'end; '];

hslimenu=uicontrol(cslicesgui,'Style','popup','Visible','on',...
	'position',[08 65 100 22],'BackgroundColor',Bcolor,...
	'TooltipString','Select Operation',...
	'String',...
	['Image Operation...|AVG Slices|ADD Slices|',...
	'EXTRACT Slices|CROP Slices|',...
	'PAD Slices|SHIFT Slices'],...
	'Tag','SliceMenu',...
    	'callback', cbSliMenu);

       
cbcrop=['VoxX1=1; VoxY1=1;',...
 'XYstr=num2str([VoxX1,VoxY1]),',...
 'ok=0;',...
 'while (ok==0)',...
   'vp_keyin(''Enter Lower Left VOX X,Y coordinates [X1, Y1] of Included Space | q'',XYstr);',...
   'if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')), return,',...
   'else XYs=str2num(Keyin); VoxX1=XYs(1); VoxY1=XYs(2); ',...
   	'ok =1;',...
	'if (VoxY1 < 1 ) vp_message(''Y must be >= 1''); ok=0; end;',...
	'if (VoxX1 < 1 ) vp_message(''X must be >= 1''); ok=0; end; ',...
   'end;',...
 'end; VoxX1,VoxY1, ',...
 'ok=0;',...
 'VoxX2=VoxX1+1; VoxY2=VoxY1+1;',...
 'XYstr=num2str([VoxX2,VoxY2]);',...
 'while (ok==0)',...
 '  vp_keyin(''Enter Upper Right VOX X,Y coordinates [X2, Y2] of Included Space | q'',XYstr);',...
 '  if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')), return,',...
 '  else XYs=str2num(Keyin); VoxX2=XYs(1); VoxY2=XYs(2); ',...
   	'ok =1;',...
 '	if (VoxY2 < VoxY1 ) vp_message(''Upper Y must be >= Lower Y''); ok=0; end;',...
 '	if (VoxX2 < VoxX1 ) vp_message(''Upper X must be >= Lower X''); ok=0; end;',...
 '  end;',...
 ' end; VoxX2, VoxY2, ',...
 'set(hop,''string'',[''('',num2str(VoxX1),'','',num2str(VoxY1),'') -- ('',num2str(VoxX2),'','',num2str(VoxY2),'')'']);',...
];

hop=uicontrol(cslicesgui,'style','edit','string','    ',...
	'TooltipString','Specify AVG, ADD, EXT, CRP, PAD or SHF',...
	'position',[105 65 145 20],'BackgroundColor',bkgcolor,...
	'CallBack',['']);

uicontrol(cslicesgui,'string', 'Initial Z Plane',...
	'TooltipString','Initial Slice Number',...
	'position',[10 40 75 20],'BackgroundColor',frgcolor);
	
hplane0=uicontrol(cslicesgui,'style','edit','string',Plane0,...
	'TooltipString','Initial Slice Number',...
	'position',[85 40 35 20],'BackgroundColor',bkgcolor,'callback',cbplane0);

uicontrol(cslicesgui,'string', 'Final Z Plane',...
	'TooltipString','Final Slice Number',...
	'position',[140 40 75 20],'BackgroundColor',frgcolor);
hmaxpln=uicontrol(cslicesgui,'style','edit','string',MaxPlane,...
	'TooltipString','Final Slice Number',...
	'position',[215 40 35 20],'BackgroundColor',bkgcolor,'callback',cbmaxpln);


uicontrol(cslicesgui,'String','PROCESS',...
	'Position',[190 10 60 25],...
	'CallBack',['out1=vp_cslicespro(FNames,OP,2,Plane0,MaxPlane,VoxX1,VoxY1,VoxX2,VoxY2);',...
	'if out1==0, delete(cslicesgui); end;']);
	
cbpad=['VoxX1=0; VoxY1=0; VoxZ1=0;',...
 'XYZstr=num2str([VoxX1,VoxY1,VoxZ1]);',...
 'ok=0;',...
 'while (ok==0)',...
   'vp_keyin(''Enter +/- Zero PAD thickness (directions determined by sign) [DX, DY, DZ] | q'',XYZstr);',...
   'if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')), return,',...
   'else XYZs=str2num(Keyin); VoxX1=XYZs(1); VoxY1=XYZs(2); VoxZ1=XYZs(3); ',...
   	'ok =1; VoxX2=VoxZ1;',...
   'end;',...
 'end; VoxX1,VoxY1, VoxZ1, ',...
 'set(hop,''string'',[''DX='',num2str(VoxX1),'', DY='',num2str(VoxY1),'', DZ='',num2str(VoxX2)]);',...
];

cbshift=['VoxX1=0; VoxY1=0; VoxZ1=0;',...
 'XYZstr=num2str([VoxX1,VoxY1,VoxZ1]);',...
 'ok=0;',...
 'while (ok==0)',...
   'vp_keyin(''Enter +/- Zero SHIFT data (directions determined by sign) [DX, DY, DZ] | q'',XYZstr);',...
   'if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')), return,',...
   'else XYZs=str2num(Keyin); VoxX1=XYZs(1); VoxY1=XYZs(2); VoxZ1=XYZs(3); ',...
   	'ok =1; VoxX2=VoxZ1;',...
   'end;',...
 'end; VoxX1,VoxY1, VoxZ1, ',...
 'set(hop,''string'',[''DX='',num2str(VoxX1),'', DY='',num2str(VoxY1),'', DZ='',num2str(VoxX2)]);',...
];

uicontrol(cslicesgui,'String','CANCEL',...
	'Position',[10 10 60 25],...
	'CallBack',['set(cslicesgui,''UserData'',-1); delete(cslicesgui);']);
	
