function [out1, out2]=vp_cslicespro(NameLst, imop, dataop,plane1,maxplane,x1,y1,x2,y2)
% Perform slice operations on ANALYZE Format images
%_______________________________________________________________________
%
% vp_cslicespro performs slice operations such as averaging, addition, 
% extraction of a set of planes from ANALYZE format images individually
% or cropping of a rectangular area within Z plane limits.
% Additional operations are pading or shifting of data.
% Slice data can be returned or saved as a separate file for each image.
% Slices are scaled and converted to real values before each operation is
% performed.
%
% imop='AVG':	Average Slices
% imop='ADD':	Add Slices
% imop='EXT':   Extract Slices
% imop='CRP':   Crop and Extract Slices
% imop='PAD':   PAD image with zero values before or after outer
%		limit voxels by the specified +/- DX, DY, DZ
% imop='SHF':   Shift image voxels by DX, DY, DZ but not origin
%		The original volume size and origin is maintained
%		but the volume within the plane limits is shifted
%		by the specified +/- DX, DY, DZ
%
% dataop=1  :  Return DATA without writing image (m=1)
% dataop=2  :  Write image
% dataop=3  :  Return DATA and write image (m=1)
% plane1,maxplane: Initial and Final Z plane
% x1,y1,x2,y2: Diagonal coordinates of crop rectangle
% x1,y1,x2: Correspond to values DX, DY, DZ for padding or shifting
% out1   :  Return value
% out2   :  Return DATA
%_______________________________________________________________________
% vp_cslicespro		C.4.4	P. Spetsieris	11/23/99
% vp_cslicespro		Ver.5.8	P. Spetsieris	4/05/07
% vp_cslicespro		Ver.6.1	P. Spetsieris	3/11/11
% vp_cslicespro		Ver.6.4	P. Spetsieris	3/10/15

global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE GENTIM SUBID EXPDATE hrname
global HelpMode

if HelpMode, vp_help(mfilename), return ; end;

out1=0;
out2=0;

m= size(NameLst,1);
dim3=0;

if m <=0,
    out1=-1;
    vp_note(['ERR: No Names Specified.']);
    return;
end;
	
if (m > 1 & dataop ~=2),
    out1=-1;
    vp_note(['ERR: Cannot return data for multiple images.']);
    return;
end;
	
h=waitbar(0,['Processing SLICE ' imop ' ...']);
for i=1:m

	%while(findobj('Tag','slprogui')),pause(1),end;
  
	waitbar((i-.8)/m,h);
	SDATA=[];
	glmaxdat=0;
  
	FName=deblank(NameLst(i,:));
	
	set(hrname,'string',FName);
	
  	%P=strtok(FName,'.');
  	P=vp_nm('dnm',FName);
	
	vp_hread([P '.hdr']);
	 
	if plane1 > DIM(3), vp_note(['Initial Plane Out of Bounds']); out1=-1; return; end; 
	if maxplane > DIM(3), vp_note(['Final Plane Out of Bounds']); out1=-1; return; end; 
	
	if (~strcmp(imop,'PAD') & ~strcmp(imop,'SHF'))
	  if x1 > DIM(1), vp_note(['X1 Out of Bounds']); out1=-1; return; end; 
	  if x2 > DIM(1), vp_note(['X2 Out of Bounds']); out1=-1; return; end; 
	
	  if y1 > DIM(2), vp_note(['y1 Out of Bounds']); out1=-1; return; end; 
	  if y2 > DIM(2), vp_note(['y2 Out of Bounds']); out1=-1; return; end; 
	
	end
	
	name=[P '.img'];
	
	ImageSize=DIM(1)*DIM(2)*vp_roifuns('bytes',MATYPE);
	%ImageSize=DIM(1)*DIM(2)*DIM(3);

	fid=fopen(name,'r');
 
	i=0;
	for PlaneNo=plane1:maxplane
  	  fseek(fid,ImageSize*(PlaneNo-1),'bof');

	  DATA=[];
	  RDATA=[];
	
	  DATA	= fread(fid,[DIM(1),DIM(2)],MATYPE);
	  
	  
	  %maxin=max(RDATA),
	  %minin=min(RDATA),
	  
	  if strcmp(imop,'EXT') 
	       i=i+1;
	       SDATA(:,:,i)=DATA;
	  elseif strcmp(imop,'CRP') 
	       i=i+1;
	       SDATA(:,:,i)=DATA(x1:x2,y1:y2);
	  elseif strcmp(imop,'PAD') 
	       if (PlaneNo == 1)
		dim1=DIM(1); dim2=DIM(2); dim3=DIM(3);
		if (x1 ~= 0), dim1=DIM(1)+abs(x1); end;
		if (y1 ~= 0), dim2=DIM(2)+abs(y1); end;
		if (x2 ~= 0), dim3=(maxplane-plane1 +1)+abs(x2); end;
		SDATA=zeros(dim1,dim2,dim3);
		i=0; 
		if x2  < 0; i=abs(x2); end;
	        j1=1;
	        if x1 < 0, j1=1 -x1; end;
		j2=j1+DIM(1)-1;
	        k1=1;
	        if y1 < 0, k1=1 -y1; end;
		k2=k1+DIM(2)-1;
	       end
	
	       if (PlaneNo <= maxplane)
	         i=i+1;
		 %j1,j2,k1,k2,i,
	         SDATA(j1:j2,k1:k2,i)= DATA(1:DIM(1),1:DIM(2));
	       else
		 if x2 > 0; i=i+x2; end;
               end

	  elseif strcmp(imop,'SHF') 
	       if (PlaneNo == 1)
		dim1=DIM(1); dim2=DIM(2); dim3=DIM(3);
		SDATA=zeros(dim1,dim2,dim3);
		i=0; 
		dim1 = dim1 -abs(x1);
		dim2 = dim2 -abs(y1);
		dim3 = dim3 -abs(x2);
		
		if x2  > 0; i=x2; end;
		
	        j1=1; a1=x1+1;
	        if x1 < 0, j1=1 -x1; a1=1;  end;
		j2=j1+dim1-1; a2=a1+dim1-1; 
		
	        k1=1; b1=y1+1;
	        if y1 < 0, k1=1 -y1; b1=1; end;
		k2=k1+dim2-1; b2=b1+dim2-1;
	       end
	
	       
	       if (PlaneNo <= maxplane)
		 
	         if ( (x2 < 0) & (PlaneNo < (abs(x2)+1) ) ) ; %skip
		 else
	           i=i+1;
		   if i <= DIM(3) 
		     %a1,b1, a2, b2, j1,j2,k1,k2,i,
	             SDATA(a1:a2,b1:b2,i)= DATA(j1:j2,k1:k2);
		   end;
	         end
               end

	  else
	    RDATA=(double(DATA))*SCALE;
	    if PlaneNo==plane1
	       SDATA=RDATA;
	    else
	  	SDATA=SDATA+RDATA;
	    end
	  end
	  
	  %maxout=max(SDATA),
	  %minout=min(SDATA),
	  %maxout=max(max(SDATA(:,:,PlaneNo))),
	  %minout=min(min(SDATA(:,:,PlaneNo))),
	  %keyboard;
	end % for PlaneNo
	
	fclose(fid);


	if ~strcmp(imop,'PAD') 
	  nplanes=maxplane-plane1+1;
	elseif ~strcmp(imop,'SHF') 
	  nplanes=DIM(3);
	else, nplanes=dim3; end;
	if strcmp(imop,'AVG') 
	  if nplanes <=0,
		out1=-1;
    		vp_note(['ERR: Invalid Plane Nos.']);
		return;
	  end;
	  SDATA=SDATA/nplanes;
	  DESCRIP=['ScAnVP AVGed Slice', num2str(plane1),'-',num2str(maxplane)];
	elseif strcmp(imop,'ADD') 
	  DESCRIP=['ScAnVP ADDed Slice', num2str(plane1),'-',num2str(maxplane)];
	elseif strcmp(imop,'EXT') 
	  DESCRIP=['ScAnVP Extracted Slices', num2str(plane1),'-',num2str(maxplane)];
	elseif strcmp(imop,'CRP') 
	  DESCRIP=['ScAnVP Cropped Slices', num2str(plane1),'-',num2str(maxplane)];
	elseif strcmp(imop,'PAD') 
	  DESCRIP=['ScAnVP Padded Slices', num2str(plane1),'-',num2str(maxplane),' by ',num2str(x1),',',num2str(y1),',',num2str(x2)];
	elseif strcmp(imop,'SHF') 
	  DESCRIP=['ScAnVP Shifted Slices', num2str(plane1),'-',num2str(maxplane),' by ',num2str(x1),',',num2str(y1),',',num2str(x2)];
	else
	  vp_note(['Invalid image operation ' imop]);
	  out1=-1;
	  return;
	end

	if dataop ~=2, out2=SDATA; end;
	if dataop==1, return, end;

	%WRITE IMAGE

	%P=strtok(name,'.');
	P=vp_nm('dnm',name);
	%P=P(1:max([findstr(P,'\') 0]));

	%nameimg=[P 'S' imop '.img'],
	glmaxdat=max(max(SDATA));
	if strcmp(imop,'EXT')
	  glmaxdat=max(glmaxdat);
	  nameimg=[P 'EX.img'],
	elseif strcmp(imop,'CRP')
	  glmaxdat=max(glmaxdat);
	  nameimg=[P 'XR.img'],
	elseif strcmp(imop,'PAD')
	  glmaxdat=max(glmaxdat);
	  nameimg=[P 'PAD.img'],
	elseif strcmp(imop,'SHF')
	  glmaxdat=max(glmaxdat);
	  nameimg=[P 'SHF.img'],
	else
	  nameimg=[P 'SL.img'],
	end
	if glmaxdat == 0,
	  vp_note(['WARNING: 0 maximum']);
	  waitfor(findobj('Name','note'));
	  %while(findobj('Name','Message')),pause(1),end;
	end
	
	% Do NOT Change to TYPE 64 for extraction padding, shifting or cropping
	if (~strcmp(imop,'EXT') & ~strcmp(imop,'PAD') & ~strcmp(imop, 'SHF') & ~strcmp(imop,'CRP'))
 	  glmindat=min(min(SDATA));
	  if (TYPE==2 & glmindat < 0) , TYPE=64; end;
	  if (abs(glmindat) > glmaxdat),
	    glmaxdat=abs(glmindat);
	  end;
	  %For version 4.77 and above save double
	  TYPE=64;
	end

	fidop=fopen(nameimg,'w');
	
	if TYPE==2
	  datype='uchar';
	  glmax=255;
	elseif TYPE==4
	  datype='int16';
	  glmax=32767;
	elseif TYPE==8
	  datype='int32';
	  glmax=(2^31-1);
	elseif TYPE==16
	  datype='float';
	  glmax=1;
	elseif TYPE==64
	  datype='double';
	  glmax=1;
	  %SDATA=double(SDATA);
	else
	  vp_message(['ERROR: Invalid DATA TYPE' num2str(TYPE)]);
	  out1=-1;
	  return;
	end
	
	DATA=[];
	if ( (TYPE < 16) & ~strcmp(imop,'EXT') & ~strcmp(imop,'PAD') & ~strcmp(imop,'SHF') & ~strcmp(imop,'CRP'))
	   DATA=SDATA*glmax/glmaxdat;
	else
	    DATA=SDATA;
	end
	no=fwrite(fidop,DATA,datype),
	if no <=0,
	  	vp_message(['Write ERROR no: ' num2str(no) ' pixels']) ;
		out1=-1;
		return;
	end
	
	fclose(fidop);

	if strcmp(imop,'EXT')
	  DIM(3)=nplanes;
	  ORIGIN(3)=max((ORIGIN(3)-(plane1-1)), 1);
	  if ORIGIN(3) > maxplane, ORIGIN(3)=1; end;
	elseif strcmp(imop,'CRP')
	  DIM(3)=nplanes;
	  ORIGIN(3)=max((ORIGIN(3)-(plane1-1)), 1);
	  if ORIGIN(3) > maxplane, ORIGIN(3)=1; end;
	  DIM(1)=x2-x1 +1;
	  ORIGIN(1)=max((ORIGIN(1)-(x1-1)), 1);
	  if ORIGIN(1) > x2, ORIGIN(1)=1; end;
	  DIM(2)=y2-y1 +1;
	  ORIGIN(2)=max((ORIGIN(2)-(y1-1)), 1);
	  if ORIGIN(2) > y2, ORIGIN(2)=1; end;
	elseif strcmp(imop,'PAD')
	  DIM(1:3)=[dim1,dim2,dim3];
	  if x1 < 0, ORIGIN(1)= ORIGIN(1)-x1; end;
	  if y1 < 0, ORIGIN(2)= ORIGIN(2)-y1; end;
	  ORIGIN(3)=ORIGIN(3)-(plane1-1);
	  if x2 < 0, ORIGIN(3)= ORIGIN(3)-x2; end;
	elseif strcmp(imop,'SHF')
	  % keep original parameters
	else
	  DIM(3)=1; % Write only one slice
	  VOX(3)=nplanes*VOX(3);
	  ORIGIN(3)=1;
	  if TYPE < 16
	    SCALE=glmaxdat/glmax;
	  else
	    SCALE=1.0;
	  end
	end


vp_hwrite(nameimg,DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,EXPTIM,DURATION,GENTIM,SUBID,EXPDATE,glmax);
vp_note([nameimg ' File written'],3);
	

end % for i

waitbar(1,h); pause(1); close(h);

vp_note('Slice Processing Completed',2);
