% function out1=vp_cvtNIF2ANA(opt, namelst)
%
% Converts nifti format image header ((.img, .hdr) and .nii type)
% to ScAnVP ANALYZE format. Promts for a series of nifti format
% images and converts their headers to scanvp ANALYZE format. 
% UNIX format nifti images are converted to DOS ANALYZE.
% An option is provided to flip nifti images along the X axis 
% from radiological orientation (RL) to neurological (LR) and vice-versa.
% Based on the user's knowledge of the true orientation, the flipped
% image suffix 'X' can be edited as 'LR' (Neurological) or 'RL' (Radiological) 
% or can be deleted. If 'X' is deleted the original image is replaced by 
% the flipped image, otherwise two Analyze images are output, one in the
% original orientation and one flipped.
%_______________________________________________________________________
% opt = 'select' : select images
% opt = 'convert' : convert images
% out1:  Return status
%
% File header information is converted to global variables that are
% saved in an ANALYZE format header.
% If the file is already in ANALYZE format, it will not be processed.
%_______________________________________________________________________
% vp_cvtNIF2ANA	Ver. 5.8.4	P. Spetsieris	Jan. 14, 2008
% 		Rev. 6.2.1	P.S. 	Apr. 24, 2013

function out1=vp_cvtNIF2ANA(opt, namelst)

global HelpMode FMenu NameLst
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE Message GENTIM SUBID EXPDATE
global HDRTYPE % NIFTI or ANALYZE
global VAL_OFFSET POpt


if HelpMode, vp_help(mfilename), return ; end;

if (nargin < 1), vp_cvtNIF2ANA('select'); return; end;

out1=0; VAL_OFFSET=0; FlipOrient=0; 
if strcmp(opt,'select')
%-Select image headers
%----------------------------------------------------------------------------
  if vp_filesui(Inf,'*.*','Select nifti format image headers to be converted',mfilename) < 0,
  return; end;

  m=size(NameLst,1);

  set(FMenu,'Pointer','Watch');
  h=waitbar(0,'Processing Header Files ...');

  if m ~=0, 

   prompt=['Flip input X axis orientation RL <=> LR ? [ Yes | No | EXIT]'];
   POpt=vp_opt('X',prompt,'Yes','Flip input orientation','No','Do Not Flip','EXIT','');
   if (POpt=='X' | strcmp(POpt,'EXIT')), 
        set(FMenu,'Pointer','Arrow');
	vp_note('Program Exited',2); return;
   elseif ( strcmp(POpt,'Yes') ), 
	FlipOrient=1;
        vp_note('Flipped image suffix ''X'' can be edited as ''LR'' (neurological) or ''RL'' (radiological) or erased. ');
   elseif ( strcmp(POpt,'No') ), 
	FlipOrient=0;
   end
  end

  for i=1:m
  	waitbar((i-.8)/m,h);
	filename = deblank(NameLst(i,:));
	out1 = vp_cvtNIF2ANA('convert',filename);
	if strcmp(HDRTYPE,'NIFTI')
	  if FlipOrient, 
		[out2]=vp_imflip(filename,'X',2);
		if out2 < 0, 
	  	  vp_message(Message); 
	  	  waitfor(findobj('Name','Message'));
                end;
          end
	  vp_note([HDRTYPE, ' FORMAT file PROCESSED: ',filename],1);
	else
	  vp_note([HDRTYPE, ' FORMAT file NOT PROCESSED: ',filename], 3);
	end
  end


  set(FMenu,'Pointer','Arrow');
  waitbar(1,h); close(h);

  vp_note('NIFTI1 -> ANALYZE Conversion Exited',3);
  valoffstr=sprintf('%f',VAL_OFFSET),
  if (VAL_OFFSET ~=0)vp_note(['Must add value ',valoffstr, ' to data']); end;


elseif strcmp(opt,'convert')

  m= size(namelst,1);


  for i=1:m
    VAL_OFFSET=0;
    FName=deblank(namelst(i,:));
    %P=strtok(FName,'.');
    [sfx,P]=vp_nm('sfx',FName);
	
    endian=0;
    if (strcmp(sfx,'hdr') | strcmp(sfx,'img'))
        vp_hread([P '.hdr']);
	if strcmp(MATYPE,'UNIX'), endian=1;
          vp_hread([P '.hdr'],endian); end;
	if((VAL_OFFSET ~= 0) | (endian ~= 0))
	    if endian==1
	      fid=fopen([P '.img'],'r','ieee-be');
	    else
	      fid=fopen([P '.img'],'r','ieee-le');
	    end
	    [dat, no]=fread(fid,Inf,MATYPE);
	    no,
	    readszdat=size(dat),
	    maxdat=max(dat),
	    fclose(fid);
        end;
    elseif (strcmp(sfx,'nii'))
	    vp_hread([P '.nii']);
	    if strcmp(MATYPE,'UNIX'), endian=1;
               vp_hread([P '.hdr'],endian); end;
	    if endian==1
	      fid=fopen([P '.nii'],'rb','ieee-be');
	    else
	      fid=fopen([P '.nii'],'rb','ieee-le');
	    end
	    fseek(fid,OFFSET,'bof');
	    [dat, no]=fread(fid,Inf,MATYPE);
	    no,
	    fclose(fid);
	    if(VAL_OFFSET == 0)
	      fid=fopen([P '.img'],'wb');
	      no=fwrite(fid,dat,MATYPE),
	      fclose(fid);
	    end
	else
        vp_note('NOT A RECOGNIZED NIFTI HEADER',3);
        out1= -1; return;
    end;
        
    if strcmp(HDRTYPE,'NIFTI')
	  if(VAL_OFFSET ~= 0)
	      dat=double(dat);
	      dat =SCALE*dat + VAL_OFFSET;
	      writeszdat=size(dat),
	      maxdat=max(dat),
	      TYPE=16; MATYPE='float';
	      SCALE=1; VAL_OFFSET=0;
	      GLMIN=0; GLMAX=1; 
	      fid=fopen([P '.img'],'wb');
	      no=fwrite(fid,dat,MATYPE),
	      fclose(fid);
	      vp_note('IMAGE SCALED AND OFFSET ADDED',3);
	  elseif(endian ~= 0)
	      fid=fopen([P '.img'],'wb');
	      no=fwrite(fid,dat,MATYPE),
	      fclose(fid);
	      vp_note('NIFTI UNIX IMAGE CONVERTED TO ANALYZE DOS',3);
	  end
	  OFFSET=0;
	  DESCRIP=['ScAnVP NIFTI->ANALYZE, ',deblank(DESCRIP)];
	  out1=vp_hwrite([P '.hdr'],DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,...
		EXPTIM,DURATION,GENTIM,SUBID,EXPDATE,GLMAX,GLMIN);
		
	  if out1 < 0,
		vp_message('Invalid Write Operation'); return;
	  end
    end

  end 

  if m <= 0, 
    out1=-1;
    vp_message('Empty File Set');
  end;

end;
