% ScAnVP (Scan Analysis and Visualization Processor) Ver. 7.0w - (C) 2016
% Center for Neurosciences, Feinstein Institute for Medical Research,
% Northwell Health (formerly, North Shore-LIJ Health System)
% 350 Community Drive, Manhasset, NY 11030.
%		
%
%				GENERAL DESCRIPTION OF SOFTWARE
%
%   INTRODUCTION
%
%	ScAnVP is a set of MATLAB routines intended primarily 
%	for the image processing, visualization, region of interest
%	statistics and analysis of PET functional brain images or other
%	single volume images such as perfusion ASL fMRI or VBM MRI. 
%	ScAnVP was developed as an in-house package at the North Shore-LIJ
%	Feinstein Institute (now Northwell Health) in collaboration with members 
%	of the Functional Brain imaging laboratory headed by Dr. David Eidelberg. 
%	The public release of ScAnVP software is distributed as copyright
%	software that is free under the terms of the GNU General Public  
%	License version 2 as published by the Free Software Foundation
%	(or any later version at your option). License terms can be read
%	by invoking 'show license' on the ScAnVP menu or by visiting
%	http://www.gnu.org/copyleft/. ScAnVP comes with ABSOLUTELY NO WARRANTY
%	and no implied support and is not intended for commercial use.
%
%	ScAnVP currently runs primarily as a set of MATLAB scripts in a Windows 
%	PC platform and utilize a highly interactive, user-friendly GUI interface.
%	This version of ScAnVP was tested on Windows 7 PC workstations with
%	4GB RAM utilizing MATLAB version 7.3(R2006b)at 2.3GHz or greater but is
%	not limited to these configurations. For MATLAB routines, ScAnVP assumes
%	the ANALYZE volume file	format for image input and output but nifti
%	images in (.img, .hdr) format can also be input in most routines to
%	provide compatibility with other standard neuroimaging software. A
%	separate routine is provided to convert .nii type files to ANALYZE format.
%
%	The ScAnVP distribution includes in-house software, documentation, masks,
%	color maps and executable programs. It may also include publicly distributed 
%	utilities or images such as SPM\sample mri or pet images and a modified
%	brain atlas based on the aal_MNI_V4 image accessed by the software.
%
%	A primary component of ScAnVP implements the SSM-PCA model for
%	regional and voxel based spatial covariance pattern derivation of 
%	characteristic imaging biomarkers of neurofunctional disorders such as
%	Parkinson's disease and prospective single subject evaluation for
%	differential diagnosis, treatment monitoring, and the assessment of
%	drug efficacy.
%
%	Major features of ScAnVP are listed below. Modifications and additions in 
%	sequential versions are listed in the file 'vp_revisions'.
% 
%	Send comments to: pspetsie@northwell.edu 
%
%
%   INSTALLATION
%	Installation is described in the README.txt file.
%
%   OVERVIEW
%	After installing the ScAnVP software, the ScAnVP menu can be invoked
%	by typing 'scanvp' in the MATLAB command window.
%	ScAnVP tasks are initiated by pressing buttons or selecting
%	options from the submenu controls on the main menu. 
%	The default directory is the MATLAB Current Directory and can
%	be changed using the MATLAB desktop tool or ScAnVP menu item.
%	The main function of major menu buttons is briefly described below.
%	More detailed information can be obtained for each task by setting
%	the Help Mode switch ON on the main menu and clicking on the task. 
%	This also works for some intermediate menus. In addition individual
%	'HELP' buttons are also available on task menus after initiation.
%	Tooltip strings are provided to clarify button functionality.
%	Images must be in ANALYZE format or Nifti format with some exceptions.
%	Conversion routines are provided to convert GE '.iee' format PET images
%	as well as GE MRI GENESIS format and Nifti images to ANALYZE format.
%	Unix ANALYZE image headers must be converted using the UNIX<=>DOS utility.
%	Functions are provided to convert raw count data to glucose rates,
%	perform regional graphic analysis or create voxel based Logan plots and
%	Patlak maps. 
%	Display routines allow interactive visualization of orthogonal multiple
%	slices with different color schemes, interactive thresholding, voxel
%	value interogation, area and line value plotting and cluster removal.	
%	Region of Interest (ROI) routines are provided to create a template of
%	rectangular, irregular or elliptical shaped regions on sequential
%	slices of a volume image. Standard or user-defined names can be assigned
%	to these regions and interactive options are provided for manipulation,
%	thresholding, statistical tabulation and mapping. The ROI template output
%	files can be input into group replay processing routines to create output
%	tables of statistical 2D and 3D ROI group value summaries in various formats.
%	Alternatively an AAL_MNI BRAIN ATLAS of predefined voxels is provided.
%	Information from these tables can also be input into ROI-based statistical
%	analysis routines. PCA covariance patterns derived by ROI-based SSM 
%	principal component analysis can be visualized using the GIS ROI voxel
%	map routine. More typically, voxel based SSM analysis is used to study
%	covariance structure that is not limited by predefined regional boundaries.
%	Group subject files are initially transformed into a standard spatially
%	normalized space such as the MNI space defined in SPM normalization. 
%	SSM voxel based results can be visualized using the statistical mapping
%	routine that also provides thresholding, masking and cluster removal.
%	SPM-like results of voxel-based SSM output is an additional option.
%	ScAnVP also provides various image utilities and processing routines 
%	including functions for transparent fusion of PET with coregistered
%	MRI images, group plot/files of cubic and spherical region values, and the
%	ability to save and display image files in other standard formats.
%	Selected documentation can be found under Revisions/Notes.
%	
%	
%   MENU
%	A brief description of menu items is given here. Further details
%	can be found from individual menu help.
%	Menu options are divided into the following categories:
%
%	CONVERSION, FUNCTIONALIZATION, DISPLAY, REGIONS OF INTEREST, STATISTICS,
%	IMAGE PROCESSING, UTILITIES, HELP & CONTROLS
%
%	CONVERSION:
%
%	  GE(PET)->ANA
%			Converts GE PET, iee image files to ANALYZE format.
%			(128x128, 8bit, Radiological (RL) orientation, zero negatives).
%
%	  GE(MRI)->ANA
%			Converts GE MRI, GENESIS image files to ANALYZE format.
%			(256x256, 8bit or 16bit Radiological (RL) orientation ).
%
%	  UNIX<->DOS
%			Converts ANALYZE image headers from UNIX to DOS format   
%			or vice-versa. If voxels have greater than 8bits, image
%			files are also converted. Multiple Volume images are
%			separated into single volume image files.
%
%	  NIFTI->ANA
%			Converts spm5 'nifti' image headers to ANALYZE format.
%			An option is provided to flip the image along the X Axis.
%			This enables Radiologic Oriented images (RL) output by SPM
%			to be converted to Neurological Orientation (LR, Left is Left).
%
%	FUNCTIONALIZATION:
%
%	  FUNCTIONALIZATION
%	    FDG Glucose - Blood File Input
%			Converts raw count Analyze images to Cerebral Metabolic 
%			Glucose rates (mg/100g/min). 
%
%	    FDOPA Uptake - Blood File Input
%			Converts raw count Dynamic Analyze Images to Uptake 
%			units (micro liters/g/min). 
%
%	    GET ROI PARAMETERS - Roi File Input, Varous Tracers
%			Performs ROI based graphic analysis- Produces SOR, Ki
%			and DVR Plots and an output text file.
%
%	    GET VOX PARAMETERS- Roi File Input, Various Tracers
%			Creates voxel based Logan plots and Patlak maps saved as 
%			ANALYZE DVR and Ki parameter images.
%
%
%	DISPLAY:
%
%	  IMAGE VIEW
%			Displays images in 3 orthogonal views and in sectional tile formats.
%			Image voxels can be bytes, integers, or real float or double
%			and may include negative values.
%			Images are displayed in Neurological orientation (Left axial
%			display corresponds to the anatomical left hemisphere) when the
%			saved orientation in the ANALYZE file saved on disk is correctly
%			specified by the user on the ScAnVP menu as either Neurological
%			(LR) or Radiological (RL) Input Orientation. Newer raw axial
%			images are usually saved in the radiological convention (RL): 
%			Right display is left of subject and left is right, for example,
%			GE GENESIS-MRI and IEE-PET format output images. SPM usually
%			displays images in Neurological orientation, however, on disk
%			spm5 normalized images are saved in radiological orientation
%			while spm99 normalized images are saved in neurological orientation.
%
%			Coordinates:
%			Scanvp display routines such as 'IMAGE VIEW' calculate physical
%			coordinates based on the voxel coordinates of the origin
%			and the physical dimensions of each voxel. If the images
%			are normalized, the physical coordinates of each point
%			represent the distance in mm. from the normal origin.
%			Using the SPM normalization routine, physical coordinates
%			represent coordinates in a Talairach or MNI based normal space
%			assuming the images are displayed in Neurological orientation.
%			For Radiologic oriented images, the positive X Axis corresponds 
%			to the negative Talairach axis. In ScAnVP, images specified as
%			RL are automatically flipped L<=>R for display only to enable
%			proper coordinate and anatomical labeling.
%			
%			Plot:
%			A plot option creates a 3D plot of a rectangular area's intensities.
%			A 2D plot option plots a line segment profile.
%
%			Rotate:
%			The rotate view option allows sectional views of a plane rotating 
%			around an axis parallel to the X,Y or Z axis to be displayed and
%			animated (movie).
%			
%			Crop/Zoom:
%			Images can be cropped and zoomed in any orthogonal view and saved.
%
%           Clusters:
%			The Del Clusters options allows the user to remove 2D/3D isolated
%			clusters less than a given diameter from an image or composite
%           PET/MRI map.
%
%			Dynamic/Multiple Images
%			Sectional slices from multiple images or dynamic sets of the same
%			subject can be displayed in various formats.
%
%	  TRANSPARENCY
%			Works similar to 'IMAGE VIEW' but displays superimposed PET/MRI views
%			transparently. The user can vary the degree of transparency.
%
%	UTILITIES
%			Various utilities can be invoked from the main menu or from 
%			a control bar on top of image figures. They include operations for
%			smoothing, reading a value of a point on an image, text placement,
%			colormap selection and colorbar placement, colormap manipulation,
%			thresholding and normalization of multiple image figures to a common
%			Min/Max range, deletion and movement of component elements, line and
%			surface plot creation, saving MATLAB figures in various other
%			standard image formats and displaying different format images as well
%			as previously created movies in a MATLAB figure.  
%
%       COLOR
%			Allows interactive selection of a color map for application
%			to a data figure. A separate 'col' directory is provided for
%			saving or retrieving user color maps. Screen background GUI.
%
%       RANGE
%			FULL: Interactive variation of the upper and lower thresholds
%			and center of the colormap of a data image.
%			SEMI: Separate manipulation of the upper and lower half of the
%			colormap
%			NORM: Normalization of all images in a figure to a common scale.
%                               
%       VALUE
%			Allows the user to select a point on a data image and display
%			its value and coordinates in voxel and physical units. It also
%			displays the default Neurological ATLAS label designation.
%			ScAnVP displays images in the Neurological orientation assuming
%			their input orientation is properly specified. 
%
%       TEXT
%			Allows the user to place text at a point on a data image
%
%       MOVIE
%			Plays previously created movies of a sequence of MATLAB image
%			frames saved in a .mat file. The 'IMAGE VIEW' ROTATE option can
%			be used to create movies.
%
%       Marker
%			Places a fixed size (default 8mm) geometric marker at a 
%			specified or pointed-to location on a displayed image.
%
%
%	REGIONS OF INTEREST: 
%
%	  ROI (Template)
%			This routine allows the user to interactively define 
%			rectangular, rlliptical or irregular regions of interest
%			on sequential slices of a single subject image file.
%			The regions are named using standard or user names.
%			Statistical data can be tabulated and viewed and mapped.
%			Thresholding and editing options are provided. The data 
%			is saved in files (Name.Plane.roi) that contain the 
%			statistical and geometric information. These files can
%			be used as region definition templates for group processing.
%
%	  GROUP-ROI (Template and ATLAS ROIs)
%			Various options are provided to replay .roi templates on group
%			member normalized image sets. Three dimensional thresholding
%			options and interactive editing is included. Output can be
%			saved in several different formats for individual or summary type
%			reports. In addition to text .roi templates the program can use
%			as input an image roi ATLAS where each brain area corresponds to
%			a coded voxel value listed in a corresponding label index file.
%			The default atlas file may be referred to by other routines. 
%			This atlas was originally based on the aal_MNI_V4.img and can be
%			used on any images spatially normalized to MNI space 79x95x68 or x69.
%
%	  MASK
%			Uses a reference image to allow the user to interactively create
%			a 0/1 8bit output ANALYZE format image mask across planes with
%			values of one for every voxel that is included within regions
%			and zero for voxels excluded.
%
%	  VOI PLOT/SAVE
%			Gets values for specified points or average values within 3D
%			spherical or cubic volumes of interest for a set of image files
%			and plots or saves the values in a file.
%
%	STATISTICAL MAPS:
%
%	  STATMAP
%			Utilizes Analyze format statistical output, Mask and Background files,
%			to create a masked, thresholded, statistics over background file
%			with exclusive and inclusive voxel selection and various orthogonal 
%			viewing options similar to 'IMAGE VIEW' including 2D and 3D cluster removal.  
%
%	  GIS ROI MAP
%			Creates a statistical voxel map of a regional GIS PC vector
%			resulting from SSM analysis of ROI template group data. Options
%			are provided to manipulate the visual representation over an MRI
%			background image (PET/MRI). Output is saved in ANALYZE format.
%
%	  GIS ATLAS MAP
%			This routine creates an ANALYZE image map of GIS ROI weight
%			output from SSM ATLAS GROUP ROI analysis. The voxel output image
%			can be input into the 'Statistical Map' routine to create
%			a PET/MRI image map. 
%
%	  SPM-like RESULTS
%			Provides an interactive gui for displaying SSM or other voxel
%			based statistics similar to that produced by SPM results.
%
%	  GROUP MASKS
%			Group threshold masks (0|1) images can be created and applied in
%			various applications. A default gray matter mask (PETmask.img) along 
%			with several other masks are provided under the scanvp\masks 
%			directory. This default mask can be replaced by any user mask provided 
%			that it has the same dimensions as images used in processing. Provided
%			are masks labeled with the characters 99 or 5 to indicate the version 
%			of spm used to normalize the files that were used to generate the mask
%			and the intended version of normalized images for which it should be used.
%			In this version the default mask is symmetrical and compatible with spm5
%			(identical to the label '5' mask which has 69 planes).
%			
%		
%	  RATIO MAPS 
%			Creates an image from the corresponding values of two sets of images.
%		
%		
%	STATISTICS:
%
%	  SPM
%			Invokes the SPM MATLAB package when available.
%
%	  SSM   (See SSM HELP and DOCUMENTATION)
%			Includes several options for both VOXEL and ROI based SSM-PCA.
%
%	    PCA
%			SSM Principal Componant Analysis (PCA) is used for the derivation
%			of group invariant subprofiles (GIS networks) and associated
%			subject scores, These network PC patterns may be significantly
%			related to disease specific metabolic or other physiological
%			processes. Subject scores for these patterns determined by the
%			algorithm can be used in differential diagnosis, patient monitoring
%			and drug performance evaluations. 
%
%	    SUBTRACTION-PCA
%			Principal Component Analysis of dual condition subtraction data. 
%			For log transformed data (the default) this is equivalent to a ratio
%			operation.
%
%	    O-PCA
%			Orthogonal PCA analysis (O-PCA) of data where the covariant
%			contribution of a particular GIS component is removed 
%			to enable other components to be revealed.
%
%	    TPR
%			Topographic Profile Rating (TPR) of new subjects for the
%			determination of disease specific GIS Scores that evaluate 
%			disease severity on an individual, prospective basis.
%			Both highly automated and interactive procedures are available
%			for reference based scoring and reference recalibration.
%			Repeated measures for independent and same group repeated
%			trials are saved in text files and displayed as scatter plots.
%
%	UTILITIES:
%
%	DISPLAY SCORES
%	  Displays bar and scatter graphs of subject scores for SSM GIS score files.
%	GENERATE MEAN
%	  Creates Voxel Based Group Log Mean Image for SSM Statistical Analysis.
%	ROC CHARACTERISTICS
%	  Displays the ROC characteristics of an SSM score file.
%	GROUP THRESHOLD MASK
%	  Creates Voxel Based Data Threshold Mask from Group Data.
%	EXCLUSIVE MASK
%	  Creates Voxel Based Exclusive Mask from Data or GIS File.
%	MASK GROUP DATA
%	  Masks Data Images Using Pre-Defined Mask
%	LINEAR GIS ADDITION
%	  Creates a Composite Voxel GIS Pattern From More Than One PC.
%	GIS CORRELATION
%	  Calculates Pearson's Correlation of a Set of GIS or Other Image Vectors
%
%	IMAGE PROCESSING:
%
%			Options include SINGLE, MULTI, or PAIR-WISE operations.
%			SINGLE operations act on individual images within a selected group
%			separately. MULTI operations act on a group of images to produce
%			a single output image. PAIR-WISE operations act on pairs of images
%			from two corresponding sets. 
%
%			Operations include SCALAR and IMAGE BASED arithmetic, image SCALING,
%			CROPPING, GLOBAL or VECTOR NORMALIZATION and Z-SCORE conversion.
%			Some of the other available options are listed below.
%
%	  APPEND
%			Concatenates ANALYZE images into one by appending component planes.
%
%	  AVERAGE
%			Scales voxel values and averages component image planes.
%
%	  ADD
%			Scales voxel values and adds component image planes.
%
%	  SUBTRACT
%			Scales voxel values and subtracts from the first image. If negative
%			values occur, the resulting image is saved as same type if possible.
%
%	  DIVIDE
%			Scales voxel values and divides into the corresponding voxels of the
%			first image. For zero voxels the result is set to zero.
%
%	  EXTRACT SLICES
%			Extract a range of slices from an ANALYZE image to create a new file.
%			Combines two or more slices within a file into a sigle slice file.
%
%	  CONVERT TYPE
%			Converts image TYPE. For example, a 16bit image to 8bit and vice-versa.
%
%	  FLIP AXIS/VIEW
%			Flips X, Y or Z axis or converts from Axial to Sagittal or Coronal views.
%
%	  INTERPOLATE
%			Change image dimensions using tri-linear or nearest neigbor interpolation
%			and save new file.
%	  SMOOTH
%			Smooth image using user specified box convolution kernel.
%
%	  MARKER
%			A square marker is placed at the same position in each plane.
%
%	  NEGS.->ZERO
%			Sets all negative values to zero.
%
%	  EXCLUDE VOX 
%			Excludes voxels from a file that are common to a reference.
%	  HEMI IMAGE 
%			Display only Right or Left Side of Image.
%
%	  MULT, THRESH, MASK, LOG, GLOBAL NORM...
%	    MULTIPLY BY SCALAR
%			Multiply image values by floating point number.
%	    THRESHOLD LOWER VALUES
%			Set values less than a lower threshold to zero.
%	    EXCLUDE THRESHOLD RANGE
%			Set values within given range to zero.
%	    MASK
%			Set values to 0 in image corresponding to 0 in 0|1 mask
%	    GLOBAL NORMALIZATION
%			Values are masked and then thresholded by setting the lower
%			threshold to a fraction of the image maximum within the masked
%			values. The values are then divided by the remaining non-zero
%			voxel mean.
%	    MASK CONVERSION WITH EXCLUDED RANGE
%			Set values within given range to zero and convert to 0|1 mask.
%	    LOGARITHMIC CONVERSION
%			Convert image values > 0 to their natural logarithm.
%			All other values are set to 0. 
%
%
%	HELP:
%
%	  HELP ON/OFF
%			HELP ON/OFF is a button which operates like a switch on the ScAnVP menu.
%			When ON, help is displayed in the desktop HELP or MATLAB command window
%			when a task is picked on the ScAnVP main menu. The help is the description
%			of the MATLAB routine found at the start of each .m file. The information
%			may include the time the original version of this routine was created.
%			When OFF ScAnVP tasks run in the normal mode.
%			In addition, many button options provide a 'TooltipString' to clarify
%			their use. A 'help' button is provided on many application menus as well.
%			This button is not a switch and is not intended for use while processing.
%
%
%	  GENERAL
%			A general description of ScAnVP features and routines (this file).
%
%	  REVISIONS
%			A file listing major revisions of each new version of ScAnVP.
%
%	  NOTES
%			A file listing revisions for versions of ScAnVP prior to 5.0.
%
%	  DEFAULT NAMES
%			A file listing default naming conventions for output files.
%
%	  ANALYZE HDR
%			Lists header parameters of a user selected ANALYZE file in the 
%			MATLAB command window.
%
%	  FIX HDR
%			Displays header parameters of a user selected ANALYZE file and 
%			allows modifications to be made and saved. New header files can
%			be created. Global variable defaults for ANALYZE Headers are
%			initialized in the vp_defaults.m file.
%
%	  FIX TIMES
%			Alows the user to save time fields from a set of ANALYZE headers in a
%			file or to restore time data from a file.
%
%	  FIX ORIGIN
%			Alows the user to correct the X Y Z voxel coordinates of the
%			origin of ANALYZE image headers.
%
%	  ERRATA
%			Lists known problems with ScAnVP routines that need to be addressed
%			or that have been recently fixed.
%
%
%	CONTROLS:
%
%	  RESTART
%			In case of an operational error, press RESTART. The RESTART button
%			will abort tasks, clear the screen and restart ScanVP.
%			Also used to set the file browser to the MATLAB current directory.
%
%	  MESSAGE
%			Displays the last ScAnVP status message and last MATLAB error
%			when the user clicks on the latter field.
%
%	  CURRENT DIRECTORY
%			Invokes the MATLAB file browser and allows the user to
%			change the current directory and view listed files.
%			The file selection browser defaults to the MATLAB current
%			directory on start up. It serves as a default output directory.
%		
%
%	  QUIT
%			Aborts tasks, clears the screen and exits ScAnVP
%
%________________________________________________________________________________________
%	vp_general.m  ScAnVP 7.0w       Phoebe Spetsieris, Ph.D. Jan. 28, 2016
%
%   Phoebe G. Spetsieris, Ph.D. 
%   Research Associate Investigator
%   Center for Neurosciences 
%   Functional Brain Imaging Laboratory 
%   The Feinstein Institute for Medical Research
%   Northwell Health
%   pspetsie@northwell.edu

	vp_help vp_general; 
