% vp_giscorr
% Calculates Correlation of a Set of GIS or Other Image Vectors 
%_______________________________________________________________________
% The user selects two or more spatially normalized input images (e.g. GISs) 
% that are converted to vector form by the routine. The correlation matrix
% of Pearson's correlations between vectors (r) is calculated along
% with p values and the R_squared = r.*r matrix is output as well.
% The routine calls the MATLAB 'corr' routine that computes a square matrix
% containing the pairwise linear correlation coefficient between each pair
% of input vectors. Two sets of values are output in the command window.
% The first refers to the correlation of the raw vectors including 0 values.
% For the second calculation only the non-zero voxels that are common to
% to all images are considered. A common non-zero mask is derived
% and the vectors are reduced to contain only common non-zero elements 
% before the calculation is made. The number of common voxels is output.
% The p values test the hypothesis of no correlation (see also 'help corr').
% A scatter plot is also output for corresponding voxel values and the  
% the least squares fit line for each pair of reduced vectors. 
% If the user enters a threshold (e.g. 1.0), the non-zero space is re- 
% evaluated for each pair of images separately and an OR operation is
% performed on the non-zero common voxels. In contrast to AND operations, 
% the threshold is satisfied by either one OR the other image voxel of 
% each pair. AND operations are not used because the assumption that both
% voxels have to be above threshold can lead to erroneously inflated results.
% Thus, for high thresholds it would exclude pairs where only one value is
% low that may actually indicate low correlation. Only corresponding voxels
% that satisfy the OR criterion in an absolute value sense for a pair of two 
% images are considered on a voxel by voxel basis. The number of corresponding
% voxels in the correlation is different for each image pair.
% Optionally, results of the linear correlation can be graphically
% displayed and a histogram of image values is shown.
% 
% The Jackard Similarity Index is also evaluated if a theshold is entered.
% This index is considered the ratio of the sum of the common red voxels
% plus the common blue voxels that satisfy the threshold divided by the
% total voxels above or below the threshold in either one of the images.
%_______________________________________________________________________
% vp_giscorr 	ScAnVP	ver. 6.1	P. Spetsieris	1/18/2010
%                       ver. 6.2        7/7/2011, 11/27/2012, 2/8/2013
%                       ver. 7.0        1/8/16

global HelpMode NameLst POpt

if HelpMode, vp_help(mfilename), help('corr'); return ; end;

gisfiles=[]; gisimgs=[]; rho=[]; R2=[];

% Get the image filenames 
prompt=['Select (GIS) Image components for Correlation Calculation'];
disp(prompt);
if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
gisfiles=NameLst;

%prompt=['Select (GIS) Image components for Correlation Calculation'];
%disp(prompt);
%gisfiles=get_file(Inf,'*.img',prompt);

if isempty(gisfiles), return; end;

sub= size(gisfiles,1);

for i=1:sub
  % reshape into column vector
  [out1, gisimgs(:,i)]=vp_volfuns('volvect',gisfiles(i,:));
  if (out1 < 0) vp_message(['ERROR reading image', num2str(i)]); return, end;
end

%Calculate Correlation

[raw, pval]= corr(gisimgs);

disp('RAW Values R_squared ')
R2_raw= raw.*raw,


disp('Extract Common MULTI-IMAGE Mask For Img Vectors');
m0imgs=(gisimgs ~= 0);
s1=size(m0imgs,1); s2=size(m0imgs,2);
mm=ones(s1,1); for (i=1:s2), mm(:,1)=mm(:,1).*m0imgs(:,i); end; 
vimgs=[]; for (i=1:s2), vimgs(:,i)=mm.*gisimgs(:,i); end;

s1=size(gisimgs,1); 
k=0; vkimgs=[];

for (i=1:s2), vkimgs(:,i)=nonzeros(vimgs(:,i)); end; 

RawImageSize=s1,
CommonVoxels0=size(vkimgs,1),

disp('Correlation r,R2 of MULTI-IMAGE Common NON-ZERO Values');
[r, p]= corr(vkimgs),
R2=r.*r,

DisplayGraphs=0;

nn=0; dd=0.1;
prompt=[' Turn Display ON? [Y|N]' ];
POpt=vp_opt('X',prompt,'YES','Display Graphs','NO','SKIP Display');

if (POpt=='X'), DisplayGraphs=0;
elseif strcmp(POpt,'YES'),
     DisplayGraphs=1; 
end

if DisplayGraphs
 nn=0; dd=0.1;
 for i=1:s2
     xname=vp_nm('nm',gisfiles(i,:)); xname=strrep(xname,'_','-');
     figure; hist(vkimgs(:,i), 100); title(['Histogram of Vector ', num2str(i),': ',xname]);
     pos=get(gcf,'position'); dd=pos(3)/20; pos1=pos(1) + nn*dd; pos2=pos(2) - nn*dd;
     set(gcf,'position',[pos1,pos2,pos(3),pos(4)]); nn=nn+1;
 end;
end;

Thresh=0.0; OrOp=0;
keyin=vp_keyin('Enter Lower Threshold to Exclude Voxel Pairs:  ','0.0');
if ~isempty(keyin), Thresh=str2num(keyin); else return; end;

if Thresh < 0, Thresh=0 - Thresh; end;

if or( (Thresh ~= 0), (s2 > 2) ) 

 %Perform for all cases where Thresh > 0 and include Thresh=0 if more than two files 
 % If Thresh=0 and there are only two images, the result would be the same as above

 OrOp=1; % Exclude voxels if both below Thresh (Include if one is above)

 %prompt=['Include Voxels Above |Thresh| In Either Image? (Y=Either OR Op, N=Both Images (AND Op) [<Y>|N])'];
 % SKIP THE OPTIONAL AND OPERATION - IF needed remove the Comment sign in these 8 statements
 %POpt=vp_opt('X',prompt,'YES','OR OP - Voxels Above Thresh On Either Image','NO','AND OP- Both Voxels in each pair');
 %if (POpt=='X'), OrOp=1;
 %elseif strcmp(POpt,'NO'),
 %    OrOp=0; 
 %    disp('Image AND Operation') %Pick Voxels that are 1 in both masks
 %end

 if OrOp, 
     disp('Image OR Operation') %Pick Voxels that are 1 in either mask
 end; %if Or

end; %if or

if or( (Thresh ~= 0), (s2 > 2)) %  (case 0 was already done for two images)

  r=zeros(s2,s2); for i=1:s2, r(i,i)=1; end;
  R2=r; JackI=r;

  disp(['Thresh= ',num2str(Thresh)]); 
  disp('Extract Common PAIR-WISE Non-Zero Mask For Img Vectors');
  disp(' ');

  % Absolute Values:
  %Either greater than positive or lower than negative threshold

    for(i=1:s2-1),
     for (j=1:s2),
       if( j > i ), 
	  mm0=ones(s1,1); mm0(:,1)=m0imgs(:,i).*m0imgs(:,j); 
          v0imgs=[]; v0imgs(:,1)=mm0.*gisimgs(:,i); v0imgs(:,2)=mm0.*gisimgs(:,j);
          mimgs=[]; mimgs=or( (v0imgs > Thresh), (v0imgs < (0-Thresh)));
          rimgs=[]; rimgs=(v0imgs > Thresh);
          bimgs=[]; bimgs=(v0imgs < (0-Thresh));
	  
          mm=ones(s1,1); 

          % Red/Blue areas
          rmm=ones(s1,1); 
             rmm(:,1)=rimgs(:,1).*rimgs(:,2); 
          bmm=ones(s1,1); 
             bmm(:,1)=bimgs(:,1).*bimgs(:,2); 

	  if OrOp==0 % AND Operation
             mm(:,1)=mimgs(:,1).*mimgs(:,2); 
	  else
	     mm(:,1)= or(mimgs(:,1), mimgs(:,2)); 
	  end %if Or

	  vimgs=[];
          vimgs(:,1)=mm(:,1).*gisimgs(:,i); 
          vimgs(:,2)=mm(:,1).*gisimgs(:,j); 
          vkimgs=[]; vkimgs(:,1)=nonzeros(vimgs(:,1));
                     vkimgs(:,2)=nonzeros(vimgs(:,2));

          % Total Joint Significant Space either red or blue in at least one image
          CommonVoxels=size(vkimgs,1);

          % Total Joint Common Significant Red Space
          RedJointVoxels=sum(rmm);
          BlueJointVoxels=sum(bmm);

           
          if CommonVoxels == 0, 
            vp_note(['No Voxels Satisfy Thresh= ',num2str(Thresh),' for images ',num2str(i),' ',num2str(j)], 2); 
          else
            JackardIndx=(RedJointVoxels + BlueJointVoxels)/CommonVoxels;

  	    i;j; [rij, p]= corr(vkimgs);
            disp(['r',num2str(i),num2str(j),'=',num2str(rij(1,2)),', p=',num2str(p(1,2)),', Total Thresh Voxs.=',num2str(CommonVoxels)]);
            disp(['Joint RedVoxels=',num2str(RedJointVoxels),' BlueVoxels=',num2str(BlueJointVoxels),', JackardIndex=',num2str(JackardIndx)]);

	    r(i,j)=rij(1,2); r(j,i)=rij(2,1);
	    JackI(i,j)=JackardIndx; JackI(j,i)=JackI(i,j);
	    if (DisplayGraphs), %plot
    		figure; plot(vkimgs(:,1), vkimgs(:,2), '.b'),
     		drawnow; % try this instead of the i,j printout trick
     		pos=get(gcf,'position'); dd=pos(3)/20; pos1=pos(1) + nn*dd; pos2=pos(2) - nn*dd;
     		set(gcf,'position',[pos1,pos2,pos(3),pos(4)]); nn=nn+1;
     		hl=lsline; set(hl,'linewidth',2); set(hl,'color',[0 0 0]);
     		xname=vp_nm('nm',gisfiles(i,:)); xname=strrep(xname,'_','-');
     		yname=vp_nm('nm',gisfiles(j,:)); yname=strrep(yname,'_','-');
     		xlabel(['Image Vector ', xname ]);
     		ylabel(['Image Vector ', yname ]);
     		title(['Linear Plot of Image Vector ', num2str(j), ' vs. ', num2str(i),', Com.Vox.=',num2str(CommonVoxels)]); 
		rji=round(rij(1,2)*1000)/1000; Rsq=round(rij(1,2)*rij(1,2)*1000)/1000;
		if(OrOp)
		 legend(['Thresh=', num2str(Thresh),' (OR Op)', ' CORR: r=',num2str(rji),', Rsq=',num2str(Rsq),', p=',num2str(p(1,2))]);
		else
		 legend(['Thresh=', num2str(Thresh),' (AND Op)', ' CORR: r=',num2str(rji),', Rsq=',num2str(Rsq),', p=',num2str(p(1,2))]);
		end; %Or
	    end; % if Display
          end % if Common
       end; % if j
     end; %for j
    end; %for i
   
    disp(' ');
    disp('Correlation r,R2 of PAIR-WISE Common NON-ZERO Values');
    r,
    R2=r.*r,
    JackI,

end %if or


if and((Thresh == 0), (s2==2)) 
 if(DisplayGraphs)
  %plot
  nn=0; dd=0.1;

  for(i=1:s2-1),
   i;  %i,j printout added because of a timing bug on some computers 
   for (j=1:s2),
    if( j>i ),
     j;
     figure; plot(vkimgs(:,i), vkimgs(:,j), '.b'),
     drawnow; % try this instead of the i,j printout trick
     pos=get(gcf,'position'); dd=pos(3)/20; pos1=pos(1) + nn*dd; pos2=pos(2) - nn*dd;
     set(gcf,'position',[pos1,pos2,pos(3),pos(4)]); nn=nn+1;
     hl=lsline; set(hl,'linewidth',2); set(hl,'color',[0 0 0]);
     %xlabel(['Image Vector ', vp_nm('nm',gisfiles(i,:)) ]);
     %ylabel(['Image Vector ', vp_nm('nm',gisfiles(j,:)) ]);
     xname=vp_nm('nm',gisfiles(i,:)); xname=strrep(xname,'_','-');
     yname=vp_nm('nm',gisfiles(j,:)); yname=strrep(yname,'_','-');
     xlabel(['Image Vector ', xname ]);
     ylabel(['Image Vector ', yname ]);
     title(['Linear Plot of Image Vector ', num2str(j), ' vs. ', num2str(i),', Com.Vox.=',num2str(CommonVoxels0)]); 
     %legend(['THRESH=0, CORR: r=',num2str(r(i,j)),', Rsq=',num2str(r(i,j)*r(i,j)),', p=',num2str(p(i,j))]);
     rji=round(r(i,j)*1000)/1000; Rsq=round(r(i,j)*r(i,j)*1000)/1000;
     legend(['THRESH=0, CORR: r=',num2str(rji),', Rsq=',num2str(Rsq),', p=',num2str(p(i,j))]);
    end; %if j > i
   end; % for j
  end; %for i
 end; % Display Graphs
end; %if and
