% vp_groi.m
% Get Region of interest
% 
% The user interface for the roi template definition routine 'vp_groi'
% allows the interactive definition of regular (Rectangular or Elliptial)
% and Irregular (Polygonal) regions and region specific thresholds. Once
% defined, various options exist for modifying a region, deleting,
% moving or displaying its statistical information or mapping its
% threhold pixels (move, reshape, statistics, map, relabel, rethresh,
% delete). The user must select the mode of operation and then
% pick the region. Other menu options are used to create statistical
% reports and regional threhold pixel maps for all planar regions.
% Old templates can be read in with or without voxel size fitting
% and y coordinate flipping. Care must be taken to avoid overwriting
% existing regions which will render them invisible but may still
% be included on reports and templates.
%
% Every new region is assigned a corresponding variable threshold value
% defined below. Regions input from a template maintain the preassigned
% threshold values unless reset (rethresh):
%
% VARIABLE THRESHOLD 0-100
% Only pixels which satisfy the following regional threshold criterium
% are processed:
%
%         Min <= Pix Value <= Max
%
%         where, Max = Current Plane Region Maximum
%                Min = Max - (Threshold/100)*(Max-GMIN)
%                GMIN = Global Min for all pixels on all planes.
% 			
% On reports each region is assigned a label number and name and
% left (L), right(R) or central orientation (C) as selected by the user.
% The first 100 label numbers are reserved for standard label names
% as displayed by the labeling routine. The users can define their
% own labels and these are given sequential numbers above 100 and are saved
% in each template to be recalled each time the template is re-read with
% the 'roi' or 'grouproi' routines. The user is forced to define a region
% label name and orientation everytime a new region is defined.
% The same non-standard labels may not have the same label numbers in
% templates defined on separate runs of the program. Group roi routines
% take this into account by considering the label name as the defining
% criterion on input and may reassign label numbers.
% Regional specifications are saved in pixel units with the origin of
% the XY coordinate system in the lower left corner for newer templates
% and upper right for older templates. Conversions can be made by setting
% 'FLIP' ON. FIT can be set to convert templates using Physical VOXEL
% dimensions and the pixel coordinates of the ORIGIN a fixed point in the
% subjects image space. Using the FIT option the same template can be
% used for corregistered PET and MRI images even if the raster sizes
% and number of planes differ.
%
% STANDARD ROI LABELS:
%
% 2   Hemisphere                45  Posterior Temporal
% 5   Cerebellar Hemisphere     50  Anterior Cingulate
% 6   Vermis                    51  Posteror Cingulate
% 10  Pons                      54  Middle Frontal
% 15  Midbrain                  55  Lateral Premotor
% 20  Basal Ganglia             56  Sensori-Motor
% 21  Caudate Nucleus           57  Superior Frontal
% 22  Putamen                   60  Calcarine
% 25  Thalamus                  61  Temporo-Occipital
% 29  Inferior Temporal         65  Cuneus
% 30  Hippocampus               69  Parieto-Occipital
% 35  Middle Temporal           70  Inferior Parietal
% 36  Superior Temporal         75  Paracentral
% 39  Orbito-Frontal            80  White Matter
% 40  Operculum              91-98  Other Stndrd. Roi
% 
% TEMPLATE REGION SPECIFICATIONS:
%
% Rectangular - saves diagonal coordinates
% 	re label#[R/L]  X1 Y1 X3 Y3 
%
% Elliptical - saves Origin, Point on One Axis, and opposite Radius
% 	el label#[R/L]  X0 Y0 X1 Y1 R2 
%
% Irregular - saves Initial Point coordinates, No. of Points and
%		Point Dx, Dy increments from Initial Point.
%
%	ir label#[R/L]  X1 Y1 (n+1) 0 0 (X2-X1) (Y2-Y1) ... 0 0
%
% 
% (see also vp_roi.m)
%__________________________________________________________________________
% vp_groi.m	C.0 	P. Spetsieris	 Nov. 13, 1998

global RoiNo RoiTyp RegDef RoiThresh RoiChoice RoiOrient FI Lh
global ActRoi ActHand LabIds LabNames Picking herror herr roigui
global NonStdLabs NonStdNo hlabmenu2 NonStdLabStr RoiDiam
global DIM Fitting Flipping Ecolor hroidel FName GMIN GMAX

if HelpMode, vp_help(mfilename), return ; end;

Message0=['............................................ '];
Message=Message0;

RoiFName=[];

RoiChoice=1;

RoiOrient=' ';

Picking='ON' ;

Fitting=1;	
Flipping=0;	

LabNames=str2mat(blanks(40),blanks(40));
LabIds=str2mat(blanks(4),blanks(4));

StaLabStr=['2  Hemisphere|5  Cerebellar Hemisphere|6  Vermis|10  Pons|',...
'15  Midbrain|20  Basal Ganglia|21  Caudate Nucleus|22  Putamen|',...
'25  Thalamus|29  Inferior Temporal|30  Hippocampus|35  Middle Temporal|',...
'36  Superior Temporal|39  Orbito-Frontal|40  Operculum|',...
'45  Posterior Temporal|50  Anterior Cingulate|51  Posteror Cingulate|',...
'54  Middle Frontal|55  Lateral Premotor|56  Sensori-Motor|57  Superior Frontal|',...
'60  Calcarine|61  Temporo-Occipital|65  Cuneus|69  Parieto-Occipital|',...
'70  Inferior Parietal|75  Paracentral|80 White Matter|',...
'91  Other Stndrd. Roi|92  Other Stndrd. Roi|',...
'93  Other Stndrd. Roi|94  Other Stndrd. Roi|95  Other Stndrd. Roi|',...
'96  Other Stndrd. Roi|97  Other Stndrd. Roi|98  Other Stndrd. Roi'];

Fcolor = [0.5 0.7 0.8];
Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];

groigui = figure('Color',Fcolor,'Name','gROIgui',...
'NumberTitle','off','Position',[30,370,260,260],...
'Units','Pixels');
	
RoiTyp='r';
RoiThresh=100;

RegDef0 =[0 50 50 10 10 0] ;
RegDef = RegDef0;

RoiNo=0; ActRoi=0;

RoiDiam=10;

XTicks=[];
YTicks=[];
j=1;
for i=2:2:DIM(1),XTicks(j)=i;j=j+1;end;
j=1;
for i=2:2:DIM(2),YTicks(j)=i;j=j+1;end;

%NonStdLabs=str2mat('Non Standard Label                      ');
%NonStdLabStr=['Non Standard Label                      '];
%NonStdNo=101;

%cbRoiFName=['RoiFName=get(hname,''string'');'];
cbRoiFName=['RoiFName=get(hname,''string'');','Message=Message0 ;',...
	'[out1,Message]=vp_roirpt(''read'',RoiFName); ',...
	'set(herror,''string'',['' '',Message]);'] ;
	
cbPRoiName=['RoiFName=get_file(1,''.roi'',''Please Select Roi Template File'',[]); ',...
	'if ~isempty(RoiFName),',...
	'set(hname,''string'',RoiFName); ','Message=Message0 ;',...
	'[out1,Message]=vp_roirpt(''read'',RoiFName); ',...
	'set(herror,''string'',['' '',Message]); end;'] ;
	

cbRoiThresh=['RoiThresh=str2num(get(hRoiThresh,''string'')); ',...
	['if(RoiThresh < 0) | (RoiThresh > 100) Message=''Error: Thresh Out of Bounds'';',...
	'else Message=Message0; end; '],...
		'set(herror,''string'',['' '',Message]);'];

cbSliRoiThresh=['RoiThresh=ceil(get(gco,''Value''));, set(hRoiThresh,''string'',RoiThresh);',...
	['if(RoiThresh < 0) | (RoiThresh > 100) Message=''Error: Thresh Out of Bounds'';',...
	'else Message=Message0; end; '],...
		'set(herror,''string'',['' '',Message]);'];

cbroi=['RoiTyp=get(hroi,''string''); ',...
	['if(RoiTyp ~= ''r'') & (RoiTyp ~= ''e'') & (RoiTyp ~= ''i'') ',...
	'& (RoiTyp ~= ''c'')Message=''Error: Invalid Roi Type'';',...
	'else Message=Message0; end; '],...
	'set(herror,''string'',['' '',Message]);, vp_roicb(RoiTyp);'];

cbRoiDiam=['RoiDiam=str2num(get(hRoiDiam,''string'')); ',...
	['if(RoiDiam < .1) | (RoiDiam > DIM(1)*VOX(1)),Message=''Error: Diameter Out of Bounds'';',...
	'else Message=Message0; end; '],...
		'set(herror,''string'',['' '',Message]);'];

cbRoiMenu = ['RoiChoice=get(hroimenu,''Value'');';];
%uicontrol(groigui,'string', 'IMPORT ROIS'
	
%cbRoiDel = ['no=str2num(get(hroidel,''String'')); no0=no;',
cbRoiDel = ['snos=get(hroidel,''String''); j=findstr(snos,''-'');',...
		'if isempty(j), no1=str2num(snos); no2=no1; else,'...
		'no1=str2num(snos(1:j-1)); no2=str2num(snos(j+1:length(snos))); end;',...
		'for no=no1:no2, no0=no;',...
		'set(herror,''string'',''PROCESSING '');',...
		'lh=findobj(''Tag'',''roi''); nr=size(lh);',...
		'for i=nr(1):-1:1,',...
		'udata=get(lh(i),''UserData'');',...
		'nroi=udata(1); if nroi==no, delete(lh(i));',...
		'set(herror,''string'',[''Roi '', num2str(no), '' Deleted'']);',...
		'no=-1; drawnow; end; end; if no==no0,',...
		'set(herror,''string'',[''Roi '', num2str(no), '' Not Found'']);',...
		'pause(1); end; end;'];
		
%		'set(herror,''string'',''Non-Existant Roi''); end;'];

	
uicontrol(groigui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[208 237 40 20],'callback','vp_help(''vp_groi'');');
	
uicontrol(groigui,'Style','Text','string',...
	['FNAME: ' FName],...
	'position',[10 222 250 15],...
	'TooltipString','Image File Name',...
	'FontName','Times','FontSize',9,'FontAngle','normal',...
	'BackgroundColor',Fcolor);
	 
uicontrol(groigui,'Style','Text','string',...
	['GLOBAL Min,Max: (' sprintf('%.2f, ',GMIN) sprintf('%.2f',GMAX) ')'],...
	'position',[10 207 250 15],...
	'TooltipString','Global Data Minimum and Maximum',...
	'FontName','Times','FontSize',9,'FontAngle','normal',...
	'BackgroundColor',Fcolor, 'Callback',...
	'set(hroierr,''string'',''Thresh Min =Reg Max- Thresh/100(Reg Max - Gl Min)'');' );
	 
uicontrol(groigui,'string', 'Thresh 0-100',...
	'TooltipString','Set Percent of Max Threshold range [Min Max]',...
	'position',[10 180 75 25],'BackgroundColor',Bcolor);
	
uicontrol(groigui,'Style','slider', 'Position', [85 180 134 25], 'Min',0,'Max',100,...
	'TooltipString','Set Percent of Max Threshold. Min=Max -Thresh/100)(Max-GMIN)',...
	'Value',RoiThresh,'Callback',cbSliRoiThresh);
	
%	'TooltipString','Set Percent of Max Threshold. Min=(1-Thresh/100)Max',
hRoiThresh=uicontrol(groigui,'style','edit','string',RoiThresh,...
	'TooltipString','Set Percent of Max Threshold. Min=Max -Thresh/100)(Max-GMIN)',...
	'position',[219 180 30 25],'BackgroundColor',Kcolor,'callback',cbRoiThresh);

uicontrol(groigui,'string', 'RECT',...
	'TooltipString','Enter two points for rectangular roi',...
	'position',[10 150 40 25],'BackgroundColor',Bcolor, 'callback',...
	['RoiTyp=''r''; vp_roicb(''r'');']);
	
%	['RoiTyp=''r''; figure(FI); set(FI,''ButtondownFcn'',''nrect=rbbox(1)'');, waitforbuttonpress;']);
%uicontrol(groigui,'string', 'ELLI','position',[80 90 70 25]
uicontrol(groigui,'string', 'ELLI','position',[50 150 40 25],...
	'TooltipString','Enter two points for center and long axis of elliptical roi',...
	'BackgroundColor',Bcolor, 'callback',...
	 ['RoiTyp=''e''; vp_roicb(''e'');'] );
	
uicontrol(groigui,'string', 'CIRC','position',[90 150 40 25],...
	'TooltipString','Enter one point for center of circular roi',...
	'BackgroundColor',Bcolor, 'callback',...
	 ['RoiTyp=''e''; vp_roicb(''c'');'] );
	 
hRoiDiam=uicontrol(groigui,'style','edit','string',num2str(RoiDiam),'Tag','RoiDiam',...
	'TooltipString','Enter diameter of circular roi in physical units (mm)',...
	'position',[130 150 40 25],'BackgroundColor',Kcolor,'callback',cbRoiDiam);
	
uicontrol(groigui,'string', 'IRR ',...
	'TooltipString','Enter irregular roi vertices',...
	'position',[170 150 40 25],'BackgroundColor',Bcolor, 'callback',...
	['RoiTyp=''i''; vp_roicb(''i''); '] );

hroi=uicontrol(groigui,'style','edit','string',RoiTyp,'Tag','roied',...
	'TooltipString','Enter roi type ''r'' ''e'' ''c'' or ''i''',...
	'position',[210 150 40 25],'BackgroundColor',Kcolor,'callback',cbroi);

uicontrol(groigui,'string', 'TEMPLATE',...
	'TooltipString','Import ''.roi'' template file',...
	'position',[10 120 70 25],'BackgroundColor',Bcolor,...
	'callback',cbPRoiName);
	
uicontrol(groigui,'Style','Check','String','FIT',...
	'TooltipString','Resize Template to Fit Image Dimensions',...
	'Value',Fitting,'Position',[150 90 45 25],...
	'CallBack',[ 'if ~Fitting, Fitting=1; else Fitting=0, end;',...
	'set(hroierr,''string'',''     '');' ]);
	
uicontrol(groigui,'Style','Check','String','FLIP',...
	'TooltipString','Flip Correction for Y',...
	'Value',Flipping,'Position',[195 90 45 25],...
	'CallBack',[ 'if ~Flipping, Flipping=1; else Flipping=0, end;',...
	'set(hroierr,''string'',''     '');' ]);
	
hname=uicontrol(groigui,'style','edit','string',RoiFName,...
	'TooltipString','Key in ''.roi'' template file name',...
	'position',[80 120 170 25],'BackgroundColor',Kcolor,'callback',cbRoiFName);

%	'FontName','MS Sans Serif','FontSize',8,'FontAngle','italic','FontWeight','demi',
uicontrol(groigui,'Style','Text','string', 'DELETE','position',[10 85 70 25],...
	'TooltipString','Delete by Roi No.',...
	'FontName','Times','FontSize',9,'FontAngle','normal',...
	'BackgroundColor',Fcolor, 'Callback',...
	'set(hroierr,''string'',''Enter Roi No. or range N1-N2 to Delete'');' );
%	 ['no=get(hroidel,''Value''); delete(no);'] );
	 
hroidel=uicontrol(groigui,'style','edit','string',num2str(RoiNo),'Tag','RoiDelNo',...
	'TooltipString','Enter Roi Number to Delete',...
	'position',[70 90 50 25],'BackgroundColor',Kcolor,'callback',cbRoiDel);
	
hmapon=uicontrol(groigui,'string', 'THRESHOLD PIXEL MAP',...
	'TooltipString','Display evaluated threshold pixels',...
	'position',[10 65 150 20],'BackgroundColor',Bcolor,'Visible','on',...
	 'callback',...
	 ['vp_roimap(''display'',0)']);
	 
	 
%	 ['vp_roimap display; set(hroimenu,''Enable'',''off'');',...
%	 'set(hmapon,''Visible'',''off''); set(hmapoff,''Visible'',''on'');']);
	
%hmapoff=uicontrol(groigui,'string', 'PIXEL MAP OFF',...
%	'position',[10 60 95 20],'BackgroundColor',Bcolor,'Visible','off',...
%	 'callback',...
%	 ['set(hroimenu,''Enable'',''on'');',...
%	 'set(hmapoff,''Visible'',''off''); set(hmapon,''Visible'',''on'');']);
	

%hroimenu=uicontrol(FI,'Style','popup','Visible','off','String','move|modify|stat|delete',
hroimenu=uicontrol(groigui,'Style','popup','Visible','on','String',...
	'move|reshape|statistics|map|relabel|rethresh|delete',...
	'Tag','RoiMenu',...
	'TooltipString','MENU: set mode for pick roi operations.',...
    	'Position',[170 25 82 60],'callback',cbRoiMenu);

uicontrol(groigui,'String','REPORT',...
	'Position',[10 35 55 25],...
	'TooltipString','Report Roi Info. and Statistics',...
	'CallBack',['set(gcbf,''Pointer'',''watch'');',...
	'vp_roidrpt; set(gcbf,''Pointer'',''arrow'');',...
	'set(herror,''string'',['' '',Message]);']);

uicontrol(groigui,'String','GRID',...
	'Position',[65 35 30 25],...
	'TooltipString','Hit(1): Set Axis On; Hit(2): Set Grid On Every Other Pixel;',...
	'CallBack',['figure(FI); axis on; grid;']);

uicontrol(groigui,'String','OFF',...
	'Position',[95 35 25 25],...
	'TooltipString','Set Axis/Grid Off',...
	'CallBack',['figure(FI); axis off;']);

hsaver=uicontrol(groigui,'string', 'SAVE/NEXT',...
	'position',[120 35 70 25],'BackgroundColor',Bcolor,'Visible','on',...
	'TooltipString','Go to SAVE/SKIP & NEXT plane options.',...
	 'callback',...
	 ['Message=Message0 ; ',...
	 'vp_roirpt(''savexit''); ',...
	 'set(herror,''string'',['' '',Message]);']);
	
uicontrol(groigui,'String','QUIT',...
	'Position',[190 35 60 25],...
	'TooltipString','End task and Exit',...
	'CallBack',['delete(FI);',...
  	'delete(findobj(''Name'',''REPORT''));',...
  	'delete(findobj(''Name'',''Statistics''));',...
  	'delete(findobj(''Name'',''ReLabel''));',...
  	'delete(findobj(''Name'',''ReThresh''));',...
  	'delete(findobj(''Name'',''Threshold MAP''));',...
  	'delete(findobj(''Name'',''Save Rois''));',...
  	'delete(findobj(''Name'',''LABELgui''));',...
	'delete(gcbf); delete(roigui);']);
	
	
%	'CallBack',['set(groigui,''UserData'',1); ', 'delete(groigui)']);

herror = uicontrol(groigui,'string', ['MSG: ..............................'],...
	'TooltipString','Report any errors found.',...
	'position',[10 10 240 25],...
	'CallBack',['set(herror,''string'',['' '',Message]);'],...
	'BackgroundColor',Ecolor);


% LABEL CONTROLS
%
%hlabelgui = figure('Color','w','Name','LABELgui',...
%'NumberTitle','off','Position',[100,200,200,100],...
%'Units','Pixels','Visible','off');
	
hlabelgui = figure('Color',Fcolor,'Name','LABELgui',...
'NumberTitle','off','Position',[100,200,200,140],...
'Units','Pixels','Visible','off');
	
cbRoiOrient = ['RoiOr=get(hroiorient,''Value''); if RoiOr==2, RoiOrient=''L'',',...
	'elseif RoiOr==1, RoiOrient='' '', else RoiOrient=''R'', end;'];

uicontrol(hlabelgui,'Style','text','string','Orientation',...
	'TooltipString','Select orientation ''left'', ''right'', or ''center''',...
	'position',[2 115 70 15]);
	
hroiorient=uicontrol(hlabelgui,'Style','popup','Visible','on','String','center|left|right',...
	'Tag','RoiOrient',...
	'TooltipString','Select orientation ''left'', ''right'', or ''center''',...
	'BackgroundColor',[0.9 0.9 1],...
    	'Position',[75 75 82 60],'callback',cbRoiOrient);

%	'[LabIds(RoiNo,:) LabNames(RoiNo,:)]=vp_roifuns(''labid'',labno), '
%	'[labid labname]=vp_roifuns(''labid'',labno), ',
%	'LabIds=str2mat(LabIds, labid); ',
%	'LabNames=str2mat(LabNames, labname); ',

cblabmenu = ['labno=get(hlabmenu,''Value'') ,',...
	'[LabIds(ActRoi,:) LabNames(ActRoi,:)]=vp_roifuns(''labid'',labno), ',...
	'set(findobj(''Name'',''gROIgui''),''Visible'',''on'');',...
	'set(findobj(''Name'',''LABELgui''),''Visible'',''off'');',...
	'Picking=''ON''; set(ActHand,''Color'',[1 1 1],''LineWidth'',[0.5]);',...
	]; 
	
cblabmenu2 = ['labno=get(hlabmenu2,''Value'') ,',...
	'nid=labno+100, LabNames(ActRoi,:)=NonStdLabs(labno,:), ',...
	'labid= strcat(num2str(nid), RoiOrient); siz=size(labid);',...
	'LabIds(ActRoi,:)= [labid blanks(4-siz(2))];',...
	'set(findobj(''Name'',''gROIgui''),''Visible'',''on'');',...
	'set(findobj(''Name'',''LABELgui''),''Visible'',''off'');',...
	'Picking=''ON''; set(ActHand,''Color'',[1 1 1],''LineWidth'',[0.5]);',...
	]; 
	
uicontrol(hlabelgui,'Style','text','Units','pixels','String','Standard Labels',...
	'TooltipString','Select One of the Standard labels',...
    	'Position',[2 95 196 15]);
	
uicontrol(hlabelgui,'Style','text','Units','pixels','String','Non Standard Labels',...
	'TooltipString','Specify non-standard label',...
    	'Position',[2 50 196 15]);

hlabmenu=uicontrol(hlabelgui,'Style','popup','Visible','on','String',StaLabStr,...
	'Tag','RoiMenu','Units','pixels',...
	'BackgroundColor',[0.9 0.9 1],...
	'TooltipString','Select standard label',...
    	'Position',[2 30 196 60],'callback',cblabmenu);
	
hlabmenu2=uicontrol(hlabelgui,'Style','popup','Visible','on','String',NonStdLabStr,...
	'Tag','RoiMenu2','Units','pixels',...
	'BackgroundColor',[0.9 0.9 1],...
	'TooltipString','Select previously defined non-standard label',...
    	'Position',[2 -15 196 60],'callback',cblabmenu2);
	


%hlabmenu=uicontrol(hlabelgui,'Style','popup','Visible','on','String',...
%['2  Hemisphere|5  Cerebellar Hemisphere|6  Vermis|10  Pons|',...
%'15  Midbrain|20  Basal Ganglia|21  Caudate Nucleus|22  Putamen|',...
%'25  Thalamus|29  Inferior Temporal|30  Hippocampus|35  Middle Temporal|',...
%'36  Superior Temporal|39  Orbito-Frontal|40  Operculum|',...
%'45  Posterior Temporal|50  Anterior Cingulate|51  Posteror Cingulate|',...
%'54  Middle Frontal|55  Lateral Premotor|56  Sensori-Motor|57  Superior Frontal|',...
%'60  Calcarine|61  Temporo-Occipital|65  Cuneus|69  Parieto-Occipital|',...
%'70  Inferior Parietal|75  Paracentral|80 White Matter|',...
%'90  Other Stndrd. Roi|91  Other Stndrd. Roi|92  Other Stndrd. Roi|',...
%'93  Other Stndrd. Roi|94  Other Stndrd. Roi|95  Other Stndrd. Roi|',...
%'96  Other Stndrd. Roi|97  Other Stndrd. Roi|98  Other Stndrd. Roi'],...
%	'Tag','RoiMenu',...
%	'BackgroundColor',[0.9 0.9 1],...
%	'Position',[2 2 196 60],'callback',cblabmenu);


%cblabedit=['labname=get(gcbo,''string'') ,',...
%	'siz=size(labname); labname=[labname blanks(40-siz(2))], ',...
%	'ActRoi, LabNames(ActRoi,:)=labname,',...
%	'LabIds(ActRoi,:)= strcat(''101'',RoiOrient) ,',...
%	'set(findobj(''Name'',''gROIgui''),''Visible'',''on'');',...
%	'set(findobj(''Name'',''LABELgui''),''Visible'',''off'');',...
%	'Picking=''ON''; set(ActHand,''Color'',[1 1 1],''LineWidth'',[0.5]);',...
%	]; 
	
uicontrol(hlabelgui,'Style','text','string','Other Specify',...
	'TooltipString','Key in new label',...
	'position',[2 2 70 15]);
	
cblabedit=['labname=get(gcbo,''string'') ,',...
	'siz=size(labname); labname=[labname blanks(40-siz(2))], ',...
	'nid=NonStdNo+1;',...
	'for i=1:98, if strcmp(vp_roifuns(''labname'',i),labname), nid=i ; end; end;',... 
	'for i=101:NonStdNo, if strcmp(NonStdLabs(i-100,:),labname), nid=i ; end; end;',... 
	'if nid == NonStdNo+1, NonStdNo=nid;',...
	'NonStdLabs=str2mat(NonStdLabs,labname),',...
	'NonStdLabStr=strcat(NonStdLabStr,''|'',labname);',...
	'set(hlabmenu2,''String'',NonStdLabStr);',...
	'end;',...
	'ActRoi, LabNames(ActRoi,:)=labname;',...
	'labid= strcat(num2str(nid), RoiOrient); siz=size(labid);',...
	'LabIds(ActRoi,:)= [labid blanks(4-siz(2))];',...
	'set(findobj(''Name'',''gROIgui''),''Visible'',''on'');',...
	'set(findobj(''Name'',''LABELgui''),''Visible'',''off'');',...
	'Picking=''ON''; set(ActHand,''Color'',[1 1 1],''LineWidth'',[0.5]);',...
	]; 
	
	
hlabel=uicontrol(hlabelgui,'style','edit','string','Non Standard Label',...
	'TooltipString','Key in any non-standard or standard label',...
	'position',[75 2 123 20],'BackgroundColor',[0.9 .9 1.0],'callback',cblabedit);
	
       
% THRESHOLD MAP CONTROLS

%	FMap = figure('Color',[0 0 1]*.8,...
%	'Name','Threshold Map',...
%	'NumberTitle','off',...
%	'Position',[50,50,512,512],...
%	'Resize','on',...
%	'Tag','RoiMap',...
%	'Units','Pixels',...
%	'Pointer','Cross',...
%	'Visible','off');
	
%	uicontrol(FMap,'String','hot ',...
%	'Position',[355 10 45 25],...
%	'CallBack','colormap(hot)');
%
%   	uicontrol(FMap,'String','gray ',...
%	'Position',[310 10 45 25],...
%	'CallBack','colormap(gray)');

%	uicontrol(FMap,'String','CLEAR ',...
%	'Position',[255 10 55 25],...
%	'CallBack',['vp_roimap(''abort'');']);
	
%	uicontrol(FMap,'String','UPDATE ',...
%	'Position',[200 10 55 25],...
%	'CallBack',['vp_roimap(''display'',0);']);
	
figure(FI);
set(gca,'Xtick',XTicks);
set(gca,'Ytick',YTicks);

