% vp_grouproi.m
% Group Region of interest analysis
%___________________________________________________________________________
% vp_grouproi is the region of interest (ROI) GROUP REPLAY routine.
% It can be run on a single subject or a group of coregistered subjects.
% The user picks the members of the group and selects one of two types
% of templates, either a set of text (.roi) templates, within a range of
% planes, or a Brain Atlas image(a default ROIAtlas.img is provided). If
% the Global Normalization Option is ON each image is divided by its masked
% mean. A common mask must be previously selected for this option.
%
% TXT .ROI TEMPLATES: For .roi files, the user selects the min and max
% plane limit files and all in between files are processed. These
% files which include planar region coordinate definitions and variable
% regional threshold values are created using 'ROI (Template)'. The program
% processes all the planes within the limits. In the AUTOMATIC mode,
% regions can not be modified and the input images are assumed to be
% registered or normalized. In INTERACTIVE mode the user has various
% options to change the size and orientation of each region to fit
% individual differences but not to delete it.
% Threshold can be specified using either variable or fixed percent of
% the region max or fixed minimum. Variable values must be specified
% in the input template. Fixed value options can be selected by the user.
%
% ROI ATLAS IMAGE TEMPLATES: In the ROI BRAIN ATLAS image, regions are
% pre-defined on each plane using a fixed coded index value for each 
% voxel within a given region type. Limit planes are user specified.
% The names of the regions are listed numerically in a separate text
% file with the name AtlasNameLabs.txt that must accompany the atlas.
% Following an initial text line defining the file, each region is 
% listed in numerical order as num name or num name_L or num name_R.
% Interactive mode, the "fit" option, variable thresholding and .roi 
% type TEMPLATE output are not operational for the ATLAS option.
%
% For both .roi and atlas templates the user must insure that the
% template(s), disk image files and any mask are in the same Right-Left (RL)
% or Left-Right (LR) orientation (i.e., Radiological or Neurological).
% To assist the user, an option is provided to check the assumed input 
% orientation setting and reset it, if needed, on the main menu. If the
% input setting is correct, the display will be Neurologically (LR) oriented,
% i.e. right side is anatomical right. Text .roi templates should be
% created in the Neurological orientation to insure proper processing
% within the display. For Atlas regions, display is Neurological but
% processing occurs in matched subject/Atlas disk file orientation. Thus,
% Atlas files are provided in both RL and LR orientation to match input.
%
% Regions can be specified as either 2D or 3D. For 3D regions
% all regions having the same label across planes are considered
% in determining the region maximum.
% The maxima are found in a preliminary run across all planes and voxels 
% which satisfy the following threshold criteria whether they are 
% 2D or 3D are processed:
%
%                Min <= Pix Value <= Max
%		 where, Min = Max - (Thresh/100)*(Max-GMIN)
%
% Variable Thresh for 3D regional data, is evaluated based on the maximum
% for that region over all the planes that contain that region label.
% GMIN is the subjects global minimum file data value and is usually  0,
% except when negative data values exist or when files have been cropped
% or processed in other ways. The Global Min & Max are displayed by the
% group roi processing routine and are written in the'Template' type (.roi)
% reports. Previous versions of ScAn/VP considered GMIN to be zero and may
% yield different results in the previously mentioned exceptional cases.
% Negative values can be set to zero using the image processing 'Zero Negs'
% option.
%
% Options are provided for 'fitting' roi templates to coregistered,
% similarly oriented images even if they do not have the same dimensions.
% The program looks for the coordinates of the physical origin which must
% be properly set in the header. The 'flipping' option allows older '*.roi'
% templates that were processed in the 'matrix' coordinate system to be
% processed by flipping Y coordinates to correspond to the Cartesian convention.
%
% ROI REPORT
% Reports can be produced in either GROUP SUMMARY(_SUM.txt) or SUBJECT(_SAS.txt) 
% format. TEMPLATE output reports are similar to the input 'Template' format and
% follow the 'Name.Pln.roi' naming convention. PLANAR reports record results
% plane by plane, whereas COMPOSITE reports average results across planes for
% each region having the same label. The GROUP SUMMARY reports (_SUM.txt 
% and _SUM_c.txt) tabulate mean values for each region type for each subject
% or for the composite group. GROUP SUMMARY PLANAR reports are saved in the 
% file named 'name_SUM.txt' and GROUP SUMMARY COMPOSITE reports are saved in
% 'name_SUM_c.txt' where name is the first group member name selected.
% For 3D data the summary reports are named 'name_SUM3D(_c).txt'. The
% SUBJECT (_SAS.txt) type option outputs subject reports summarizing the means
% and number of threshold pixels for each plane and each region separately. The
% COMPOSITE option for SUBJECT reports gives the mean and NThPX values for each
% region type and the mean over all regions for left, right center and total.
% Reports are overwritten with each program use, unless renamed by the user.
% For GROUP SUMMARY COMPOSITE reports, a separate report (_SUMn_c.txt) is output 
% which includes in parenthesis the number of threshold pixels for each mean value 
% reported (for ATLAS porcessing the standard deviation is also printed). 
% The *SUMn_c.txt reports give the total number or threshold pixels greater or
% equal to the minimum threshold pixels. When the minimum threshold of max
% value is set to 0, that number should be the total number of pixels if there
% are no negative values. If there are negatives, you must choose the Global Minimum
% as the Fixed Threshold Minimum value. When there are no negatives, use the Fixed
% Threshold and set it to 100% to get all the voxels. 
%______________________________________________________________________________
% vp_grouproi.m	C.0 	P. Spetsieris	Mar. 29, 1999
%          Rev. 5.8	P. Spetsieris	Feb. 22, 2007
%          Rev. 7.0	P. Spetsieris	Oct. 19, 2015


global FI FNames FName roigui hroierr Automatic PlaneNo0 PlaneNo
global RoiFNames TemplRpt SASRpt SAS_cRpt CompRpt PlaneRpt SUMRpt
global ThreshMode FixedThresh FixedThMin
global HelpMode htemp Fitting Flipping Ecolor
global GNop MaskOpt MaskImg TemplateOpt
global AtlasNm AtlasDIM DefAtlasNm gPrevDirs
global InputOrient DisplayOrient FMenu

if HelpMode, vp_help(mfilename), return ; end;

%                (1 - Thresh/100)*Max <= Pix Value <= Max,

roigui=findobj('Name','GROUP ROIgui');
if ~isempty(roigui)
	if(strcmp(get(roigui,'Visible'),'on'))
		figure(roigui);
	end
	return
end

grprogui=findobj('Name','grprogui');
if ~isempty(grprogui)
	if(strcmp(get(grprogui,'Visible'),'on'))
		figure(grprogui);
	end
	return
end

vp_imframe;

FName=[];
RoiFNames=[' ',' '];

WndNo=0;
WndScl=4;
PlaneNo=1;
Message0=['............................................ '];
Message=Message0;
Automatic=1;
IdentGroup=0;
DefTemplate=0;
TemplRpt=0;
SASRpt=0;
SAS_cRpt=0;
PlaneRpt=0;
CompRpt=1;
SUMRpt=1;
ThreshMode=2; %VARIABLE %of 3D REG MAX  
	
FixedThresh=100;
FixedThMin=0;
Fitting=0;
Flipping=0;
GNop=0; % global normalization ON=1; OFF=0;
MaskOpt=0; MaskImg=[];

Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];
Fcolor = [.8,.8,1];

roigui = figure('Color',Fcolor,'Name','GROUP ROIgui',...
'NumberTitle','off','Position',[100,200,250,295],...
'Units','Pixels');
	

cbtext= ['text(18,75,''Report Type'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

cbtext2= ['text(10,98,''Threshold'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

eval(cbtext);

eval(cbtext2);

axis off;

pcbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented disk input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented disk input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

uicontrol(roigui,'string', 'Check RL/LR Input File Orientation',...
	'TooltipString',['Check: Radiological (Right is Left) or Neurological ',...
    '(Right is Right) Disk Input File Orientation.'],...
	'position',[10 265 185 20],...
	'BackgroundColor',Ecolor,'callback',pcbImgOrient);

pcbgroup=['if InputOrient==1, nmprompt=''Please Select Group Files (Radiological (RL) Input)'';',...
    'else nmprompt=''Please Select Group Files (Neurological (LR) Input)''; end;',...
    'FNames=get_file(Inf,''.img'',nmprompt,[]); ',...
	'if ~isempty(FNames),',...
	'FName=FNames(1,:); vp_fimage;',...
	'figure(roigui); eval(cbtext); axis off;',...
	'if isempty(Message), IdentGroup=1; PlaneNo=max(PlaneNo-1,1); Message=FName; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), IdentGroup=1; end;',...
	'set(hroierr,''string'',['' '',Message]); else, IdentGroup=0;',...
    'Message=['' No Files Selected'']; set(hroierr,''string'',Message); ',...
    'end;'];

uicontrol(roigui,'string', 'SELECT GROUP',...
	'TooltipString','Identify group files for replay',...
	'position',[10 240 95 20],'BackgroundColor',Bcolor,'callback',pcbgroup);
	
uicontrol(roigui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[200 265 40 20],'callback','vp_help(''vp_grouproi'');');
	       
cbRptMask = ['MaskOpt=get(hrptmask,''Value'')-1; if MaskOpt == 0, MaskImg=[]; ',...
	'elseif MaskOpt == 1, MaskImg=get_file(1,''.img'',''Select MASK Image In Input Orientation'',[]);',...
	'FName=MaskImg; vp_fimage;',...
	'Message=[''Mask: '',vp_nm(''nm'',MaskImg);]; end;',...
	'if isempty(MaskImg), MaskOpt=0; Message=[''No Mask'']; end;',...
	'set(hroierr,''string'',Message); '];
	
hrptmask=uicontrol(roigui,'Style','popup','Visible','on',...
	'TooltipString','Select Mask For Input Images',...
	'String',...
	' NO MASK | PRE-MASK IMAGES','Tag','RptMask',...
    	'Position',[10 212 110 25],'callback',cbRptMask);
	
cbGNon=['  GNop=1; vp_note([''Global Normalization Option ON. Each Image Divided by its masked Mean.''],2);'];
	
cbGNoff=[' GNop=0; vp_note([''Global Normalization Option OFF''],2); '];

hGNon=uicontrol(roigui,'String','Global Normalization','Style','Checkbox',...
	'FontWeight','Normal',...
	'TooltipString','ON/OFF Switch-  (ON: Global Normalization, Divide each image by its masked non-zero voxel mean )',...
	'Units','pixel',...
	'Position',[125 215 125 019],...
	'value',GNop,...
	'CallBack',['if GNop==0, ',...
	'eval(cbGNon);',...
	'else ',...
	'eval(cbGNoff);',...
	' end; ']);
       
cbRoiFName='';
	
%	'[nam,sfx]=strtok(RoiFNames(1,:),''.''); [pln,sfx]=strtok(sfx,''.'');',...
%	'if isempty(sfx), q=max([find(isletter(nam)==1) 0]); ',...
%	'sz=size(nam); PlaneNo=str2num(nam(q+1:sz(2))) ; nam=nam(1:q);',...
%	'else, PlaneNo=str2num(pln); end;',...
%

%'AtlasNm=get_file(Inf,''.img'',''please select ATLAS Template Image'',atlasdir,[]);',...
%
cbAtlas=['w=which(''scanvp''); atlasdir=[w(1:(max(findstr(w,''\''))-4)),''templates\Brain_Atlas''],',...
	'if InputOrient==1, atlasnm=[DefAtlasNm,''_RL69'']; vp_note(''Select Radiological RL Atlas'',3);',...
    'else atlasnm=[DefAtlasNm,''_LR68'']; vp_note(''Select Neurological LR Atlas'',3); end;',...
    'AtlasNm=get_file(1,''.img'',[''Suggested ATLAS Template Image: '',atlasnm],atlasdir,[]);',...
	'if ~isempty(AtlasNm), FName=AtlasNm; vp_fimage; AtlasDIM=DIM;',...
    'nds=size(gPrevDirs,1); gPrevDirs(nds,:)=[];',...
	'vp_note(''Only FIXED %of REG MAX or FIXED Min Options valid for ATLAS regions'',2);',...
	'ThreshMode=4; set(hthrmenu,''Value'',ThreshMode); eval(cbThrMenu); ',...
	'DefTemplate=1; end;',...
	' '];

cbPRoiName=['TemplateOpt= get(htemp,''value'');',...
	'if TemplateOpt== 2,',...
	'RoiFNames=get_file(2,''.roi'',''Please Select Roi Template File Limits (2 files)'',[]); ',...
	'elseif TemplateOpt==3  RoiFNames=get_file(1,''.roi'',''Please Select Roi Template File'',[]); end; ',...
	'if ((TemplateOpt==2) | (TemplateOpt==3)),',...
	'  if ~isempty(RoiFNames),',...
	'  [dnm,dnm,dir,nam,sfx]=vp_nm(''dnm'',RoiFNames(1,:)), ',...
	'  [nam,pln]=strtok(nam,''.''), ',...
	'  if isempty(pln), q=max([find(isletter(nam)==1) 0]), ',...
	'  sz=size(nam); PlaneNo=str2num(nam(q+1:sz(2))) , nam=nam(1:q),',...
	'  else, PlaneNo=str2num(pln(2:length(pln))); end;',...
	'  nam=[dir,nam], PlaneNo,',...
	'  set(hname,''string'',nam); ','Message=Message0 ; vp_fimage;',...
	'  set(hroierr,''string'',['' '',Message]);',...
	'  if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=FName;',...
	'  elseif ~strcmp(Message,''Maximum Plane Reached''), end;',...
	'  end;',...
	'  figure(roigui); eval(cbtext); axis off;',...
	'  set(hplane,''string'',PlaneNo); DefTemplate=1;',...
	'  set(hroierr,''string'',['' '',Message]);',...
	'elseif TemplateOpt==4, eval(cbAtlas); end;'] ;
	
%uicontrol(roigui,'string', 'TEMPLATE',...
%	'position',[10 163 75 25],'BackgroundColor',Bcolor,...
%	'callback',cbPRoiName);
	
htemp=uicontrol(roigui,'Style','Popup','string',...
    'PLANAR/ATLAS TEMPLATE | PLANAR Template Limits (.roi) | SINGLE PLANE Template (.roi) | ATLAS Template (.img)  ',...
	'TooltipString','Select Planar Limit .roi Templates, Single Plane .roi Template or Volume .img Template',...,...
	'position',[10 185 180 25],'BackgroundColor',Bcolor,...
	'callback',cbPRoiName);
	
%	'callback',['if get(htemp,''value'')==1, eval(cbPRoiName), else eval(cbP1RoiName), end;']);




hname=uicontrol(roigui,'style','text','string',RoiFNames(1,:),...
	'position',[10 163 230 25],'BackgroundColor',Fcolor,'callback',cbRoiFName);

	
uicontrol(roigui,'Style','Check','String','FIT Roi',...
	'TooltipString','Resize ROI Template to Fit Image Dimensions',...
	'Value',Fitting,'Position',[195 202 55 12],...
	'CallBack',[ 'if ~Fitting, Fitting=1; else Fitting=0, end;',...
	'set(hroierr,''string'',''     '');' ]);
	
uicontrol(roigui,'Style','Check','String','FLIP Y',...
	'TooltipString','ROI Flip Correction for Y (UP/DOWN)',...
	'Value',Flipping,'Position',[195 190 55 12],...
	'CallBack',[ 'if ~Flipping, Flipping=1; else Flipping=0, end;',...
	'set(hroierr,''string'',''     '');' ]);
	

pcbplane=['Message=Message0; PlaneNo=PlaneNo+1; vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hroierr,''string'',[''MSG:'',Message]);'];

%	'set(hroierr,''string'',[''MSG:'',Message]);'];

uicontrol(roigui,'string', 'VIEW PLANE++',...
	'TooltipString','Select Plane for viewing only.',...
	'position',[100 142 85 20],'BackgroundColor',Bcolor,'callback',pcbplane);


uicontrol(roigui,'Style','popup','Visible','on',...
	'TooltipString','Select INTERACTIVE to modify ROIS or view Threshold MAP.',...
	'String','AUTOMATIC | INTERACTIVE ',...
	'position',[10 137 90 25],'BackgroundColor',Bcolor,'callback',...
	['val=get(gco,''value''); if val==1, Automatic=1; else,',...
	'Automatic=0; end;']);

%uicontrol(roigui,'string', 'AUTOMATIC',...
%	'TooltipString','Select INTERACTIVE to modify region positions or outline. ',...
%	'position',[10 137 90 25],'BackgroundColor',Bcolor,'callback',...
%	['if Automatic==1, Automatic=0; set(gco,''string'',''INTERACTIVE''); else,',...
%	'Automatic=1; set(gco,''string'',''AUTOMATIC''); end;']);


cbplane=['PlaneNo=str2num(get(hplane,''string'')); Message=Message0;',...
	'vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hroierr,''string'',[''MSG:'',Message]);'];

hplane=uicontrol(roigui,'style','edit','string',PlaneNo,...
	'position',[185 142 55 20],'BackgroundColor',Kcolor,'callback',cbplane);


cbRptMenu = ['SUMRpt=get(hrptmenu,''Value'');';];

cbThrMenu = ['ThreshMode=get(hthrmenu,''Value''); if ThreshMode > 4, vp_thmin;',...
	'elseif ThreshMode > 2, vp_thresh;',...
	'set(hroierr,''string'',Message); end;'];

	
hthrmenu=uicontrol(roigui,'Style','popup','Visible','on','Value',ThreshMode,...
	'TooltipString','Select method for selecting threshold minimum',...
	'String',...
	['VARIABLE %of 2D REG MAX|VARIABLE %of 3D REG MAX|FIXED %of 2D REG MAX|',...
	'FIXED %of 3D REG MAX|FIXED MINIMUM THRESHOLD'],'Tag','ThrMenu',...
    	'Position',[76 107 164 25],'callback',cbThrMenu);
	
hrptmenu=uicontrol(roigui,'Style','popup','Visible','on',...
	'TooltipString','Select Report Format',...
	'String',...
	'GROUP SUMMARY|SUBJECT (SAS)','Tag','RptMenu',...
    	'Position',[90 82 150 25],'callback',cbRptMenu);
	
	
uicontrol(roigui,'Style','Check','String','TEMPLAT',...
	'TooltipString',['Full Report Template Style Format-',...
	'Warning Overites input template if subject the same name'],...
	'Value',TemplRpt,'Position',[10 62 72 23],...
	'CallBack',[ 'if ~TemplRpt, TemplRpt=1, else TemplRpt=0, end; ',...
	'set(hroierr,''string'',''    '');' ]);
	
uicontrol(roigui,'Style','Check','String','PLANAR',...
	'TooltipString','Plane by Plane Report Format',...
	'Value',PlaneRpt,'Position',[86 62 65 23],...
	'CallBack',[ 'if ~PlaneRpt, PlaneRpt=1; else PlaneRpt=0, end;',...
	'set(hroierr,''string'',''     '');' ]);
	
uicontrol(roigui,'Style','Check','String','COMPOSITE',...
	'TooltipString','Mean Across Planes For Each Roi - Report Format',...
	'Value',CompRpt,'Position',[155 62 84 23],...
	'CallBack',[ 'if ~CompRpt, CompRpt=1, else CompRpt=0, end; ',...
	'set(hroierr,''string'',''    '');' ]);


uicontrol(roigui,'String','PROCESS REPORT',...
	'TooltipString','Initiate Processing.',...
	'Position',[10 35 110 25],...
	'CallBack',[ 'if ~IdentGroup, set(hroierr,''string'',''ERR: Group Not Defined''); return; end; ',...
	'if ~DefTemplate, set(hroierr,''string'',''ERR: Templates Not Defined''); return; end; ',...
	'if ~Automatic & (ThreshMode==2 | ThreshMode==4 | TemplateOpt==4 ),',...
	' set(hroierr,''string'',''ERR: Interactive for 3D or ATLAS Not Operative''); return; end; ',...
	'if (TemplateOpt==4) & (ThreshMode < 3 ),',...
	' set(hroierr,''string'',''ERR: Variable Threshold Not Valid for ATLAS''); return; end; ',...
	'if GNop & (MaskOpt ~= 1), set(hroierr,''string'',''ERR: Must Define Mask for Global Normalization'');',...
	' return; end; ',...
	'if ~TemplRpt &~PlaneRpt & ~CompRpt, set(hroierr,''string'',''ERR: Report Type(s) Not Specified'');',...
	' return; end; ',...
	'if TemplRpt & (TemplateOpt==4), set(hroierr,''string'',''ERR: Template Report Not Valid for ATLAS'');',...
	' return; end; ',...
	'if isempty(RoiFNames(1,:)), Message=''Template Not Defined'',',...
	'set(hroierr,''string'',[''Error: '',Message]); return; end;',...
	'FName=FNames(1,:); vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1);',...
	'set(hplane,''string'',PlaneNo); PlaneNo0=PlaneNo; ',...
	'if SUMRpt == 2, SASRpt=PlaneRpt; SAS_cRpt=CompRpt; else,',...
	'SASRpt=0; SAS_cRpt=0; end;',...
	'set(roigui,''Visible'',''off''); drawnow; vp_groupro;',...
	'else set(hroierr,''string'',[''MSG: '',Message]);',...
	'if PlaneNo==DIM(3), vp_groupro, set(roigui,''Visible'',''off'');, end;',...
	' end;'] );
	
%	'CallBack',['set(roigui,''UserData'',1)']);

	
uicontrol(roigui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Position',[180 35 60 25],...
	'CallBack',['delete(gcbf);','delete(FI);']);
%	'CallBack',['set(roigui,''UserData'',1); ', 'delete(roigui)']);

hroierr = uicontrol(roigui,'string', ['MSG : ',Message],...
	'TooltipString','Report error or message field',...
	'position',[10 10 230 25],...
	'CallBack',['set(hroierr,''string'',[''MSG: '',Message]); vp_note(Message,3);'],...
	'BackgroundColor',Ecolor);

