% vp_hemimage
% Filter out Left or Right hemisphere of ANALYZE Format images
%_______________________________________________________________________
% vp_hemimage allows the user to maintain only the right or left half of
% all planes of a set of images by assigning zeros to the opposite side.
% The images are assigned a suffix of '_r' or '_l'. These designations
% correspond to the anatomical right and left hemisphere if the "Input
% Orientation" of the file on disk (LR or RL) is correctly designated on
% the scanvp menu. In some diseases the most affected side may be either the
% right or the left in individuals. For group processing programs that
% require a common non-zero masked space it may be possible to process right
% and left images separately (e.g. TPR). However, if the user unavoidably
% needs to include all images together some of the images may be flipped
% externally Left <--> Right along the X axis so that the side of interest
% occupies the same non-zero voxel space in subsequent group analysis. For
% this purpose the _r and _l specification refers to assigned voxel space
% independent of anatomical designation which would have to be noted
% otherwise by the user, e.g. with a capital R or L in the image name.
% The program assumes the "Input Orientation" to be the same, either
% "Neurological (LR)" or "Radiological(RL)" for all files in group
% processing and display regardless of any external flipping operations.
%
% The R/L half limit (Xhalf) demarcation is determined by dividing the X
% voxel dimension in half and rounding that number to the nearest voxel.
% In "Neurological" designated voxel space, the left hemisphere includes
% all voxels with an x coordinate less or equal to Xhalf. The right
% hemisphere then assumes all voxels with an x coordinate greater than
% Xhalf. For "Radiological" input the same is true but images are internally
% flipped prior to this operation. The program is unaware of any previous
% external flipping performed separately by the user for study purposes.
%
% For SPM normalized images, Xwid= 79, Xhalf = round(39.5) = 40. In this
% case the user should be aware that Left and Right sides have different
% widths. Any external flipping should be performed prior to halving the image
% to avoid any group processing artifacts. Thus, in voxel space, if the user
% flips and halves the images, the result can depend on the operation order: 
% 1) If some images are pre-flipped and then the left half is saved, all the 
% halved images will have 40 significant x coordinates in the left half and
% the right half will have all zeros. The 40th column remains unchanged
% irregardless of the image's original orientation.
% 2) If only the right half is saved, all the images will have 39 significant
% x coordinates on the right of the 40th column which will be zeroed.
% 3) If either side is halved first and then optionally flipped
% externally, an original left half image will have 40 significant x
% coordinates and an original right side images will have 39. Thus for a 
% group of images the 40th columnn may be zeroed or processed inconsistently.
%_______________________________________________________________________
% vp_hemimage	ScAnVP	5.2	P. Spetsieris	9/18/03
%               ScAnVP Rev. 7.0 P.Spetsieris 11/16/15

global HelpMode FMenu Message NameLst POpt InputOrient DisplayOrient
global ORIGIN

if HelpMode, vp_help(mfilename), return ; end;

out1=0;
op='R';

cbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

prompt=['HEMI-IMAGE Display/Save Which Side?'];
optR=['RIGHT DISPLAY, LEFT ZERO'];
optL=['LEFT DISPLAY, RIGHT ZERO'];
optO=['CHECK INPUT ORIENTATION'];

POpt=vp_opt('X',prompt,'Right',optR,'Left',optL,'Check Orient',optO,'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_hemimage'); vp_hemimage; 
  return; end;
if(strcmp(POpt,'Check Orient')),
  eval(cbImgOrient); vp_hemimage; 
  return; end;
if(strcmp(POpt,'Right')),
  vp_note(optR,2);
  op='R'; end;
if(strcmp(POpt,'Left')),
  vp_note(optL,2);
  op='L'; end;

if vp_filesui(Inf,'.img','Select images for HEMI-ZERO operation',mfilename) < 0 ,return; end;

m= size(NameLst,1);
dim3=0;

if m <=0,
  return;
end;

set(FMenu,'Pointer','Watch');
h=waitbar(0.1,'Processing HEMI Images...');
	
for i=1:m

  	FName=deblank(NameLst(i,:));
	
	waitbar((i-.8)/m, h);
	
	[out1,hfimg]=vp_hemipro(FName,op,1,(InputOrient ~= DisplayOrient));
	
	if out1 < 0, 
	  vp_note(Message);
	  %disp(['vp_hemimage ABORTED ']);
	  waitfor(findobj('Name','Note'));
	  %while(findobj('Name','Message')), pause(1); end;
	else
        PlaneNo=ORIGIN(3);
        fi=vp_fimg(hfimg,PlaneNo,1,4,0);
	end
end

waitbar(1,h); pause(1); close(h);
set(FMenu,'Pointer','Arrow');
%disp('END vp_hemimage');
vp_note('HEMI-IMAGES PROCESSING COMPLETE',3);


