% vp_imageop
% Perform operations on individual ANALYZE Format images.
%_______________________________________________________________________
% User selects an image operation to perform on a sequence of images.
% vp_imageop calls vp_imcvt to perform image operations such as multiply
% by scalar, threshold, mask, calculate image parameters and log 
% transformation on each image separately. 
% The image is scaled or converted to real values before each operation is
% performed. If there are negative values in addition to positive values
% for an 8bit output image the negative values are set to zero.
% If AUTO naming is enabled (default), the processed images are assigned a
% character 'imop' as a suffix with noted exceptions.
% For arithmetic operations, the suffix 'A' is followed by a suffix
% M=Multiply, D=Divide, A=Add and S= Subtract. For logarithm operations
% negative and zero values are assigned zero in the ouput. The letter 'L'
% is assigned as a prefix. For smoothing, the prefix 's' is assigned.
% For subject global normalization created by dividing each value by the
% subject's mean thresholded value, a 'g' prefix is assigned.
% For masking a suffix _m is assigned.
% For parameter calculation no output text file is created.
% 
% Interactive renaming is enabled unless set to automatic.
%
% Note,
%  Mask and threshold options can be opted out by pressing CANCEL.
%
% imop
%  Operations noted by a suffix assigned to output names.
%  A=Arithmetic Multiply, D=Divide, A=Add, S=Subtract by scalar(s)
%	Either one constant is entered or multiple constants
%       for operation by a different constant for each image.
%  T=Threshold- Set voxels < T value to 0
%  X=Exclude Range of values- (Set values in range [T1,T2] to Zero)
%  K=Create a 0|1 mask by assigning 0 to all exclusive values [T1,T2]
%    and then setting 1 to all non-zero values (Original 0's are retained).
%    For an inclusive mask, values can be single values Ni or a range Nj:Nk
%    separated by commas, e.g. A) -Inf:2,2:Inf implies that a mask is created 
%    for absolute value greater than 2 and B) 3,6:7,9. implies that only values in range
%    6 to 7 and the values 3 and 9 are set to 1. Values can be integer or real.
%    (0 values are not modified, even if specified in the inclusive range).
%
%  Operations noted by a prefix assigned to output names.
%  L=Natural Logarithm (Zero and Negatives set to Zero)
%  s=Smoothing using Box Filter [k,k,k] or [k,l,m], assigns 's' prefix.
%  g=Global normalization, dividing by the subject mean thresholded value.
%    The lower threshold is determined as a fraction of subject's masked image
%    max. The fraction (default .0) includes all positive values (Zero values
%    are excluded) but can be set to a greater value, e.g. .35. Mask is optional.
%  v=Vector normalization. Devide by the sqrt of the sum of the values squared.
%  z=Z-score normalization. Subtract the mean and devide by the standard deviation.
%  p=Image Mean Values (GMR) parameter: Images are optionally masked and thresholded
%    and the mean and global values of the masked, thresholded images are reported.
%  a=Image assymetry index (AI), AI=2(R-L)/(R+L) with optional mask/threshelding. 
%    AI is evaluated from global totals Left and Right of X axis midpoint.
%    In evaluating the assymetry index (AI) option, the Neurological (L/R)
%    or Radiological (R/L) input orientation of image files on disk as indicated
%    on the ScAnVP menu is taken into account and should be correctly specified.
%  m=Mask image using input mask and/or fractionally threshold as for option 'g'.
%    Any assymetric mask applied to an image is assumed to be similarly oriented. 
%
%  Any images resulting from these operation are output in the same orientation
%  as the original image on which operations are performed regardless of 
%  whether the "Input" orientation setting on the ScAnVP menu is specified. 
%_______________________________________________________________________
% vp_imageop		C.4.7	P. Spetsieris	1/28/02
%			Updated 5.1 3/19/03  by P.S.
%			Updated 5.3 6/03/04  by P.S.
%			Updated 5.4 11/30/04 by P.S.
%			Updated 5.6 10/27/05 by P.S.
%			Updated 5.7 5/31/06  by P.S.
%			Updated 5.8 3/22/07  by P.S.
%			Updated 6.2 10/02/12 by P.S.
%			Updated 6.4 3/09/15 by P.S.
%			Updated 7.0 01/14/16 by P.S.

global HelpMode FMenu Message Ecolor Op ImOpFactor Mask Keyin
global hwaitbar AutoName TotValues TotCount TotStd ArithmeticOp
global AssymIndx LeftVal RightVal

if HelpMode, vp_help(mfilename),  return ; end;

IdentGroup=0;
DefOp=0;
out1=0;
Op='M';
ImOpFactor=1;
ArithmeticOp='A';
Message=['MSG:.................'];
Bcolor=[0.8 .8 .8];
AutoName=1;
Mask=[];
TotValues=[];
TotCount=[];
TotStd=[];
AssymIndx=[];
LeftVal=[];
RightVal=[];


imageopgui = figure('Color',[0.9 .8 .8],'Name','Image Operation','Tag','imageopgui',...
'NumberTitle','off','Position',[100,200,350,75],'UserData',0,...
'Units','Pixels');
	
cbOpsMenu = ['val=get(hopsmenu,''Value''); ',...
	'Message=[''Select Operation'' ]; ',...
	'if val == 1, Op='' ''; return; ',...
	'elseif val==2, Op=''A''; ',...
 	' vp_keyin(''Select Op A=Add, S=Sub., M=Mult., D=Div. ? [A,S,M,D | q]'',''A'');',...
	'   if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')), return;',...
 	'   else ArithmeticOp=Keyin,',...
	'     if isempty(findstr(''ASMD'',ArithmeticOp)),',...
	'     vp_message(''Invalid OP''); return; end;',...
	'   end;',...
	' vp_scalarfactors;',...
	'elseif val==3, Op=''T''; ',...
 	' vp_keyin(''Enter Lower Threshold [num | q]'',''0'');',...
	'elseif val==4, Op=''X''; ',...
 	' vp_keyin(''Enter Excluded Range [T1,T2| q]'',''0'');',...
	'elseif val==5, Op=''K''; ',...
	' vp_note(''Use Comma To Denote Range. All other values except zeros will be set to 1. '',3); ',...
 	' vp_keyin(''Enter Mask Excluded Range [T1,T2| q]'',''0'');',...
	'elseif val==6, Op=''Y'';',...
	' vp_note(''Use Colon To Denote Range and Comma To Separate Single Values'',3); ',...
 	' vp_keyin(''Enter Mask Inclusive Values [Ni,Nj:Nk,Nm| q]'',''0'');',...
	'elseif val==7, Op=''L''; DefOp=1; ',...
	' Message=[''Op: Log (Zero values <=0) '']; ',...
	' set(herr,''string'',Message); return; ',...
	'elseif val==8, Op=''s''; ',...
 	' vp_keyin(''Size of Convolution Kernel? [k or [k,l,m]| q]'',''3'');',...
	'elseif (val >= 9 & val <= 14), Op=''g''; ',...
	' if val==10, Op=''v'', end;',...
	' if val==11, Op=''z'', end;',...
	' if val==12, Op=''p'', end;',...
	' if val==13, Op=''a'', end;',...
	' if val==14, Op=''m'', end;',...
	' maskname=[];',...
	' if val~=14,',...
	'  maskname=get_file(1,''*mask*.img'',''Select (OPTIONAL) Mask or Cancel'');',...
	' else,',...
	'  maskname=get_file(1,''*mask*.img'',''Select Mask '');',...
	' end;',...
	' if isempty(maskname);',...
	' else, [out1 Mask]=vp_volfuns(''volvect'',maskname);',...
	'    if (out1 < 0) vp_message(''ERROR reading mask'');',...
	'    return, end;',...
	' end;',...
	' if ( (val~=10) & (val~=11) )',...
 	'   vp_keyin(''Lower Threshold Factor of Image Max or Cancel(-Inf)? [f | q]'',''.0'');',...
	'   if (isempty(Keyin) ), Keyin=''-Inf''; end; ',...
	' else Keyin=Op;',...
	' end;',...
	'else, Op='' ''; return; ',...
	'end;',...
	'if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')),',...
 	'return, else, if strcmp(Op,''Y''), ImOpFactor=Keyin; ',...
	' else ImOpFactor=str2num(Keyin); end; end;',...
	'if strcmp(Op,''T''),',...
	' Message=[''Op: Zero Values less than '' Keyin]; ',...
	'elseif strcmp(Op,''X''),',...
	' Message=[''Op: Zero Values in Range: '' Keyin]; ',...
	'elseif strcmp(Op,''K''),',...
	' Message=[''Op: Create 0|1 Mask Excluding Range: '' Keyin]; ',...
	'elseif strcmp(Op,''Y''),',...
	' Message=[''Op: Create 0|1 Mask Including Range: '' Keyin]; ',...
	'elseif strcmp(Op,''A''),',...
	' Message=[''Op: Arithmetic Operation '' ArithmeticOp '' by Factor(s) '' Keyin]; ',...
	'elseif strcmp(Op,''g''),',...
	' Message=[''Global Subject Normalization, Dividing by Mean of Values >  '' Keyin, '' of Sub. Max'']; ',...
	'elseif strcmp(Op,''v''),',...
	' Message=[''Vector Image Normalization, Dividing by Sqrt of Sum of Square Values. '']; ',...
	'elseif strcmp(Op,''z''),',...
	' Message=[''Z-Score Image Normalization, (Val -Mean)/STD of Non Zero Values'']; ',...
	'elseif strcmp(Op,''p''),',...
	' Message=[''Op: Process Masked Values >  '' Keyin, '' of Sub. Max'']; ',...
	'elseif strcmp(Op,''a''),',...
	' Message=[''Op: Process Left vs. Right Masked Values >  '' Keyin, '' of Sub. Max'']; ',...
	'elseif strcmp(Op,''m''),',...
	' Message=[''Op: Mask Image, Allow Values >  '' Keyin, '' of Sub. Max'']; ',...
	'elseif strcmp(Op,''s''),',...
	' if size(ImOpFactor)==1,',...
	'  Message=[''Op: Smooth, Box Convl. Size '' Keyin ''x'' Keyin ''x'' Keyin ]; ',...
	' else,',...
	'  Message=[''Op: Smooth, Box Convl. Size '' Keyin ]; end; ',...
	'else, Op='' ''; return; ',...
	'end;',...
	'if val ~=1, DefOp=1;',...
	'set(herr,''string'',Message); end; '];

%PS Add Read Scalars option
%cbscalarfactors=[',...
% 	' vp_keyin(''Scalar Factor(s) ? [num | q]'',''1'');',...
%'];

hopsmenu=uicontrol(imageopgui,'Style','popup','Visible','on',...
	'position',[112 50 138 20],'BackgroundColor',Bcolor,...
	'TooltipString','Select Operation',...
	'String',...
	['Image Operation...|Mult,Div,Add,Sub by Scalar|Threshold Lower Values|',...
	'Exclude Thresh Range|Exclusive Mask Creation|Inclusive Mask Creation|',...
	'Natural Log|Smooth Box Convl.|',...
	'Global Normalization |Vector Normalization |Z-score Normalization |',...
	'Image Global Mean Sdev |Image Assymetry Index | Mask Image ...'],...
	'Tag','OpsMenu',...
    	'callback', cbOpsMenu);

pcbnames=['fimagelst=[]; fimagelst = get_file(Inf,''.img'',''Select images'',[]);',...
	  'if ~isempty(fimagelst), IdentGroup=1; TotValues=[]; TotCount=[]; TotStd=[];',...
	  'AssymIndx=[]; LeftVal=[]; RightVal=[];  end;'];
	
uicontrol(imageopgui,'string', 'SELECT IMAGES',...
	'TooltipString','Pick Images for Application',...
	'position',[10 50 100 20],'BackgroundColor',Bcolor,'callback',pcbnames);
	
uicontrol(imageopgui,'String','PROCESS',...
	'TooltipString','Initiate Processing.',...
	'Position',[10 25 65 20],...
	'CallBack',[ 'if ~IdentGroup, set(herr,''string'',''ERR: Images Not Defined''); return; end;',...
	'if ~DefOp, set(herr,''string'',''ERR: Data Operation Not Defined''); return; end;',...
	'set(FMenu,''Pointer'',''Watch'');',...
	'm=size(fimagelst,1); hwaitbar=waitbar(0,''Processing Image Operation ...'');',...
	'if or(strcmp(Op,''p''),strcmp(Op,''a'')), in3=1, else in3=2; end;',...
	'for i=1:m, fnme=[]; fnme=deblank(fimagelst(i,:));',...
	' if strcmp(Op,''A''), n=length(ImOpFactor), ',...
	'   if (n== m),',...
	'     [out1, out2, out3, out4]=vp_imcvt(fnme,Op,in3,ImOpFactor(i),AutoName,Mask,ArithmeticOp);',...
	'   elseif (n== 1),',...
	'     [out1, out2, out3, out4]=vp_imcvt(fnme,Op,in3,ImOpFactor,AutoName,Mask,ArithmeticOp);',...
	'   else,',...
	'      set(herr,''string'',''ERR: Must enter either 1 Constant or 1 for each image''); return;',...
	'   end; ',...
	' else,',...
	'   [out1, out2, out3, out4]=vp_imcvt(fnme,Op,in3,ImOpFactor,AutoName,Mask,ArithmeticOp);',...
	' end,'...
	' waitbar((i-.8)/m,hwaitbar);',...
	' if out1<0, set(herr,''string'',['' '',Message]); pause(2); ',...
	' else, set(herr,''string'',['' Processing '',num2str(m)]); pause(1); end;',...
	' if strcmp(Op,''p''), TotValues(i,:)=out2; TotCount(i,:)=out3; TotStd(i,:)=out4; end;',...
	' if strcmp(Op,''a''), AssymIndx(i,:)=2*(out3-out2)/(out2+out3); LeftVal(i,:)=out2; RightVal(i,:)=out3; end;',...
	'end; waitbar(1,hwaitbar); pause(1); close(hwaitbar);',...
	'if strcmp(Op,''p''), TotValues.*TotCount, TotValues, TotStd, TotCount,',...
	' flname=[''GMean'',num2str(m),''Values'',num2str(ImOpFactor),''.txt''];',...
 	' if ~AutoName, vp_keyin(''Enter Filename : '',flname);',...
	'   if (~isempty(Keyin) ), flname=Keyin; end; end; ',...
	' LabLst=[]; LabLst=strvcat(''Total'', ''Mean'',''StDev'',''nVoxels''); DatValues=[TotValues.*TotCount, TotValues, TotStd, TotCount];',...
	' vp_write_XYZfile(flname,''Global TOTAL, MEAN, STDEV and No of Thresholded VOXELS'',LabLst,DatValues,cellstr(fimagelst)); ',...
	' set(herr,''string'',['' Results File: '',flname]); ',...
	'elseif strcmp(Op,''a''), AssymIndx, LeftVal, RightVal,',...
	' flname=[''AssymIndx'',num2str(m),''Values'',num2str(ImOpFactor),''.txt''];',...
 	' if ~AutoName, vp_keyin(''Enter Filename : '',flname);',...
	'   if (~isempty(Keyin) ), flname=Keyin; end; end; ',...
	' LabLst=[]; LabLst=strvcat(''ASSYM.'', ''Left'',''Right'',''Global '');',...
    '           DatValues=[AssymIndx, LeftVal, RightVal, (RightVal+LeftVal)];',...
	' vp_write_XYZfile(flname,''ASSYMETRY INDEX 2x(Right-Left) / (Right+Left) and Left, Right and Total(R+L)GLOBAL VALUES'',',...
    '           LabLst,DatValues,cellstr(fimagelst)); ',...
	' set(herr,''string'',['' Results File: '',flname]); ',...
	'else,',...
	' set(herr,''string'',['' Processing Complete, Check: '', out4]); ',...
	'end;',...
	'set(FMenu,''Pointer'',''Arrow'')';] );

	
uicontrol(imageopgui,'Style','Check','String','AUTO',...
	'TooltipString','Automate Output Naming',...
	'Value',AutoName,'Position',[76 25 50 20],...
	'CallBack',[ 'AutoName=get(gco,''value''),']);
	
uicontrol(imageopgui,'string', 'HELP',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[128 25 50 20],'callback','vp_help(''vp_imageop'');');
	
uicontrol(imageopgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Position',[190 25 60 20],...
	'CallBack',['delete(imageopgui);',...
	'set(FMenu,''Pointer'',''Arrow'')';] );

herr = uicontrol(imageopgui,'string', [' ',Message],...
	'TooltipString','Report error or message field',...
	'position',[10 0 330 20],...
	'CallBack',['set(herr,''string'',['' '',Message]);'],...
	'BackgroundColor',Ecolor);


