function [out1, out2, out3, out4]=vp_imcvt(fname, imop, in3, in4, in5, mask, in6,in7)
% function [out1, out2, out3, out4]=vp_imcvt(fname, imop, in3, in4, in5, mask, in6, in7)
% Perform image conversion operation on an ANALYZE Format image
%_______________________________________________________________________
% vp_imcvt performs image operations such as zero negatives, 8->16bit
% 16->8bit, 8->double or scalar multiplication etc. on an ANALYZE format images.
% Except for noted cases such as multiplication and division by positive scalar,
% the image is scaled and converted to real values before each operation is
% performed and then reset to the requested conversion type or to the original
% type if the operation cannot affect values. For most arithmetic, logarithmic 
% and normalization operations, the output values will be saved as 'float'.
% For integer conversion, if values are less than the maximum for that type,
% scaling is set to 1, if parameter in6 is set to 1 or if output is a mask.
% If there are negative values, and an 8bit output is requested the
% negative values are set to zero. The processed images are usually assigned
% a suffix. For arithmetic operations the suffix A is assigned preceded
% by a suffix denoting the operation M,D,A or S.
% In the following noted cases a prefix character 'imop' is assigned.
% For Natural Log transformation, all values less or equal to zero are set
% to zero. A prefix character 'L' is assigned. 'K' is used for 'Y' operations.
% For smoothing operations a prefix character 's' is assigned.
% For global normalization a prefix character 'g' is assigned.
% For global normalization a suffix '_m' is assigned.
% For parameter calculation the mean (masked), thresholded value is returned.
%
% imop
%  Z=ZERONEGS
%  A=Arithmietic Operation: Multiply,Divide, Add, Subtract by scalar	
%  T=Set values lower than threshold value to 0.
%  X=Set values in range [T1,T2] to 0.
%  Y=Create 0:1 Mask setting values (except 0) to 1 specified by format:
%	 e.g, Ni,Nj:Nk,Nm implies Include Ni,Nm and range Nj to Nk.
%  K=Create 0|1 Exclusive Mask setting values in range [T1,T2] to 0.
%  B=8bit
%  S=16bit
%  I=32bit
%  F=32bit Float
%  D=64 Double
%  L=Natural Logarithm
%  s=Smooth using kxlxm Box filter convolution.
%  m=Mask image with specified input mask and/or fractional threshold of image max.
%  g=global normalization dividing by subject's thresholded, (masked), non-zero values mean.
%  v=vector normalization dividing by the square root of the sum of the squared values.
%  z=z-scored of image: transformation of non-zero values -> (val-mean)/std of non-zero values.
%  a=calculate subject thresholded, (masked) image params: assymetry index, left, right globals
%  p=calculate subject thresholded, (masked) image parameters: global max, mean value, stdev, nxpls
%
% in3=1  :  Return DATA without writing image
% in3=2  :  Return DATA and write image
% in4    :  Threshold (Opt T, g, p), Scalar value (Opt A)
%	    Range (T1,T2) (Opts X and K)
%	    Size of Convolution kernel for Opt s.
% in5    :  Automate Image Naming On/Off
% in6	 :  A: Arithmetic Operation M,D,A,S=Multiply,Divide,Add  
% in7    :  Integer Output: Rounding Selected (1=True, 0=False), Scale=1 
% mask	 :  Input mask image vector for options 'm' and optionally for 'g','a','p','v','z'. 
% out1   :  Return status
% out2   :  Return DATA
% out3   :  Return SCALE, for imop=L, return File Name
% out4   :  Return Output File Name; If imop is 'p', out4=stdev.
%_______________________________________________________________________
% vp_imcvt	ScAnVP C.4.4	P. Spetsieris	1/11/00
%			Revised 5.1 3/19/03, 4/9/03 by P.S.
%			Revised 5.3 6/03/04 by P.S.
%			Revised 5.4 3/16/05 by P.S.
%			Revised 5.6 10/04/05 by P.S.
%			Revised 5.7 7/18/06 by P.S.
%			Revised 5.8 4/27/07 by P.S.
%			Revised 5.9 3/14/08 by P.S.
%			Revised 6.1 1/11/11 by P.S.
%			Revised 6.2 10/2/12 by P.S.
%			Revised 6.2.1 4/22/13 by P.S.
%			Revised 6.4 3/9/15 by P.S.
%			Revised 7.0 1/14/16 by P.S.

global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE Message GENTIM SUBID EXPDATE
global HelpMode TYPE0 InputOrient

if HelpMode, vp_help(mfilename), return ; end;

out1=0;
out2=0;
out3=0;
out4='';
autonaming=0;
iroundscl=0; %integer roundoff, scale=1

RDATA=[];
datavect=[];

imop,

  	fname=deblank(fname);
  	P=vp_nm('dnm',fname);
	
	TYPE0=0; % Types whose SCALE may change
	glmaxdat=0;
	
	vp_hread([P '.hdr']);
	
	name=[P '.img'];
  
	ImageSize=DIM(1)*DIM(2)*DIM(3);

	fid=fopen(name,'r');
 

	DATA=[];
	RDATA=[];
	
	DATA	= fread(fid,MATYPE);
	  
	fclose(fid);
	
	RDATA=(double(DATA))*SCALE;
	if strcmp(imop,'A'),
	 if strcmp(in6,'M'),
	     RDATA=RDATA*in4;
	 elseif strcmp(in6,'D'),
	     RDATA=RDATA/in4;
	 elseif strcmp(in6,'A'),
	     RDATA=RDATA+in4;
	 elseif strcmp(in6,'S'),
	     RDATA=RDATA-in4;
	 end;
	end;
	SCALE,
	  
	glmaxdat=max(RDATA),
	glmindat=min(RDATA),
	glmax=GLMAX;
	  
	if exist('in5'), autonaming=in5; end;
	autonaming,
	if exist('in7'), iroundscl=in7; end;
	iroundscl,
	
	if strcmp(imop,'Z') 
		
		if glmindat >= 0
		  Message=['No Negatives Found ' name];
		  out1=-1;
		  disp(Message);
		  out2=DATA;
		  out3=SCALE;
		  return;
		else
	  		RDATA=(RDATA>0).*RDATA;
			glmaxdat=max(RDATA),
			glmindat=min(RDATA),
			
			if ((TYPE <16) & (SCALE > 0))
			  TYPE0 = TYPE;
	  		  DATA=(DATA>0).*DATA;
		        end


			DESCRIP=['ScAnVP Negatives Zeroed Image'];
			datype=MATYPE;
		end
	elseif strcmp(imop,'L') 
		
	  	RDATA(find(RDATA<=0))=exp(0);
	  	RDATA=log(RDATA);
		glmaxdat=max(RDATA),
		glmindat=min(RDATA),
		DESCRIP=['ScAnVP Natural Log Image'];
		TYPE=16,
		datype='float';
		glmax=1;
		
	elseif strcmp(imop,'T') 
		if glmindat > in4 
		  Message=['No Values Less or Equal To ' num2str(in4) ' Found for ' name];
		  out1=-1;
		  disp(Message);
		  out2=DATA;
		  out3=SCALE;
		  return;
		else
		  RDATA=(RDATA > in4).*RDATA ; 
		  glmaxdat=max(RDATA),
		  glmindat=min(RDATA),

		  if ((TYPE <16) & (SCALE > 0))
			TYPE0 = TYPE;
	  		DATA=(DATA > in4/SCALE).*DATA;
		  end

		  DESCRIP=['ScAnVP Lower Thresholded Image'];
		  datype=MATYPE;
		end
		  
	elseif strcmp(imop,'m') 
	
		  mask = (mask ~= 0); % make sure mask is 0/1 binary;

  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else,
		   Message=['Empty mask'];
		   disp(Message);
		   out1=-1;
		   return;
		  end

		% Remove any NaN numbers
		  datavect(isnan(datavect))=0;

		  glmaxdat=max(datavect),
		  if in4==-inf, minval=min(datavect)
		  else, minval=in4*glmaxdat, end;
		  
		  datavect=(datavect > minval).*datavect ; 
		  datavect=reshape(datavect,1,prod(DIM));
		  glmaxdat=max(datavect),
		  glmindat=min(datavect),
		  RDATA=reshape(datavect,DIM(1:3));
		  
		  if ((TYPE <16) & (SCALE > 0))
			TYPE0 = TYPE;
		  	DATA=reshape(DATA,1,prod(DIM));
                        % O.K use datavect > minval to pick the voxels 
		  	DATA=(datavect > minval).*DATA ; 
		  	DATA=reshape(DATA,DIM(1:3));
		  end

		  DESCRIP=['ScAnVP Masked, Thresholded Image'];
		  datype=MATYPE;
		  
	elseif strcmp(imop,'g') 
	
		  mask = (mask ~= 0); % make sure mask in 0/1 binary;
  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else
     			datavect=RDATA;
		  end
		  
		  glmaxdat=max(datavect),
		  
		  if in4==-inf, minval=min(datavect)
		  else, minval=in4*glmaxdat, end;
		  datavect=(datavect > minval).*datavect ; 
		  
		  %PS Don't count 0's in mean
		  nonzpos=find(datavect); nonzsum=size(nonzpos,1),
		  if (nonzsum == 0)
		      vp_message(['Error: Data has no positive elements ']);
		      return;
		  end
		  datavect=reshape(datavect,1,prod(DIM));
		  datamean=sum(datavect)/nonzsum,
		  
		  datavect=datavect/datamean;
		  
		  glmaxdat=max(datavect),
		  glmindat=min(datavect),
		  
		  RDATA=reshape(datavect,DIM(1:3));
		  DESCRIP=['ScAnVP Masked, Globally Normalized Image'];
		  TYPE=16,
		  datype='float';
		  glmax=1;
		  
	elseif strcmp(imop,'v') 
	
		  mask = (mask ~= 0); % make sure mask in 0/1 binary;
  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else
     			datavect=RDATA;
		  end
		  
		  datavect=reshape(datavect,1,prod(DIM));

		  % divided by sqrt of sum of elements squared.
		  disp('vector normalization ');
		  datanorm=norm(datavect),
		  if datanorm ~= 0
			datavect=datavect/datanorm;
		  else
			vp_note(['Vector Normalization Error: Vector Norm is Zero ']);
			return;
		  end
		  
		  glmaxdat=max(datavect),
		  glmindat=min(datavect),
		  
		  RDATA=reshape(datavect,DIM(1:3));
		  DESCRIP=['ScAnVP Masked, Vector Normalized Image'];
		  TYPE=16,
		  datype='float';
		  glmax=1;
		  
	elseif strcmp(imop,'z') 
	
		  mask = (mask ~= 0); % make sure mask in 0/1 binary;
  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else
     			datavect=RDATA;
		  end
		  
		  datavect=reshape(datavect,1,prod(DIM));

		  Dmask=datavect ~= 0; % Get the intrisic Non-zero mask
		  [no, mn, st]=vp_math('statNo0',datavect); %Non-zero statistics
		  if st ~= 0
		      datavect= (datavect - mn)/st;
		      datavect=datavect.*Dmask; % zero out z-scored 0s 
		  else
		      vp_note(['Vector Z-score Error: STD = 0 ']);
		      return;
		  end
		  
		  glmaxdat=max(datavect),
		  glmindat=min(datavect),
		  
		  RDATA=reshape(datavect,DIM(1:3));
		  DESCRIP=['ScAnVP Masked, Z-scored Image'];
		  TYPE=16,
		  datype='float';
		  glmax=1;
		  
	elseif strcmp(imop,'p') 
	
		  mask = (mask ~= 0); % make sure mask in 0/1 binary;
  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else
     			datavect=RDATA;
		  end
		  
		  glmaxdat=max(datavect),
		  
		  if in4==-inf, minval=min(datavect)
		  else, minval=in4*glmaxdat, end;
		  datavect=(datavect > minval).*datavect ; 
		  
		  %PS Don't count 0's in mean

		  %nonzpos=find(datavect); nonzsum=size(nonzpos,1),

		  %datavect=reshape(datavect,1,prod(DIM));
		  %datamean=sum(datavect)/nonzsum,
		  
		  [nonzsum, datamean, stdev]=vp_math('statNo0',datavect),  %Non-zero statistics
		  if (nonzsum == 0)
		      vp_message(['Error: Data has no positive elements ']);
		      return;
		  end

		  
		  %datavect=datavect/datamean; % Do not normalize
		  
		  glmaxdat=max(datavect);
		  glmindat=min(datavect);
		  
		  RDATA=reshape(datavect,DIM(1:3));
		  DESCRIP=['ScAnVP Masked, Globally, Normalized Image'];
		  TYPE=16,
		  datype='float';
		  glmax=1;
		  
	elseif strcmp(imop,'a') 
	
		  mask = (mask ~= 0); % make sure mask in 0/1 binary;
  		  if ~isempty(mask);
     			datavect=RDATA.*mask;
		  else
     			datavect=RDATA;
		  end
		  
		  glmaxdat=max(datavect),
		  
		  if in4==-inf, minval=min(datavect)
		  else, minval=in4*glmaxdat, end;
		  datavect=(datavect > minval).*datavect ; 
		  
		  %PS Check for 0 Image 
		  nonzpos=find(datavect); nonzsum=size(nonzpos,1),
		  if (nonzsum == 0)
		      vp_message(['Error: Data has no positive elements ']);
		      return;
          end

          % Initially consider only orientation in disk file.
          % Ignore anatomical orientation.
          leftval=0; rightval=0; 

          midxl=round(DIM(1)/2),

          % If the midpoint is between right and left count it on both sides
          % but take half the value. 
          if DIM(1)/2 ~= midxl, midxr=midxl, else midxr=midxl + 1, end;

		  datavect=reshape(datavect,DIM(1:3));
          for i=1:midxl, for j=1:DIM(2), for k=1:DIM(3),
             leftval=leftval+datavect(i,j,k); end; end; end;
          % Correct for common midvalue
          if midxl == midxr, for j=1:DIM(2), for k=1:DIM(3),
             leftval=leftval-datavect(midxl,j,k)/2; end; end; end;  
     
          for i=midxr:DIM(1), for j=1:DIM(2), for k=1:DIM(3),
             rightval=rightval+datavect(i,j,k); end; end; end;
          % Correct for common midvalue
          if midxr == midxl, for j=1:DIM(2), for k=1:DIM(3),
             rightval=rightval-datavect(midxr,j,k)/2; end; end; end;   
     
          % Correct for Anatomical Orientation
          % If Radiological disk file, Flip Right and Left
          if InputOrient==1,
              savleft=leftval; leftval=rightval; rightval=savleft; 
          end;
          AssymIndx=2*(rightval-leftval)/(rightval + leftval),

		  glmaxdat=max(datavect);
		  glmindat=min(datavect);
		  
		  RDATA=reshape(datavect,DIM(1:3));
		  DESCRIP=['ScAnVP Masked, Thresholded Image'];
		  TYPE=16,
		  datype='float';
		  glmax=1;

	elseif strcmp(imop,'X') 
		t1=in4(1); t2=in4(2);
		if ( (glmindat>t2) | (glmaxdat<t1) ) 
		  Message=['Data out of Range: (' num2str(t1) ', ',num2str(t2) ') Found for ' name];
		  out1=-1;
		  disp(Message);
		  out2=DATA;
		  out3=SCALE;
		  return;
		else
		  Message=['Data Range: (' num2str(glmindat) ', ' num2str(glmaxdat) ') Exclude (' num2str(t1) ', ' num2str(t2) ') for ' name];
		  disp(Message);

		  RDATA=( (RDATA>t2) + (RDATA<t1) ).*RDATA ; 
		  glmaxdat=max(RDATA),
		  glmindat=min(RDATA),

		  if ((TYPE <16) & (SCALE > 0))
			TYPE0 = TYPE;
		  	DATA=( (DATA>t2/SCALE) + (DATA<t1/SCALE) ).*DATA ; 
		  end

		  DESCRIP=['ScAnVP Excl. Range Image'];
		  datype=MATYPE;
		end
		
	elseif strcmp(imop,'K') 
		t1=in4(1); if length(in4) > 1, t2=in4(2);
		else t2=t1; end;
		%RDATA=( (RDATA>t2) + (RDATA<t1) ) ; 
		RDATA=( (RDATA>t2) + (RDATA<t1) ).*RDATA ; 
		RDATA=( RDATA ~= 0 ) ; 
		glmindat=min(RDATA),
		glmaxdat=max(RDATA),
		DESCRIP=['ScAnVP Excl. 0|1 Mask'];
		%datype=MATYPE;
		TYPE=2,
		datype='uchar';
		glmax=255;

	elseif strcmp(imop,'Y') 

		skey=in4,
		ind=findstr(skey,',');
 		lk=length(skey);
		li=length(ind);
		nkey=li+1;
		n2=-1;
		datavect=reshape(RDATA,prod(DIM),1);
		SDATA=zeros(size(datavect));
		for k=1:nkey
		  n1=n2+ 2;
		  if (k < nkey), n2=ind(k) -1;
	 	  else, n2=lk; end;
		  sk=skey(n1:n2),
		  ink=findstr(sk,':');
		  if isempty (ink) % assume one value
			t1=str2num(sk);
			if (t1 ~=0), % except 0 values
			  %foundt1=max(datavect==t1);
			  SDATA= SDATA + (datavect==t1);
			end
			
		  else  % assume range
			t1=str2num(sk(1:ink(1)-1)),
			lsk=length(sk);
			sk(ink(1)+1);
			sk(lsk);
			t2=str2num(sk((ink(1)+1):lsk)),
			%foundranget1_t2=max( (datavect>=t1) & (datavect<=t2) ) ; 
			if and(t1<=0, t2>=0) % case range includes 0
			  %foundzeros=max( datavect==0);
			  SDATA= SDATA + ( (datavect>=t1) & (datavect<0) ) ; 
			  SDATA= SDATA + ( (datavect>0) & (datavect<=t2) ) ; 
			else
			  SDATA= SDATA + ( (datavect>=t1) & (datavect<=t2) ) ; 
			end 
		  end;
		  
		end %for k

		%Convert all > 0 values to 1
		SDATA = SDATA > 0;

		glmindat=min(SDATA);
		glmaxdat=max(SDATA),
		if glmaxdat==0, out1=-1; 
		 Message=['No values found for ',P];
  		 disp(Message);
		 vp_message(Message);
		return; end;
		RDATA=reshape(SDATA,DIM(1:3));
		DESCRIP=['ScAnVP Incl. 0|1 Mask'];
		%datype=MATYPE;
		TYPE=2,
		datype='uchar';
		glmax=255;
		
	elseif strcmp(imop,'s') 
		xdata=reshape(RDATA,DIM(1),DIM(2),DIM(3));
		xdata=smooth3(xdata,'box',in4) ; 
		RDATA=reshape(xdata,prod(DIM(1:3)),1);
		glmaxdat=max(RDATA),
		glmindat=min(RDATA),
		if size(in4) ==1
		  DESCRIP=['ScAnVP Smoothed Size ' num2str(in4) ' Cubic Box Conv.'];
		else
		  DESCRIP=['ScAnVP Smoothed [' num2str(in4) '] Box Conv.'];
		end
		TYPE=16,
		datype='float';
		glmax=1;
		
	elseif strcmp(imop,'A') 
		if strcmp(in6,'M')
		  DESCRIP=['ScAnVP Scalar Multiplied image'];
		elseif strcmp(in6,'D')
		  DESCRIP=['ScAnVP Scalar Divided image'];
		elseif strcmp(in6,'A')
		  DESCRIP=['ScAnVP Scalar Added image'];
		elseif strcmp(in6,'S')
		  DESCRIP=['ScAnVP Scalar Subtracted image'];
		end;

		if ( (TYPE < 16) & strcmp(in6,'M') ) 
		  TYPE0 = TYPE;
	  	  SCALE=SCALE*in4;
		  datype=MATYPE;
		elseif ( (TYPE < 16) & strcmp(in6,'D') ) 
		  TYPE0 = TYPE;
	  	  SCALE=SCALE/in4;
		  datype=MATYPE;
		else
		  % Play it safe. Convert to float.
		  TYPE=16,
		  datype='float';
		  glmax=1;
		end

		
	else
		TYPE0 = TYPE,
		if strcmp(imop,'B') 
		  TYPE=2,
		  datype='uchar';
		  glmax=255;
		elseif strcmp(imop,'S') 
		  TYPE=4,
		  datype='int16';
		  glmax=32767;
		elseif strcmp(imop,'I') 
		  TYPE=8,
		  datype='int32';
		  glmax=(2^31-1);
		elseif strcmp(imop,'F') 
		  TYPE=16,
		  datype='float';
		  glmax=1;
		elseif strcmp(imop,'D') 
		  TYPE=64,
		  datype='double';
		  glmax=1;
		else
		  Message=['ERROR: Invalid IMOP: ' imop];
		  disp(Message);
		  out1=-1;
		  return;
		end
  		DESCRIP=['ScAnVP Converted to ' datype ' Image'];
	end
	  
	  %maxout=max(RDATA),
	  %minout=min(RDATA),
	  %maxout=max(max(RDATA(:,:,PlaneNo))),
	  %minout=min(min(RDATA(:,:,PlaneNo))),
	  %keyboard;
	

TYPE,datype,DESCRIP,

if glmaxdat == 0,
  Message=['WARNING: 0 maximum'];
  disp(Message);
  vp_message(Message);
  while(findobj('Name','Message')),pause(1),end;
end

if (TYPE==2 & glmaxdat*glmindat < 0), 
 %they can both be negative but limits cannot have opposite signs
	RDATA=(RDATA>0).*RDATA;
	glmindat=min(min(min(RDATA))),
  	vp_note(['8bit Image Negatives were set to Zero'],2);
	DESCRIP=['ScAnVP Negatives Zeroed, Converted to 8Bit Image'];
end;

if (abs(glmindat) > glmaxdat),
	glmaxdat=abs(glmindat);
end;

	
if TYPE0 ~= 0 % May not need to change DATA range
 if TYPE >= TYPE0, %more bits 
   if TYPE < 16 %integer types
     if iroundscl==1, %rounding selected
       DATA=[];
       DATA=round(RDATA);
       SCALE=1;
     end
      %Otherwise will save the same data as old type
   else %float type
      DATA=[];
      DATA=RDATA;
      SCALE=1.0;
   end
 else % less bits
    DATA=[];
    if or((glmaxdat > glmax), iroundscl==0),
     DATA=RDATA*glmax/glmaxdat;
     SCALE=glmaxdat/glmax;
    else %Dangerous roundoff if data small
     % iroundscl=1, rounding selected
     DATA=round(RDATA);
     SCALE=1;
   end
 end
else %TYPE0=0
 DATA=[];
 if TYPE < 16 %integer types
   if or(strcmp(imop,'K'),strcmp(imop,'Y'))
     DATA=RDATA;
     SCALE=1;
   else 
   %if (glmaxdat > glmax)
    if or((glmaxdat > glmax), iroundscl==0),
      DATA=RDATA*glmax/glmaxdat;
      SCALE=glmaxdat/glmax;
    else %Dangerous roundoff if data small
     % iroundscl=1, rounding selected
     DATA=round(RDATA);
     SCALE=1;
    end
   end
 else
  DATA=RDATA;
  SCALE=1.0;
 end
end

out2=DATA; out3=SCALE;
if strcmp(imop,'a'), out2=leftval; out3=rightval; end;
if strcmp(imop,'p'), out2=datamean; out3=nonzsum; out4=stdev; end;
if in3==1, return, end;

%WRITE IMAGE

savdir=[pwd, '\'];

nam=vp_nm('nm',P);
if (strcmp(imop,'L') | strcmp(imop,'s') | strcmp(imop,'g') | ...
    strcmp(imop,'v') | strcmp(imop,'z') ),
 nameimg=[savdir imop nam '.img'];
elseif  strcmp(imop,'m') 
 nameimg=[savdir nam '_m.img'];
elseif  strcmp(imop,'A') 
 nameimg=[savdir nam in6 imop '.img'];
elseif  strcmp(imop,'Y') 
 nameimg=[savdir nam 'K.img'];
else
 %nameimg=[P imop '.img'];
 nameimg=[savdir nam imop '.img'];
end
fidop=fopen(nameimg,'w');


no=fwrite(fidop,DATA,datype);
if no <=0,
  	Message=['Write ERROR no: ' num2str(no) ' pixels'] ;
	disp(Message);
	out1=-1;
	return;
end
	
fclose(fidop);

out4=nameimg;
if(autonaming)
	if (vp_hwrite(nameimg,DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,...
	EXPTIM,DURATION,GENTIM,SUBID,EXPDATE) ) < 0,
	vp_message(['hwrite err for ',nameimg]); out1=-1; 
	end;
	return;
end %if autonaming

Bcolor = [.8,1,1];

imcvtgui = figure('Color',Bcolor,'Name',[imop ' cvt'],'Tag','imcvtgui',...
'NumberTitle','off','Resize','on','Position',[100,200,250,60],'UserData',0,...
'Units','Pixels');

% Save default header before user name read, to keep consistent with image saved.
% Read user name. If not the default name. Warn if only .hdr or .img doesn't exist,
% since if there aren't permissions to overwrite, only the header or image will be
% overwritten and the other will be incorrect.

%	'nameimg=deblank(nameimg); namehdr0=[strtok(nameimg,''.'') ''.hdr''];',...
%
cbfname=['staterr=0; hrname=findobj(''tag'',''namefield''); nameimg=get(hrname,''userdata''),',...
	'nameimg=vp_nm(''dnm'',nameimg); namehdr0=[nameimg ''.hdr'']; nameimg=[nameimg ''.img''];',...
	'if (vp_hwrite(nameimg,DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,',...
	'EXPTIM,DURATION,GENTIM,SUBID,EXPDATE) ) < 0,',...
	'vp_message(''hwrite err''); staterr=1; ',...
	'else, ',...
	  'name=get(hrname,''string''),',...
	  'name=vp_nm(''dnm'',name); namehdr=[name ''.hdr''];',...
	  'name=[name ''.img''];',...
	  'if ~strcmp(name,nameimg),',...
	     'if (exist(name,''file'') & ~exist(namehdr,''file'')) | ',...
	       ' (~exist(name,''file'') & exist(namehdr,''file''))  ',...
	       'vp_message([''Remove Invalid File '' name '' .hdr or .img'']);',...
	       ' staterr=3; ',...
	     'else, [s,dosmsg]=dos([''move '' nameimg '' '' name]); ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	       '[s,dosmsg]=dos([''move '' namehdr0 '' '' namehdr]); ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	     'end; ',...
	     'if staterr==0,  nameimg=name;  end;',...
	  'end; ',...
	'end;',...
	'if staterr==0,',...
	  'set(gco,''string'',[nameimg ''   FILE SAVED'']); Message=[''FILE SAVED'']; ',...
	  'disp(Message); pause(2); set(findobj(''tag'',''imcvtgui''),''UserData'',1); ',...
	' end; '];
	
       
       
%	'set(hroierr,''string'',[''MSG: '',Message]);'] ;
	
uicontrol(imcvtgui,'style','text','string','ENTER NAME: ','units','normalized',...
	'position',[10/250 25/60 125/250 25/60],'BackgroundColor',Bcolor);
	
hrname=uicontrol(imcvtgui,'style','edit','tag','namefield','string',nameimg,...
	'units','normalized','userdata',nameimg,...
	'position',[10/250 5/60 225/250 25/60],'BackgroundColor','w','callback',cbfname);



while( get(findobj('tag','imcvtgui'),'UserData')==0) ,pause(1); end;
out4=nameimg;
delete( findobj('tag','imcvtgui') );

%while( findobj('tag','imcvtgui') )pause(1); end;

