% vp_imfun7.m
% Interface to imfun.exe (Opt 7a|b) Dynamic Patlak method voxel based
% functionalization using blood file input data. P. Spetsieris 1999
% 
% DYNAMIC IMAGE FUNCTIONALIZATION
% Dynamic Patlak method voxel based functionalization of PET image files
% in accordance with one of the following menu options :
%
%  a--Patlak (mg/100g/min)         Metabolism (FDG)
%  b--Patlak (micro liters/g/min)  Uptake (FDOPA)
% 
% ANALYZE format image frames are input and an ANALYZE format image
% is output. Negative values are zeroed. File values are multiplied
% by a scale factor and converted to 8bits. The 'real' functional
% range is saved in the header. Correponding blood files should
% have the same name with a '.blf' suffix and be in the image
% directory which also serves as the ouput directory.
%  
% Format for input name.blf text data file is:
%    Plasma Blood file : (Option 1) START TIMES,CP,INTEGRAL CP
%    (Name.blf)      or: (Option 2) Tn,Cn
%  
% Blood files first line contain the plasma glucose concentration
% start-stop times (min) and an arbitrary comment followed by
% sequential blood curve data.
%
% Option 1 Blood File
% Refers to a file containing the start time TP for each
% frame and externally fitted values for CP and INTEGRAL CP at
% midpoint of each scan separated by space or commas. The first
% line of the file is as above where the start time refers to
% the first frame for processing. The start and stop frames for
% processing are entered by the user.
%  
% Option 2 Blood File
% Contains original blood curve data Tn, Cn and can be used only
% with contiguous scans. CP and integral CP values at scan midpoints
% are derived by the program using linear interpolation and numerical
% integration. Start times for each frame are calculated from the
% value for the start frame in the blood file and the scan durations
% stored in the Analyze headers.
%
% For Patlak uptake(Opt b), background subtraction values for each frame
% can be read from a file Name.bak (first line is a comment).
%
% Functional output images are
% assigned an 'f' prefix.
%  
% See dos('...scanvp\bin\imfun h') for further options using command line usage.
%___________________________________________________________________________
% vp_imfun7.m 5.0                          Phoebe Spetsieris Feb. 2, 2003
%

global Frame0 MaxFrame BlfName PatlakOpt Message
global HelpMode Ecolor DATA XYZDATA FrameNo 
global hblfname hFrame0 hmaxfrm Message0
global FI 


if HelpMode, vp_help(mfilename), return ; end;

BlfName=[];
PatlakOpt=[];

%Set user interface controls

bkgcolor = [.9,.9,.8];
Message0=['....'];
Message=Message0;

Frame0=1;
MaxFrame = 1 ;

delete(findobj('Name','PatlakBlfGui'));

cbFrame0=['Frame0=str2num(get(hFrame0,''string''));'];
cbmaxfrm=['MaxFrame=str2num(get(hmaxfrm,''string''));'];

patlakblfgui = figure('Color',[.5,.6,.6],'Name','PatlakBlfGui',...
'NumberTitle','off','Position',[100,200,250,154],...
'Units','Pixels');
	
uicontrol(patlakblfgui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[200 129 40 20],'callback',...
	'vp_help(''vp_imfun7''); ');

cbPblfname=['BlfName=get_file(1,''.blf'',''Please Select Blood File'',[]); ',...
	'if ~isempty(BlfName),',...
	'set(hblfname,''string'',BlfName); ',...
	'vp_dispfile(BlfName); ',...
	'if isempty(Message), Message=Message0; end; end;',...
	];
	
uicontrol(patlakblfgui,'string', 'Name.blf FILE',...
	'position',[10 104 75 20],'BackgroundColor',bkgcolor,...
	'ToolTipString',...
	'Blood file (name.blf) should have the same name as image frames.',...
	'callback',cbPblfname);
	
cbblfname=['BlfName=get(hblfname,''string'');','Message=Message0 ;',...
	'if ~isempty(BlfName),',...
	'vp_dispfile(BlfName); ',...
	'if isempty(Message), Message=Message0; end; end;',...
	];
	
hblfname=uicontrol(patlakblfgui,'style','edit','string',BlfName,...
	'ToolTipString',...
	'Blood file (name.blf) should have the same name as image frames.',...
	'position',[85 104 155 20],'BackgroundColor',bkgcolor,'callback',cbblfname);
	
uicontrol(patlakblfgui,'string', 'Initial Frame',...
	'TooltipString','Initial Process Frame Number',...
	'position',[10 35 75 20],'BackgroundColor',bkgcolor);
	
hFrame0=uicontrol(patlakblfgui,'style','edit','string',Frame0,...
	'TooltipString','Initial Process Frame Number',...
	'position',[85 35 35 20],'BackgroundColor',bkgcolor,'callback',cbFrame0);

uicontrol(patlakblfgui,'string', 'Final Frame',...
	'TooltipString','Final Process Frame Number',...
	'position',[130 35 75 20],'BackgroundColor',bkgcolor);
hmaxfrm=uicontrol(patlakblfgui,'style','edit','string',MaxFrame,...
	'TooltipString','Final Process Frame Number',...
	'position',[205 35 35 20],'BackgroundColor',bkgcolor,'callback',cbmaxfrm);

cbPOpts = ['if isempty(BlfName), vp_note(''Blood Text File Not Defined'',2); else,',...
	' POpt=get(hPatlakMenu,''Value''); ',...
	' if POpt== 2, PatlakOpt=''a'';',...
	'elseif POpt== 3, PatlakOpt=''b''; ',...
	'else ; end; end;'];
	
hPatlakMenu=uicontrol(patlakblfgui,'Style','PopUp','String',['Patlak Options|',...
	'Metabolism (FDG) - (mg/100g/min)|',...
	'Uptake (FDOPA) - (microliters/g/min)'],...
	'TooltipString',['1: Gives Voxel Cerebral Metabolic Rates Glucose |',...
	'2: Voxel Uptake Values.'],...
	'Position',[10 60 210 20],...
	'CallBack', cbPOpts,...
	'BackgroundColor',bkgcolor);

uicontrol(patlakblfgui,'String','CANCEL',...
	'Position',[10 10 60 20],...
	'CallBack',['delete(gcbf);']);
	
	%'[s,w]=dos(cmdline,''-echo''),',...
	%
uicontrol(patlakblfgui,'String','PROCESS',...
	'Position',[180 10 60 20],...
	'CallBack',['if isempty(BlfName), vp_note(''Select Blood File'',2);',...
	'elseif isempty(PatlakOpt), vp_note(''Select PatlakOpt '',2);',...
	'elseif Frame0==MaxFrame, vp_note(''Set Frames'',2);',...
	'else,',...
	'cmdline=[''imfun '',BlfName,'' '',num2str(7),PatlakOpt,',...
	''' '',num2str(Frame0),'' '',num2str(MaxFrame),',...
	''' > PATfunoutput.txt''],',...
	'h=waitbar(0.5,''Processing PATLAK Functionalization'');',...
	'[s,w]=vp_dos(cmdline),',...
	'waitbar(1,h); pause(1); close(h);',...
	'if s < 0,',...
	'vp_note(''PROCESS FAILED, Check MATLAB Command Window and PATfunoutput.txt.'');',...
	'else,',...
	'vp_note(''PROCESS COMPLETE Check PATfunoutput.txt'');',...
	'end; end;']);
