% function val=vp_imvalue(no);
%
% User picks a point on a data image and its value, coordinates and default
% ATLAS region label designation is displayed. It is assumed that the image
% is always displayed in the Neurological orientation (LR) and the referenced
% ATLAS has the same orientation ('LR') as the image display. It is also,
% assumed that the image and ATLAS are coregistered although a linear scaling
% difference in size is allowed.
% X, Y, Z coordinate values correspond to Talairach coordinates if the file
% is normalized and in the Neurological Orientation. If the file was normalized
% and saved in the Radiological orientation (RL) it is flipped along the X axis
% (RL => LR) upon display assuming the 'Input Orientation' is set to 'RL'
% on the main scanvp menu. If the input orientation is set to LR, then the
% corresponding X Axis Talairach coordinate X will be equal to -X of the
% displayed coordinate and the referenced display ATLAS label will be
% incorrect at least with regard to the L/R designation.
%_______________________________________________________________________
% vp_imvalue.m  ScAnVP 	Ver. C.4.4 Phoebe Spetsieris  Jan. 27, 2000
%  Rev. Ver. 6.2, 6.2.1, by P.S.  Jul. 5, 2011, Apr. 29, 2013,
%  Rev. Ver. 7.0 by P.S. Oct. 8, 2015 

function [out1] = vp_imvalue(no)

global HelpMode UFI DIM VOX ORIGIN DefAtlasNm

if HelpMode, vp_help(mfilename), return ; end;

out1=0; 
viewno=0;

TxColor=[0.8 0.2 1];

	
%if (~exist('FI') | isempty(FI) ), 
if  isempty(UFI),  
  UFI=gcf;
end 
      
figure(UFI);
       
if nargin < 1
  vp_note('PICK IMAGE POINT', 1);
end

[x,y]=ginput(1);

voxsave=VOX;

if strcmp(get(gco,'type'),'image')
  cdata=get(gco,'CDATA');
  cdata=cdata';
  
  sz=size(cdata);
  mx=max(max(cdata));
  mn=min(min(cdata));
  
  x=round(x), y=round(y),
  value=cdata(x,y),
  udata=get(gca,'UserData');
  if ~isempty(udata),
  
    viewno=udata(1); plno=udata(2); vox=udata(3:5); xyzor=udata(6:8);
    if viewno==1
  	xmm=(x-xyzor(1))*vox(1); ymm=(y-xyzor(2))*vox(2);
	zmm=(plno-xyzor(3))*vox(3);
        xvox=x; yvox=y; zvox=plno;
    elseif viewno==2,
  	ymm=(x-xyzor(2))*vox(2); zmm=(y-xyzor(3))*vox(3);
	xmm=(plno-xyzor(1))*vox(1);
        xvox=plno; yvox=x; zvox=y;
    elseif viewno==3,
  	xmm=(x-xyzor(1))*vox(1); zmm=(y-xyzor(3))*vox(3);
	ymm=(plno-xyzor(2))*vox(2);
        xvox=x; yvox=plno; zvox=y;
    end

    w=which('scanvp'); %find default atlas template
    atlasfile=[w(1:(max(findstr(w,'\'))-4)),'templates\Brain_Atlas\',[DefAtlasNm,'_LR68.img']];

    saveglobhdr=vp_globhdr('save'); % save the image Analyze header globals

    [out1 atlasdata]=vp_volfuns('xyzdata',atlasfile);
     
    vox,
    VOX,
    xvox=round(xvox*vox(1)/VOX(1)),
    yvox=round(yvox*vox(2)/VOX(2)), zvox=round(zvox*vox(3)/VOX(3)),
    atlasindx=atlasdata(xvox,yvox,zvox),
    if atlasindx==0, atlaslabel='Undefined',
    else
        [out1 message]= vp_atlaslabel(atlasfile, atlasindx);
        %if (out1==0), atlaslabel=message, else, vp_note(message,2); end;
        if (out1==0), atlaslabel =[message,' (Display Atlas ''LR'')'],
        else, vp_note(message,2); end;
    end;

    out1=vp_globhdr('restore',saveglobhdr); % restore the image Analyze header globals
    if (out1 ~= 0), vp_note('Check for possible error in image parameters',2); end;

  end
  
else
	out1=-1;
	vp_message('Not an ''image'' point');
	return;
end

himv=findobj('Name','ImValue');

position=[200,200,350,120];

if ~isempty(himv); position=get(himv,'Position'); delete(himv); end;


        himv = figure('Color','w','Name','ImValue',...
	'NumberTitle','off','Position',position,...
	'Units','Pixels','Visible','on');
	
	axis off;

	uicontrol(himv,'String','VALUE',...
	 'Position',[35 5 55 20],...
	 'TooltipString','Get Value/Spec. of point on screen',...
	 'CallBack','nargin, vp_imvalue(0);');
	
	uicontrol(himv,'String','HELP',...
	'Position',[105 5 55 20],...
	'CallBack',['vp_help(''vp_imvalue''); ',...
	'vp_imvalue(0);'] );

	uicontrol(himv,'String','RETURN',...
	'Position',[175 5 55 20],...
	'CallBack',[' ; ',...
	'delete(findobj(''Name'',''ImValue''));'] );

	
	coord=[' (',num2str(x) ', ' num2str(y),')'];
	if viewno == 1, view ='Axial XY,  ';
	elseif viewno==2, view = 'Sagittal YZ,  ';
	elseif viewno==3, view = 'Coronal XZ,  ';
	else view= 'Not Defined,   ';
	end
	
        textstr=['View: ', view, 'pix',coord];
	text(0.0,.98, textstr,...
	'FontName','Times','FontSize',10,...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Left',...
	'Color',TxColor);

	if viewno~=0,
	
	  %textstr=['PlaneNo :   ',num2str(plno), '  ', coord ];
	  textstr=['PlaneNo :   ',num2str(plno)];
	  text(0.0,.40, textstr,...
	  'FontName','Times','FontSize',10,...
	  'VerticalAlignment','Middle',...
	  'HorizontalAlignment','Left',...
	  'Color',TxColor);
	
	  textstr=['x= ',num2str(xmm),', y= ',...
		num2str(ymm),', z= ',num2str(zmm), '  (mm)'];
	  text(0.0,.80, textstr,...
	  'FontName','Times','FontSize',10,...
	  'VerticalAlignment','Middle',...
	  'HorizontalAlignment','Left',...
	  'Color',TxColor);
	end
	
	
	
	textstr=['Value: ',num2str(value), ',   Min,Max(', num2str(mn),...
	', ',num2str(mx),')'];

	
	%textstr=['Value: ',num2str(ImValue), '   Reset to: ',...
	%	num2str(RoiThresh)] ;
	text(0.0,.60, textstr,...
	'FontName','Times','FontSize',10,...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Left',...
	'Color',TxColor);


	atlaslabel=strrep(atlaslabel,'_',' ');
	textstr=['Ref. Atlas Label: ', atlaslabel];
	text(0.0,.20, textstr,...
	'FontName','Times','FontSize',10,...
	'VerticalAlignment','Middle',...
	'HorizontalAlignment','Left',...
	'Color',TxColor);
       
	








