% vp_imview.m
%
% 'IMAGE VIEW' Displays three orthogonal sections (Axial, Coronal and Sagittal)
% of an ANALYZE format image through the origin defined in the header
% or through the center point if the origin is not defined. Options
% are provided for modifying the origin, the view point, for rotation
% around any axis and for volume cropping. Modified images can be saved.
% The user can display multiple sequential sections with various
% formating and slice skip options. Dynamic or multi-image displays
% of the same sectional slice are also supported. In addition, options
% are provided for 3D plots of planar surface values, 2D and 3D cluster
% removal as well as linear profile plots.
%
% Images are displayed in Neurological Orientation (Right is Anatomical Right)
% as long as the "Input Orientation" (i.e. the saved orientation of the ANALYZE
% file on disk is correctly specified as Neurological (LR) or Radiological (RL).
% Newer raw images are usually stored in the Radiological convention. Thus axial
% stored right is left of subject for GE GENESIS-MRI and IEE-PET formats. SPM99
% normalized images are stored in the Neurological convention while spm5 normalized
% images are stored in Radiological format although displayed neurologically.
% This version of ScAnVP follows the Neurological convention for display
% purposes and automatically flips the Radiological Input specified files.
% Image processing utilities are provided on the ScAnVP menu to (externally) 
% flip the original image along the X,Y or Z orthogonal axes or to save
% Sagittal or Coronal views. However, 'IMAGE VIEW' assumes axial input planes
% and will not label the displayed 'Axial', 'Sagittal' and 'Coronal'
% sections properly, if the input image is not in the Axial orientation.
%
% Physical coordinates displayed by 'IMAGE VIEW' are based on the voxel coordinates 
% of the origin ORIGIN and the physical dimensions of each voxel (VOX).
% If the images are normalized using spm99, the physical position of the 
% origin should correspond to that defined by the normalized space of the
% spm template which is Talairach based in spm99. When a spm99 normalized
% image is displayed by 'IMAGE VIEW', the cross in the three orthogonal views
% should initially point to the Talairach origin and coordinates of other
% points are in mm from the origin in Talairach space. Spm5 normalized files
% conform to the MNI template which approximates Talairach space.
%
% Thus, X, Y, Z coordinate values correspond to Talairach coordinates if the file
% is normalized and in the Neurological Orientation (e.g. spm99). If the file was 
% normalized and is in the Radiological orientation (e.g. spm5) it will be flipped 
% along the X axis (RL <=> LR) prior to display if its original input orientation
% is specified on the main menu, otherwise, X=-X is the corresponding X Axis
% Talairach coordinate. Internal flipping does not affect the file on disk.
%
%___________________________________________________________________________
% vp_imview.m	ScAnVP C.0	P. Spetsieris  April 1, 1999
%		rev. 5.2 by P.S		Nov. 13, 2003
%		rev. 6.1 by P.S		Jun. 3, 2011
%		rev. 7.0 by P.S		Oct. 29, 2015

global WinView  WinFormat Plane0 MaxPlane MaxDim Opt FName Time Message
global DIM VOX SCALE TYPE OFFSET ORIGIN GCM SliceDelta
global HelpMode Ecolor DATA XYZDATA PlaneNo 
global hrname hview hform hplane0 hmaxpln Message0
global XYZDATA0 DIM0 VOX0 XYZOR0 PlaneX PlaneY PlaneZ RotAngle
global DESCRIP EDIT InputOrient DisplayOrient


if HelpMode, vp_help(mfilename), return ; end;

FName=[];
RotAngle=0;

%Set image user interface controls
vp_imui;

cbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) Disk Input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) Disk Input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];


uicontrol(imviewgui,'string', 'For Left-Right Display, Check Input.',... 
	'TooltipString',['For Neurological Display (LR),',...
    'check Radiological or Neurological ',...
    ' Input File Orientation on Main Menu.'],...
	'position',[10 192 190 20],...
	'BackgroundColor',Ecolor,'callback',cbImgOrient);

uicontrol(imviewgui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[202 192 40 20],'callback','vp_help(''vp_imview'');');
	

cbPName=['FName=get_file(1,''.img'',''Please select image'',[]); ',...
	'if ~isempty(FName),',...
	'Opt=1; Time=Time+1; vp_imviewpro; colormap(GCM); ',...
	'set(hrname,''string'',FName); ',...
	'set(hview,''string'',WinView); ',...
	'set(hform,''string'',WinFormat); ',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'if isempty(Message), Message=Message0; end; end;',...
	];
	
uicontrol(imviewgui,'string', 'PICK IMAGE',...
	'position',[10 213 75 20],'BackgroundColor',bkgcolor,...
	'callback',cbPName);
	
cbFName=['FName=get(hrname,''string'');','Message=Message0 ;',...
	'if ~isempty(FName),',...
	'Opt=1; Time=Time+1; vp_imviewpro; colormap(GCM); ',...
	'set(hview,''string'',WinView); ',...
	'set(hform,''string'',WinFormat); ',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'if isempty(Message), Message=Message0; end; end;',...
	];
	
hrname=uicontrol(imviewgui,'style','edit','string',FName,...
	'position',[85 213 155 20],'BackgroundColor',bkgcolor,'callback',cbFName);

%	'set(hmaxpln,''string'',Plane0); ',...
%
cbDyNames=['DyFNames=get_file(inf,''.img'',''Please select image set'',[]); ',...
	'if ~isempty(DyFNames), ',...
	'Opt=3; vp_imviewpro; colormap(GCM); ',...
	' end;',...
	];
	
uicontrol(imviewgui,'string', 'DYNAMIC or MULTI-IMAGES',...
	'TooltipString','Display Multiple or Dynamic Images at set Plane',...
	'position',[10 172 160 18],'BackgroundColor',bkgcolor,...
	'callback',cbDyNames);
	
cb2D3Dopts = ['if isempty(FName), vp_note(''Image Not Defined'',2); else,',...
	' ImOpt=get(hclustmenu,''Value''); ',...
	' if ImOpt== 2, ',...
	'vp_remclust(''0'',1,2); ',...
	'elseif ImOpt== 3, ',...
	'vp_remclust(''0'',2,2); ',...
	'elseif ImOpt== 4, ',...
	'vp_remclust(''vp_imviewpro'',1,3); ',...
	'elseif ImOpt== 5, ',...
	'vp_remclust(''vp_imviewpro'',2,3); ',...
	'elseif ImOpt== 6, ',...
	'UFI=FI; vp_lineplot; ',...
	'elseif ImOpt== 7, ',...
	'UFI=FI; vp_surfplot; ',...
	'else ; end; colormap(GCM); end;'];
	
%	'Del. Clusters 2D-pnt|Del. Clusters 2D-key|',...
%	'Del. Clusters 3D-key|Surface Plot of area'],...
%	'TooltipString',['1: Remove 2D clusters - Define Sample Roi |',...
%	'2: Remove 2D clusters - key diameter |',...
%	'3: Remove 3D clusters - key diameter |',...
%	'4: Display surface plot of values in 2D area'],...
%

hclustmenu=uicontrol(imviewgui,'Style','PopUp','String',['CLUST/PLOT...|',...
	'Del 2D Clusters|',...
	'Del 2D Clust PET/MRI|',...
	'Del 3D Clusters|',...
	'Del 3D Clust PET/MRI|',...
	'Plot Line Segment|',...
	'Surface Plot of Area'],...
	'TooltipString',['1: Remove 2D clusters - over 0 |',...
	'2: Remove 2D clusters over Background |',...
	'3: Remove 3D clusters - over 0 |',...
	'4: Remove 3D clusters over Background |',...
	'5: Profile plot of line segment |',...
	'6: Surface plot of 2D area'],...
	'Position',[70 11 109 20],...
	'CallBack', cb2D3Dopts,...
	'BackgroundColor',bkgcolor);
	
	
	
uicontrol(imviewgui,'String','DISPLAY',...
	'Position',[180 10 60 20],...
	'CallBack',['if isempty(FName), vp_note(''Select Image'',2); else,',...
	' Opt=2; vp_imviewpro; colormap(GCM); set(imviewgui,''UserData'',1); end;']);
	

