function [out1, out2]=vp_mimpro(NameLst, imop, in3, in4, in5, in6, in7, in8)
% Perform image operation on ANALYZE Format images
%_______________________________________________________________________
%function [out1, out2]=vp_mimpro(NameLst, imop, in3, in4, in5, in6, in7, in8)
%
% vp_mimpro performs corresponding voxel image operations such as 
% addition, subtraction, division, multiplication etc. on a set of
% of multiple ANALYZE format images. For a single image the operation
% may be vector normalization, masking, Z transform or multiplication.
% Each is scaled and converted to real values before each operation is
% performed. If there are resulting negative values and the original
% data is 8bit, the result is converted to double.
% Values divided by zero are set to zero.
% For mixed subjects, some of the output header parameters may refer
% only to the last subject processed. 
%
% NameLst   :   First Argument is Name List
% imop      :   Second Argument
%
% imop='AVG':	Average Images
% imop='ADD':	Add Images
% imop='SUB':	Subtract Images
% imop='DIV':	Divide Images
% imop='EXC':	3D Exclude common voxels
% imop='MLT':	Multiply Images
% imop='OP':	Processed Image
% imop='EXS':	Exclude same sign voxels
%
% in3=1  :  Return DATA without writing image
% in3=2  :  Write image and return File Name (return Data if auto naming off)
% in3=3  :  Return DATA and write image
% in4	 :  Set of scaling factors for imops other than EXC.  
%	    precision for imop='EXC' 1=1pixel, 2=2pixel, etc.
% in5    :  reserved
% in6    :  1= automatically assign output names, 0= interactive naming
%        :  2= preset output name stored in global OutImg
% in7	 :  Not utilized by 'EXC' Option
% in7=0	 :  No input or output normalization 
% in7=1  :  globally normalize input values.
% in7=2  :  vector normalize input values.
% in7=3  :  Z score output. (Only non-zero values are processed)
% in7=4  :  globally normalize input and Z score output.
% in7=5  :  vector normalize input and Z score output.
% in7=12 :  vector normalize output.
% in7=13 :  globally normalize input and vector normalize output.
% in7=14 :  vector normalize input and vector normalize output.
% in8    :  data mask file for imop other than 'EXC'
% out1   :  Return value
% out2   :  Return Output File Name or DATA
%_______________________________________________________________________
% vp_impro	C.4.4	P. Spetsieris	11/23/99
% vp_mimpro  ScAnVP 5.6	P. Spetsieris	10/21/05
% rev  ScAnVP 5.8	P. Spetsieris	04/12/07
% rev  ScAnVP 6.2 	P. Spetsieris	04/28/12
%  (Added option for preset output name stored in OutImg if AutoName=2)
%  Corrections for niftii images

global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE GENTIM SUBID EXPDATE
global HelpMode Message OutImg

if HelpMode, vp_help(mfilename), return ; end;

%Defaults
out1=0;
out2=0;
gnorm=0; % 1=perform global normalization, default 0=off; 
vinnorm=0; % 1=perform vector input normalization, default 0=off; 
	   % Constant coefficients will also be normalized;
voutnorm=0; % 1=perform vector input normalization, default 0=off; 
zscout=0; % 1=perform output zscore of image, default 0=off; 
autonaming=1; %Automatic naming default ON
maskfile=[]; % mask data


m= size(NameLst,1);
dim3=0;
sfactor=ones(1,m);


if m == 1,
 imop='OP';
end;

if m < 1,
  vp_note(['ERROR: Invalid number of files: ',num2str(m)]);
  out1=-1;
  return;
end;

if ( strcmp(imop,'EXC') & (m ~=2) ),
  vp_note(['ERROR: Enter two files.']);
  out1=-1;
  return;
end;

if nargin > 3, sfactor=in4; end;
if (~strcmp(imop,'EXC') & m ~=size(sfactor,2))
  vp_note(['ERROR: No. of Images (',num2str(m),') Not Equal to No. of Scale Factors (',num2str(sfactor,2),').']);
  out1=-1;
  return;
end 

if nargin > 5, autonaming=in6; end;
if nargin > 6,
	in7,
	if ((in7 == 1)|(in7==4)|(in7==13))
	gnorm=1, end;
	if ((in7 == 2)|(in7==5)|(in7==14))
	  vinnorm=1, ssf2=sum(sfactor.* sfactor); 
	  sfactor=sfactor/(sqrt(ssf2));
	end;
	if ((in7 == 12)|(in7==13)|(in7==14))
	voutnorm=1, end;
	if ((in7 == 3)|(in7==4)|(in7==5))
	zscout=1, end;
end;
if nargin > 7, if ~isempty(in8), maskfile=in8; end; end;
autonaming,
gnorm, vinnorm, voutnorm, zscout,


if ~isempty(maskfile)
  [out1 maskdata]=vp_volfuns('xyzdata',maskfile);
  if out1 < 0, vp_message('ERROR: reading MASK file'); return; end;
  size(maskdata),
  maskdata = (maskdata ~= 0);
  maskdata=reshape(maskdata,prod(size(maskdata)),1);
end

for i=1:m

  FName=deblank(NameLst(i,:));
  P=vp_nm('dnm',FName);
  
  vp_hread([P '.hdr']);
  
  if i > 1
    if ~isequal(VOX, vox1)
	vox1,
	VOX,
  	vp_note(['ERROR: Incompatible Voxel sizes']);
	out1=-1;
	return;
    end
  
    if ~isequal(DIM(1:2), dim1)
	dim1,
	DIM(1:2),
  	vp_note(['ERROR: Incompatible Planar Dimensions']);
	out1=-1;
	return;
    end
  
    if ~isequal(DIM(3), dim3)
	dim3,
	DIM(3),
  	vp_note(['ERROR: Incompatible No. of Planes']);
	out1=-1;
	return;
    end
  
    if(TYPE ~= typ1)
	typ1,
	TYPE,
  	vp_note(['WARNING: Different TYPES']);
	waitfor(findobj('Name','Note'));
	
    end
    
    types(i)=TYPE;
  
  else
    vox1=VOX;
    dim1=DIM(1:2);
    dim3=DIM(3);
    typ1=TYPE;
    types(i)=TYPE;
  end
  
  disp(' ');
end %for i


PlaneNo=1;

SDATA=[];

glmaxdat=0;
	
if strcmp(imop,'EXC') 
  FName=deblank(NameLst(1,:));
  P=vp_nm('dnm',FName);
  vp_hread([P '.hdr']);
  name=[P '.img'];
  ImageSize=DIM(1)*DIM(2)*DIM(3);
  [out1 axyzdata]=vp_volfuns('xyzdata',name);
  if out1 < 0, vp_message('ERROR: reading Image 1'); return; end;
  
  BName=deblank(NameLst(2,:));
  P=vp_nm('dnm',BName);
  name=[P '.img'];
  [out1 bxyzdata]=vp_volfuns('xyzdata',name);
  if out1 < 0, vp_message('ERROR: reading Image 2'); return; end;
 
  prec=round(in4(1,1)),
  [out1 SDATA]=vp_volfuns('excludata',axyzdata,bxyzdata,prec);
  if out1 < 0, vp_message('ERROR: in exclude operation'); return; end;
  
else
   h=waitbar(0,['Processing Image ' imop  ' ...']);
   for i=1:m
  	waitbar((i-1)/m,h);
  	FName=deblank(NameLst(i,:));
  	P=vp_nm('dnm',FName);
	vp_hread([P '.hdr']);
	name=[P '.img'];
	ImageSize=DIM(1)*DIM(2)*DIM(3);


	%%fid=fopen(name,'r');
	DATA=[];
	RDATA=[];
	%%DATA	= fread(fid,MATYPE);
  	[out1, DATA]=vp_volfuns('volvect',NameLst(i,:));
	
	%Mask
	if ~isempty(maskfile)
    	  if ~isequal(size(maskdata), size(DATA))
	      vp_note(['Error: Image Data and Mask sizes differ ']);
	      out1=-1; return;  
	  else
	      DATA=DATA.*maskdata; 
	  end
	end
	
	%Globally Normalize
	if (gnorm | vinnorm) > 0
	 if (gnorm==1) % Global mean normalization
	    disp('global mean normalization ');
	    % Don't count 0's in mean
	    nonzpos=find(DATA); nonzsum=size(nonzpos,1),
	    if (nonzsum == 0)
	      vp_note(['Error: Data has no non-zero elements ']);
	      oug1=-1; return;
	    end
	    datavect=reshape(DATA,1,prod(DIM));
	    datamean=sum(datavect)/nonzsum,
	    if datamean ~= 0
	      DATA=DATA/datamean;
	    else
	      vp_note(['Global Normalization Error: Data Mean is Zero ']);
	      out1=-1; return;
	    end
	 end
	 if (vinnorm==1), % Vector input normalization 
	    datavect=reshape(DATA,1,prod(DIM));
	 	    % divided by sqrt of sum of elements squared.
	    disp('vector normalization ');
	    datanorm=norm(datavect),
	    if datanorm ~= 0
	        DATA=DATA/datanorm;
	    else
	        vp_note(['Vector Normalization Error: Vector Norm is Zero ']);
	        oug1=-1; return;
	    end
	 end
	  
	 % normalization will remove the DATA SCALE factor
	 SCALE=sfactor(1,i); % set to external input scale factor
	  
	else
	  %SCALE=SCALE*sfactor(1,i);
	  %Note, vp_volfuns pre-scales data, even though SCALE is not pre-set to 1;

	  SCALE=sfactor(1,i);  
	end;
	
	RDATA=(double(DATA))*SCALE;
	SCALE,
	  
	maxin=max(RDATA),
	minin=min(RDATA),
	  
	if i==1
	   SDATA=RDATA;
	else
	  if strcmp(imop,'SUB') 
	     SDATA=SDATA-RDATA;
	  elseif strcmp(imop,'DIV') 
	      n=ImageSize;
	      for p=1:n,
		   if RDATA(p) == 0, SDATA(p)=0;
		   else SDATA(p)=SDATA(p)/RDATA(p);
		   end;
	      end
	  elseif strcmp(imop,'MLT') 
	      n=ImageSize;
	      for p=1:n,
		   SDATA(p)=SDATA(p).*RDATA(p);
	      end
	  elseif strcmp(imop,'EXS') 
	      n=ImageSize;
	      for p=1:n,
		if SDATA(p).*RDATA(p) > 0, SDATA(p) = 0; end;
	      end
	  else
	  	SDATA=SDATA+RDATA;
	  end
	end
	%%fclose(fid);
   end %for

   waitbar(1,h); pause(1); close(h);
   
end; %if


if m > 1
  if strcmp(imop,'AVG') 
    SDATA=SDATA/m;
    DESCRIP=['ScAnVP Averaged Image'];
    i=i+1; types(i)=16;
  elseif strcmp(imop,'ADD') 
    DESCRIP=['ScAnVP ADDed Image'];
    i=i+1; types(i)=16;
  elseif strcmp(imop,'SUB') 
    DESCRIP=['ScAnVP SUBtracted Image'];
    i=i+1; types(i)=16;
  elseif strcmp(imop,'DIV') 
    DESCRIP=['ScAnVP DIVided Image'];
    i=i+1; types(i)=16;
  elseif strcmp(imop,'MLT') 
    DESCRIP=['ScAnVP Multiplied Image'];
    i=i+1; types(i)=16;
  elseif strcmp(imop,'EXC') 
    DESCRIP=['ScAnVP EXCluded data Image'];
  elseif strcmp(imop,'EXS') 
    DESCRIP=['ScAnVP EXCluded Same Sign data'];
  end
else
    DESCRIP=['ScAnVP Processed Image'];
end

%Mask output image
if ~isempty(maskfile)
      SDATA=SDATA.*maskdata; 
end


if (zscout==1 ), % Z score 
    % subtract mean and divide by std of non-zero values
    disp('Z-score');
    datavect=reshape(SDATA,1,prod(DIM));

    %if std(datavect) ~= 0
    %  SDATA= (SDATA - mean(datavect))/std(datavect);
    %else
    %  vp_note(['Vector Z-score Error: STD = 0 ']);
    %  return;
    %end

    Dmask=SDATA ~= 0; % Get the intrisic Non-zero mask
    [no, mn, st]=vp_math('statNo0',datavect);
    if st ~= 0
      SDATA= (SDATA - mn)/st;
      SDATA=SDATA.*Dmask; 
    else
      vp_note(['Vector Z-score Error: STD = 0 ']);
      out1=-1; return;
    end

elseif (voutnorm==1 ), % Output Normalization 
    datavect=reshape(SDATA,1,prod(DIM));
    % divided by sqrt of sum of elements squared.
    disp('vector output normalization ');
    datanorm=norm(datavect),
    if datanorm ~= 0
    SDATA=SDATA/datanorm;
    else
	vp_note(['Vector Normalization Error: Vector Norm is Zero ']);
	out1=-1; return;
    end
end

if in3 ~=2, out2=SDATA; end;
if in3==1, return, end;

%WRITE IMAGE
% Use Maximum Type

TYPE=max(types);
if (gnorm|vinnorm|voutnorm|zscout), if(TYPE < 16),TYPE=16; end; end;


FName=deblank(NameLst(1,:));
savdir=pwd;
nam=vp_nm('nm',FName);
P=[savdir,'\',nam];

if autonaming ~=2
 if strcmp(imop,'EXC'),
   OutImg=[P 'X.img'];
 else
   if gnorm
     OutImg=[P ,'g',imop '.img'];
   else
     OutImg=[P imop '.img'];
   end
 end
end

glmaxdat=max(max(max(SDATA))),
glmindat=min(min(min(SDATA))),

OutImg,
fidop=fopen(OutImg,'w');
fidop,

if glmaxdat == 0,
  vp_note(['WARNING: 0 maximum']);
  waitfor(findobj('Name','Note'));
end

if (TYPE==2 & glmindat < 0) ,
  	TYPE=16;
end;

if (abs(glmindat) > glmaxdat),
	glmaxdat=abs(glmindat);
end;


if TYPE==2
	datype='uchar';
	glmax=255;
elseif TYPE==4
	datype='int16';
	glmax=32767;
elseif TYPE==8
	datype='int32';
	glmax=(2^31-1);
elseif TYPE==16
	datype='float';
	glmax=1;
elseif TYPE==64
	datype='double';
	glmax=1;
	%SDATA=double(SDATA);
else
	  vp_note(['ERROR: Invalid TYPE' num2str(TYPE)]);
	  out1=-1;
	  return;
end
	
DATA=[];
if ( (TYPE < 16) & (glmaxdat > glmax))
    DATA=SDATA*glmax/glmaxdat;
    SCALE=glmaxdat/glmax;
else
  DATA=SDATA;
  SCALE=1.0;
end

no=fwrite(fidop,DATA,datype);

if no <=0,
  	vp_note(['Write ERROR no: ' num2str(no) ' pixels']) ;
	out1=-1;
	return;
else
 vp_note(['Image: ',OutImg,' SAVED'], 3);
end
	
fclose(fidop);

if(autonaming)
	if (vp_hwrite(OutImg,DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,...
	  EXPTIM,DURATION,GENTIM,SUBID,EXPDATE) ) < 0,
	  vp_message(['hwrite err for ',OutImg]); out1=-1; 
	else
	  if (in3==2 & autonaming) out2=OutImg; end;
	end;
	return;
end %if autonaming


Bcolor = [.8,1,1];

progui = figure('Color',Bcolor,'Name',imop,'Tag','progui',...
'NumberTitle','off','Position',[100,200,420,60],...
'Units','Pixels');
	

cbFName=['staterr=0; hrname=findobj(''tag'',''namefield''); OutImg=get(hrname,''userdata''),',...
	'OutImg=deblank(OutImg); namehdr0=[vp_nm(''dnm'',OutImg) ''.hdr''];',...
	'if (vp_hwrite(OutImg,DIM,VOX,SCALE,TYPE,OFFSET,ORIGIN,DESCRIP,',...
	'EXPTIM,DURATION,GENTIM,SUBID,EXPDATE) ) < 0,',...
	'vp_message(''hwrite err''); staterr=1; ',...
	'else, ',...
	  'name=get(hrname,''string''),',...
	  'name=deblank(name); namehdr=[vp_nm(''dnm'',name) ''.hdr''];',...
	  'name=[vp_nm(''dnm'',name) ''.img''];',...
	  'if ~strcmp(name,OutImg),',...
	     'if (exist(name,''file'') & ~exist(namehdr,''file'')) | ',...
	       ' (~exist(name,''file'') & exist(namehdr,''file''))  ',...
	       'vp_message([''Remove Invalid File '' name '' .hdr or .img'']);',...
	       ' staterr=3; ',...
	     'else, [s,dosmsg]=dos([''move '' OutImg '' '' name]), ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	       '[s,dosmsg]=dos([''move '' namehdr0 '' '' namehdr]), ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	     'end; ',...
	     'if staterr==0,  OutImg=name;  end;',...
	  'end; ',...
	'end;',...
	'if staterr==0,',...
	  'set(gco,''string'',[OutImg ''   FILE SAVED'']); Message=[''FILE SAVED'']; ',...
	  'disp(Message); pause(2); delete(findobj(''tag'',''progui'')); ',...
	' end; '];
	
       
uicontrol(progui,'style','text','string','ENTER NAME: ',...
	'position',[10 25 125 25],'BackgroundColor',Bcolor);
	
hrname=uicontrol(progui,'style','edit','tag','namefield','string',OutImg,...
	'userdata',OutImg,...
	'position',[10 5 400 25],'BackgroundColor','w','callback',cbFName);



waitfor(findobj('tag','progui'));
