function [gisout, gismatrix, label, ori] = vp_readgis(display,gisfile)
% Determine type of gis and read regional weights.
%
% Reads a user selected formatted or unformatted gis file
% of SSM ROI based analysis output containing GIS regional 
% weights and returns type of GIS file and regional values.
% If the name of the file is not input, the routine prompts
% for user input.
% Unformatted gis files are text files of regional weights
% for one or more GISs listed as a matrix with a separate
% column for each GIS. Formatted GIS files are output by
% SSM ROI PCA analysis routines. The first line specifies the
% number of principal components (GISs) and regions in each GIS. 
% The fourth line lists the 'Variance Accounted For' the GIS.
% The sixth line lists the 'Scan Mid Time (min)'.
% The region name, orientation and weights for all components
% are listed in sequential rows beginning with the seventh line.
% Region names may be listed only as 'region no' with no orientation.
% Formatted files containing weights for a single GIS may contain
% the specification NRois: # in the first line followed by
% sequential lines with orientation, specific region names and
% weight values.
% 
% Input:
% display: 0=No display, 1=bar graph
% gisfile: Prompts if not provided
%
% Output:
% gisout= GIS file type
%  gisout=1; %formatted
%  gisout=2; %unformatted
%  gisout=3; %semiformatted type [region, weight] 
%  gisout=4; %formatted single gis
%  gisout=5; %formatted single gis NRois: type
%  gisout=-1; %error
%
% gismatrix=GIS matrix, label = roi id labels, ori= roi orientation C,R,L
%
% Also sets the globals GisWeight GisOri and GisLab for a selected single gis
%________________________________________________________________________
% function [gisout,gismatrix,label,ori]=vp_readgis(display,gisfile)  ScAnVP 5.3 P. Spetsieris 3/22/04
%
%

global HelpMode Keyin
global GisOri GisLab GisWeight

if HelpMode, vp_help(mfilename); return ; end;

IdentGIS=0;
gisout=-1;
gismatrix=[];
gistype=['Unformatted'];
ori=[]; label=[]; 
if nargin <= 0
  display=0;
end
if nargin <= 1
  gisfile=get_file(1,'*txt','Select ROI Based SSM GIS File'),
end

vp_dispfile(gisfile);

%Determine type of file
    fid=fopen(gisfile);
    [gistype,count]=fscanf(fid,'%s',1),
    if count==0 
        vp_note('Empty File');
	return;
    end
    if ~strcmp(gistype,'SSM'), 
      if strcmp(gistype,'NRois:'), 
        [nrs,count]=fscanf(fid,'%s',1),
        gisout=5; %formatted single gis
        gistype=['Formatted1'];
	fclose(fid);
        vp_note(['Detected Formatted Single GIS File Type NRois: ' nrs],2);
      else
        numstr=str2num(gistype);
        if ~isempty(numstr)
          gisout=2; %unformatted
          gistype=['Unformatted'];
	  fclose(fid);
          vp_note('Detected Unformatted File',2);
        else
	  if strcmp(gistype,'region')
            [str,count]=fscanf(fid,'%s',1);
	    if count==0, vp_note('EOF encountered'); return; end;
	    numstr=str2num(str); %region no.
	    if ~isempty(numstr)
              gistype=['SemiFormatted '];
              gisout=3; %semiformatted type [region, weight] 
              vp_note('Detected [Region Weight] Semi-Formatted File Type',2);
	    else
              gistype=['Unknown'];
              vp_note('Err: 77 Unknown or Outdated File Type');
	      return;
	    end
	  else
            gistype=['Unknown'];
            vp_note('Err: 82 Unknown or Outdated File Type');
	    return;
	  end
        end
      end
    else % SSM formatted 
      [gistype,count]=fscanf(fid,'%s',1),
      if strcmp(gistype,'ROI'), 
        gisout=1; %formatted
        vp_note('Detected SSM ROI GIS Formatted File',2);
      else
        gisout=-1;
        vp_note([gistype,': Not a Standard GIS Formatted File']);
	return;
      end;
    end


if gisout==3  % Semi-Formatted [Region Weight] Type
 num_gis=1; ngis=1; noroi=1;
 [str,count]=fscanf(fid,'%s',1);
 if count==0, vp_note('EOF encountered'); return; end;
 numstr=str2num(str);
 if ~isempty(numstr) gismatrix(noroi,ngis)=numstr;
 else vp_note('Err: 106 Unknown or Outdated File Type'); return; end;
 while 1
    [str,count]=fscanf(fid,'%s',1);
    if ~strcmp(gistype,'region')
	vp_note('Err: 110 Unknown or Outdated File Type'); return;
    end;
    if count==0, %EOF 
      break;
    else
      [str,count]=fscanf(fid,'%s',1); % dump region no.
      if count==0, 
         vp_note(['Abrupt EOF. Incomplete DATA']);
    	 return; 
      else
         [str,count]=fscanf(fid,'%s',1);
         if count==0, vp_note('EOF encountered'); return; end;
         numstr=str2num(str);
         if ~isempty(numstr)
           noroi=noroi+1;
 	   gismatrix(noroi,ngis)=numstr;
         else
            vp_note(['Invalid String: ',str,' Detected']); return; 
	    break;
         end
      end;
    end;
 end %while
 if noroi==1, vp_note(['vp_readgis Error: Single Region Weight: ',num2str(gismatrix(1,1))]);
 else
   if display >=1
     % Display bar graph
     gisfig=figure('name',['SSM WEIGHTS for GIS Network ' gisfile]); bar(gismatrix(:,1));
     xlabel('Region '); ylabel(['SSM WEIGHTS  -  GIS Network']);
   end
 end
 return;
elseif gisout==2  % Unformatted
 try
  gismatrix=load(gisfile);
 catch
  vp_message('INVALID FILE TYPE ');
  return;
 end
 sz=size(gismatrix);
 norois=sz(1), num_gis=sz(2), 
 
elseif gisout==5  % Formatted NRois type
  [a, Message]=vp_gisfun('readgis',gisfile);
  if a, vp_dispfile(gisfile), 
    gismatrix=GisWeight'; ngis=1; num_gis=1; 
    ori=GisOri; label=GisLab;
  else vp_message('ERR: Invalid file type 5');
    return;
  end;
  
else % Formatted

  a=''; count=1;
  while (~strcmp(a,'Components') & count==1)
    if (strcmp(a,'Component') & count==1)
      gisout=4;
      [ngis,count]=fscanf(fid,'%d',1);
      num_gis = 1;
      break;
    else
     [a,count]=fscanf(fid,'%s',1);
    end
  end;
  
  if gisout==1
  
    [a,count]=fscanf(fid,'%c',3);
    if count ~=3, vp_note('vp_readgis Error 158: Invalid File'); return; end;
  
    [num_gis,count]=fscanf(fid,'%d',1);
    if count ~=1, vp_note('vp_readgis Error 161: Invalid File'); return; end;
  end
  
  while (~strcmp(a,'Regions') & count==1)
    [a,count]=fscanf(fid,'%s',1);
  end;
  [a,count]=fscanf(fid,'%c',3);
  if count ~=3, vp_note('vp_readgis Error 167: Invalid File'); return; end;
  
  [norois,count]=fscanf(fid,'%d',1)
  if count ~=1, vp_note('vp_readgis Error 170: Invalid File'); return; end;
  
  if gisout==1
    [dump,count]=fscanf(fid,'%s',(2+2*num_gis))
    if count ~=(2+2*num_gis), vp_note('vp_readgis Error 200: Invalid File'); return; end;
  else
    [dump,count]=fscanf(fid,'%s',4)
    if count ~=4, vp_note('vp_readgis Error 203: Invalid File'); return; end;
  end
  for noroi=1:norois,
    labname=[]; ori(noroi,:)='C';
	
    [name,count]=fscanf(fid,'%s',1);
    if count ~=1, vp_note('vp_readgis Error 176: Invalid File'); return; end;
    
    if strcmp(name,'region')
      [dump,count]=fscanf(fid,'%s',1); % dump region no.
      labname=['region ',str2num(dump)];
      for j=1:num_gis, [gismatrix(noroi,j),count]=fscanf(fid,'%f',1);
        if count ~=1, vp_note('vp_readgis Error 181: Invalid File'); return; end;
      end;
    else % other name
      labname=name; numstr=[];
      while isempty(numstr), % dump name and orientation
        [str,count]=fscanf(fid,'%s',1),
        if count==0, vp_note('EOF encountered'); return; end;
        numstr=str2num(str);
        if ~isempty(numstr) gismatrix(noroi,1)=numstr;
      
	  if gisout==1
            for j=2:num_gis, [gismatrix(noroi,j),count]=fscanf(fid,'%f',1);
               if count ~=1, vp_note('vp_readgis Error 192: Invalid File'); return; end;
            end;
	  end
	else 
	  % get name and orientation
	   if (strcmp(str,'R') | strcmp(str,'L') | strcmp(str,'C'))
			ori(noroi,:)=str;  
	   else labname=[labname ' ' str], end;
	   
        end % name and orientation
      end % while 
    end; % other name
    len=length(labname),
    if len > 1
     while isspace(labname(1)), labname= [labname(2:len) blanks(1)]; end;
    end
    labname=[labname blanks(40-len)];
    if (noroi==1), label(noroi,:)=labname; else label = strvcat(label,labname); end;
  end; % for noroi
  ori=char(ori), label,
end; %formatted
  
GisWeight=gismatrix(:,1); GisLab=label; GisOri=ori;

%  Select Gis #
if display >=1
 if num_gis==1
    gisfig=figure('name',['ROI Weights GIS Network ' gisfile]); bar(gismatrix(:,1));
    xlabel('ROI No.'); ylabel(['ROI WEIGHT  -  GIS Network']);
 else
   ngis=0;
   Keyin=num2str(ngis);
   while ~(isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),

     Keyin=num2str(ngis+1);
     prompt=['GIS NO.? [1-',num2str(num_gis),'| q]'];
     vp_keyin(prompt,Keyin);

     if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
     else
       ngis=str2num(Keyin);
       if (ngis > num_gis | ngis < 1), vp_note('vp_readgis Error: Out of Range');
       ngis=num_gis-1;
       else
         % Display bar graph
         gisfig=figure('name',['ROI Weights GIS ',num2str(ngis) ' Network ' gisfile]); bar(gismatrix(:,ngis));
         xlabel('ROI No.'); ylabel(['ROI WEIGHT  -  GIS ',num2str(ngis),' Network']);
	 GisWeight=gismatrix(:,1);
	 Message=['Selected Gis No: ',num2str(ngis)];
	 disp(Message);
       end
     end 
  
   end
 end 
end


