% vp_roi.m
% Region of interest analysis
%___________________________________________________________________________
% ROI is the region of interest definition (TEMPLATE creation) routine.
% Users can create a multiplanar template of irregular, elliptical,
% and rectangular regions saved as separtate files for each plane of
% interest. Regions can be drawn on a PET image on a coregistered
% MRI image. Various options are provided to manipulate the regions,
% delete or change their size and orientation, label with standard and
% non-standard names, display statistics with and without thresholding,
% import other templates, and display selected (thresholded) pixels.
% Results are saved for each plane in (Name.Pln.roi) files, which can be
% modified at any time, using the same program. These files contain
% statistical data as well as geometric information which can be used
% to recreate the regions for group replay (see vp_grouproi).
% Images should be displayed in Neurological Orientation (LR) so that
% the anatomical right side is displayed on the right side of the display
% and avoid confusion in prospective use or in the group roi replay routine.
% To insure LR display, the user can check the "Input File Orientation"
% (An RL input file is flipped to LR for display).
% The GROUP ROI routine uses the templates to generate 2D and 3D reports.
% The 'fitting' option allows the import of templates created with other
% coregistered images even if they do not have the same dimensions. The
% program looks for the coordinates of the physical origin which must be
% properly set in the ANALYZE header and is recorded in the .roi file. The
% 'flipping' option allows older '*.roi' templates that were processed in
% the 'matrix' coordinate system to be processed by flipping Y coordinates
% to correspond to the Cartesian convention. Statistical data is ignored
% when .roi files are used as templates but the original threshold value
% is applied to the new data unless reset for each input region or using
% group roi threshold optiions. 
% (see also vp_groi.m and vp_grouproi.m )
%__________________________________________________________________________
% vp_roi.m	C.0 	P. Spetsieris	Nov. 12, 1998
%      Rev. 7.0	    P. Spetsieris	Jan. 05, 2016

global PlaneNo FI
global FName roigui hroierr Ecolor GMIN GMAX
global NonStdLabs NonStdNo hlabmenu2 NonStdLabStr
global ThreshMode FixedThMin
global InputOrient DisplayOrient FMenu
global HelpMode

if HelpMode, vp_help('vp_roi'); return ; end;

roigui=findobj('Name','ROIgui');
if ~isempty(roigui)
	if(strcmp(get(roigui,'Visible'),'on'))
		figure(roigui);
	end
	return
end

groigui=findobj('Name','gROIgui');
if ~isempty(groigui)
	if(strcmp(get(groigui,'Visible'),'on'))
		figure(groigui);
	end
	return
end

NonStdLabs=str2mat('Non Standard Label                      ');
NonStdLabStr=['Non Standard Label                      '];
NonStdNo=101;

vp_imframe;

FName=[];
WndNo=0;
WndScl=4;
PlaneNo=1;
Thresh=20;
ThreshMode=1;
FixedThMin=-Inf;
Message0=['............................................ '];
Message=Message0;

roigui = figure('Color',[0.5 0.7 0.8],'Name','ROIgui',...
'NumberTitle','off','Position',[100,200,250,140],...
'Units','Pixels');
	
Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];

pcbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented disk input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented disk input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

uicontrol(roigui,'string', 'Check RL/LR Input File Orientation',...
	'TooltipString',['Check: Radiological (Right is Left) or Neurological (Right is Right)',...
    'Disk Input File Orientation.'],...
	'position',[10 115 185 20],...
	'BackgroundColor',Ecolor,'callback',pcbImgOrient);

uicontrol(roigui,'string', 'Help',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'position',[200 115 40 20],'callback','vp_help(''vp_roi'')');
	
cbPName=['FName=get_file(1,''.img'',''please select image'',[]); ',...
	'if ~isempty(FName),',...
	'vp_fimage;',...
	'set(hrname,''string'',FName); ',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'[GMAX GMIN]=vp_roifuns(''gminmax''),',... 
	'set(hroierr,''string'',[''MSG:'',Message]); end;'];
	
uicontrol(roigui,'string', 'PICK IMG',...
	'TooltipString','Select Analyze Format Image',...
	'position',[10 85 60 25],'BackgroundColor',Bcolor,...
	'callback',cbPName);
	
cbFName=['FName=get(hrname,''string'');','Message=Message0 ;',...
	'vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'[GMAX GMIN]=vp_roifuns(''gminmax''),',... 
	'set(hroierr,''string'',[''MSG:'',Message]);'];
	
hrname=uicontrol(roigui,'style','edit','string',FName,...
	'TooltipString','Key in Analyze Format Image Name',...
	'position',[72 85 168 25],'BackgroundColor',Kcolor,'callback',cbFName);

%uicontrol(roigui,'string', 'Threshold 0-100 %',...
%	'position',[10 110 160 25],'BackgroundColor',Bcolor);

%hthresh=uicontrol(roigui,'style','edit','string',Thresh,...
%	'position',[180 110 60 25],'BackgroundColor',Kcolor,'callback',cbThresh);


pcbplane=['Message=Message0; PlaneNo=PlaneNo+1; vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hroierr,''string'',[''MSG:'',Message]);'];

uicontrol(roigui,'string', 'NEXT PLANE>>',...
	'TooltipString','Skip to Next Plane',...
	'position',[10 60 160 25],'BackgroundColor',Bcolor,'callback',pcbplane);


cbplane=['PlaneNo=str2num(get(hplane,''string'')); Message=Message0;',...
	'vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hroierr,''string'',[''MSG:'',Message]);'];

hplane=uicontrol(roigui,'style','edit','string',PlaneNo,...
	'TooltipString','Key in Plane No.',...
	'position',[180 60 60 25],'BackgroundColor',Kcolor,'callback',cbplane);


uicontrol(roigui,'String','DRAW/GET ROIS',...
	'TooltipString','Go to Roi Template Definition Options.',...
	'Position',[10 35 100 25],...
	'CallBack',['vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1);',...
	'set(hplane,''string'',PlaneNo); ',...
	'vp_groi, set(roigui,''Visible'',''off'');',...
	'else set(hroierr,''string'',[''MSG: '',Message]);',...
	'if PlaneNo==DIM(3), vp_groi, set(roigui,''Visible'',''off'');, end;',...
	' end; '] );
%	'CallBack',['set(roigui,''UserData'',1)']);

uicontrol(roigui,'String','QUIT',...
	'TooltipString','EXIT routine.',...
	'Position',[180 35 60 25],...
	'CallBack',['delete(gcbf);','delete(FI);']);
%	'CallBack',['set(roigui,''UserData'',1); ', 'delete(roigui)']);

hroierr = uicontrol(roigui,'string', ['MSG : ',Message],...
	'position',[10 10 230 25],...
	'CallBack',['set(hroierr,''string'',[''MSG: '',Message]);'],...
	'BackgroundColor',Ecolor);

