function [pval]=vp_scatternplot(scores,subn, ntrials,paired,netnam,labelx,labely)
% function [pval]=vp_scatternplot(scores,subn, ntrials,paired,labelx,labely,netnam)
% Plots a scatter plot of repeated (trial or independent group) measure scores.
%_______________________________________________________________________
% This function plots SSF or other group scores for repeated 
% measures (paired=1) or independent groups (paired=0) starting 
% with an initial set for baseline or reference labelled trial 1 and
% followed by an optional number of measurement trials for the same or 
% different groups. Scores for each group are shown in comparison with
% the initial trial and a composite is presented for all trials.
% Connecting lines are optional for same group repeated measures.
% The number of subjects in each set is input as well as a X axis and
% Y axis label. Defaults are provided. The mean and std of the initial 
% and subsequent trial values are plotted alongside as errorbars.
% The plot legend specifies the mean and std for each subject set. 
% P-values result from paired value t-tests of each trial pair for
% paired=1 and result from student t-tests for different groups. They  
% are displayed for each trial with respect to the initial condition
% in the figure legend and are printed for each condition and previous 
% conditions in the command window. 
%
% scores: Vector including all scores
% subn:  Number of subjects in each set (constant for paired) 
% ntrials:   Number of trials (score sets)
% paired: 1=repeat same group, 0=independent groups
% netnam: Title, e.g. Network Name
% labelx, labely: Axis Labels
%
% Defaults: subn = number of scores. ntrials=scores/subn(1), paired=0
%
% pval: Returns the matrix pij of p values between sets i,j of trial scores
%_______________________________________________________________________
% vp_scatternplot.m   ScAnVP ver. 6.2    P. Spetsieris   Mar 18, 2012
% 	 based in part on vp_scatterplot.m   P. Spetsieris   Nov. 22, 2006
% 	 Rev.6.3    P. Spetsieris   Dec. 13, 2013

global P2 POpt

pval=[];

if nargin  < 1
 vp_note('Error: Must Input Score File ',3);
 return;
end

if nargin  < 2
  sub =length(scores);
  subn=sub;
else
  sub = subn(1);
end

if sub <=1,
    vp_note('Error: Number of subjects per trial must be greater than 1',3);
    return; 
end;

if nargin  < 3
  ntrials = round(length(scores)/sub) ;
end

if nargin  < 4
  paired=0; % default independent group input
end

if paired
 if sub*ntrials ~= length(scores)
   vp_note(['ERROR: No. of Scores: ',num2str(length(scores)), ' not equal to Subjects*Trials: ',num2str(sub*ntrials)],3);
   return; 
 end;
else
 if length(scores) ~= sum(subn)
  vp_note(['WARNING No. of scores: ', num2str(length(scores)),...
  ' Not equal to total subjects: ', num2str(sum(subn))],3);
 end
end


if nargin  < 5
  netnam=['Scores for ', num2str(ntrials),' trials']; % Unspecified Network 
else
  if isempty(netnam)
    netnam=['Scores for ', num2str(ntrials),' trials']; % Unspecified Network 
  end;
end

if nargin  < 6
  labelx='   TRIAL 1'; % X Coordinate  
  for i=2:ntrials
   labelx= [labelx, '             TRIAL ',num2str(i)];
  end
end

if nargin  < 7
  labely='SCORE'; % Y Coordinate 
end

norms=sub; % Number of baseline subjects.

% nosubs: Total number of subjects in first and second condition
nosubs=sub;

if ntrials > 1
 if paired
  nosubs=2*sub;
 else
  nosubs=subn(1) + subn(2);
 end
end

% create scatter plot
%xpos=ones(length(scores) + 3, 1);
xpos=ones(length(scores), 1);
ebar=zeros(size(scores));
pval=ones(ntrials,ntrials);


if nosubs > norms
  for i=norms+1:nosubs, xpos(i)=3; end;
end

%if ntrials > 2
%   %for i=nosubs+1:nosubs+sub, xpos(i)=5; end;
%   for i=nosubs+1:sub*ntrials, xpos(i)=3 + 2*(1+ floor((i-nosubs)/sub)); end;
%end

if ntrials > 2
  xposi=xpos(i); 
  for j=3:ntrials

   if paired, subnj=sub
   else subnj=subn(j); end;

   for k=1:subnj
      xpos(i+k)= xposi+2;
   end

   i=i+subnj;
   xposi=xposi+2;

  end
end

xpos, % Debug

%append to end
i=i+1;
i1=i(1,1); % In case i is an array.
xpos(i)=1.5;
scores(i)=mean(scores(1:norms));
ebar(i)=std(scores(1:norms));

if nosubs > sub
i=i+1;
i2=i(1,1); %In case i is an array.
xpos(i)=3.5;
scores(i)=mean(scores(norms+1:nosubs));
ebar(i)=std(scores(norms+1:nosubs));

  if paired
  % Need not test for equal Variance
    ev2='';
    [H2,P2,CI2,STATS2]=ttest(scores(1:norms),scores(norms+1:nosubs), .05),
    disp('paired ttest');
  else
    % Independent Groups
    % test equal variances
    [Hf2,Pf2]= ftest_levene(scores(1:norms), scores(norms+1:nosubs), .05), 
    if Hf2 == 0, % equal variances
      ev2='(ev)'; disp('Equal Variance, ttest2:');
      [H2,P2,CI2,STATS2]=ttest2(scores(1:norms),scores(norms+1:nosubs), .05),
    else
      ev2='(uev)'; disp('Unequal Variance, tutest2:');
      [H2,P2,CI2,STATS2]=tutest2(scores(1:norms),scores(norms+1:nosubs), .05),
    end
  end 
  pval(1,2)=P2; pval(2,1)=P2;
end %if nosubs

nsubs=nosubs;
% pvalues with respect to baseline (trial 1)

if ntrials > 2
 i3=i(1,1)+1; %In case i is an array.

 for j=3:ntrials
  i=i+1;
  xpos(i)= xpos(i-1) + 2;

  if paired, subnj=sub
  else subnj=subn(j); end;

  scores(i)=mean(scores(nsubs+1:nsubs+subnj));
  ebar(i)=std(scores(nsubs+1:nsubs+subnj));
  if paired
    ev3='';
    [H3,Pj,CI3,STATS3]=ttest(scores(1:norms),scores(nsubs+1:nsubs+subnj), .05),
    disp('paired ttest');
  else
    % Independent Groups
    % test equal variances
    [Hf3,Pf3]= ftest_levene(scores(1:norms), scores(nsubs+1:nsubs+subnj), .05), 
    if Hf3 == 0, % equal variances
      ev3='(ev)'; disp('Equal Variance, ttest2:');
      [H3,Pj,CI3,STATS3]=ttest2(scores(1:norms),scores(nsubs+1:nsubs+subnj), .05),
    else
      ev3='(uev)'; disp('Unequal Variance, tutest2:');
      [H3,Pj,CI3,STATS3]=tutest2(scores(1:norms),scores(nsubs+1:nsubs+subnj), .05),
    end
  end
  pval(1,j)=Pj; pval(j,1)=Pj;
  nsubs=nsubs+subnj;
 end %for j
end
hfigscat=figure;
%scatter(xpos',scores,'filled');
%errorbar(xpos(:,1),scores,ebar,'bo');

errorbar(xpos(:,1),scores,ebar,'ko');

%set(gca,'XLim',[0,6.5]);
set(gca,'XTickLabel',[]);

%Normal Style
set(gca,'FontSize',16);
set(gca,'FontWeight','Bold');
ch=get(gca,'Children');
set(ch,'MarkerSize', 8);
set(ch,'LineWidth',2);

%Demo Style
%set(gca,'FontSize',24);
%set(gca,'FontWeight','Bold');
%ch=get(gca,'Children');
%set(ch,'MarkerSize', 14);
%set(ch,'LineWidth',4);

xlabel(labelx);
ylabel(labely);
legstr=[''];

%DEBUG
%disp('DEBUG');
%i1,i2,i3,
%i1=i1(1,1),i2=i2(1,1), i3=i3(1,1),
%disp('END DEBUG');

if ntrials > 2
 
 i=i3;
 for k=3:ntrials
    legstr=[legstr, '   C',num2str(k),':[',num2str(round(100*scores(i))/100),', ',...
           num2str(round(100*ebar(i))/100),', ',num2str(round(10000*pval(1,k))/10000),']'];
    i=i+1;
 end
end

if ntrials > 1
  i=i2;
  %
  legstr=['   C2:[',num2str(round(100*scores(i))/100),', ',num2str(round(100*ebar(i))/100),', ',...
         num2str(round(10000*P2)/10000),']',legstr];
end

i=i1;
legstr=['B:[',num2str(round(100*scores(i))/100),', ',num2str(round(100*ebar(i))/100),']',legstr];

legstr=['(Mean,STD,p):   ',legstr];

hlg=legend(legstr,'location','NorthOutside');

set(gcf,'name',netnam);

lnfig=gcf;

%xpos, % Debug

if ntrials > 2
  %display p value matrix for each set of conditions 
  for j=2:ntrials-1
   for k=3:ntrials
    if k>j
     j,k
     if paired
       %[H23,P23,CI23,STATS23]=ttest(scores(norms+1:norms+sub),scores(nosubs+1:nosubs+sub), .05),
       [H23,P23,CI23,STATS23]=ttest(scores((j-1)*sub+1:j*sub),scores((k-1)*sub+1:k*sub), .05),
     else
       % Independent Groups
       % test equal variances
       ij=sum(subn(1:j-1)), subnj=subn(j),  ik= sum(subn(1:k-1)), subnk=subn(k),
       %[Hf23,Pf23]= ftest_levene(scores((j-1)*sub+1:j*sub), scores((k-1)*sub+1:k*sub), .05), 
       [Hf23,Pf23]= ftest_levene(scores(ij+1:ij+subnj), scores(ik+1:ik+subnk), .05), 
       if Hf23 == 0, % equal variances
         ev23='(ev)'; disp('Equal Variance, ttest2:');
         %[H23,P23,CI23,STATS23]=ttest2(scores((j-1)*sub+1:j*sub),scores((k-1)*sub+1:k*sub), .05),
         [H23,P23,CI23,STATS23]=ttest2(scores(ij+1:ij+subnj),scores(ik+1:ik+subnk), .05),
       else
         ev23='(uev)'; disp('Unequal Variance, tutest2:');
         %[H23,P23,CI23,STATS23]=tutest2(scores((j-1)*sub+1:j*sub),scores((k-1)*sub+1:k*sub), .05),
         [H23,P23,CI23,STATS23]=tutest2(scores(ij+1:ij+subnj),scores(ik+1:ik+subnk), .05),
       end
     end
     pval(j,k)=P23; pval(k,j)=P23;
    end
   end
  end
  %disp('paired ttest p12, p13, p23');
  %P12=P2, P13=P3, P23
  if(paired)
    disp('Paired Trials ttest p Matrix pij values for trial i by j ');
  else
    disp('Independent Trials ttest2 p Matrix pij for trials i by j');
  end
  pval,
  
 if paired
   for si=1:sub(1,1) %In case sub is now multidimensional 
    %Display Connecting Lines (State to Next State)
    prompt=['Display Composite Connecting Lines? [Y|N|SKIP]'];
    POpt=vp_opt('X',prompt,'YES','Display Lines Connecting Consecutive States','NO','Skip One','SKIP','');
    if strcmp(POpt,'YES'), 
      figure(lnfig);
      for j=1:ntrials-1
        %disp(['subject ',num2str(si),' segment ',num2str(j)]);
        x1=xpos(si+(j-1)*sub(1,1));
        y1=scores(si+(j-1)*sub(1,1));
        x2=xpos(si+j*sub(1,1));
        y2=scores(si+(j)*sub(1,1));
        line([x1, x2],[y1, y2],'LineStyle',':','LineWidth',2,'Color',[0 0 0]);
        %Demo line([x1, x2],[y1, y2],'LineStyle',':','LineWidth',4,'Color',[0 0 0]);  
      end %for j
    elseif strcmp(POpt,'SKIP'), 
      break;
    end; %if strcmp
   end; % for i
 end; % if paired
elseif ntrials==2 
 if paired,
   for si=1:norms(1,1) % in case norms is multidimentsional
    %Display Connecting Lines (State 1 and 2 )
    prompt=['Display Connecting Lines? [Y|N|SKIP]'];
    POpt=vp_opt('X',prompt,'YES','Display Lines Connecting States','NO','Skip One','SKIP','');
    if strcmp(POpt,'YES'), 
      figure(lnfig);
      line([xpos(si),xpos(si+norms(1,1))],[scores(si), scores(si+norms(1,1))],'LineStyle',':','LineWidth',2,'Color',[0 0 0]);
      %Demo line([xpos(si),xpos(si+norms(1,1))],[scores(si), scores(si+norms(1,1))],'LineStyle',':','LineWidth',4,'Color',[0 0 0]);
    elseif strcmp(POpt,'SKIP'), 
     break;
    end; %if strcmp
   end; %for si
 end %if paired
 %?end

end %if ntrials


