% vp_ssm_ROC.m
% ROC Charasteristic of SSM Score File
%
% This routine reads the score values of an SSM formated or unformatted score
% file (subjects vs. scores) and outputs several plots including the
% bar and scatter plot of scores and Z-scores, and ROC AUC and accuracy
% vs score cutoff plots. The sensitivities vs specificity values are
% plotted and printed as a text file.
% The user can specify a value for the specificity and the sensitivity is
% interpolated from adjacent plot values.
% For score files for which the mean control score is higher than the mean
% non-control score, the user is given the option to evaluate the parameters of
% the negative of score values and display accuracy as function of the negative cutoff.
% 
% ScAnVP 6.2  P.Spetsieris, Ph.D.     10/22/2012
%		Rev. 6.2.1 by P.S. Apr. 27, 2013
%		Rev. 6.2.2 by P.S. Dec. 2, 2013
%_______________________________________________________________________

global Keyin

if HelpMode, vp_help(mfilename), return ; end;


[mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2]=vp_ssm_scores;
if mode < 0, return; end;

netnam=vp_nm('nm',ssffile);

if isempty(Zssfs), SSFs=ssfscores, else SSFs=Zssfs; end;

ZRadius=0;
[nrsubs,k]=size(SSFs);
nnorms=norm2-norm1+1; nabnorms=nrsubs-nnorms;

if (nnorms <= 1), vp_note('SCORE FILE MUST CONTAIN CONTROLS',3); return; end;

meanctrl=mean(SSFs(norm1:norm2,ngis)),
abnSSFs=SSFs;
abnSSFs(norm1:norm2,:)=[];
meanabn=mean(abnSSFs),

NegValues=0;
if(meanabn < meanctrl)
    vp_note('NOTE: MEAN CONTROL SCORE GREATER THAN MEAN NON-CONTROL SCORE', 3);
    keyin=vp_keyin('Routine will Evaluate Score Negatives? y=OK| n=no| q=quit: ', 'y');

    if ~isempty(keyin),
     if strcmp(keyin,'q'), return; 
     else, 
         if or(strcmp(keyin,'y'), strcmp(keyin,'yes'))
            NegValues=1;
         end
     end;
    else
     NegValues=1;
    end;
end

if (NegValues==1)
     vp_note(['Computing AUC of Score Negatives '], 3);
     SSFs=0-SSFs;
else
     vp_note(['Computing AUC Characteristics of Score Values'], 2);
end

npts=60;

  minz=min(SSFs(:,ngis));
  maxz=max(SSFs(:,ngis));
  dz=(maxz-minz)/npts ; % Plot increment (Z range/npts)
  %ratio controls diagnosed, ratio diseased diagnosed,
  rdiagc=[]; rdiagd=[]; rfalsep=[]; rtruep=[]; 
  %total diagnosed, ratio total diagnosed
  tdiag=[]; rtdiag=[];
  CLR=[];
  diagnratio=[];
  tdmax=0;
  zrad=minz-dz; zradi=[]; 
  pLR=[]; % positive Likelihood ratio
  LLR=[]; % Log Likelighood ratio
  ComLR=[]; % Total Likelihood ratio
  rWZ=[]; % reference weighted Z value
  
  for i=1:(npts+3)
    zradi(i)=zrad;
    ndiagc=0; ndiagd=0;
    for k=1:nrsubs % for each subject
      if ( (k>=norm1) & (k<=norm2) ) % if subject in normal set 
       % and if diagnosed as normal increment correct normal diagnosis
       if SSFs(k,ngis) <= zrad, ndiagc=ndiagc+1; end; 
      else %if abnormal and diagnosed correctly increment correct abnormals
       if SSFs(k,ngis) > zrad, ndiagd=ndiagd+1; end;
      end
    end
    % record correct diagnostic rations for ith value of Zrad
    rdiagc(i)=ndiagc/nnorms; rdiagd(i)=ndiagd/nabnorms;
    tdiag(i)=ndiagc + ndiagd;
    rtdiag(i)=tdiag(i)/nrsubs;
    rWZ(i)=1-rdiagd(i);
    % evaluate a 95% corrected total diagnostic Likelihood Ratio
    CLR(i)=(rtdiag(i) -.05)/((nrsubs-tdiag(i))/nrsubs + .05);
    if (ndiagd <=0), 
      if (ndiagc <=0), diagnratio(i)=0; 
      %replace infinite value with a linear extension of the lower value
      else, diagnratio(i)=2*diagnratio(i-1) - diagnratio(i-2); end;
    else diagnratio(i)=rdiagc(i)/rdiagd(i); end;
    if diagnratio(i) <= 1, ilow=i; end; 
    if tdiag(i) > tdmax , tdmax=tdiag(i); imax=i; nmaxc=ndiagc; nmaxd=ndiagd; end;
    if tdiag(i) == tdmax , imax2=i; nmaxc2=ndiagc; nmaxd2=ndiagd; end;
    if rdiagd(i) > .95, ig95=i; Zg95=zrad; ng95c=ndiagc; ng95d=ndiagd; end;
    %if diagnratio(i) > 2, break; end;
    zrad=zrad+dz; % Value of the Zradius variable 
  end
 
  % find exact value of ZRad for balanced diagnosis, or interpolate 
  if diagnratio(ilow)== 1, ZRadius=zradi(ilow), perdiag= round(rdiagc(ilow)*100),
   perdiagc=perdiag; perdiagd=perdiag;
  else ZRadius=  zradi(ilow) + ...
   (1-diagnratio(ilow))*(zradi(ilow+1)-zradi(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ;
   %ZRadius=(round(ZRadius*10000))/10000;
   perdiagc = rdiagc(ilow) + ...
    (1-diagnratio(ilow))*(rdiagc(ilow+1)-rdiagc(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ,
   perdiagd = rdiagd(ilow) + ...
    (1-diagnratio(ilow))*(rdiagd(ilow+1)-rdiagd(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ,
   perdiag= round((perdiagc + perdiagd)*100/2) ,
  end
  
  ZRadius=(round(ZRadius*100))/100;
  
  ZTot=zradi(imax), nmaxc, nmaxd,
  ZTot2=zradi(imax2), nmaxc2, nmaxd2,
  %ZTot=(round(ZTot*10000))/10000;
  %ZTot2=(round(ZTot2*10000))/10000;
  ZTot=(round(ZTot*100))/100; senstot=rdiagd(imax); senstot=(round(senstot*100))/100;
        spectot=rdiagc(imax); spectot=(round(spectot*100))/100;
  ZTot2=(round(ZTot2*100))/100;  senstot2=rdiagd(imax2); senstot2=(round(senstot2*100))/100;
        spectot2=rdiagc(imax2); spectot2=(round(spectot2*100))/100;

  cutspec=' CUTOFF' ; if NegValues, cutspec='NEG of CUTOFF'; end;
  fig1=figure('name',['DIAGN ACCURACY For ',netnam]);
    %plot(zradi,rtdiag,'k-',zradi,rdiagc,'ok-',zradi,rdiagd,'^k-','LineWidth',1.45,'MarkerSize',5);
    plot(zradi,rtdiag,'k-',zradi,rdiagc,'ok-',zradi,rdiagd,'^k-','LineWidth',1.45,'MarkerSize',5);
    set(gca,'YLim',[0,1.3]);
    xlabel([cutspec,': Rt(sens,spec)=[', num2str(ZTot),'(',num2str(senstot),',',num2str(spectot),') to ',...
       num2str(ZTot2),'(',num2str(senstot2),',',num2str(spectot2),')],  (Rb=',num2str(ZRadius),...
    ', sens=spec=',num2str(perdiag/100),')'],'FontWeight','bold');
    ylabel('ACCURACY','FontWeight','bold');
    line([ZRadius,ZRadius],[0,perdiag/100],'LineStyle',':');
    line([ZTot,ZTot],[0,max(rtdiag)],'LineStyle','-.');
    line([ZTot2,ZTot2],[0,max(rtdiag)],'LineStyle','-.');
  %hlg=legend('TOTAL REF DIAGNOSTIC RATIO (CONTROLS+PATIENTS)','REF DIAGNOSTIC RATIO CONTROLS (TRUE NEGATIVES)','REF DIAGNOTIC RATIO PATIENTS (TRUE POSITIVES)');
  hlg=legend('TOTAL ACCURACY (CONTROLS+PATIENTS)','SPECIFICITY CONTROLS (TRUE NEGATIVES)','SENSITIVITY PATIENTS (TRUE POSITIVES)');
  set(hlg,'FontWeight','bold');
  set(fig1,'color',[ 1 1 1]);
  fig1b=figure('name',['TOTAL TRUE DIAGN ACCURACY For ',netnam]);
    nply=8;
    %pt1=12; pt2=32;
    pt1=imax -10; if pt1< 1, pt1= 1; end;
    pt2= imax2+ 10; if pt2 > npts, pt2= npts; end;
    if pt2 < 1, pt2=10; end;
    plot(zradi,rtdiag,'ok-','LineWidth',1.45,'MarkerSize',2);
    [ply1,s1]=polyfit(zradi(pt1:pt2),rtdiag(pt1:pt2),nply);
    h1=refcurve(ply1);
    [y1,delta1]=polyval(ply1,zradi(pt1:pt2),s1),
    [maxy1,indx1]=max(y1),
    z1=zradi(pt1-1+min(indx1)),
    set(gca,'XLim',[zradi(pt1),zradi(pt2)],'YLim',[.5,1.2]);
    xlabel([' SCORE-CUTOFF    ']); if NegValues, xlabel(['NEG of SCORE-CUTOFF    ']); end;
    ylabel([' TOTAL ACCURACY ']);
    hlg=legend(['Total ACCURACY, ',...
    'Max= ',num2str(round(rtdiag(imax)*100)/100),' at ',num2str(ZTot)],...
    [num2str(nply),' degree polynomial fit, Max=',num2str(round(maxy1*100)/100),' at ', num2str(round(z1*100)/100)]);
    
  %fig2=figure;
  fig2=figure('name',['ROC CURVE For ',netnam]);
  pos1=get(fig1,'position');
  pos1b=get(fig1b,'position');
  pos2=get(fig2,'position');
  pos2(1)=pos1(1) + pos1(3);
  pos1b(1)=(pos1(1) + pos1(3)/2);
  set(fig2,'position',pos2,'color',[1 1 1]);
  set(fig1b,'position',pos1b,'color',[1 1 1]);
  
  % plot ROC curve
  mxi=length(rdiagc);
  for i =1:mxi, pLR(i)=0; ComLR(i)=0; end;
  for i =1:mxi
   rfalsep(i)=1-rdiagc(mxi+1-i);
   rtruep(i)=rdiagd(mxi+1-i);
   if (rfalsep(i) > 0) pLR(mxi+1-i)=rtruep(i)/rfalsep(i); 
   elseif (rtruep(i) > 0) pLR(mxi+1-i)=0; end;
   LLR(mxi+1-i)=log(rtruep(i)) - log(rfalsep(i)); 
   %if ( (rfalsep(i)+1-rtruep(i)) ~= 0)
    
    n= 1.0 + (rtruep(i) - rfalsep(i)) ;
    d= 1.0 - (rtruep(i) - rfalsep(i)) ;
    ComLR(mxi+1-i)=n/d;
    %ComLR(mxi+1-i)=(rtruep(i)+1.0-rfalsep(i))/(rfalsep(i)+1.0-rtruep(i));
   %end;
  end
  %plot(rfalsep,rtruep,'ok-','LineWidth',4,'MarkerSize',5);
  plot(rfalsep,rtruep,'ok-','LineWidth',3,'MarkerSize',5);
  xlabel(['False Positive Ratio (1-specificity) '],'FontWeight','bold');
  ylabel(['True Positive Ratio (Sensitivity)'],'FontWeight','bold');

  %Add fitted curve
    %pt1=1; pt2=mxi;
    %[ply1,s1]=polyfit(rfalsep(pt1:pt2),rtruep(pt1:pt2),nply);
    %h1=refcurve(ply1);
    %[psensx,delta1]=polyval(ply1,0.05,s1),

  %set(gca,'YLim',[0,1.3]);
  set(gca,'YLim',[0,1.2]);
  set(gca,'XLim',[-0.01,1.01]);

  % Enhance for publication
  set(gca,'LineWidth',2)
  set(gca,'FontWeight','Bold')
  %set(gca,'FontSize',20)
  set(gca,'FontSize',18)

  %Calculate area under the curve
  acurve=0; napts=0;
  for i=1:mxi
   if (rfalsep(i) > 0) & (rfalsep(i) <= 1)
     acurve=acurve + (rtruep(i) + rtruep(i-1))*(rfalsep(i)-rfalsep(i-1))/2.0;
     napts= napts + 1;
   end
  end
  napts,
  acurve,
  acurve=(round(acurve*100))/100;
  atrapz=trapz(rfalsep,rtruep),
  ROCtitle=['ROC curve,    AUC =',num2str(acurve)];
  legend(ROCtitle);
  nm=vp_nm('nm',netnam);
  filenm=['ROCfor',nm,'.txt'];
  %vp_write_XYfile(filenm,ROCtitle,'1-specificity','sensitivity',rfalsep,rtruep);

  mxi=length(rdiagc); rzradi=zradi;
  for i=1:mxi
    rzradi(i)=zradi(mxi+1-i);
  end

  if ~NegValues,
   vp_write_XYfile(filenm,ROCtitle,'sensitivity','specificity      Z-CUTOFF',rtruep,1-rfalsep, rzradi);
  else
   vp_write_XYfile(filenm,ROCtitle,'sensitivity','specificity     NEG of Z-CUTOFF',rtruep,1-rfalsep, rzradi);
  end
  vp_note('ROC data text file saved',3);  
  
  %Determine interpolated index for specificity = 0.95 and corresponding sensitivity
  specx=0.95;


  while specx >=0

    keyin=vp_keyin('Enter Specificity > 0 | q: ', num2str(specx));
    if ~isempty(keyin),
     if strcmp(keyin,'q'), specx=-1;
     else, specx=str2num(keyin); 
         if or(specx<0, specx>1)
            vp_note(['Reset Invalid specificity ',num2str(specx)], 3);
            specx=0.95;
         end
     end;
    else
     specx=-1;
    end;

    if specx >= 0,

      xfalsep=1-specx; sensx=0; 
   
      for i=1:mxi
       if (rfalsep(i) >= xfalsep), 
           i2 = i, sensx=rtruep(i2);
           if rfalsep(i) ~= xfalsep,
             i1=max(i2-1,1); sens1=rtruep(i1), sens2=rtruep(i2), 
             if (i1 ~= i2)
               fspec1=rfalsep(i1), fspec2=rfalsep(i2),
               sensx= sens1 + ((xfalsep-fspec1))*(sens2-sens1)/(fspec2-fspec1) ; 
             end
             i1, 
           end
           break;
       end
      end

      %Display sensitivity for selected specificity;
      specx, sensx,

      accuracy=['Sens =',num2str(sensx), ',   Spec =', num2str(specx)];
      vp_note(accuracy,3);

    end %if specx

  end; %while

  %legend(['ROC curve,    AUC =',num2str(acurve),...
  %       ',  Sens =',num2str(sensx), ',   Spec =', num2str(specx)]);


