% vp_ssm_autoroirecal.m
%
% ROI based SSM automated TPR GROUP score RECALIBRATION procedure.
% This procedure is used on a pre-defined GIS to produce group reference
% scores for a new group of data derived from scanning methods or sources 
% that may differ in some details from the original used to derive the
% GIS. It saves the new reference group's SSM subject scores and mean
% files in a scanvp\networks reference directory for future automated
% TPR evaluation of similarly scanned new individual subjects.
%_____________________________________________________________________________________
%
% The user  (1) Selects a new GROUP SUMMARY COMPOSITE roi report (name_SUM(3D)_c.txt)
%               generated by GROUP-ROI processing of a new group of Talairach normalized
%               subject image files. Ideally if controls are entered, they should be
%               age matched and in equal numbers to patients which should span the full
%               range in disease severity.
%           (2) Selects a textfile containing the ROI GIS profile definition values.
%               (The GIS is defined in terms of its relative, Z-scored weights for 
%               the (nr) regions used in the PCA analysis.) The file may reside in a 
%		scanvp\networks\ subdirectory or eleswhere.
%
% The following steps summarize the evaluation method.
%           (A) Calculates new SRP by performing subject mean and group mean normalization.
%           (B) Calculates Scores for the pre-defined GIS, using the new SRP.
%           (C) Saves reference score and mean files, in a new scanvp network directory.
%
% Natural log transformation of the roi data is performed in the routine.
% The non-log option is only availabe in the interactive mode.
% Logs of negative data values are set to zero.
% Data normalization to derive the new subject residiual profile (SRP) is
% performed as a two step procedure. First the mean value over all regions
% is evaluated for each subject and used to generate within subject regional
% deviations from the mean. Second the subject's difference from the group mean
% deviations profile is evaluated as described below to generate the SRP. 
% The new SSF subject scores are evaluated using the following calculation:
% 
% 	          SSF=SRP*GIS/fact, fact=norm(GIS)*sqrt(nregions-1),
%
% where SSF is a subject vector, SRP is a matrix (nsubjects x nregions) and GIS is
% a regional vector. An output file containing the new SSF scores for each new
% subject is generated and plotted. Z scores are evaluated and offset by the mean
% Z score of any specified control subjects. Since new data is incorporated in the
% calculation of the group mean and SRP a shift may result in score values
% for any subjects that were also in the original analysis. The reference group
% can be carefully selected to avoid a significant Z-score shift. The resulting
% scores constitute a recalibration of the range of scores for the new, complete 
% group of subjects that include specified controls.
% The new reference scores and the group log mean text file are saved
% in a user-named scanvp/network reference directory for subsequent access
% by the automated reference based TPR option.
%
% ScAnVP	version 5.5	P. Spetsieris,Ph.D.    7/14/05  
% 		revision 5.6	P. Spetsieris,Ph.D.    11/18/05  
%		revision 5.7.w	P. Spetsieris,Ph.D.    11/8/06  
%		revision 5.8	P. Spetsieris,Ph.D.    6/01/07  
%		revision 6.2	P. Spetsieris,Ph.D.    4/18/12  
%_______________________________________________________________________

global HelpMode NameLst FNames 
global DIM roi SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE GENTIM SUBID EXPDATE

global POpt FMenu Keyin NameLst


if HelpMode, vp_help(mfilename), return ; end;

REFscores=0; readref_mean=0; imls=[];

POpt='GROUP RECALIBRATION',
prompt='AUTO TPR GROUP RECALIBRATION - SELECT or EXIT';
optA=['NEW GROUP recalibration, Input new group ROI SUMMARY file and pre-defined GIS, Calc. GIS Scores. '];
POpt=vp_opt('X',prompt,'GROUP RECALIBRATION',optA,'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_autoroirecal'); 
  vp_ssm_autoroirecal;
  return; end;
if(strcmp(POpt,'GROUP RECALIBRATION')),
  vp_note(optA,3);
  REFscores=0; readref_mean=0;
else
  vp_note(['Invalid Option',POpt],3);
  return;
end

im_data=[]; devs=[]; negzero_set=[];
log_transform=1;  
prompt=['Specify Composite SUMMARY NEW GROUP ROI file'];

if vp_filesui(1,'*_c*.txt',prompt,mfilename) < 0 ,return; end;
roimatrix_name=NameLst(1,:);

length_name = length(roimatrix_name);
k = findstr(roimatrix_name,'\');
pwdd = roimatrix_name(1:max(k));
p=max([findstr(roimatrix_name,'SUM') 0]);
if p==0
 name = roimatrix_name(1:length_name-4);
else
 if roimatrix_name(p-1)=='_'
   name = roimatrix_name(1:p-2);
 else
   name = roimatrix_name(1:p-1);
 end
end

lmean_name = strcat(name, '_tprlmean.txt');
mean_name = strcat(name, '_tprmean.txt');

[file_list,roi_list,roimatrix]=vp_read_grprpt([roimatrix_name]);
if isempty(roimatrix)
  set(FMenu,'Pointer','Arrow');
  vp_message(['Error: Could Not Read New Report: ',roimatrix_name]); return;
end
roimatrix=roimatrix'; 
voxno=size(roimatrix,1);
sub=size(roimatrix,2);

if ( sub <= 1)
  set(FMenu,'Pointer','Arrow');
  vp_message(['ERROR Subjects=',num2str(sub),...
  ' Enter GROUP file to RECALIBRATE scores or exit and select REFERENCE option']);
  return;
end
  
if ( sub <= 5)

   prompt=['WARNING Limited Subjects = ',num2str(sub),', For RECALIBRATION Continue or Exit? [Y|N|EXIT]'];
	    POpt=vp_opt('X',prompt,'YES','Continue','NO','Exit - use REFERENCE Option','EXIT','');
	    if strcmp(POpt,'YES'), 
	    else
  		set(FMenu,'Pointer','Arrow');
        	REFCALformat=0;
  		return;
	    end;
end


POpt='STANDARD ROI GIS Directory',
prompt=['AUTO ROI RECALIBRATION: STANDARD or OTHER REFERENCE GIS DIRECTORY?'];
optA=['Specify scanvp\networks Directory GIS.'];
optB=['Specify Other Directory Reference GIS.'];
POpt=vp_opt('X',prompt,'STANDARD ROI GIS',optA,'OTHER REFERENCE ROI GIS',optB, 'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_autoroirecal'); 
  vp_ssm_autoroirecal;
  return; end;
if(strcmp(POpt,'STANDARD ROI GIS')),
  vp_note(optA,3);
  w=which('scanvp');
  netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
elseif(strcmp(POpt,'OTHER REFERENCE ROI GIS')),
  netdir=[];
  vp_note(optB,3);
else
  vp_note(['Invalid Option ',POpt],3);
  return;
end

%w=which('scanvp');
%netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
	
prompt=['Select pre-defined GIS vector text file for TPR RECALIBRATION'];
disp(prompt);

gis_name=get_file(1,'*gis*.txt',prompt,netdir,[ ]);
if isempty(gis_name), return; end;

[gisout gismatrix]=vp_readgis(0,gis_name);
if (gisout < 0), 
  vp_message('Error reading GIS'); return; end;

num_gis = size(gismatrix,2); % PS
if num_gis > 1
  vp_message('Error: File not a valid single ROI GIS text file'); 
  return; 
end;
ngis=1;

gis=gismatrix(:,ngis);

num_roi = size(gis,1); % PS

if voxno ~= num_roi,
  set(FMenu,'Pointer','Arrow');
  vp_message('Error: Number of GIS regions do not agree with new data'); return;
end

[gisdir,gisdnm,gisdir,gisnm]=vp_nm('dir',gis_name),


%ln=length(gisnm);
%ngisch=gisnm(ln);
%ngisno=str2num(ngisch);
%if isempty(ngisno), ngisno=1;
% gisroot=gisnm;
%else
%   ngisch=gisnm(ln-1);
%   ngisn1=str2num(ngisch);
%   gisroot=gisnm(1:(ln-1));
%   if ~isempty(ngisn1)
%    ngisno=ngisno+10*ngisn1; % allow at most 2 digits
%    gisroot=gisnm(1:(ln-2));
%   end
%end

[ngisno, gisroot]=vp_num3str(gisnm);
ngisch=num2str(ngisno);

k0=findstr(gisroot,'gis');
if ~isempty(k0),
    k0=k0-1;
    if (gisroot(k0)== '_')k0=k0-1; end;
    gisroot=gisroot(1:k0);
    kfound=k0;
    for (i=1:k0), k=k0-i+1; 
     if gisroot(k) ~= '_', j=str2num(gisroot(k));
       if isempty(j), kfound=k, break; end; 
     end;
     if kfound ~= k0, break; end;
    end;
    gisroot=gisroot(1:kfound),
end

readref_mean=0; %  read reference mean option

log_transform=1; %  log transformation option

if log_transform
  roimatrix(find(roimatrix<=0))=exp(0); % PS Set all values <=0
  roimatrix=log(roimatrix); % PS perform log transformation
end;

hwaitbar=waitbar(0,'Processing ROI TPR ...');
waitbar(.2,hwaitbar);

%Compute within image mean vector
meanvect=mean(roimatrix);

waitbar(.3,hwaitbar);

%Compute within-subject deviations from mean of image
%i.e., normalize to whole brain
meanmat=ones(voxno,sub);
for x=1:sub
   meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
end
devs=roimatrix-meanmat;

waitbar(.4,hwaitbar);

%Compute mean image of voxels across all new subjects
mean_refimg=mean(devs,2);

%PS Save Row Vector mean image for subsequent (AUTO)TPR analysis
mean_vec=mean_refimg';
if log_transform
    save(lmean_name,'mean_vec','-ascii');
else
    save(mean_name,'mean_vec','-ascii');
end

for k=1:sub, devs(:,k)=devs(:,k)-mean_refimg; end;
srp=devs';

waitbar(.5,hwaitbar);

%generate SSF values for a known GIS pattern based
%on SRP calculated in the new population.

num_roi = size(gis,1); % PS
if voxno ~= num_roi,
  set(FMenu,'Pointer','Arrow');
  vp_message('Error: Number of GIS regions do not agree'); return;
end

% calculate new ssf
ssf=srp*gis;

%PS normalize ssf values
ssf=ssf/(norm(gis)*sqrt(num_roi -1));

ssf_name = strcat(name, '_tprRScores',ngisch,'.txt');
Zssf_name = strcat(name, '_tprZScores',ngisch,'.txt');
% Save Gis Recalibrated Scores below

waitbar(1,hwaitbar); pause(1); close(hwaitbar);

disp ('   ***************************************************     ')
disp(' ');
disp('             RESULTS OF GIS2SSM TPR')
disp(' ');
disp('Newly Calculated SSF')
ssf
ssftprfig=figure('name',['TPR SCORES GIS ',num2str(ngis)]); bar(ssf); xlabel('New Subject'); ylabel('New Subject SSF SCORE');
pos=get(ssftprfig,'position');
if sub < 15
  pos(3)=max(pos(3)/3, pos(3)*(sub/15)); % use 15 as a standard number of subjects
end

set(ssftprfig,'position',pos);
    
% Z transformation 

 getscorefile=0; 
    
 refsub=sub;


% Specify Controls Option
prompt=['Specify Controls? '];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'SPECIFY CONTROLS','','NO CONTROLS','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_help('vp_ssm_autoroirecal'); 
   end
end
  
if (strcmp(POpt,'SPECIFY CONTROLS'))
 nnorms=1; %determine later
else
 % No Specific Controls
 nnorms=0; norm1=0; norms=0; Zoffset(1:nosubs)=0;
end


if nnorms > 0

 no=1;
 while no~= 2 % Must enter at least two numbers even if the same
   Keyin=['1 - ',num2str(refsub)];
   prompt=['Specify Control Subject Index Range Within (Ref.) subjects [n1-n2] | q]   <',Keyin,'>   '];
   vp_keyin(prompt,Keyin);

   if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	      return;
   else
      
    Keyin=strrep(Keyin,' ','');
    [nn,no]=sscanf(Keyin,'%d-%d'),
    norm1=nn(1)
    if no ==2  norm2=nn(2) 
        if (norm1 < 1 | norm1 > refsub), vp_note('Error: Number norm1 out of range',2);
          no=1;
        elseif (norm2 < 1 | norm2 > refsub), vp_note('Error: Number norm2 out of range',2);
          no=1;
        elseif (norm1 == 1 & norm2 == refsub), vp_note('Error: All subjects cannot be controls',2);
          no=1;
        end %if
    end; %if
   end %if 
 end % while
	
end

ssfscores=ssf;
[nosubs, num_gis]=size(ssfscores),
ngis=1;
	
%Normalize SSF Scores (Z-score for gis over ref subjects)
Zssfs=zeros(size(ssfscores));
Zssfs(:,ngis)=(ssfscores(:,ngis)-mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 

% Offset calculated for each gis

if norm2 > 0
 vp_note(['Control Subject Range: ',num2str(norm1),' - ',num2str(norm2)],2);
 if norm2 ~= norm1
      Zoffset=mean(Zssfs(norm1:norm2,:)),
 else
      Zoffset=Zssfs(norm1,:),
 end
 Zssfs(:,ngis)=Zssfs(:,ngis)-Zoffset(ngis)*ones(nosubs,1);
end
 
 Zssf=Zssfs;
    
 zscorefig=figure('name','TPR Z SCORES'); bar(Zssf); xlabel('New Subject'); ylabel('New Subject Z SCORE');
 vp_write_ssmfile(ssf_name,ngisno,-1,file_list,ssf,1,norm1,norm2)
 vp_write_ssmfile(Zssf_name,ngisno,-1,file_list,Zssf,3,norm1,norm2)
 vp_dispfile(Zssf_name);
    
 yo=max(Zssf)/4;
 figure(zscorefig);
 if norm2 > 0
   for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
 end
 set(gca,'color',[1.0 .9 .9]);
 
 
 REFCALformat=0;
 prompt=['Save In New Reference Network? [Y|N|EXIT]'];
 POpt=vp_opt('X',prompt,'YES','REF-Network','NO','Working Directory Only','EXIT','');
 if strcmp(POpt,'YES'), 
        REFCALformat=1;
 else
  % saved above
 end;

 if (REFCALformat==1)

   nnorms = norm2 - norm1 + 1;
   if nnorms > 0
     studyname = [gisroot,'_RE_',num2str(nnorms),'_',num2str(sub-nnorms)];
   else
     studyname = [gisroot,'_RE_',num2str(sub)];
   end
   vp_keyin('[Assign Output Files Study Name | q]',studyname);
   if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
   return, else studyname=Keyin; end;
    
   netprename=gisdir(1:length(gisdir)-1);
   netpredir=vp_nm('dir',netprename);
   pwdd = [netpredir,studyname];
  
   while (exist(pwdd) == 7)
     vp_keyin(['NAME-IN-USE, Assign Output Files Study Name | q'],studyname);
     if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
     return, else studyname=Keyin; end;
     pwdd = [netpredir,'\',studyname];
   end
  
   [ret,Message]= mkdir(netpredir,studyname),
   
   ssf_name = strcat(pwdd, ['\',studyname,'_RScores',num2str(ngisno),'.txt']);
   Zssf_name = strcat(pwdd, ['\',studyname,'_ZScores',num2str(ngisno),'.txt']);
   
   % Save Gis Reference Scores
   vp_write_ssmfile(ssf_name,ngisno,-1,file_list,ssf,1,norm1,norm2)
   vp_write_ssmfile(Zssf_name,ngisno,-1,file_list,Zssf,3,norm1,norm2)

   % Save Gis Network
   gisfilnm=[pwdd,'\',studyname '_gis',num2str(ngisno),'.txt'],  
   vp_write_ssmfile(gisfilnm,ngisno,-1,roi_list,gis(:,ngis),2)
	
  %Save mean reference vector

  %PS Save Row Vector mean image for subsequent TPR analysis
  %PS
  lmean_name=[pwdd,'\',studyname '_lmean.txt'], 
  mean_name=[pwdd,'\',studyname '_mean.txt'], 
  if log_transform
    save(lmean_name,'mean_vec','-ascii');
  else
    save(mean_name,'mean_vec','-ascii');
  end

  set(FMenu,'Pointer','Arrow');
  vp_message(['END CHECK: ',ssf_name,', ']);

end
