% vp_ssm_autoroitpr
% ROI based SSM Automatic Topographic Profile Rating (TPR)
% Performs evaluation of new subject(s) SSM subject scores for a specific GIS
% New subjects should be scanned with the same parameters as 
% the reference subjects.
%_______________________________________________________________________
%
% The user selects as input (1) a new GROUP SUMMARY COMPOSITE roi report 
% name_SUM(3D)_c.txt) generated by GROUP-ROI processing of one or more 
% new Talairach normalized subject image files and selects (2) a predefined
% network GIS for evaluation of new subject scores for the GIS associated
% disease or condition.
%
% The new subject report need not include subjects from the previous study
% since each subject is evaluated individually on a prospective basis.
% However, new subjects should be scanned with the same parameters as 
% the reference subjects.
% The GIS definition text file originates from an original PCA analysis
% and contains the relative weights for the regions included in the roi
% template file used in the analysis. These should be the same regions
% defined in the template used to generate the new subject SUMMARY report.
% The network GIS directory must contain besides 1) the GIS definition file,
% the following text files also generated by the original PCA analysis or
% a TPR recalibration procedure:
% 2) the log data group mean regional profile to be used as a reference from 
% which the new SRP values will be calculated and 3) the reference score file
% (...RScores not ...ZScores) indicating control subject indeces.
% These files are selected automatically by the routine once the name of the
% network gis is identified and naming conventions are followed as indicated.
%
% For example to test for the hypothetical network 1 derived with the study
% STUDY for the disease\condition COND, the scanvp directory under networks
% could be named COND and contain the following files named as indicated.
%
%	EXAMPLE
%	directory:	scanvp\networks\COND\ROI\STUDY
%	contents:
%			gis file:               STUDY_gis1.txt
%			log mean file:          STUDY_lmean.txt
%						 (or STUDY_SUM(3D)_c.txt)
%			Reference score file:   STUDY_RScores1.txt
%
% If a second network can be defined for the same condition it can be included in
% the same directory with a corresponding name, e.g., STUDY_gis2,
% STUDY_RScores2.txt. Network names can be renamed by the user other than 1, 2, etc.
% that are the indeces assigned by the PCA analysis routine.
%
% The following procedure is used to derive new subject scores. Natural log
% transformation of the NEW subject SUMMARY report data is performed by the 
% MATLAB routine. Logs of negative data values are set to zero. Optional
% non-log transformation is only available in the interactive version.
% Normalization is performed as a two step procedure. The mean value over
% all regions is evaluated for each subject and used to generate within 
% subject regional deviations from the mean. The group mean regional
% profile of the reference data (STUDY_lmean.txt) is then read and subtracted
% from the corresponding new subject values to generate the new subject residual
% profile (SRP).
% The new SSF subject scores are determined using the following calculation:
% 
% 	          SSF=SRP*GIS/fact, fact=norm(GIS)*sqrt(nr-1),
%
% where SSF is an nsx1 vector, SRP is a nsxnr matrix and GIS is a nrx1 vector.
%                 ns=number of new subjects, nr=number of regions.
% 
% Output files containing the new SSF scores and Z-scores for each new subject
% is generated and plotted. Results are saved in the input file directory.
%
% Recalibration: If the original 'lmean' and 'RScores' are not available they
%		 can be regenerated using a Reference SUMMARY COMPOSITE file,
%		 STUDY_SUM(3D)_c, derived using the same set of ROIs as in the gis.
%		 This file is derived from an equivalent group of normals and
%		 subjects indicative of the condition. The 'lmean' file is derived 
%		 using PCA and reference scores are gererated using the interactive
%		 TPR recalibration option. If the new reference group is not similar
%		 to that used in the original analysis or if different conditions
%		 or thresholds prevail, some shift in the score assignments is
%		 expected within limits.
%_______________________________________________________________________
% vp_ssm_autoroitpr      ScAnVP 5.3   P.Spetsieris   8/5/2004
%				5.4   P.S. 9/21/2004
%				5.6   P.S. 11/18/2005
%				5.8   P.S. 7/09/07
%_______________________________________________________________________

global POpt FMenu Keyin NameLst
global GisOri GisLab GisWeight

if HelpMode, vp_help(mfilename); return ; end;

vp_note('WARNING! New and Reference subjects should be similarly scanned',3);
prompt=['AUTOTPR: Specify Composite Summary NEW (Group) ROI file'];
if vp_filesui(1,'*_c*.txt',prompt,mfilename) < 0 ,return; end;
roimatrix_name=NameLst(1,:);

POpt='STANDARD ROI GIS Directory',
prompt=['AUTO ROI TPR: STANDARD or OTHER REFERENCE GIS DIRECTORY?'];
optA=['Specify scanvp\networks Directory GIS to Calculate TPR scores.'];
optB=['Specify Other Directory Reference GIS to Calculate TPR scores.'];
POpt=vp_opt('X',prompt,'STANDARD ROI GIS',optA,'OTHER REFERENCE ROI GIS',optB, 'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_autoroitpr'); 
  vp_ssm_autoroitpr;
  return; end;
if(strcmp(POpt,'STANDARD ROI GIS')),
  vp_note(optA,3);
  w=which('scanvp');
  netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
elseif(strcmp(POpt,'OTHER REFERENCE ROI GIS')),
  netdir=[];
  vp_note(optB,3);
else
  vp_note(['Invalid Option ',POpt],3);
  return;
end

%w=which('scanvp');
%netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
	
prompt=['AUTOTPR: Enter GIS file name'];
%if vp_filesui(1,'*gis*.txt',prompt,mfilename,netdir) < 0 ,return; end;
%gis_name=NameLst(1,:);

gis_name= get_file(1,'*gis*.txt',prompt,netdir,[ ]);  
[dnm,dnm,dir]=vp_nm('dnm',gis_name);
p=max([findstr(gis_name,'gis') 0]);
l=length(gis_name);
condnm=gis_name(1:p-1);
gisid=gis_name(p+3:l-4);

[gisout gismatrix]=vp_readgis(0,gis_name);
if (gisout < 0), 
  vp_message('Error reading GIS'); return; end;

%PS 
num_gis = size(gismatrix,2); % PS
if num_gis > 1
  prompt=['GIS NO.? [1-',num2str(num_gis),'| q]'];
  vp_keyin(prompt,'1');

  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  return, end;
  ngis=str2num(Keyin);
else
  ngis=1;
end

gis=gismatrix(:,ngis);

%PS num_roi = length(gis);
num_roi = size(gis,1); % PS

%reffile_name= get_file(1,'*.txt','REFERENCE (GROUP) ROI or MEAN file',netdir,[ ]);  
%reffile_name= get_file(1,'*lmean.txt','REFERENCE MEAN file',netdir,[ ]);  
%reffile_name= get_file(1,'*lmean*.txt','REFERENCE (GROUP) ROI or MEAN file',dir,[ ]);  

reffile_name=[condnm,'lmean.txt'],
if exist(reffile_name) ~=2
 reffile_name=[condnm,'lnmean.txt'],
 if exist(reffile_name) ~=2
  reffile_name=[condnm,'SUM_c.txt'];
  if exist(reffile_name) ~=2
    reffile_name=[condnm,'SUM3D_c.txt'];
    if exist(reffile_name) ~=2
      vp_message(['Error: Could Not find either ',condnm,'lmean, SUM_c or SUM3D_c.txt file']); return;
    end
  end
 end
end

if isempty(reffile_name), return; end;

length_name = length(roimatrix_name);
k = findstr(roimatrix_name,'\');
pwdd = roimatrix_name(1:max(k));
p=max([findstr(roimatrix_name,'SUM') 0]);
if p==0
 outnm = roimatrix_name(1:length_name-4);
else
 if roimatrix_name(p-1)=='_'
   outnm = roimatrix_name(1:p-2);
 else
   outnm = roimatrix_name(1:p-1);
 end
end
ssf_name = strcat(outnm, '_tprScores.txt');
Zssf_name = strcat(outnm, '_tprZScores.txt');


[file_list,roi_list,roimatrix]=vp_read_grprpt([roimatrix_name]);
if isempty(roimatrix)
  set(FMenu,'Pointer','Arrow');
  vp_message(['Error: Could Not Read New Report: ',roimatrix_name]); return;
end
roimatrix=roimatrix';

readref_mean=0; % PS add read reference mean option

%PS test if ref_file is a group report or log data mean.
fd=fopen(reffile_name);
line1=fgets(fd); line2=fgets(fd); fclose(fd);
if line2==-1 %EOF - Assume Group Mean File
 mean_refimg=load([reffile_name]);
 readref_mean=1;
 refvoxno=size(mean_refimg,2);
else readref_mean=0;
end

if readref_mean==0
[refile_list,ref_list,refmatrix]=vp_read_grprpt([reffile_name]);
if isempty(refmatrix)
  set(FMenu,'Pointer','Arrow');
  vp_message(['Error: Could Not Read Reference Report: ',reffile_name]); return;
end
refmatrix=refmatrix';
end

log_transform=1; % PS Force log transformation 

%prompt=['Log-transform the data? neg. values & logs will be set to zero [Y|N|EXIT]'];
%POpt=vp_opt('X',prompt,'YES','Perform Natural Log Transformation','NO','Pre Transformed Data','EXIT','');
%if (POpt=='X' | strcmp(POpt,'EXIT')), 
%	set(FMenu,'Pointer','Arrow');
%	vp_note('Program Exited',2); return;
%elseif strcmp(POpt,'NO'), log_transform=0,
%else log_transform=1;
%end;

if log_transform
  roimatrix(find(roimatrix<=0))=exp(0); % PS Set all values <=0
  roimatrix=log(roimatrix); % PS perform log transformation
  if readref_mean==0
    refmatrix(find(refmatrix<=0))=exp(0); % PS Set all values <=0
    refmatrix=log(refmatrix); % PS perform log transformation
    refvoxno=size(refmatrix,1);
  end
end;

voxno=size(roimatrix,1);
sub=size(roimatrix,2);
if voxno ~= num_roi,
  set(FMenu,'Pointer','Arrow');
  vp_message('Error: Number of GIS regions do not agree'); return;
end

if voxno ~= refvoxno,
  set(FMenu,'Pointer','Arrow');
  vp_message('Error: Number of new regions do not agree with ref'); return;
end

hwaitbar=waitbar(0,'Processing ROI TPR ...');
waitbar(.2,hwaitbar);

if readref_mean==0
    refsub=size(refmatrix,2);
    refmeanvect=mean(refmatrix);
    meanrefmat=ones(refvoxno,refsub);
    for x=1:refsub
       meanrefmat(:,x)=refmeanvect(1,x)*meanrefmat(:,x);
    end
    refdevs=refmatrix-meanrefmat;
    %Compute mean image of voxels across ref subjects
    mean_refimg=mean(refdevs');
end

%Compute within image mean vector
meanvect=mean(roimatrix);

waitbar(.3,hwaitbar);

%Compute within-subject deviations from mean of image
%i.e., normalize to whole brain
meanmat=ones(voxno,sub);
for x=1:sub
   meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
end
devs=roimatrix-meanmat;

waitbar(.4,hwaitbar);

%Compute new subject deviations from mean reference image
  mean_refimg=mean_refimg';
  mean_refimgmat=ones(voxno,sub);
  for x=1:voxno
     mean_refimgmat(x,:)=mean_refimg(x,1)*mean_refimgmat(x,:);
  end
  devs=devs-mean_refimgmat;

srp=devs';
waitbar(.5,hwaitbar);


%generate SSF values for a known GIS pattern based
%on SRP calculated in the new population.

% calculate new ssf
ssf=srp*gis;

%PS normalize ssf values
ssf=ssf/(norm(gis)*sqrt(voxno-1));

%write_srp_file(ssf_name,1,file_list,ssf)
vp_write_srpfile(ssf_name,1,file_list,ssf)

waitbar(1,hwaitbar); pause(1); close(hwaitbar);

disp ('   ***************************************************     ')
disp(' ');
disp('             RESULTS OF GIS2SSM TPR')
disp(' ');
disp('Newly Calculated SSF')
ssf
ssftprfig=figure('name','TPR SCORES'); bar(ssf); xlabel('New Subject'); ylabel('New Subject SSF SCORE');
pos=get(ssftprfig,'position');
%pos(3)=pos(3)*(sub/15); % use 15 as a standard number of subjects
if sub < 15
  pos(3)=max(pos(3)/3, pos(3)*(sub/15)); % use 15 as a standard number of subjects
end
set(ssftprfig,'position',pos);
    
% END GIS2SSF CODE

% Z transformation, read REF Score file.
%ssffile=[dnm,'RScores.txt'];
ssffile=[condnm,'RScores',gisid,'.txt'];
if ~exist(ssffile),
 vp_message(['Could not find REFerence Scores file: ',ssffile,' CANNOT CALCULATE Z Scores']); 
 return; 
end;
[mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2] =vp_ssm_scores(-2,ssffile)
    [nosubs, num_gis]=size(ssfscores),
    %if (norm1==0 | norm2==0)
    % set(FMenu,'Pointer','Arrow');
    % vp_message(['Invalid Reference File: ',ssffile,' Control Subjects Not Specified']);
    %return; end;

    %Normalize SSF Scores (Z-score for gis over ref subjects)
    Zssfs(:,ngis)=(ssfscores(:,ngis)-mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 

    % Offset calculated for each gis

    Zoffset(1:num_gis)=0;
    if norm2 > 0
     if norm2 ~= norm1
      Zoffset=mean(Zssfs(norm1:norm2,:)),
     else
      Zoffset=Zssfs(norm1,:),
     end
    end;
    
    %Zssfs=Zssfs-Zoffset;
    Zssfs(:,ngis)=Zssfs(:,ngis)-Zoffset(ngis)*ones(nosubs,1);

    % Use the same mean, std and offset as the reference to calculate new Z scores.
    
    %Zssf=zscore(ssf),
    %Zssf=(ssf - mean(ssf))./std(ssf),
    Zssf=(ssf - mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 
    Zssf=Zssf-Zoffset(ngis)*ones(sub,1);;
    
    zscorefig=figure('name','TPR Z SCORES'); bar(Zssf); xlabel('New Subject'); ylabel('New Subject Z SCORE');
    vp_write_srpfile(Zssf_name,1,file_list,Zssf)
    vp_dispfile(Zssf_name);
    
    pos=get(zscorefig,'position');
    pos(3)=max(pos(3)/3,pos(3)*(sub/nosubs));
    set(zscorefig,'position',pos);
   
    %Display Combined Scores (Group + New Subjects)
    ZCssf=[Zssfs(:,ngis); Zssf;];
    ZCscorefig=figure('name','TPR COMBINED Z SCORES'); bar(ZCssf);
    if norm2 > 0
      xlabel('Reference and New Subject(s) c: control, s: new subject'); ylabel('COMBINED GROUP Z SCORES');
      for i=norm1:norm2, text(i,1,'c','color','r'); end; %Mark controls
    else
      xlabel('Reference and New Subject(s), s: new subject'); ylabel('COMBINED GROUP Z SCORES');
    end
    for i=nosubs+1:nosubs+sub, text(i,1,'s','color','m'); end; %Mark new subjects
    set(gca,'color',[.9 1.0 .9]);
    
    pos=get(ZCscorefig,'position');
    pos(1)=pos(1)+2*pos(3)/3;
    set(ZCscorefig,'position',pos);


    % create scatter plot
    if norm2 > 0
      labelx=['CONTROLS                REFERENCE SUBJECTS            NEW SUBJECTS'];
    else
      labelx=['REFERENCE SUBJECTS            NEW SUBJECTS'];
    end
    labely=['REFERENCE AND NEW SUBJECT Z - SCORES'];
    p=max([findstr(dnm,'networks') 0]);
    l=length(dnm);
    netnam=dnm(p+9:l);
    vp_scatterplot(ZCssf,norm2,nosubs,sub,labelx,labely,netnam)


 % write diagnostic report
   DiagnOpt=0; %OMIT
   if (norm2 > 0 & DiagnOpt)
     p=max([findstr(dnm,'networks') 0]);
     l=length(dnm);
     netnam=dnm(p+9:l);

     DIAGNrpt_name = strcat(outnm, '_DIAGNrpt.txt');
 
     Keyin='0'; clear cycle;
     %Min Z for Total TRUE Max
     diagntype=4;
     imls=file_list;
     vp_ssm_prediagnrpt;
 
     %vp_DIAGNrpt(DIAGNrpt_name,netnam,file_list,Zssf,'ROI based Z Scores',Zssfs(:,ngis),norm1,norm2);
 
     vp_MDiagnRpt(DIAGNrpt_name,netnam,imls,Zssf,'VOXEL based Z Scores',Zssfs(:,ngis),norm1,norm2,nosubs,diagntype,ZRadius,1);
     vp_dispfile(DIAGNrpt_name);
 
     vp_message(['END CHECK: ',ssf_name,' and ',DIAGNrpt_name]);
   else

     vp_message(['END CHECK: ',ssf_name]);
   end; % diagn rpt Opt
   
set(FMenu,'Pointer','Arrow');



