% vp_ssm_autovoxrecal.m
%
% Voxel based SSM automated TPR GROUP score RECALIBRATION procedure.
% This procedure is used on a pre-defined GIS to produce group reference
% scores for a new group of data derived from scanning methods or sources 
% that may differ in some details from the original used to derive the
% GIS. It saves the new reference groups' SSM subject scores, mask and mean
% files in a reference directory for future automated TPR evaluation of
% similarly scanned new individual subjects. The new reference directory 
% will be under scanvp\networks, if the original reference files are there.
%_____________________________________________________________________________________
%
% The user  (1) Selects a data set of images to perform TPR evaluation of SSF scores 
%               for a predefined GIS. If controls are entered, the set should ideally
%               contain equal numbers of subjects and controls.
%           (2) Selects a GIS image file defining a disease specific profile.
%           (3) Uses GIS intrinsic mask or selects an additinal predefined mask
%
% The following steps summarize the evaluation method.
%           (A) Calculates SRP for new group of images using combined mask.
%           (B) Calculates Scores for pre-defined GIS, using new SRP.
%           (C) Saves reference score files, mean image and mask in a new
%               scanvp\network directory.
%
% 
% SRP is calculated for a new user seleceted reference group of patients (and controls).
% It is assumed that images have been smoothed and Talairach normalized before analysis.
% SRP is determined as in PCA analysis using the combined GIS and image intrinsic masks
% and optional user specified mask. The SRP, mask and reference mean images are saved.
% The resultant output mask generated is input into the subsequent recalibration
% steps. Scores are generated using the SRP and input GIS resulting in a recalibration
% of the range of scores for a new, complete group of subjects that may include 
% controls. The resulting reference scores, GIS and mean file can be saved in a new  
% reference directory for subsequent single or multi-subject AUTO-TPR.
% Since SRP is obtained from direct PCA analyis involving new data not in the original
% analysis (used in determing the GIS), the new data is incorporated in the
% calculation of the group mean and SRP which may result in a shift in score values
% previously calculated for subjects that were in the original analysis. 
%
% The following calculation is used to define the new scores for the pre-defined GIS.
% The GIS values are relative voxel weights that are Z-scored with respect to their
% mean value by the routine before output.
% 
% 	          SSF=SRP*GIS/norm(GIS)
%
% where SSF is a vector (nsubs), SRP is a matrix (nsubs,nvoxs) and GIS is a vector (nvoxs).
%
% An output file containing the new SSF scores for each new subjects is generated
% and plotted. Z scores are generated and offset by the mean Z score of the control
% subjects. Results are saved in a user-named scanvp/network reference directory 
% for subsequent access by the automated reference based TPR option.
%
% ScAnVP	version 5.5	P. Spetsieris,Ph.D.    5/12/05  
%		revision 5.6	P. Spetsieris,Ph.D.    11/21/05  
%		revision 5.7.w	P. Spetsieris,Ph.D.    11/08/06  
%		revision 5.8    P. Spetsieris,Ph.D.    6/01/2007
%		revision 6.2    P. Spetsieris,Ph.D.    3/12/2012
%		revision 6.2.2  P. Spetsieris,Ph.D.    8/15/2013
%_______________________________________________________________________

global HelpMode FMenu NameLst FNames POpt
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE GENTIM SUBID EXPDATE

if HelpMode, vp_help(mfilename), return ; end;

REFscores=0; readref_mean=0; imls=[];
norm1=0; norm2=0;

POpt='Standard GIS Network',
prompt='AUTO TPR GROUP RECALIBRATION - SELECT or EXIT';
optA=['NEW GROUP recalibration, Input pre-defined standard scanvp network GIS, Calculate new reference scores. '];
optB=['NEW GROUP recalibration, Input pre-defined GIS, Calc. GIS Score on new group images. '];
POpt=vp_opt('X',prompt,'Standard GIS Network',optA,'Other Reference GIS Network',optB, 'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_autovoxrecal'); 
  vp_ssm_autovoxrecal;
  return; end;
if(strcmp(POpt,'Standard GIS Network')),
  vp_note(optA,3);
  REFscores=0; readref_mean=0;
  w=which('scanvp');
  netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
elseif(strcmp(POpt,'Other Reference GIS Network')),
  netdir=[];
  vp_note(optB,3);
else
  vp_note(['Invalid Option',POpt],3);
  return;
end

prompt=['Select pre-defined GIS For TPR RECALIBRATION'];
disp(prompt);

gisfile=get_file(1,'*gis*.img',prompt,netdir,[ ]);
if isempty(gisfile), return; end;

maskname=[]; masking=0;
im_data=[]; devs=[]; negzero_set=[];
log_transform=1; % PS 

workdir=pwd;
%Get the IMAGE FILENAMES {FName} 
% Controls Option
%-----------------------------------------------------------------------
prompt=['Specify Controls? '];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'SPECIFY CONTROLS','','NO CONTROLS','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_help('vp_ssm_autovoxrecal'); 
   end
end
  
if (strcmp(POpt,'SPECIFY CONTROLS'))
 nnorms=1; % determine number later
else
 % No Specific Controls
 nnorms=0;
end

if nnorms > 0
  %PS get the image CONTROL filenames {FName} 
  prompt=['Select ONLY CONTROL GROUP New Image Data set'];
  if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
  disp(prompt);

  FNames=cellstr(NameLst);
  nnorms= size(NameLst,1);
end

%PS get the image NON-CONTROL filenames {FName} 
 if nnorms > 0
  prompt=['Select ONLY NON-CONTROL GROUP New Image Data set'];
 else
  prompt=['Select New Group Image Data set'];
 end
 if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
disp(prompt);

m= size(NameLst,1);
if nnorms > 0
  NameLst=cellstr(NameLst);
  for i=1:m, FNames(nnorms+i,:)=NameLst(i,:); end; 
else
  FNames=NameLst;
end

FNames=char(FNames),
sub= size(FNames,1),

if nnorms > 0
  prompt1=['Subjects: ',num2str(m),' Controls: 1 - ',num2str(nnorms)];
else
  prompt1=['Subjects: ',num2str(m)];
end

[gisdir,gisdnm,gisdir,gisnm]=vp_nm('dir',gisfile),

[ngisno, gisroot]=vp_num3str(gisnm);

k0=findstr(gisroot,'gis');
if ~isempty(k0),
    k0=k0-1;
    if (gisroot(k0)== '_')k0=k0-1; end;
    gisroot=gisroot(1:k0);
    kfound=k0;
    for (i=1:k0), k=k0-i+1; 
     if gisroot(k) ~= '_', j=str2num(gisroot(k));
       if isempty(j), kfound=k, break; end; 
     end;
     if kfound ~= k0, break; end;
    end;
    gisroot=gisroot(1:kfound),
end

[out1 gisimg]=vp_volfuns('volvect',gisfile);
if (out1 < 0) vp_message('ERROR reading gisfile');
return, end;

% normalize column vector to
% have Euclidean norm=1

normgis=norm(gisimg); % PS save norm
gisimg=gisimg/normgis;

if nnorms > 0
  studyname = [gisroot,'_RE_',num2str(nnorms),'_',num2str(sub-nnorms)];
else
  studyname = [gisroot,'_RE_',num2str(sub)];
end

vp_keyin('[Assign Output Files Study Name | q]',studyname);
if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
return, else studyname=Keyin; end;
    
%pwdd = [netdir,'\',studyname];
netprename=gisdir(1:length(gisdir)-1);
netpredir=vp_nm('dir',netprename);
pwdd = [netpredir,studyname];
  
while (exist(pwdd) == 7)
  vp_keyin(['NAME-IN-USE, Assign Output Files Study Name | q'],studyname);
  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  return, else studyname=Keyin; end;
  pwdd = [netpredir,'\',studyname];
end
  
%[ret,Message]= mkdir(netpredir,studyname),
  
im_data=zeros(prod(DIM), sub, 'single');

h=waitbar(0,'Processing images ...');
set(FMenu,'Pointer','Watch');

%%%% Reading in images
% reshape 3 dimensional images to one-dimensional vectors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:sub
  msg=['reading image ' FNames(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,h,['Reading image ',num2str(i)]);
  [out1 im_data(:,i)]=vp_volfuns('volvect',FNames(i,:));
  if (out1 < 0)vp_message(['ERROR reading im_data file ',FNames(i,:)]);
  return, end;
end

waitbar(1,h); pause(1); close(h);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PS Add masking option
prompt=['Mask Data'];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'MASK','','GIS INTRINSIC MASK','Mask derived from GIS','EXIT','','Help','');
   if(strcmp(POpt,'X'))
     set(FMenu,'Pointer','Arrow');
   return; end;
   if(strcmp(POpt,'EXIT'))
     set(FMenu,'Pointer','Arrow');
   return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify an existing [0|1] mask image or auto-calculate using GIS');
   end
end
  
%Calculate intrinsic mask 
gismask=(gisimg ~= 0); %GIS mask

% MUST also consider what happens if images have other zero values
intrimask=(im_data(:,1) ~= 0).*gismask;
for k=2:sub
     intrimask=(im_data(:,k) ~= 0).*intrimask;
end

if (strcmp(POpt,'MASK'))
   vp_note('Mask must have the same dimensions as the image files.',5);
   maskname=get_file(1,'*mask*.img','Select Mask');
   if isempty(maskname), return; end;
   %PS

   [out1 mask]=vp_volfuns('volvect',maskname);
   if (out1 < 0) vp_message('ERROR reading mask');
   return, end;
   
   mask=mask.*intrimask;
   masking=1;
else
% intrinsic mask 
  mask=intrimask;
  masking=1;
end

mask=(mask ~=0); % In case MASK is not 0/1


% MASK IMAGES
   h=waitbar(0,'Masking images ...');
   for k=1:sub
     msg=['Masking Image ',num2str(k)];
     waitbar((i-.8)/sub,h,msg);
     im_data(:,k)=im_data(:,k).*mask;
   end
   waitbar(1,h); pause(1); close(h);

% LOG TRANSFORMATION 
 
 if log_transform
   h=waitbar(.4,'Processing Log Transformation of Images ...');
   %negzero_set=find(im_data <=0);
   for p=1:sub % Save memory
     negzero_set=find(im_data(:,p) <=0);
     im_data(negzero_set,p)=exp(0); % PS Set all values <=0
   end
   %im_data(negzero_set)=exp(0); % PS Set all values <=0
   negzero_set=[]; % free memory
   im_data=log(im_data); % PS perform log transformation
   waitbar(1,h); pause(1); close(h);
 end;

 % End of data scanning and log-transform 

% CALCULATE SRP

  h=waitbar(0,'Calculating Means ...');
  waitbar(.2,h);
  voxno=size(im_data,1);
  sub=size(im_data,2);

  %Compute within image mean vector
  if masking
  %PS Don't count 0's in mean
    maskpos=find(mask); masksum=size(maskpos,1),
    if (masksum == 0)
      set(FMenu,'Pointer','Arrow');
      vp_message(['Error: Mask has no positive elements ']);
      close(h);
      return;
    end
    meanvect=sum(im_data)/masksum;
  else
    meanvect=mean(im_data);
  end %if masking

  waitbar(.5,h);

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  %PS Replace because of memory restrictions
  %meanmat=ones(voxno,sub); 
  %for x=1:sub
   % meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
   % meanmat(:,x)=meanmat(:,x).*mask; % mask matrix
  %end
  waitbar(.8,h);
  
  %devs=im_data-meanmat;
  %meanmat=[]; % free memory
  
  %PS Save memory
  for x=1:sub, im_data(:,x) = (im_data(:,x) - meanvect(1,x)).*mask ; end;

  waitbar(1,h); pause(1); close(h);

   % CALCULATE MEAN REFERENCE DEVIATIONS IMAGE

   h=waitbar(.1,'Calculating SRP - Please Wait ...');

   %PS mean_img=mean(im_data');
   mean_img=mean(im_data,2); % Take mean of each row to form column
   
   waitbar(.2,h);
   
  %Save mean reference image
   
   TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mean Image']; 
   filnm=[workdir,'\',studyname '_lnmean.img'], 
   out1=vp_imsave(filnm,reshape(mean_img,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _lnmean file');
      set(FMenu,'Pointer','Arrow');
      return;
   end;
  
  %Save mask mage
   
   %TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mask Image']; 
   TYPE=2; SCALE=1; DESCRIP=['ScAnVP - SSM 0/1 Mask Image']; 
   filnm=[workdir,'\',studyname 'mask.img'], 
   out1=vp_imsave(filnm,reshape(mask,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _mask file');
      set(FMenu,'Pointer','Arrow');
      return;
   end;
  
  %Compute subject deviations from mean image (SRP)
  %mean_img=mean_img';
  
  waitbar(.3,h);
  for x=1:sub, %PS
   im_data(:,x)=im_data(:,x) -mean_img;
   waitbar(.3+.6*x/sub,h);
  end;
  
  %devs = srp' 
  waitbar(1,h); pause(1); close(h);

%end %PS END IF NOT OPT A

 if masking
  h=waitbar(0,'Masking SRP ...');
  for k=1:sub
     msg=['Masking SRP Sub. ',num2str(k)];
     waitbar((i-.8)/sub,h,msg);
     %devs(:,k)=devs(:,k).*mask;
     im_data(:,k)=im_data(:,k).*mask;
  end
  waitbar(1,h); pause(1); close(h);
 end % if masking

%ssf=devs'*gisimg,
ssf=im_data'*gisimg,

im_data=[]; %PS free memory

disp(' ');
disp('Calculated SSF')

ssfvoxtprfig=figure('name','SSF SCORES'); bar(ssf); xlabel('SUBJECTS'); ylabel('SSF SCORE');
pos=get(ssfvoxtprfig,'position');

if sub < 5, pos(3)=pos(3)*sub/5; 
      set(ssfvoxtprfig,'position',[pos(1),pos(2),pos(3),pos(4)]);
end;

REFCALformat=0;
if sub > 1

    %Display Scatter Plot
    if nnorms > 0,
      vp_scatterplot(ssf, nnorms, sub, 0, 'CONTROLS             SUBJECTS', 'SSF SCORE', studyname);  
    else
      vp_scatterplot(ssf, nnorms, sub, 0, '          SUBJECTS', 'SSF SCORE', studyname);  
    end;

    prompt=['Save As Recalibrated REFrence Scores? [Y|N|EXIT]'];
    POpt=vp_opt('X',prompt,'YES','REF-Scores','NO','TPR format','EXIT','');
    if strcmp(POpt,'YES'), 
        REFCALformat=1;
	if sub < 5
	    prompt=['Save Ref. Score? WARNING Limited Subjects = ',num2str(sub),', Continue or Exit and use the B or C REFERENCE options? [Y|N|EXIT]'];
	    POpt=vp_opt('X',prompt,'YES','REF-Scores','NO','TPR format','EXIT','');
	    if strcmp(POpt,'YES'), 
	    else
        	REFCALformat=0;
	    end;
	end
    else
    end;
end

for i=1:sub, imls{i}=FNames(i,:); end; % Create cell array

% save ssf simple tpr format for all options
if sub==1
  ssf_name = strcat(workdir, ['\',studyname,'_VtprScore.txt']);
else
  ssf_name = strcat(workdir, ['\',studyname,'_VtprScores.txt']);
end
vp_write_srpfile(ssf_name,1,imls,ssf)

if (REFCALformat==1)
  ssf_name = strcat(pwdd, ['\',studyname,'_VScores',num2str(ngisno),'.txt']);
  Zssf_name = strcat(pwdd, ['\',studyname,'_VZScores',num2str(ngisno),'.txt']);
  %save later
end
   
% PS add Z transformation option

getzscores=0;

if sub == 1
     vp_note(['To derive a Z score for a single subject use Options B or C. ']);
elseif (sub > 1)
  	%z score option may be misleading for small number of subjects
  	%if recalibrating, get Z scores without asking 
	
	if (sub < 5 & REFCALformat==0)
	    prompt=['Z Score? WARNING Limited Subjects = ',num2str(sub),', Continue or Exit and use the B or C REFERENCE options? [Y|N|EXIT]'];
	    POpt=vp_opt('X',prompt,'YES','Get Z-Scores','NO',['Don''t Z-Score'],'EXIT','');
	    if strcmp(POpt,'YES'), 
	      %getzscores=1;
	      %Zssf=(ssf - mean(ssf))./std(ssf),
	      %zscorefig=figure('name','SSF Z SCORES','position',[pos(1)+50,pos(2)+50,pos(3),pos(4)]);
	      %bar(Zssf); xlabel('Subject'); ylabel('Subject Z SCORE');
	    else
	      set(FMenu,'Pointer','Arrow');
	      vp_message(['END CHECK: ',ssf_name,', ']);
	      return;
	    end;
	    
	end
    
	getzscores=1;
%	Zssf=(ssf - mean(ssf))./std(ssf),

 
	%Normalize SSF Scores (Z-score for gis over ref subjects)
	Zssf=[];
	Zssf=(ssf-mean(ssf))./std(ssf), 

	norm1=0; norm2=0;
	% Offset calculated for gis
	if nnorms > 0
	  norm1=1; norm2=nnorms;
	  vp_note(['Control Subject Range: ',num2str(norm1),' - ',num2str(norm2)],2);
	
	  if norm2 ~= norm1
	      Zoffset=mean(Zssf(norm1:norm2,:)),
	  else
	      Zoffset=Zssf(norm1,:),
	  end
    
	  Zssf=Zssf-Zoffset;
	end

	zscorefig=figure('name','SSF Z SCORES','position',[pos(1)+50,pos(2)+50,pos(3),pos(4)]);
	bar(Zssf); xlabel('Subject'); ylabel('Subject Z SCORE');
	yo= round(max(Zssf)/4);
	%if nnorms > 0
	%  for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
	%end
	%set(gca,'color',[.9 1.0 .9]);
end


if REFCALformat % Save in recalibrated format directory

   [ret,Message]= mkdir(netpredir,studyname),

   % Save Gis Reference Scores
   %vp_write_ssmfile([refname,'_VScores',num2str(ngis),'.txt'],ngis,vaf(ngis),file_list,ssf(:,ngis),1,norm1,norm2)
   vp_write_ssmfile(ssf_name,ngisno,-1,imls,ssf,4,norm1,norm2)
   if getzscores == 1
	vp_write_ssmfile(Zssf_name,ngisno,-1,imls,Zssf,5,norm1,norm2)
   end
   vp_dispfile(Zssf_name);

   %Save gis image
   
   TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM GIS Image']; 
   filnm=[pwdd,'\',studyname '_gis',vp_num3str(ngisno),'.img'],  
   out1=vp_imsave(filnm,reshape((normgis*gisimg),DIM(1:3)), 1), 
   if (out1 < 0),
        vp_message('ERROR writing _gis file');
        set(FMenu,'Pointer','Arrow');
        return;
   end;

  %Save mean reference image
   
   %mean_img=mean_img';
   TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mean Image']; 
   filnm=[pwdd,'\',studyname '_lnmean.img'], 
   out1=vp_imsave(filnm,reshape(mean_img,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _lnmean file');
      set(FMenu,'Pointer','Arrow');
      return;
   end;
  
  %Save mask image
   
   %TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mask Image']; 
   TYPE=2; SCALE=1; DESCRIP=['ScAnVP - SSM 0/1 Mask Image']; 
   filnm=[pwdd,'\',studyname 'mask.img'], 
   out1=vp_imsave(filnm,reshape(mask,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _mask file');
      set(FMenu,'Pointer','Arrow');
      return;
   end;
  

else
  if getzscores == 1
     if sub==1
       Zssf_name = strcat(workdir, ['\',studyname,'_VtprZScore.txt']);
     else
       Zssf_name = strcat(workdir, ['\',studyname,'_VtprZScores.txt']);
     end
     vp_write_srpfile(Zssf_name,1,imls,Zssf)
     vp_dispfile(Zssf_name);
  end
end

set(FMenu,'Pointer','Arrow');

if REFCALformat % Save in recalibrated directory
 vp_message(['END CHECK: ',pwdd,' and ',workdir]);
else
 vp_message(['END CHECK: ',workdir]);
end

