% vp_ssm_autovoxtpr
% Voxel based SSM Automatic Topographic Profile Rating (TPR)
% Performs evaluation of new subject(s) SSM subject scores for a specific
% GIS using pre-defined reference data. New subjects should be scanned with
% the same parameters as the reference subjects.
%_______________________________________________________________________
%
% Input:
% The user selects as input
%  (1) One or more new Talairach normalized subject image files.
%  (2) A predefined network GIS image for evaluation of new subject 
%    scores for the disease or condition associated with the GIS.
%    The GIS can optionally reside in a scanvp\network directory and contain
%    reference files generated by a previous group PCA analysis that 
%    generated the GIS or that resulted from a recalibration procedure. 
%
% The new subject set need not include subjects from the previous study
% since each subject is evaluated individually on a prospective basis.
% However, new subjects should be scanned with the same parameters as 
% as the reference subjects.
% It is assumed that the same Talairach or other normalization procedure
% is used for all new subjects as was used in the reference study.
% The GIS definition image file results from an original PCA analysis
% and contains the relative voxel weights for that GIS.
% For automated operation, the network GIS directory must contain besides
% 1) the GIS definition image file, the following files also generated
% by the original PCA analysis:
% 2) the log data group mean image file (lnmean) to be used as a reference
% from which the new SRP values will be calculated. 
% 3) The original reference score file (...VScores not ...ZScores)
% indicating control subject indeces.
% These files are selected automatically by the routine once the name of the
% network GIS is identified and naming conventions are followed as indicated.
% If these files do not exist, a recalibration procedure can be performed.
% 
% Method:
% The following procedure is used to derive new subject scores.
% New subject image files and reference GIS network image file are selected.
% Log Mean image file is automatically read from the reference GIS directory.
% The intrinsic non-zero voxel mask is determined from the reference mean file.
% The intrinsic gis mask should be the same but is multiplied with the mean
% mask for cases where these files originate from different analyses.
% New images are masked and natural log transformation is performed.
% Logs of negative data values are set to zero. (Optional non-log
% transformation is only available in the interactive version.)
% Data mean centering normalization is performed as a two step procedure.
% The mean value over all non-zero voxels is evaluated for each new subject
% and used to generate within subject regional deviations from the mean.
% The group log mean image values of the reference data (GISName_lnmean.img)
% are then subtracted from the corresponding new subject values to generate
% the new subject residual profile (SRP).
% The new SSF subject scores are determined using the following calculation:
% 
% 	          SSF=SRP*GIS/norm(GIS),
%
% where SSF is a vector (no of new subs.), SRP is a matrix (nsubs, nvoxs) and
% GIS is a vector (nvoxs).
% 
% An output file containing the new SSF scores for each new subject is generated
% and plotted. The Z scores based on the reference score mean and std. values
% are also generated for the new subjects. and if controls exist, they are offset
% by the reference score control mean. 
% Results are saved in the user working directory.
%_______________________________________________________________________
% vp_ssm_autovoxtpr      ScAnVP 5.4   P.Spetsieris   1/25/2005
% 			 Rev. 5.6     P.Spetsieris   11/21/2005
% 			 Rev. 5.7     P.Spetsieris   12/31/2005
% 			 Rev. 5.8     P.Spetsieris   6/01/2007
% 			 Rev. 6.2     P.Spetsieris   3/05/2012
% 			 Rev. 6.2.2   P.Spetsieris   8/21/2013
%_______________________________________________________________________

global POpt FMenu Keyin NameLst
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global HelpMode

if HelpMode, vp_help(mfilename); return ; end;

im_data=[]; imls=[]; ssf=[];

vp_note('WARNING! New and Reference subjects should be similarly scanned',3);
prompt=['AUTOTPR: Select NEW Subjects Image Files'];
if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
disp(prompt);

imfiles_B=NameLst;
sub= size(NameLst,1);
log_transform=1; % PS Force log transformation 

pwdd=pwd;
if sub==1
   img1=imfiles_B(1,:);
   
   outnm=vp_nm('nm',img1);
else
  outnm=['S',num2str(sub)];
end


POpt='Standard GIS Network',
prompt=['AUTOTPR: STANDARD or OTHER REFERENCE GIS?'];
optA=['Input scanvp\networks directory Reference GIS, Calculate new group TPR scores.'];
optB=['Input Other Directory Reference GIS, Calc. new group TPR scores '];
POpt=vp_opt('X',prompt,'STANDARD GIS Network',optA,'OTHER REFERENCE GIS Network',optB, 'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_autovoxtpr'); 
  vp_ssm_autovoxtpr;
  return; end;
if(strcmp(POpt,'STANDARD GIS Network')),
  vp_note(optA,3);
  w=which('scanvp');
  netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
elseif(strcmp(POpt,'OTHER REFERENCE GIS Network')),
  netdir=[];
  vp_note(optB,3);
else
  vp_note(['Invalid Option',POpt],3);
  return;
end

prompt=['AUTOTPR: Select PCA GIS For TPR'];
disp(prompt);

gis_name= get_file(1,'*gis*.img',prompt,netdir,[ ]);  
if isempty(gis_name), return; end;
[dnm,dnm,dir]=vp_nm('dnm',gis_name);
p=max([findstr(gis_name,'gis') 0]);
l=length(gis_name);
condnm=gis_name(1:p-1);
gisid=gis_name(p+3:l-4);
if length(gisid) > 1
  ngisid=str2num(gisid);
  if ~isempty(ngisid) % only if numeric
    gisid=num2str(ngisid);
  end
end

p=max([findstr(dnm,'networks') 0]);
l=length(dnm);
netnam=dnm(p+9:l);

[out1 gisimg]=vp_volfuns('volvect',char(gis_name));
if (out1 < 0) vp_message('ERROR reading gisfile');
return, end;

% reshape into column vector and normalize to
% have Euclidean norm=1
normgis=norm(gisimg); % PS save norm
gisimg=gisimg/normgis;
gismask=(gisimg ~= 0);

num_vox = size(gisimg,1); % PS

% READ MEAN DEVIATIONS REFERENCE IMAGE
meanfile=[condnm,'lnmean.img'];
if exist(meanfile) ~=2
 meanfile=[condnm,'lmean.img'];
 if exist(meanfile) ~=2
   vp_message(['Error: Could Not find either ',condnm,'lnmean.img or ',meanfile,' file']);
   return;
 end
end
  
 
[out1 mean_img]=vp_volfuns('volvect',meanfile);
if (out1 < 0) vp_message('ERROR reading mean file');
return, end;
 
refvoxno = size(mean_img,1); % PS

% DERIVE MASK FROM MEAN IMAGE FILE
mask = (mean_img ~= 0);

% INCORPORATE GIS MASK
mask = mask.*gismask;

masksum=sum(mask),
if (masksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: Mask has no positive elements ']);
   return;
end
   
   
h=waitbar(0,'Processing images ...');

im_data=zeros(prod(DIM), sub, 'single');

% Reading in images

for i=1:sub
  msg=['reading image ' imfiles_B(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,h,['Reading image ',num2str(i)]);
  [out1 im_data]=vp_volfuns('volvect',imfiles_B(i,:));
  if (out1 < 0) vp_message(['ERROR reading image data file ',num2str(i)]); return, end;

  %Incorporate Intrinsic Data Mask for Incomplete Data Cases
  submask=(im_data ~= 0).*mask;
  submasksum=sum(submask);
  if (submasksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: Mask has no positive elements ']);
   return;
  end
   
  if submasksum ~= masksum,
    submasksum,
    masksum,
    set(FMenu,'Pointer','Arrow');
    vp_message(['Error: Subject ',num2str(i), ' :',imfiles_B(i,:),' Voxel Space Does not Span Reference Mask Space']); return;
  end

  % Masking images
  dim=size(mask);
  msg=['Masking Image ',num2str(i)];
  waitbar((i-.6)/sub,h,msg);
  im_data=im_data.*mask;


  % - Assume Group Mean File

  if log_transform
   	waitbar((i-.4)/sub,h,'Processing Log Transformation of Image ...');
   	negzero_set=find(im_data<=0);
   	im_data(negzero_set)=exp(0); % PS Set all values <=0
   	negzero_set=[]; % free memory
   	im_data=log(im_data); % PS perform log transformation
  end;
  % End of data scanning and log-transform 

  waitbar((i-.2)/sub,h,'Processing Mean ...');

  % CALCULATE SRP

  voxno=size(im_data, 1);

  if voxno ~= num_vox,
 	 set(FMenu,'Pointer','Arrow');
 	 vp_message('Error: Number of GIS voxels do not agree with data'); return;
  end

  if voxno ~= refvoxno,
  	set(FMenu,'Pointer','Arrow');
 	vp_message('Error: Number of voxels do not agree with ref. mean'); return;
  end


  %Compute within image mean vector
  %PS Don't count 0's in mean
  meanvect=sum(im_data)/masksum;


  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  %for x=1:sub, im_data(:,x) = (im_data(:,x) - meanvect(1,x)).*mask ; end;
  im_data = im_data - meanvect.*mask ; 

  waitbar(i/sub,h,'Calculating Image SSF...');
  
  %Compute subject deviations from mean image
  im_data=(im_data -mean_img).*mask;

  %generate SSF values for a known GIS pattern based
  %on SRP calculated in the new population.

  % calculate new ssf
  ssf(i)=im_data'*gisimg,
  
end % for subject i

ssf=ssf';

im_data=[]; %PS free memory

for i=1:sub, imls{i}=imfiles_B(i,:); end; % Create cell array

p=max([findstr(dnm,'networks') 0]);
l=length(dnm);
netnam=dnm(p+9:l);
nm=vp_nm('nm',netnam); 
p=max([findstr(nm,'_') 3]);
flnm=[nm(1,1:p-1),outnm];
if sub==1
  ssf_name = strcat(pwdd, ['\',flnm,'_S1_VtprScore.txt']);
  Zssf_name = strcat(pwdd, ['\',flnm,'_S1_VtprZScore.txt']);
else
  ssf_name = strcat(pwdd, ['\',flnm,'_VtprScores.txt']);
  Zssf_name = strcat(pwdd, ['\',flnm,'_VtprZScores.txt']);
end


vp_write_srpfile(ssf_name,1,imls,ssf)
vp_dispfile(ssf_name);

waitbar(1,h); pause(1); close(h);

disp ('   ***************************************************     ')
disp(' ');
disp('             RESULTS OF GIS2SSM TPR')
disp(' ');
disp('Newly Calculated SSF')
ssf
ssftprfig=figure('name','TPR SCORES'); bar(ssf); xlabel('New Subject'); ylabel('New Subject SSF SCORE');
pos=get(ssftprfig,'position');
%pos(3)=pos(3)*(sub/15); % use 15 as a standard number of subjects
if sub < 15
  pos(3)=max(pos(3)/3, pos(3)*(sub/15)); % use 15 as a standard number of subjects
end
set(ssftprfig,'position',pos);
    
% END GIS2SSF CODE

% Z transformation, read REF Score file.
ssffile=[condnm,'VScores',gisid,'.txt'];
if exist(ssffile) ~=2
   vp_message(['Error: Could Not find REFerence file ',ssffile,' file']); return;
end
  
[mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2] =vp_ssm_scores(-2,ssffile)
[nosubs, num_gis]=size(ssfscores),
%if (norm1==0 | norm2==0)
%     set(FMenu,'Pointer','Arrow');
%     vp_message(['Invalid Reference File: ',ssffile,' Control Subjects Not Specified']);
%return; end;

%Normalize SSF Scores (Z-score for gis over ref subjects)
Zssfs(:,ngis)=(ssfscores(:,ngis)-mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 

% Offset calculated for each gis

Zoffset(1:num_gis)=0;
if norm2 > 0
 if norm2 ~= norm1
      Zoffset=mean(Zssfs(norm1:norm2,:)),
 else
      Zoffset=Zssfs(norm1,:),
 end
 Zssfs(:,ngis)=Zssfs(:,ngis)-Zoffset(ngis)*ones(nosubs,1);
end
    
% Use the same mean, std and offset as the reference to calculate new Z scores.
    
%Zssf=zscore(ssf),
%Zssf=(ssf - mean(ssf))./std(ssf),
Zssf=(ssf - mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 
Zssf=Zssf-Zoffset(ngis)*ones(sub,1);;
    
zscorefig=figure('name','TPR Z SCORES'); bar(Zssf); xlabel('New Subject'); ylabel('New Subject Z SCORE');
vp_write_srpfile(Zssf_name,1,imls,Zssf)
vp_dispfile(Zssf_name);
    
pos=get(zscorefig,'position');
%pos(3)=pos(3)*(sub/nosubs);
pos(3)=max(pos(3)/3, pos(3)*(sub/nosubs));
    %maxz=max(Zssfs(:,ngis));
    %minz=min(Zssfs(:,ngis));
    %maxn=max(Zssf); minn=min(Zssf);
    %if (maxn-minn) > .5*(maxz-minz)
    %  pos(4)=pos(4)*((maxn-minn)/(maxz-minz));
    %else
    %  pos(4)=.5*pos(4);
    %end
set(zscorefig,'position',pos);
   
%Display Combined Scores (Group + New Subjects)
ZCssf=[Zssfs(:,ngis); Zssf;];
ZCscorefig=figure('name','TPR COMBINED Z SCORES'); bar(ZCssf);
if norm2 > 0
  xlabel('REFERENCE and NEW SUBJECT(S), Controls: __ __, New Subjects: --'); ylabel('COMBINED GROUP Z SCORES');
else
  xlabel('REFERENCE and NEW SUBJECT(S), New Subjects: --'); ylabel('COMBINED GROUP Z SCORES');
end
%for i=norm1:norm2, text(i,-1,'c','color','r'); end; %Mark controls
%ypos=min(ZCssf)/2;
ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
if norm2 > 0
  text(norm1-.5,ypos,'<','color','b');  %Mark controls
  text(norm2,ypos,'>','color','b');  %Mark controls
  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
end
%for i=nosubs+1:nosubs+sub, text(i,-1,'s','color','m'); end; %Mark new subjects
%text((nosubs+1),ypos,'<','color','m'); %Mark new subjects
text((nosubs+0.8),ypos,'<','color','b'); %Mark new subjects
text((nosubs+sub),ypos,'>','color','b'); %Mark new subjects
line([nosubs+1,nosubs+sub],[ypos,ypos],'LineStyle',':');
set(gca,'color',[.9 1.0 .9]);
    
pos=get(ZCscorefig,'position');
pos(1)=pos(1)+2*pos(3)/3;
set(ZCscorefig,'position',pos);

% create scatter plot
if norm2 > 0
  labelx=['CONTROLS                REFERENCE SUBJECTS            NEW SUBJECTS'];
else
  labelx=['REFERENCE SUBJECTS            NEW SUBJECTS'];
end
labely=['REFERENCE AND NEW SUBJECT Z - SCORES'];
vp_scatterplot(ZCssf,norm2,nosubs,sub,labelx,labely,netnam)

set(FMenu,'Pointer','Arrow');


