% vp_ssm_preDIAGNrpt.m
% SSM TPR Pre Diagnostic Report Interface
%
% This routine determines the ZScore Cut-off radius (ZRa) assuming various
% preselected options. Graphs are produced of the diagnostic ratios for
% patients and controls and total TRUE true diagnoses versus Z-cutoff.
% ROC curves are plotted. Criteria include reference values for the minimum
% cut-off that gives maximum or balanced TRUE patient/normal diagnosis.
% For option 7, the cut-off is determined as the maximum true patient plus
% true normal diagnoses of the reference subjects by curve fitting of the
% reference data.
% Option 8, determines cut-off from a combined patient and normal TRUE
% Likelihood ratio.
%
% Option
% 0: Normals are assumed to lie within the REF normal Z score limits
% 1: Normals are assumed to have a Z score less than 1 (1STD of the combined REF subjects)
% 2: Normals are assumed to have a Z score less than 2 STD of the normal REF subjects
% 3: Normals are assumed to have a Z score less than the value (Z-cutoff) that gives equal
% TRUE diagnostic ratios for both Normal and Patient REF subjects.
% If this option is selected a graph is produced for the diagnostic ratios versus Z-cutoff.
% 4: Z-cutoff is selected as the lowest value that gives the maximum true patient + normals.
% 5: R is determined to be the value of Z when the %of REF patients diagnosed TRUE = 95.  
% 6: The user can overide the default Z-cutoff generated by Option 3 with a keyin.  
% 7: Z-cutoff is selected as the curve-fitted value that gives the maximum true diagnosis
% for patients + normals
% 8: Z-cutoff is selected as the value that gives maximum TRUE Likelihood Ratio determined
%    as ComLR=(sensitivity + specificity)/(2-sensitivity - specificity) or the ratio of
%    combined TRUE/FALSE ratios at each point of the ROC curve.
%    A corrected Combined Likelihood Ratio is used:
%		CLR =  (sens + spec -.05)/(2 -sens-spec +.05) 
%    Also displayed are:
%    The positive Likelihood Ratio for values of of the Z where false positives are not 
%    zero, is estimated as +LR=sensitivity/(1-specificity) or the ratio of y/x at each point
%    of the ROC curve. This value blows up for specificity=1. The Log LR is calculated
%    as LLR = log(sensitivity) - log(1-specificity).
%
% A reference weighted Z-score ratio is calculated as the summed ratio of reference patients
% that have values less or equal to Z for each GIS to be used in comparing Z values in 
% conflicting diagnoses of subjects scoring positively on multiple tests.
% 
% ScAnVP 5.7  P.Spetsieris, Ph.D.     6/02/2006
%		Rev. 5.8	3/17/07
%		Rev. 6.0	1/23/09  (Write ROC, CLR plot values to file)
%		Rev. 6.2	10/20/12 (ROC values file lists sens vs. spec. not 1-spec.)
%_______________________________________________________________________

global ResolMode CutOffType pLR

if HelpMode, vp_help(mfilename), return ; end;

%diagntype=-1;
%diagntype=7;
if ~exist('cycle'), cycle=0; end;
%while (diagntype < 0 | diagntype > 5)
%  prompt=['Create DIAGNOSTIC Report ?  [ y=REPORT | q]'];
%  vp_keyin(prompt,'y');
%  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
%    break,
%  else
%    % diagntype=round(str2num(Keyin));
%    if strcmp(Keyin,'y'), diagntype= 4; end;
%    if strcmp(Keyin,'yes'), diagntype= 4; end ;
%  end;
%end

ZRadius=0;
[nrsubs,k]=size(Zssfs);
nnorms=norm2-norm1+1; nabnorms=nrsubs-nnorms;

npts=60;
if (diagntype >=3 & cycle== 0)

  cycle=cycle+1
  minz=min(Zssfs(:,ngis));
  maxz=max(Zssfs(:,ngis));
  dz=(maxz-minz)/npts ; % Plot increment (Z range/npts)
  %ratio controls diagnosed, ratio diseased diagnosed,
  rdiagc=[]; rdiagd=[]; rfalsep=[]; rtruep=[]; 
  %total diagnosed, ratio total diagnosed
  tdiag=[]; rtdiag=[];
  CLR=[];
  diagnratio=[];
  tdmax=0;
  zrad=minz-dz; zradi=[]; 
  pLR=[]; % positive Likelihood ratio
  LLR=[]; % Log Likelighood ratio
  ComLR=[]; % Total Likelihood ratio
  rWZ=[]; % reference weighted Z value
  
  for i=1:(npts+3)
    zradi(i)=zrad;
    ndiagc=0; ndiagd=0;
    for k=1:nrsubs % for each subject
      if ( (k>=norm1) & (k<=norm2) ) % if subject in normal set 
       % and if diagnosed as normal increment correct normal diagnosis
       if Zssfs(k,ngis) <= zrad, ndiagc=ndiagc+1; end; 
      else %if abnormal and diagnosed correctly increment correct abnormals
       if Zssfs(k,ngis) > zrad, ndiagd=ndiagd+1; end;
      end
    end
    % record correct diagnostic rations for ith value of Zrad
    rdiagc(i)=ndiagc/nnorms; rdiagd(i)=ndiagd/nabnorms;
    tdiag(i)=ndiagc + ndiagd;
    rtdiag(i)=tdiag(i)/nrsubs;
    rWZ(i)=1-rdiagd(i);
    % evaluate a 95% corrected total diagnostic Likelihood Ratio
    CLR(i)=(rtdiag(i) -.05)/((nrsubs-tdiag(i))/nrsubs + .05);
    if (ndiagd <=0), 
      if (ndiagc <=0), diagnratio(i)=0; 
      %replace infinite value with a linear extension of the lower value
      else, diagnratio(i)=2*diagnratio(i-1) - diagnratio(i-2); end;
    else diagnratio(i)=rdiagc(i)/rdiagd(i); end;
    if diagnratio(i) <= 1, ilow=i; end; 
    if tdiag(i) > tdmax , tdmax=tdiag(i); imax=i; nmaxc=ndiagc; nmaxd=ndiagd; end;
    if tdiag(i) == tdmax , imax2=i; nmaxc2=ndiagc; nmaxd2=ndiagd; end;
    if rdiagd(i) > .95, ig95=i; Zg95=zrad; ng95c=ndiagc; ng95d=ndiagd; end;
    %if diagnratio(i) > 2, break; end;
    zrad=zrad+dz; % Value of the Zradius variable 
  end
 
  % find exact value of ZRad for balanced diagnosis, or interpolate 
  if diagnratio(ilow)== 1, ZRadius=zradi(ilow), perdiag= round(rdiagc(ilow)*100),
   perdiagc=perdiag; perdiagd=perdiag;
  else ZRadius=  zradi(ilow) + ...
   (1-diagnratio(ilow))*(zradi(ilow+1)-zradi(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ;
   ZRadius=(round(ZRadius*10000))/10000;
   perdiagc = rdiagc(ilow) + ...
    (1-diagnratio(ilow))*(rdiagc(ilow+1)-rdiagc(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ,
   perdiagd = rdiagd(ilow) + ...
    (1-diagnratio(ilow))*(rdiagd(ilow+1)-rdiagd(ilow))/(diagnratio(ilow+1)-diagnratio(ilow)) ,
   perdiag= round((perdiagc + perdiagd)*100/2) ,
  end
  
  ZTot=zradi(imax), nmaxc, nmaxd,
  ZTot2=zradi(imax2), nmaxc2, nmaxd2,
  ZTot=(round(ZTot*10000))/10000;
  ZTot2=(round(ZTot2*10000))/10000;
  if cycle <= 1,
  fig1=figure('name',['REF DIAGN RATIOS For ',netnam]);
    %plot(zradi,rdiagc,'ok-',zradi,rdiagd,'^k-',zradi,diagnratio,'k-','LineWidth',1.5);
    plot(zradi,rtdiag,'k-',zradi,rdiagc,'ok-',zradi,rdiagd,'^k-','LineWidth',1.45,'MarkerSize',5);
    set(gca,'YLim',[0,1.3]);
    %set(gca,'XLim',[-1.5,1.5]);
    %xlabel(['    R     Rt= [',sprintf('%.2f',ZTot),' - ',sprintf('%.2f',ZTot2),'], (Rb= ',num2str(ZRadius),')'],...
    %
    xlabel([' Z-CUTOFF     ZRt= [',num2str(ZTot),' - ',num2str(ZTot2),'],     (ZRb= ',num2str(ZRadius),')'],...
    'FontWeight','bold'); ylabel('REF RATIO','FontWeight','bold');
    %line([ZRadius,ZRadius],[0,1],'LineStyle',':');
    line([ZRadius,ZRadius],[0,perdiag/100],'LineStyle',':');
    line([ZTot,ZTot],[0,max(rtdiag)],'LineStyle','-.');
    line([ZTot2,ZTot2],[0,max(rtdiag)],'LineStyle','-.');
  hlg=legend('TOTAL REF DIAGNOSTIC RATIO (CONTROLS+PATIENTS)','REF DIAGNOSTIC RATIO CONTROLS (TRUE NEGATIVES)','REF DIAGNOTIC RATIO PATIENTS (TRUE POSITIVES)');
  set(hlg,'FontWeight','bold');
  set(fig1,'color',[ 1 1 1]);
  %fig1b=figure;
  fig1b=figure('name',['TOTAL TRUE REF DIAGN RATIO For ',netnam]);
    nply=8;
    %pt1=12; pt2=32;
    pt1=imax -10; if pt1< 1, pt1= 1; end;
    pt2= imax2+ 10; if pt2 > npts, pt2= npts; end;
    if pt2 < 1, pt2=10; end;
    plot(zradi,rtdiag,'ok-','LineWidth',1.45,'MarkerSize',2);
    [ply1,s1]=polyfit(zradi(pt1:pt2),rtdiag(pt1:pt2),nply);
    h1=refcurve(ply1);
    [y1,delta1]=polyval(ply1,zradi(pt1:pt2),s1),
    [maxy1,indx1]=max(y1),
    z1=zradi(pt1-1+min(indx1)),
    set(gca,'XLim',[zradi(pt1),zradi(pt2)],'YLim',[.5,1.2]);
    xlabel([' Z-CUTOFF    ']);
    ylabel([' TOTAL REF DIAGN RATIO ']);
    hlg=legend(['Total REF Diagnostic Ratio, ',...
    'Max= ',num2str(round(rtdiag(imax)*100)/100),' at ',num2str(ZTot)],...
    [num2str(nply),' degree polynomial fit, Max=',num2str(round(maxy1*100)/100),' at ', num2str(round(z1*100)/100)]);
    
  %fig2=figure;
  fig2=figure('name',['ROC CURVE For ',netnam]);
  pos1=get(fig1,'position');
  pos1b=get(fig1b,'position');
  pos2=get(fig2,'position');
  pos2(1)=pos1(1) + pos1(3);
  pos1b(1)=(pos1(1) + pos1(3)/2);
  set(fig2,'position',pos2,'color',[1 1 1]);
  set(fig1b,'position',pos1b,'color',[1 1 1]);
  
  % plot ROC curve
  mxi=length(rdiagc);
  for i =1:mxi, pLR(i)=0; ComLR(i)=0; end;
  for i =1:mxi
   rfalsep(i)=1-rdiagc(mxi+1-i);
   rtruep(i)=rdiagd(mxi+1-i);
   if (rfalsep(i) > 0) pLR(mxi+1-i)=rtruep(i)/rfalsep(i); 
   elseif (rtruep(i) > 0) pLR(mxi+1-i)=0; end;
   LLR(mxi+1-i)=log(rtruep(i)) - log(rfalsep(i)); 
   %if ( (rfalsep(i)+1-rtruep(i)) ~= 0)
    
    n= 1.0 + (rtruep(i) - rfalsep(i)) ;
    d= 1.0 - (rtruep(i) - rfalsep(i)) ;
    ComLR(mxi+1-i)=n/d;
    %ComLR(mxi+1-i)=(rtruep(i)+1.0-rfalsep(i))/(rfalsep(i)+1.0-rtruep(i));
   %end;
  end
  plot(rfalsep,rtruep,'ok-','LineWidth',1.45,'MarkerSize',2);
  %plot(rfalsep,rtruep,'*k-','LineWidth',1.45,'MarkerSize',5);
  xlabel(['False Positive Ratio (1-specificity) '],'FontWeight','bold');
  ylabel(['True Positive Ratio (Sensitivity)'],'FontWeight','bold');
  set(gca,'YLim',[0,1.3]);
  set(gca,'XLim',[-0.01,1.01]);
  %Calculate area under the curve
  acurve=0; napts=0;
  for i=1:mxi
   if (rfalsep(i) > 0) & (rfalsep(i) <= 1)
     acurve=acurve + (rtruep(i) + rtruep(i-1))*(rfalsep(i)-rfalsep(i-1))/2.0;
     napts= napts + 1;
   end
  end
  napts,
  acurve,
  acurve=(round(acurve*100))/100;
  atrapz=trapz(rfalsep,rtruep),
  ROCtitle=['ROC curve,    AUC =',num2str(acurve)];
  legend(ROCtitle);
  nm=vp_nm('nm',netnam);
  filenm=['ROCfor',nm,'.txt'];
  %vp_write_XYfile(filenm,ROCtitle,'1-specificity','sensitivity',rfalsep,rtruep);
  vp_write_XYfile(filenm,ROCtitle,'sensitivity','specificity',rtruep,1-rfalsep);
  
  
  end;
  
  % OMIT THE FOLLOWING THREE PLOTS TO SIMPLIFY OUTPUT
  %fig3=figure('name',['posLIKELIHOOD CURVE For ',netnam]);
  %pos3=get(fig3,'position');
  %pos3(1)=pos3(1) + pos2(3)/2;
  %set(fig3,'position',pos3,'color',[1 1 1]);
  %plot(zradi,pLR,'ok-','LineWidth',1.45,'MarkerSize',2);
  %xlabel(['Z Cut-off Radius '],'FontWeight','bold');
  %ylabel(['Positive Likelihood Ratio'],'FontWeight','bold');
  
  %fig3b=figure('name',['LOG LIKELIHOOD CURVE For ',netnam]);
  %pos3b=get(fig3b,'position');
  %pos3b(1)=pos3b(1) + pos3(3)/2;
  %set(fig3,'position',pos3b,'color',[1 1 1]);
  %plot(zradi,LLR,'ok-','LineWidth',1.45,'MarkerSize',2);
  %xlabel(['Z Cut-off Radius '],'FontWeight','bold');
  %ylabel(['Log Likelihood Ratio'],'FontWeight','bold');
  
  
  %fig4=figure('name',['Total LIKELIHOOD CURVE For ',netnam]);
  %pos4=get(fig4,'position');
  %pos4(1)=pos4(1) + pos3b(3)/2;
  %set(fig4,'position',pos4,'color',[1 1 1]);
  %%plot(zradi,ComLR,'*k-','LineWidth',1.45,'MarkerSize',5);
  %plot(zradi,ComLR,'ok-','LineWidth',1.45,'MarkerSize',2);
  %xlabel(['Z Cut-off Radius '],'FontWeight','bold');
  %ylabel(['Total pos + neg Likelihood Ratio'],'FontWeight','bold');
  %[maxComLR,iLR]=max(ComLR);
  %zmaxComLR=zradi(iLR); % set to min Z or max Total LR Likelihood Ratio
  %hComLR=legend(['Total Likelihood Ratio (sens+spec)/(2-(sens+spec)), ',...
  %  'Max= ',num2str(round(maxComLR*100)/100),' at ',num2str(round(zmaxComLR*100)/100)]);
    
  fig5=figure('name',['Corrected Total Likelihood Ratio For ',netnam]);
  pos5=get(fig5,'position');
  %pos5(1)=pos5(1) + pos4(3)/2;
  pos5(1)=pos5(1) + pos2(3)/2;
  set(fig5,'position',pos5,'color',[1 1 1]);
  %plot(zradi,CLR,'*k-','LineWidth',1.45,'MarkerSize',5);
  plot(zradi,CLR,'ok-','LineWidth',1.45,'MarkerSize',2);
  xlabel(['Z Cut-off Radius '],'FontWeight','bold');
  ylabel(['Corrected Total TRUE/FALSE Likelihood Ratio'],'FontWeight','bold');
  [maxctLR,ictLR]=max(CLR);
  zmaxctLR=zradi(ictLR); % set to min Z or max corrected Total Likelihood Ratio
  RL=num2str(round(zmaxctLR*100)/100);
  hctLR=legend(['Corrected (Total Diagnosed Ratio -.05) /(Total Misdiagnosed +.05 ), ',...
    'Max= ',num2str(round(maxctLR*100)/100),' at ',RL]);
  
  CLRtitle=['CLR plot,    RL =',RL];
  nm=vp_nm('nm',netnam);
  filenm=['CLRfor',nm,'.txt'];
  vp_write_XYfile(filenm,CLRtitle,'Z Cut-off Radius','CLR',zradi,CLR);

  
  fig6=figure('name',['Reference Weighted Z Score For ',netnam]);
  pos6=get(fig6,'position');
  pos6(1)=pos6(1) + pos5(3)/2;
  set(fig6,'position',pos6,'color',[1 1 1]);
  %plot(zradi,rWZ,'*k-','LineWidth',1.45,'MarkerSize',5);
  plot(zradi,rWZ,'ok-','LineWidth',1.45,'MarkerSize',2);
  xlabel(['Z Value '],'FontWeight','bold');
  ylabel(['RWZI (Reference Weighted Z Score Index)'],'FontWeight','bold');
  hRWZI=legend(['RWZI : Reference Weighted Z Score Index: SUM patients <=Z / Total patients'] );
  [plyWZ,sWZ]=polyfit(zradi(1:npts),rWZ(1:npts),nply); 
  hWZ=refcurve(plyWZ);
  set(gca,'XLim',[0,max(zradi(1:(npts-4)))],'YLim',[0,1.2]); %ply error large for last 4 pts
  
  
end

%  ' TRUE at ',num2str(ZRadius),' Max(',num2str(tdmax),', ',num2str(rtdiag(imax)*100),',% Nc=',num2str(nmaxc),...
%
%prompt=['Cut-off ZScore ? [no.| q],   REF',...,
%
  
prompt=['Cut-off ZScore REF   Fitted = ', num2str(round(z1*100)/100),...
  '  Numerical Max=',num2str(round(rtdiag(imax)*1000)/10),'% (Nc=',num2str(nmaxc),...
  ', Pt=',num2str(nmaxd),') at ',num2str(ZTot),...
  ', Nrm=Pat Diag ',num2str(perdiag),...
  '% TRUE (Nc=',num2str(round(perdiagc*nnorms/100)),', Pt=',num2str(round(perdiagd*nabnorms/100)),') at ',num2str(round(ZRadius*10000)/10000),...
  ', Max Tot LR=',num2str(maxctLR),' at Z=',num2str(zmaxctLR),' ',...
  ', Patients > 95% (Nc=',num2str(ng95c), ', Pt=',num2str(ng95d),') at ',num2str(round(Zg95*100)/100)];
  %vp_keyin('Cut-off ZScore? [no.| q] ',num2str(ZRadius));
  
%if diagntype==3, zkey=ZRadius;
%elseif diagntype==4, zkey=ZTot;
%elseif diagntype==5, zkey=Zg95;
%end;

if diagntype ~= 6
  vp_note(prompt);
else
  zkey=ZTot;
  vp_keyin(prompt,num2str(zkey));
  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
    diagntype=-1;
    break,
  else
    ZRadius=str2num(Keyin);
%    ZRadiusK=str2num(Keyin);
%    if abs(ZRadiusK - ZRadius) < .01, diagntype=3; 
%    elseif abs(ZRadiusK - ZTot) < .01, diagntype=4; 
%    elseif abs(ZRadiusK - Zg95) < .01, diagntype=5; 
%    else diagntype=6; end;
%    ZRadius=ZRadiusK;
  end;
end

% Allow only types 3,4,6 and 7
% derived ZRadius for diagntype 3 (balanced TRUE) 
if diagntype==4
  ZRadius=ZTot ;   % min Z for max Total TRUE
elseif diagntype==7
  ZRadius=z1; % set to curve-fitted value for max Total
elseif diagntype==8
  ZRadius=zmaxctLR; % set to min Z for max corrected Total LR
end

% netnames(:,nn)= netnam;
% nm_list=imls;
% ssfscoress(:,nn)=Zssf;
% refscoress(:,nn)=Zssfs(:,ngis);
% scoretype='VOXEL based Z Scores';
% n1s(:,nn)=norm1;
% n2s(:,nn)=norm2;
% diagntype=7;
% radii(:,nn)=ZRadius;
% criterion=2; %criterion (1=Z score, 2=Diagnostic Index, 3=Priority GIS order, 4=Differential ZScore)
% function vp_MDiagnRpt(outfile,netnames,nm_list,ssfscoress,scoretype,refscoress,n1s,n2s,diagntype,radii,criterion)

%nm=vp_nm('nm',netnam); 
%rptnm=[nm(1,1:2),num2str(nnorms),num2str(nabnorms),outnm];
%if (diagntype >= 0 & diagntype <= 7)
  %  DIAGNrpt_name=sprintf('%s_%dDIAGNrpt.txt',rptnm,diagntype);
  %  vp_DIAGNrpt(DIAGNrpt_name,netnam,imls,Zssf,'VOXEL based Z Scores',Zssfs(:,ngis),norm1,norm2,diagntype,ZRadius);
  %  vp_dispfile(DIAGNrpt_name);
  %  vp_note(['END CHECK: ',ssf_name,' and ',DIAGNrpt_name],3);
%else
  %  vp_note(['END CHECK: ',ssf_name],3);
%end

vp_note(['CHECK: ',ssf_name],3);


set(FMenu,'Pointer','Arrow');

