% vp_ssm_roiOpca
% Perform ssm Orthogonal analysis on matrix of ROIs
%_______________________________________________________________________
% In this application of SSM analysis the covariance contribution of a specified
% GIS pattern is subtracted from a given set of ROI data before performing PCA.
% The user selects as input a GROUP SUMMARY COMPOSITE report (name_SUM_c.txt
% or name_SUM3D_c.txt) generated by  GROUP-ROI processing of a set of
% Talairach normalized subject patient and normal image files and optionally
% identifies normal or control subject indeces. 
% Optional, natural log transformation is performed in the routine
% prior to processing. Logs of negative data values are set to zero. 
% Data normalization is performed as a two step procedure. The mean
% value over all regions is evaluated for each subject and used
% to generate within subject regional deviations from the mean. 
% The mean of these deviations is then evaluated for each region over
% all subjects and subtracted from the corresponding subject value 
% to generate the subject residual profile (SRP). The contribution of
% a specified GIS is then evaluated using TPR and removed from the data.
% The new Orthogonal GIS and SSF values are evaluated as follows: 
% The sub-by-sub covariance matrix of the reduced SRP is evaluated.
% Eigenvectors and eigenvalues of the covariance matrix are computed.
% Eigenvectors (SSFs) are scaled to carry the weight of their eigenvalue.
% The percentage of 'Variance Accounted For' for each eigenvector is computed.
% The eigenvector matrix is multiplied by the SRP to generate the gismatrix array.
% GIS vector weights are normalized to correspond to Z-scores (weight - mean)/std.
% Z-scores are offset by the mean score for specified normals or controls.
% Relevant output files for SSF, SRP and GIS data are generated.
% 
% Results are saved in the input file directory. Results include a file
% containing the evaluated PCA's (GIS vectors) and a SSM_Summary text file 
% listing SSF values for all subjects and principal components and
% the "%Variance Accounted For"  for those components.
%_______________________________________________________________________
% Based in part on a routine written by Chris Habeck for voxel analysis 
% vp_roiOpca.m			ScAnVP 5.1   P.Spetsieris	7/23/2003
% vp_ssm_roiOpca.m		rev. ScAnVP 5.2   P.Spetsieris	9/22/2003
%				rev. ScAnVP 5.3   P.Spetsieris  4/13/2004
%				rev. ScAnVP 5.4   P.Spetsieris  11/21/2004
%				rev. ScAnVP 5.8   P.Spetsieris  6/07/2007

function [out1]=vp_ssm_roiOpca

global HelpMode FMenu POpt Keyin Message

if HelpMode, vp_help(mfilename), return ; end;

out1=0;
file_list=''; roi_list='';

prompt=['Orthogonal Component ROI PCA - Select Option'];
optA=['(A) Input SRP and GIS, Calc. GIS Score on SRP - reduce SRP - perform PCA on new SRP.'];
optB=['(B) Input ROI Summary File and GIS, determine SRP, Remove GIS contribution - perform PCA.'];
POpt=vp_opt('X',prompt,'Input SRP',optA,'Input ROI File',optB,'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_roiOpca'); vp_ssm_roiOpca; 
  return; end;
if(strcmp(POpt,'Input SRP')),
  vp_note(optA,3);
  opcaopt='A'; end;
if(strcmp(POpt,'Input ROI File')),
  vp_note(optB,3);
  opcaopt='B'; 
end

if opcaopt=='B',

  roimatrix_name= get_file(1,'*_c*.txt','Composite Summary GROUP ROI file',[ ]);  

  [file_list,roi_list,roimatrix]=vp_read_grprpt([roimatrix_name]);
  if isempty(roimatrix)
    set(FMenu,'Pointer','Arrow');
    vp_message('Error: Could Not Read Report'); return;
  end
  roimatrix=roimatrix';

  nrois=size(roimatrix,1);
  subs=size(roimatrix,2);
  if (subs <= 1)
    set(FMenu,'Pointer','Arrow');
    vp_message(['Error: Number Of Subjects (',num2str(subs),') Must Be > 1']);
    return;
  else
    prompt=['Read ',num2str(subs),' Subjects, ',num2str(nrois),' Regions. '];
  end

  log_transform=1; % add log transformation option
  prompt=[prompt,' Log-transform the data? neg. values and logs will be set to zero [Y|N|EXIT]'];
  POpt=vp_opt('X',prompt,'YES','Perform Natural Log Transformation','NO','Pre Transformed Data','EXIT','');
  if (POpt=='X' | strcmp(POpt,'EXIT')), 
	set(FMenu,'Pointer','Arrow');
	vp_note('Program Exited',2); return;
  elseif strcmp(POpt,'NO'), log_transform=0,
  else log_transform=1;
  end;

  if log_transform
    roimatrix(find(roimatrix<=0))=exp(0); % Set all values <=0
    roimatrix=log(roimatrix); % perform log transformation
  end;

  hwaitbar=waitbar(0,'Determining ROI SRP ...');
  waitbar(.3,hwaitbar);

  %Compute within image mean vector
  meanvect=mean(roimatrix);

  waitbar(.5,hwaitbar);

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  meanmat=ones(nrois,subs);
  for x=1:subs
     meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
  end
  devs=roimatrix-meanmat;

  waitbar(.7,hwaitbar);

  %Compute mean image of voxels across subjects
  %Needs to be NORMALIZED IMAGES i.e. devs
  mean_vec=mean(devs');

  %Compute subject deviations from mean image
  mean_vec=mean_vec';
  mean_vecmat=ones(nrois,subs);
  for x=1:nrois
     mean_vecmat(x,:)=mean_vec(x,1)*mean_vecmat(x,:);
  end
  devs=devs-mean_vecmat;
  srp=devs';

  waitbar(1,hwaitbar); pause(1); close(hwaitbar);

  
elseif opcaopt == 'A'
% disp('Enter Name of SRP i.e. SRP (sub-by-roi matrix)');
  srpname= get_file(1,'*srp*.txt','Enter srp file name (nsub-by-nroi matrix)',[ ]);  
  if isempty(srpname), return; end;
  
  %Determine type of file
  subs=0; nrois=0; mode=0;
  fid=fopen(srpname);
  if fid < 1, vp_message('Cannot Open File'); return; end;
  [str,count]=fscanf(fid,'%s',1),
  if count==0 
        vp_note('Empty File');
	fclose(fid);
	return;
  end
  numstr=str2num(str);
  if ~isempty(numstr)
        mode=2; %unformatted
	fclose(fid);
        vp_note('Detected Unformatted File',2);
  else
        subs=1;
        mode=3; %formatted type [Sub, Srp roi values] 
	if strcmp(str,'subject')
	  mode=1; %type ['subject' no (srp roi values)]
	  [str,count]=fscanf(fid,'%s',1); % dump no
	  if count==0, vp_note('EOF encountered');
	    mode=-1; fclose(fid); return;
	  end;
	end
        [str,count]=fscanf(fid,'%s',1);
	if count==0, vp_note('EOF encountered');
	  mode=-1; fclose(fid); return;
	end;
	numstr=str2num(str);
	if ~isempty(numstr)
          vp_note(['Detected [Subject SRP Values] Formatted File Type ' num2str(mode)],2);
	  % determine no. of rois
	  noroi=1; nrois=1;
	  srp(subs,noroi)=numstr;
	  while (1)
        	[str,count]=fscanf(fid,'%s',1);  
		if count==0, break; end;	 %EOF encountered
		
		numstr=str2num(str);
		if ~isempty(numstr)
	  		noroi=noroi+1;
	  		srp(subs,noroi)=numstr;
			if subs==1, nrois=noroi; end;
		else % new subject
		  if noroi ~= nrois, vp_note('Invalid File Type');
		  mode=-1; fclose(fid); return; end;
		  subs=subs+1;
		  noroi = 0;
	          if mode==1, [str,count]=fscanf(fid,'%s',1); % skip subject no
	              if count==0, vp_note('Subject Number expected, EOF encountered');
	               mode=-1; fclose(fid); return; end;
	          end;
		end
	  end
	else
	  mode=-1;
          vp_note('Unknown or Outdated File Type');
	  fclose(fid);
	  return;
	end
  end
    
  if mode==1 % Formatted ['subject' no (Roi SRP Values)] evaluated above
  elseif mode==2 % Unformatted
    srp=load([srpname]);
    nrois = size(srp,2);
    subs = size(srp,1);
  elseif mode==3 % Formatted [Name (Roi SRP Values)] evaluated above
  else
    vp_note('Invalid or Obsolete File Type');
    vp_ssm_roiopca;
    return;
  end
  vp_note([num2str(subs) ' Subjects, ', num2str(nrois), ' Regions'],2);

end % if opcaopt

% Specify Controls Option
prompt=['Specify Controls? '];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'SPECIFY CONTROLS','','NO CONTROLS','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify Control Subjects Images or Choose No CONTROLS ');
   end
end
  
if (strcmp(POpt,'SPECIFY CONTROLS'))
 nnorms=1; %determine later
else
 % No Specific Controls
 nnorms=0; norm1=0; norm2=0; Zoffset(subs)=0;
end


if nnorms > 0

 no=1;
 while no~= 2 % Must enter at least two numbers even if the same
   Keyin=['1 - ',num2str(subs)];
   prompt=['Specify Control Subject Index Range Within subjects [n1-n2] | q]   <',Keyin,'>   '];
   vp_keyin(prompt,Keyin);

   if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	      return;
   else
      
    Keyin=strrep(Keyin,' ','');
    [nn,no]=sscanf(Keyin,'%d-%d'),
    norm1=nn(1)
    if no ==2  norm2=nn(2) 
        if (norm1 < 1 | norm1 > subs), vp_note('Error: Number norm1 out of range',2);
          no=1;
        elseif (norm2 < 1 | norm2 > subs), vp_note('Error: Number norm2 out of range',2);
          no=1;
        elseif (norm1 == 1 & norm2 == subs), vp_note('Error: All subjects cannot be controls',2);
          no=1;
        end %if
    end; %if
   end %if 
 end % while
	
end

if nnorms > 0
  vp_note(['Control Subject Range: ',num2str(norm1),' - ',num2str(norm2)],2);
end

%Determining GIS Contributions

% disp('Enter Name of Gis values i.e. gis (roi-by-1 matrix)');
% disp('Enter Name of SRP i.e. SRP (sub-by-roi matrix)');

gisname= get_file(1,'*gis*.txt','Enter GIS(s) file (nroi-by-ngis matrix)',[ ]);  
if isempty(gisname), return; end;
%gismatrix=load([gisname]);
[gisout gismatrix]=vp_readgis(0,gisname);
if (gisout < 0), 
  vp_message('Error reading GIS'); return; end;


num_gis = size(gismatrix,2); 
if num_gis > 1
  prompt=['SELECT GIS For Reduction. NO.? [1-',num2str(num_gis),'| q]'];
  %vp_keyin('GIS NO.? [no.| q]','1');
  vp_keyin(prompt,'1');

  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  return, end;
  ngis=str2num(Keyin);
else
 ngis=1;
end
gis=gismatrix(:,ngis);

% num_roi = length(gis);
if nrois ~= size(gis,1),
  set(FMenu,'Pointer','Arrow');
  vp_message('Error: Number of GIS regions do not agree'); return;
end

% calculate ssf of gis
ssf=srp*gis;

% normalize ssf values
ssf=ssf/(norm(gis))^2;

% Reducing DATA

% subtract GIS-related activity
vp_note('Removing TPRd GIS pattern from the data',3);

devs = srp' - gis*ssf';
srp=devs';
   
hwaitbar=waitbar(0,'Performing ROI PCA on Reduced Data ...');
waitbar(.3,hwaitbar);

%Compute sub-by-sub Covariance matrix
%Tcovar=devs'*devs;
Tcovar=cov(devs);

waitbar(.6,hwaitbar);

%Compute Eigenvector (SSF) and Eigenvalue of Covariance matrix
[eigenvect eigenval]=eig(Tcovar);
eigenval=diag(eigenval);

%Convert Eigenvector (SSF) to carry weight of eigenvalue
%(SSF * sqrt(eigenvalue))
sqrt_eigenval=sqrt(eigenval');
sqrt_eigenvalmat=[]; % initialize
for x=1:subs
   sqrt_eigenvalmat(x,:)=sqrt_eigenval(1,:);
end
n_eigenvect=eigenvect.*sqrt_eigenvalmat;
ssf=n_eigenvect;

waitbar(.7,hwaitbar);

%Compute Percentage Variance Accounted For
percentvar=eigenval/sum(eigenval);
percentvar=percentvar';

%Multiply eigenvector matrix by devs to generate gismatrix array
gismatrix=eigenvect'*devs';
gismatrix=gismatrix';
           
waitbar(.8,hwaitbar);

%Normalize eigenvectors (Z-score region weights)
for x=1:subs
   gismatrix(:,x)=(gismatrix(:,x)-mean(gismatrix(:,x)))./std(gismatrix(:,x)); 
end

% first principal component first
gis = fliplr(gismatrix);
ssf = fliplr(ssf);
vaf = 100*fliplr(percentvar);

waitbar(.9,hwaitbar);

if opcaopt=='B'
 filnm=roimatrix_name,
else filnm=srpname,
  for i=1:subs, file_list{i}=['subject ',num2str(i)]; end;
  for i=1:nrois, roi_list{i}=['region  ',num2str(i)]; end;
  file_list=file_list'; roi_list=roi_list;
end;


% Name output files, strip 'SUM' or 'srp' from name

length_name = length(filnm);
k = findstr(filnm,'\');
pwdd = filnm(1:max(k));
p=max([findstr(filnm,'SUM') 0]);
if p==0
 p=max([findstr(filnm,'srp') 0]);
 if p==0
   name = filnm(1:length_name-4);
 else
   name = filnm(1:p-1);
 end
else
 name = filnm(1:p-1);
end
gis_name = strcat(name, 'O_gis.txt');
srp_name = strcat(name, 'O_srp.txt');
ssf_name = strcat(name, 'O_ssfRScores.txt');
%
Zssf_name = strcat(name, 'O_ssfZScores.txt');

% write relevant output files
vp_write_ssmfile(gis_name,subs,vaf,roi_list,gis,2)
vp_write_ssmfile(ssf_name,subs,vaf,file_list,ssf,1,norm1,norm2)
vp_write_srpfile(srp_name,nrois,file_list,srp)

save Ogismatrix.txt gis -ascii -tabs
save Osrp.txt srp -ascii -tabs
save OssfScores.txt ssf -ascii -tabs
save Opercentvar.txt vaf -ascii -tabs

if opcaopt=='B',
  % These vales are not effected by the orthogonal operation
  lmean_name = strcat(name, 'lmean.txt'),
  mean_name = strcat(name, 'mean.txt'),

  %Save Row Vector mean image for subsequent TPR analysis
  %
  mean_vec=mean_vec';
  if log_transform
    save(lmean_name,'mean_vec','-ascii');
  else
    save(mean_name,'mean_vec','-ascii');
  end
end

clear mean_vec mean_vecmat meanmat meanvect x
disp(' '); disp('');disp(' '); disp(''); 
disp ('   ***************************************************     ')
disp(' ');
disp('             RESULTS OF ROI ORTHOGONAL SSM ANALYSIS')
disp(' ');
disp('Unitized Subject SSF Scores (by columns):')
eigenvect
disp('Eigenvalues:')
eigenval'
disp('Subject SSF Scores carrying weight of eigenvalue (by columns):')
n_eigenvect
disp('Percentage variance accounted for by each SSF/GIS:')
percentvar
disp('Normalized GIS values are in variable "gismatrix" (by column).')
disp('Subject Residual Profiles are in variable "srp" (sub-by-roi matrix).')
disp('Weighted SSF scores are in variable "ssf" (One column per ssf).')
disp(' '); disp('');
disp ('   ***************************************************     ')

waitbar(1,hwaitbar); pause(1); close(hwaitbar);

ssfpcafig=figure('name','PCA ORTHO. SCORES for NEW GIS 1'); bar(ssf(:,1));
ylabel('PCA ORTHO. SCORE - GIS 1');
yo=max(ssf(:,1))/4;
%add Z transformation option
%Normalize SSF Scores (Z-score for each of the (n=subs) eigenvectors over subjects)
for x=1:subs % for each eigenvector
   Zssfs(:,x)=(ssf(:,x)-mean(ssf(:,x)))./std(ssf(:,x)); 
end

if nnorms > 0
  % Offset calculated for each gis
  if norm2 ~= norm1
    Zoffset=mean(Zssfs(norm1:norm2,:)),
  else
    Zoffset=Zssfs(norm1,:),
  end

  %Zssfs=Zssfs-Zoffset;
  for x=1:subs
    Zssfs(:,x)=Zssfs(:,x)-Zoffset(x)*ones(subs,1);
  end
  for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
  xlabel('Subject  (c: control)'); 
else
  xlabel('Subject  '); 
end


vp_write_ssmfile(Zssf_name,subs,vaf,file_list,Zssfs,3,norm1,norm2);
zscorefig=figure('name','PCA ORTHO. Z SCORES for NEW GIS 1'); bar(Zssfs(:,1));
ylabel('PCA ORTHO. Z SCORE - GIS 1');
yo=max(Zssfs(:,1))/4;

if nnorms > 0
  for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
  xlabel('Subject  (c: control)'); 
else
   xlabel('Subject'); 
end

pos=get(ssfpcafig,'position');
set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
figure(ssfpcafig);


%  Select Other Gis #
ngis=1; ngis0=1;
Keyin=num2str(ngis);
num_gis = size(ssf,2);
num_gis =num_gis -1; % skip the last one

netnam=['GIS ',num2str(ngis)];

if norm2 > 0
  labelx=['CONTROLS                SUBJECTS'];
else
  labelx=['             SUBJECTS'];
end

labely=['SCATTERPLOT -  Z SCORES for ',netnam];
vp_scatterplot(Zssfs(:,ngis),norm2,subs,0,labelx,labely,netnam);

while ~(isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  
  if ~strcmp(Keyin, 'Skip'),
    prompt=[' Save GIS Network ' num2str(ngis), ' Scores, Weights ? [Y|N|EXIT]'];
    POpt=vp_opt('X',prompt,'YES','Save GIS Network','NO','SKIP','EXIT','');
    if (POpt=='X' | strcmp(POpt,'EXIT')), 
	Keyin='';
    elseif strcmp(POpt,'NO'),
    else 
        % Save Gis Network
	vp_write_ssmfile([name,'O_gis',num2str(ngis),'.txt'],ngis,vaf(ngis),roi_list,gis(:,ngis),2)
        % Save Gis Reference Scores
	vp_write_ssmfile([name,'O_RScores',num2str(ngis),'.txt'],ngis,vaf(ngis),file_list,ssf(:,ngis),1,norm1,norm2)
	vp_write_ssmfile([name,'O_ZScores',num2str(ngis),'.txt'],ngis,vaf(ngis),file_list,Zssfs(:,ngis),3,norm1,norm2)
    end;
  end;

  if ~(isempty(Keyin)),
    if ngis > num_gis -1, ngis= num_gis -1; end;
    if (ngis < 1 ), ngis = 0; end;
    Keyin=num2str(ngis+1);
    prompt=['Display SCORES GIS No.? [1-',num2str(num_gis),'| q]'];
    vp_keyin(prompt,Keyin);

    if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
    else
      ngis=str2num(Keyin);
      if (ngis > num_gis | ngis < 1), vp_note('Error: Number out of range',2);
        ngis= ngis0; Keyin=['Skip'];
      else
        % Display bar graph
	netnam=['GIS ',num2str(ngis)];
        ssfpcafig=figure('name',['PCA ORTHO. SCORES for NEW ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(ssf(:,ngis));
	yo=max(ssf(:,ngis))/4;
	if norm2 > 0
	  for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
          xlabel('Subject  (c: control)'); ylabel(['PCA ORTHO. SCORE  -  ',netnam]);
	else
          xlabel('Subject'); ylabel(['PCA ORTHO. SCORE  -  ',netnam]);
	end
	
	% suppress
	%labely=['SCATTERPLOT - ORTHO. SCORES for ',netnam];
	%vp_scatterplot(ssf(:,ngis),norm2,subs,0,labelx,labely,netnam)
     
        zscorefig=figure('name',['PCA Z SCORES for ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(Zssfs(:,ngis));
	yo=max(Zssfs(:,ngis))/4;

	if norm2 > 0
	  for i=norm1:norm2, text(i,yo,'c','color','c'); end; %Mark controls
          xlabel('Subject  (c: control)'); ylabel(['PCA ORTHO. Z SCORE  -  GIS ',num2str(ngis)]);
	else
          xlabel('Subject'); ylabel(['PCA ORTHO. Z SCORE - GIS ',num2str(ngis)]);
	end

        pos=get(ssfpcafig,'position');
        set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
	
	labely=['SCATTERPLOT - ORTHO. Z SCORES for ',netnam];
	vp_scatterplot(Zssfs(:,ngis),norm2,subs,0,labelx,labely,netnam)
        ngis0=ngis;
      end
    end 
  
  end %if ~
end


%vp_message(['END CHECK: ',gis_name,', ',srp_name,', ',ssf_name,', ']);
Message=['END CHECK: ',gis_name,', ',vp_nm('nm',srp_name),', ',vp_nm('nm',ssf_name)];
vp_message(Message);
set(FMenu,'Pointer','Arrow');

