% vp_ssm_roitpr
% ROI based SSM Interactive Topographic Profile Rating (TPR)
% Performs evaluation of new subject(s) SSM subject scores for a specific GIS
%_______________________________________________________________________
%
% TPR    (Topographic Profile Rating)
%               New Subject Score Evaluation.
%               I) Interactive Options, Reference, Methods, Log. etc.
%               II) Automatic: Predefined Methods and File References.
%
% INTERACTIVE OPTIONS
% The user  (1) Selects a GIS principal component text file.
%           (2) Selects a data summary file of images to perform TPR evaluation
%		NOTE: for Recalibration, the summary file should include a compete
%		set of subjects and controls and step 3 is not needed.
%           (3) Selects a reference data summary file to calculate reference mean.
%
% Using automatic or interactive TPR, the user may determine new subject scores 
% for a previously derived GIS topographic profile using reference data
% to scale results or using new group data to recalibrate reference scores.
% The user selects as input a new GROUP SUMMARY COMPOSITE roi report 
% name_SUM(3D)_c.txt) generated by GROUP-ROI processing of ONE or more (ns)
% new Talairach normalized subject image files to be scored for their
% manifestation of the previously derived disease specific GIS.
% If previous reference data is used, a single subject or a new group can
% be evaluated with no difference in individual subject scores when obtained
% singularly or as part of a composite group. With the 'recalibration'
% option scores may shift depending on the new group selected as reference.
% Subject groups do not necessarily include subjects from the previous study.
% The reference group must be carefully selected to avoid a significant shift.
% Natural log transformation of the report data is usually performed in the
% routine unless the non-log option is selected. Logs of negative data values
% are set to zero.
% The user must select the GIS text file containing the definition of the  
% disease specific topographic profile. (The GIS is defined in terms of its
% relative, Z-scored weights for the (nr) regions used in the PCA analysis.)
% Data normalization to derive the new subject residiual profile (SRP) is
% performed as a two step procedure. First the mean value over all regions
% is evaluated for each subject and used to generate within subject regional
% deviations from the mean. Second the subject's difference from the group mean
% deviations profile is evaluated as described below to generate the SRP. 
% To derive subject scores for the GIS on the same scale as the original study,
% the user must either input the text file (study_lmean.txt) containing the
% (log) data group mean regional profile to be used as a reference from which
% the SRP values will be calculated or specify a reference GROUP SUMMARY COMPOSITE
% roi report to be used to evaluate this mean. If such a file is not available
% or if the user needs to recalibrate reference scores for the GIS, the new group data 
% can be used to generate the mean assuming that a large enough number of subjects
% including controls (similar to the original reference study used to generate the
% GIS profile) is used. Once the specified or evaluated group mean regional profile
% of the reference data is subtracted from the corresponding new subject values to
% generate the new subject residual profile (SRP), the new SSF subject scores are
% evaluated using the following calculation:
% 
% 	          SSF=SRP*GIS/fact, fact=norm(GIS)*sqrt(nr-1)
%
% where SSF is an nsx1 vector, SRP is a nsxnr matrix and GIS is a nrx1 vector.
% An output file containing the new SSF scores for each new subject is generated
% and plotted. Z scores can then be calculated based on derived or input reference
% subject scores. Control subjects are read from the input file or specified. 
% Results are saved in the input file directory.
%
% See also help on specific options.
%_______________________________________________________________________
% vp_ssm_roitpr                 ScAnVP 5.1   P.Spetsieris   5/29/2003
%                 (revision)    ScAnVP 5.3   P.Spetsieris   3/15/2004
%                 (revision)    ScAnVP 5.4   P.Spetsieris   5/18/2005
%                 (revision)    ScAnVP 5.5   P.Spetsieris   6/23/2005
%                 (revision)    ScAnVP 5.6   P.Spetsieris   11/18/2005
%_______________________________________________________________________

global HelpMode FMenu NameLst POpt
global GisOri GisLab GisWeight
global Message Ecolor Op ImOpFactor Keyin
global hwaitbar AutoTPR

if HelpMode, vp_help(mfilename); help('vp_ssm_autoroitpr'); return ; end;

out1=0;
ImOpFactor=1;
Message=['MSG:.................'];
Bcolor=[0.8 .85 .95];
%Mcolor=[0.66 .63 .80];
%Mcolor=[0.60 .62 .75];
Mcolor=[0.56 .65 .70];
Mcolor=[0.56 .70 .70];
AutoTPR=0;

tip1=['Principal Component Analysis'];
tip2=['Topographic Profile Rating (Automatic or Interactive'];
tip3=['Orthogonal Principal Component Analysis'];
tip4=['Select SSM roi Utility: Display SCORES, Generate MEAN, Thresh. MASK, Exclusive MASK'];


roissmtprgui=findobj('Name','ROI TPR');
if ~isempty(roissmtprgui)
	if(strcmp(get(roissmtprgui,'Visible'),'on'))
		figure(roissmtprgui);
	end
	return
end


roissmtprgui = figure('Color',Mcolor,'Name','ROI TPR','Tag','roissmtprgui',...
'NumberTitle','off','Position',[100,200,325,350],'UserData',0,...
'Units','Pixels');


uicontrol(roissmtprgui,'Style','Text','String','Scaled Subprofile Model',...
	'FontName','Times','FontSize',16,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[10/250 75/100 225/250 020/100],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
uicontrol(roissmtprgui,'Style','Text','String','(ROI Based TPR)',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[40/250 73/100 125/250 015/100],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
	
uicontrol(roissmtprgui,'Style','Text','String','AUTOMATED TPR',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'HorizontalAlignment','Left',...
	'Position',[30/250 59/100 120/250 015/100],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0])
	
titleAU=['USING REFERENCE FILES'];
titleAR=['GROUP RECALIBRATION'];
optAU=['(AU) Input new subject(s) and predefined GIS, determine Scores with respect to reference data.'];
optAR=['(AR) Input new GROUP and predifined GIS, determine new Scores and create reference directory. '];

uicontrol(roissmtprgui,'string', titleAU,...
	'TooltipString',optAU,...
	'Units','Normalized',...
	'position',[40/325 215/350 160/325 20/350],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_autoroitpr');
	
uicontrol(roissmtprgui,'string', titleAR,...
	'TooltipString',optAR,...
	'Units','Normalized',...
	'position',[40/325 190/350 160/325 20/350],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_autoroirecal');
	
	
uicontrol(roissmtprgui,'Style','Text','String','INTERACTIVE TPR',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'HorizontalAlignment','Left',...
	'Position',[30/250 34/100 130/250 015/100],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0])
	
titleA=['USING REFERENCE SUMMARY'];
titleB=['GROUP RECALIBRATION'];
optA=['Input GIS file, new subject and reference ROI summary files'];
optB=['Input GIS file, and ROI summary file of complete group of new subjects and controls'];

uicontrol(roissmtprgui,'string', titleA,...
	'TooltipString',optA,...
	'Units','Normalized',...
	'position',[40/325 128/350 180/325 20/350],...
	'BackgroundColor',Bcolor,'callback','REFscores=1; vp_ssm_roitprpro');
	
uicontrol(roissmtprgui,'string', titleB,...
	'TooltipString',optB,...
	'Units','Normalized',...
	'position',[40/325 103/350 180/325 20/350],...
	'BackgroundColor',Bcolor,'callback','REFscores=0; vp_ssm_roitprpro');
	
	
uicontrol(roissmtprgui,'string', 'HELP',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'Units','Normalized',...
	'position',[250/325 250/350 60/325 20/350],'callback',['vp_help(''vp_ssm_roitpr'');',...
	' vp_note(''See HELP window. Invoke Specific Task Help or Set Help Mode ON on Main Menu'');']);
	
uicontrol(roissmtprgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Units','Normalized',...
	'Position',[250/325 5/350 60/325 20/350],...
	'CallBack',['delete(roissmtprgui);',...
	'set(FMenu,''Pointer'',''Arrow'')';] );





