function [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2, vaf] =vp_ssm_scores(opt,ssffile)
% Display bar graph of SSF scores for specified SSM GISs.
%
% Reads a user selected formatted or unformatted output file
% of SSM Voxel or ROI based analysis containing SSF GIS scores
% and displays a bar graph of scores vs. subjects for a user
% specified GIS. Unformatted score files are text files of
% subject scores for one or more GISs listed as a separate
% column for each GIS. Formatted score files are output by
% SSM PCA analysis routines.
% Semi-formatted score files have two columns [subject score]
% output for a single GIS by the TPR routines.
% Formatted files may indicate the subject indeces for normals
% and may contain the string 'REF' or 'Z' indicating an initial
% reference score file or a Z score file, respectively.
%
% opt  -1: File name specified as argument 2; Do not prompt.
% opt  -2: File name specified as argument 2; Do not prompt or display
% opt   1: Prompt for ROI Based Score File
%       2: Prompt for Reference ROI Based Score File
%       3: Prompt for Voxel Based Score File
%       4: Prompt for Reference Voxel Based Score File
%	0: Prompt for any Score File
%
% Return Values: 
% mode 
%      -1: error
%       1: formatted
%       2: unformatted
%       3: TPR semiformatted [subject score] Type
%       4: REF score type 
%       5: Z score type
%
% ssfscores: SSF Score values
% Zssfs    : Z-scored SSF values
% norm1    : First Control Index
% norm2    : Last Control Index
% vaf      : %Variance-accounted For
%________________________________________________________________________
% function vp_ssm_scores  ScAnVP 5.2	P. Spetsieris	12/18/03
% function [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2, vaf] =vp_ssm_scores(opt)
%
% Revision ScAnVP 5.3 by PS 8/5/04
% Revision ScAnVP 5.4 by PS 12/15/04
% Revision ScAnVP 5.8 by PS 06/29/07
% Revision ScAnVP 6.2 by PS 09/22/11 Display scatterplot for TPR files
%

global HelpMode Keyin
global POpt NameLst

if HelpMode, vp_help(mfilename); return ; end;

%if nargin > 1, opt=-1; end;
if nargin < 2, ssffile=[]; end;
if nargin < 1, opt=0; end;
mode=-1;
ssfscores=[]; Zssfs=[];
norm1=0; norm2=0;
ngis=0; num_gis=0;
vaf=[];


  if opt==0
    ssffile=get_file(1,'*txt','Select ROI or Voxel SSF GIS Scores File'),
  elseif opt==1,
    ssffile=get_file(1,'*txt','Select ROI SSF GIS Scores File'),
  elseif opt==2
    ssffile=get_file(1,'*txt','Select REFERENCE ROI SSF GIS Scores File'),
  elseif opt==3
    ssffile=get_file(1,'*txt','Select VOX SSF GIS Scores File'),
  elseif opt==4
    ssffile=get_file(1,'*txt','Select REFERENCE VOX SSF GIS Scores File'),
  end

  if isempty(ssffile), return; end;
  SSFOpt=0;
%Determine type of file
  if (opt ~= -2) vp_dispfile(ssffile); end;
  fid=fopen(ssffile);
    [SSMType,count]=fscanf(fid,'%s',1),
    if count==0 
        vp_note('Empty File');
	return;
    end
    if ~strcmp(SSMType,'SSM'), 
      numstr=str2num(SSMType);
      SSMType=['Unformatted'];
      if ~isempty(numstr)
        mode=2; %unformatted
	fclose(fid);
        vp_note('Detected Unformatted File',2);
      else
        [str,count]=fscanf(fid,'%s',1);
	if count==0, vp_note('EOF encountered'); return; end;
	numstr=str2num(str);
	if ~isempty(numstr)
          SSMType=['TPR '];
          mode=3; %TPR semiformatted type [sub, score] 
          vp_note('Detected [Subject Score] TPR File Type',2);
	else
	  %vp_ssm_scores;
          vp_note('Unknown or Outdated File Type');
	  return;
	end
      end
    else % SSM formatted 
      [SSMType,count]=fscanf(fid,'%s',1),
      if (strcmp(SSMType,'Voxel') | strcmp(SSMType,'VOX')), 
        SSFOpt=1; mode=1; %formatted
        vp_note('Detected SSM Voxel Formatted File',2);
      elseif strcmp(SSMType,'ROI'), 
        SSFOpt=2; mode=1; %formatted
        vp_note('Detected SSM ROI Formatted File',2);
      else
        SSFOpt=0;
        vp_note('Not a Standard Formatted File');
	return;
      end;
    end


if mode==3  % TPR [subject score] Type
 num_gis=1; ngis=1; nosub=1;
 ssfscores(nosub,ngis)=numstr;
 while 1
    [str,count]=fscanf(fid,'%s',1);
    if count==0, %EOF 
      break;
    else
      [str,count]=fscanf(fid,'%s',1);
      if count==0, 
         vp_note(['Abrupt EOF. Incomplete DATA']);
    	 return; 
      else
         numstr=str2num(str);
         if ~isempty(numstr)
           nosub=nosub+1;
 	   ssfscores(nosub,ngis)=numstr;
         else
            vp_note(['Invalid String: ',str,' Detected']);
    	    return; 
	    break;
         end
      end;
    end;
 end %while
 if nosub==1, vp_note(['subject: ',SSMType,' SCORE: ',num2str(ssfscores(1,1))]);
 else
     % Display bar graph
     ngis=1;
     ssffig=figure('name',[SSMType ' SSF SCORES FILE ' ssffile]);
     bar(ssfscores(:,ngis));
     xlabel('SUBJECT'); ylabel(['SSF SCORE  -  GIS ',num2str(ngis)]);


     %Score Scatterplot
     netnam=[SSMType ' SSF SCORES FILE ' vp_nm('nm',ssffile)];
     labely=['SCORES for ', vp_nm('nm',ssffile)];
     labelx=['CONTROLS                SUBJECTS'];
     
     Keyin='0';
     prompt=['NO. of CONTROLS? [0-',num2str(nosub-1),'| q]'];
     vp_keyin(prompt,Keyin);

     if isempty(Keyin), norm2=0;
     	labelx=['               SUBJECTS'];
     elseif (strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
		 return ; 
     else
                 norm2=str2num(Keyin);
                 norm1=1; %Assume controls start with 1
     end;

     if (norm2 > nosub | norm2 < 0), vp_note('Error: Out of Range',2);

     else
     % Display scatter plot 
       vp_scatterplot(ssfscores(:,ngis),norm2,nosub,0,labelx,labely,netnam);
       pos=get(gcf,'position'); pos(1)=pos(1)+50; set(gcf,'position',pos);
     end

     ssfscores,

 end
 return;

elseif mode==2  % Unformatted
 try
  ssfscores=load(ssffile);
 catch
  %vp_ssm_scores;
  vp_message('INVALID FILE TYPE ');
  return;
 end
 sz=size(ssfscores);
 nosub=sz(1), num_gis=sz(2), 
 if num_gis > nosub,
  vp_note(['WARNING No gis:' num2str(num_gis) ' > No subjects: ' num2str(nosub)'],3);
 end;
 
 nosubs=nosub;
 
else % Formatted

  a=''; count=1;
  while (~strcmp(a,'Components') & count==1)
    if (strcmp(a,'Component') & count==1)
      [ngis,count]=fscanf(fid,'%d',1);
      num_gis = 1;
      break;
    else
     if (strcmp(a,'REF') & count==1), mode=4; end;
     if (strcmp(a,'Z') & count==1), mode=5; end;
     [a,count]=fscanf(fid,'%s',1);
    end
  end;
  
  if ((mode==1) | (mode==4) | (mode==5))
  
    if (num_gis ~= 1)
      [a,count]=fscanf(fid,'%c',3);
      if count ~=3, vp_note('Error 66: Invalid File'); return; end;
  
      [num_gis,count]=fscanf(fid,'%d',1),
      if count ~=1, vp_note('Error 69: Invalid File'); return; end;
    end
  end
  
  [a,count]=fscanf(fid,'%c',13)
  if count ~=13, vp_note('Error 71: Invalid File'); return; end;
  
  [nosubs,count]=fscanf(fid,'%d',1)
  if count ~=1, vp_note('Error 74: Invalid File'); return; end;
  
  %Check for Control indeces
  [a,count]=fscanf(fid,'%c',1),
  if strcmp(a,','),  
    [a,count]=fscanf(fid,'%s',1),
    if (strcmp(a,'Controls') & count==1)
      norm1=0; norm2=0;
      [norm1,count]=fscanf(fid,'%d',1),
      [a,count]=fscanf(fid,'%c',1),
      [norm2,count]=fscanf(fid,'%d',1),
      if (norm2 < norm1) | (norm2 > nosubs),
       vp_note(['Invalid Control Indeces: ',num2str(norm1),'-',num2str(norm2)]);
      return; end; 
    end
  else
  end
  
  %[dump,count]=fscanf(fid,'%s',(2+2*num_gis)),
  %if count ~=(2+2*num_gis), vp_note('Error 76: Invalid File'); return; end;
  [dump, count]=fscanf(fid,'%s',(2+1*num_gis)),
  if count ~=(2+1*num_gis), vp_note('Error 76: Invalid File'); return; end;
  for x=1:num_gis,
      [vaf(x),count]=fscanf(fid,'%f',1);
      if count ~=1, vp_note('Error 229: vaf incorrect'); return; end;
  end
  vaf,
  
  NameLst=[''];
  for nosub=1:nosubs,
    [name,count]=fscanf(fid,'%s',1),
    if count ~=1, vp_note('Error 79: Invalid File'); return; end;
    if(strcmp(name,'subject')),
     [subno,count]=fscanf(fid,'%d',1); % dump number of subject
     name=[name,' ',num2str(subno)];
    end;
    if nosub==1,
      NameLst=char(name);
    else
      NameLst=char(NameLst,char(name));
    end;
    
    for j=1:num_gis, [ssfscores(nosub,j),count]=fscanf(fid,'%f',1);
        if count ~=1, vp_note('Error 82: Invalid File'); return; end;
    end;
  end;
end; %formatted
  
%Normalize SSF Scores (Z-score for each of the eigenvectors over subjects)
  for x=1:num_gis % for each eigenvector
         Zssfs(:,x)=(ssfscores(:,x)-mean(ssfscores(:,x)))./std(ssfscores(:,x)); 
  end

%Calculate Z score offset if normal indeces are indicated
if (norm2 ~= 0) 
   
     % Offset calculated for each gis

     if norm2 ~= norm1
        Zoffset=mean(Zssfs(norm1:norm2,:)),
     else
        Zoffset=Zssfs(norm1,:),
     end
    
     %Zssfs=Zssfs-Zoffset;
     for x=1:num_gis
        Zssfs(:,x)=Zssfs(:,x)-Zoffset(x)*ones(nosubs,1);
     end
end

%  Select Gis #

if (num_gis==1), ngis=1; ngisp=1; end; % 1 even if original not 1
if opt == -2, Keyin='quit';
else, ngis=0; ngisp=1; Keyin=num2str(ngis);
end

while ~(isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),

  if (ngis + 1) > num_gis, Keyin=num2str(ngis);
  else Keyin=num2str(ngis+1); end;
  if num_gis ~=1
    prompt=['GIS NO.? [1-',num2str(num_gis),'| q]'];
    vp_keyin(prompt,Keyin);
  end

  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
		 ngis=ngisp ; 
  else
   ngis=str2num(Keyin);
   if (ngis > num_gis | ngis < 1), vp_note('Error: Out of Range',2);
    ngis=ngisp -1;
   else
     % Display bar graph
     % ssffig=figure('name',['SSF SCORES GIS ',num2str(ngis)]); bar(ssfscores(:,ngis));
     ngisp=ngis;
     if (mode==5)
      if num_gis == 1
       ssffig=figure('name',[SSMType ' SSF Z SCORES for GIS, ' ssffile]); bar(ssfscores(:,ngis));
      else
       ssffig=figure('name',[SSMType ' SSF Z SCORES GIS ',num2str(ngis) ', ' ssffile]); bar(ssfscores(:,ngis));
      end
      %set(gca,'color',[.9 1.0 .9]);
     else
      if num_gis == 1
       ssffig=figure('name',[SSMType ' SSF SCORES for GIS, ' ssffile]); bar(ssfscores(:,ngis));
      else
       ssffig=figure('name',[SSMType ' SSF SCORES GIS ',num2str(ngis) ', ' ssffile]); bar(ssfscores(:,ngis));
      end
     end
     %ssffig=figure('name',[ssffile ' ' SSMType ' SSF SCORES GIS ',num2str(ngis)]); bar(ssfscores(:,ngis));

     if num_gis == 1

       if norm2 ~= 0
          xlabel('SUBJECT   (Control: __ __ )'); ylabel('SSF SCORE  ');
       else
          xlabel('SUBJECT '); ylabel('SSF SCORE  ');
       end

     else
       if (norm2 ~= 0)
         xlabel('SUBJECT   (Control: __ __ )'); ylabel(['SSF SCORE - GIS ',num2str(ngis)]);
       else
         xlabel('SUBJECT'); ylabel(['SSF SCORE  -  GIS ',num2str(ngis)]);
       end
     end
     
     if (norm2 >= 0) 

      if (norm2 ~= 0) 
       ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
       text(norm1-.5,ypos,'<','color','b');  %Mark controls
       text(norm2,ypos,'>','color','b');  %Mark controls
       line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
      end

       %Score Scatterplot
       if num_gis == 1, netnam= ['GIS ', vp_nm('nm',ssffile)];
       else, netnam=['GIS ',num2str(ngis),', ',vp_nm('nm',ssffile)]; end; 
       labely=['SCORES for ',netnam];
       if (norm2 ~= 0) 
         labelx=['CONTROLS                SUBJECTS'];
       else
         labelx=['                SUBJECTS'];
       end
       vp_scatterplot(ssfscores(:,ngis),norm2,nosubs,0,labelx,labely,netnam);
       pos=get(gcf,'position'); pos(1)=pos(1)+50; set(gcf,'position',pos);
       
       % Zscores
       pos=get(gcf,'position'); pos(1)=pos(1)+50;
       if mode==5
        if num_gis == 1
         zscorefig=figure('name',['RECALCULATED Z SCORES for GIS, ' ssffile]); bar(Zssfs(:,ngis));
         if (norm2 ~= 0) 
           xlabel('SUBJECT   (Control: __ __ )'); ylabel('Z SCORE  ');
	 else
           xlabel('SUBJECT   '); ylabel('Z SCORE  ');
	 end
	else
         zscorefig=figure('name',['RECALCULATED Z SCORES GIS ',num2str(ngis), ', ' ssffile]); bar(Zssfs(:,ngis));
         if (norm2 ~= 0) 
           xlabel('SUBJECT    (Control: __ __ )'); ylabel(['Z SCORE  (for GIS ',num2str(ngis),')']);
         else
           xlabel('SUBJECT    '); ylabel(['Z SCORE  (for GIS ',num2str(ngis),')']);
         end
	end
       else
        if num_gis == 1
         ngis,
	 Zssfs,
         zscorefig=figure('name',['Z SCORES for GIS, ' ssffile]); bar(Zssfs(:,ngis));
         if (norm2 ~= 0) 
           xlabel('SUBJECT   (Control: __ __ )'); ylabel('Z SCORE  ');
	 else
           xlabel('SUBJECT    '); ylabel('Z SCORE  ');
	 end
	else
         zscorefig=figure('name',['Z SCORES for GIS ',num2str(ngis), ', ' ssffile]); bar(Zssfs(:,ngis));
         if (norm2 ~= 0) 
           xlabel('SUBJECT    (Control: __ __ )'); ylabel(['Z SCORE  (for GIS ',num2str(ngis),')']);
         else
           xlabel('SUBJECT    '); ylabel(['Z SCORE  (for GIS ',num2str(ngis),')']);
         end
	end
       end

       %set(gca,'color',[.92 .9 .95]);

       if (norm2 ~= 0) 
         ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
         text(norm1-.5,ypos,'<','color','b');  %Mark controls
         text(norm2,ypos,'>','color','b');  %Mark controls
         line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
       end

       pos=get(gcf,'position'); pos(1)=pos(1)+ 100; set(gcf,'position',pos);
       
       %Z Score Scatterplot
       labely=['Z SCORES for ',netnam];
       if (norm2 ~= 0) 
         labelx=['CONTROLS                SUBJECTS'];
       else
         labelx=['                SUBJECTS'];
       end
       vp_scatterplot(Zssfs(:,ngis),norm2,nosubs,0,labelx,labely,netnam);
       pos=get(gcf,'position'); pos(1)=pos(1)+ 150; set(gcf,'position',pos);
       
       
       %Save REF GIS
       if ((mode==4) & (num_gis > 1))
       
	  prompt=[' Save GIS Network ' num2str(ngis), ' REF Scores ? [Y|N]'];
	  POpt=vp_opt('X',prompt,'YES','Save GIS Scores','NO','SKIP');
	  if (POpt=='X'), Keyin=''; 
	  elseif strcmp(POpt,'NO'),
	  else 
	        % Save Gis Reference Scores

		p=deblank(ssffile);
		q     = length(p);
		s=max([findstr(p,'_VScores') 0]);
		if s~=0, refname=p(1:s-1);
		else
		  refname=['S',num2str(nosubs)];
		end;
		vp_write_ssmfile([refname,'_VScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,ssfscores(:,ngis),4,norm1,norm2)
		vp_write_ssmfile([refname,'_ZScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,Zssfs(:,ngis),5,norm1,norm2)
	  end;
       
       end
       
     end

     
     
   end
  end 
 
  
  if (num_gis == 1), Keyin='q';
  else
    Keyin='y';
    prompt=['MAKE A DIFFERENT SELECTION?  [ y | n ]'];
    vp_keyin(prompt,Keyin);
    if ~strcmp(Keyin,'y')
       Keyin='q';
    end
  end
  
end


