function out1=vp_ssm_subvoxpca;
% vp_ssm_subvoxpca
% User interface to invoke voxel based subtraction SSM-PCA analysis.
%_______________________________________________________________________
% vp_ssm_subvoxpca allows the user to select files interactively and invokes
% the function vp_ssmpca to perform SSM-PCA analysis on the log difference of
% two corresponding sets of ANALYZE format normalized images representing
% e.g. two different conditions of the same subject.
%
% The user picks the images to be analyzed and an optional (same size) mask file.
% Data can be raw Talairach normalized, smoothed and realigned images (NORM)
% or previously prepared (optionally log-transformed) output (PREP data).
% For PREP data no log-transform or mean offset is performed by the routine.
% A mask file can be created using other ScAnVP options or from raw data
% or can be the default gray PET mask or other predefined mask file.
% For NORM data (non-PREP), a lower threshold fraction can be specified to
% create a mask based on the combined input images. The header file of the 
% first input image is used to specify geometric specifications for all images.
% It is assumed that the images have been Talairach normalized and have 
% consistent orientation but need not have the same scalefactor.
% The program prompts for a name to be assigned to output files. The user
% interface routine then calls the MATLAB 'PCA' function to perform the analysis.
% The SRP option is used for PREP data irregardless of the preparation method:
% 'vp_ssmpca(Names.lst, [Thresh | mask] | [SRP mask] ],[NOLOG], output.txt)'
% Natural log transformation is by default performed in the ssmpca routine
% for NORM data. If requested, a 'NOLOG' command line option can be specified.
% A string 'SUBTRACTION: #' is added on the first line of the Names.lst to
% indicate subtracton of 'before (Input 1)' from 'after (Input 2)' (log) image 
% data. (Note that for LOG data subtraction of A from B is equivalent to 
% evaluating log(B/A).) This subtraction is performed before the subject and
% regional voxel mean is subtracted to generate the resultant subject residual
% profiles and subject by subject covariation matrix. 
% SRP output files are output if optionally requested by the user.
% PCA analysis of the subject by subject covariation matrix is then performed
% and results are saved in the user directory. Output includes the evaluated
% PCA's (GIS images), the data mask used by the routine, the group mean image
% of the (log transformed) data (after each subject's difference data is 
% normalized by subtracting it's voxel mean), the optional subject residual profile 
% images (deviations from the group mean image), a SSM summary text file:
% (Name_VScores.txt) listing SSF reference (REF) scores for all subjects for
% the first 16 principal components and the "%Variance Accounted For"  for
% those components. The output SSF REF Scores are then read in by the MATLAB
% routine and subject SSF Z-Scores are calculated for each of the eigenvectors.
% Individual GIS REF and Z-Score files can be saved.
%
%_______________________________________________________________________
% vp_ssm_subvoxpca	ScAnVP5.6    P. Spetsieris, Sept. 20, 2005 (calls DOS)
%	 revision	ScAnVP5.7.w  P. Spetsieris, Nov. 08, 2006
%	 revision	ScAnVP5.8    P. Spetsieris, June 5, 2007
%	 revision	ScAnVP5.9    P. Spetsieris, Mar 10, 2008
%	 revision	ScAnVP6.2    P. Spetsieris, Feb 6, 2012 (calls MATLAB function)
%	 revision	ScAnVP6.2.1  P. Spetsieris, Apr. 29, 2013 
%_______________________________________________________________________

global HelpMode NameLst POpt FMenu FName FNames MaskOpt Keyin
global MATYPE ORIGIN

if HelpMode, vp_help(mfilename), return ; end;

% get the image's filename {FName} 
%-----------------------------------------------------------------------
%FName      = FName(FName ~= ' ')

%NameLst      = get_file(Inf,'.img','Select images',[]);

PlaneNo=ORIGIN(3);

tip1=['Raw Realigned, Smoothed and Talairach Normalized'];
tip2=['Pre-Processed Data'];

rawdata=1;
prompt=['Select Talairach normalized, smoothed, raw images or pre-processed images'];
POpt=vp_opt('X',prompt,'NORM',tip1,'PREP',tip2,'EXIT','','HELP','');
if POpt=='X', 
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'PREP'),
	rawdata=0;
elseif strcmp(POpt,'EXIT'),
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'HELP'),
	vp_help('vp_ssm_subvoxpca'); vp_ssm_subvoxpca;
	return;
end;

if rawdata
  prompt=['SELECT ONLY CONDITION 1 (Talairach Normalized) IMAGES for VOX SUB SSM-PCA'];
  filter='*.img';
else
  prompt=['SELECT ONLY CONDITION 1 PREP (Talairach Normalized) IMAGES for VOX SUB SSM-PCA'];
  filter='*.img';
end

if vp_filesui(Inf,filter,prompt,mfilename) < 0 ,return; end;

ncond1= size(NameLst,1);
FNames=cellstr(NameLst);

if rawdata
  prompt=['SELECT ONLY CONDITION 2 SUBJECT (Talairach Normalized) IMAGES for VOX SUB SSM-PCA'];
  filter='*.img';
else
  prompt=['SELECT ONLY CONDITION2 SUBJECT PREP (Talairach Normalized) IMAGES for VOX SUB SSM-PCA'];
  filter='*.img';
end

if vp_filesui(Inf,filter,prompt,mfilename) < 0 ,return; end;

ncond2= size(NameLst,1);
if ncond2 ~= ncond1
  vp_note('The number of subjects for the two conditions are not equal.',2); return;
end

NameLst=cellstr(NameLst);
for i=1:ncond2, FNames(ncond1+i,:)=NameLst(i,:); end; 
NameLst=char(FNames),
m= size(NameLst,1),

if vp_checkNIFU(NameLst,5) %exit for UNIX and .nii files
 set(FMenu,'Pointer','Arrow');
return, end;

studyname = vp_nm('nm',NameLst(1,:));
vp_keyin('[Assign Output Files Study Name | q]',studyname);
if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
return, else studyname=Keyin; end;

    
prompt1=['Subjects: ',num2str(ncond1)];

log_transform=0; % PS add log transformation option
if rawdata
  log_transform=1; 
  prompt=[prompt1,',  Log-transform the data? neg. values & logs will be set to zero [Y|N|EXIT]'];
  POpt=vp_opt('X',prompt,'YES','Perform Natural Log Transformation','NO','Pre Transformed Data','EXIT','');
  if (POpt=='X' | strcmp(POpt,'EXIT')), 
	set(FMenu,'Pointer','Arrow');
	vp_note('Program Exited',2); return;
  elseif strcmp(POpt,'NO'), log_transform=0,
  else log_transform=1;
  end;
end


if rawdata
  prompt=['Select [Threshold|Default .35|Pre Defined Mask|EXIT]'];
  POpt=vp_opt('X',prompt,'Default','Thresh=.35','Thresh','Key Lower Threshold','Mask','Select Mask File','EXIT','');
else
  prompt=['Select [Mask|No Mask|EXIT]'];
  POpt=vp_opt('X',prompt,'Mask','Select Mask File','No Mask','','EXIT','');
end

if POpt=='X', 
	if rawdata
	  MaskOpt = ['T35'],
        else
	  vp_note('Program Exited',2); return;
        end
elseif strcmp(POpt,'EXIT'),
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'Mask'),
	vp_note('Mask must have the same dimensions as the image files.',3);
	MaskOpt = get_file(1,'*mask*.img','Select Mask File Name',[]);
	if (isempty(MaskOpt) ) return, end;

elseif strcmp(POpt,'Default'),
	    MaskOpt = ['T35'],
elseif strcmp(POpt,'Thresh'),
	thresh = -1;
	while (thresh < 0 | thresh >=1)
 	  vp_keyin('Lower Threshold   [Enter two digit decimal greater or equal to 0 and less than 1 | q]','.35');
	  if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
 	  return, else thresh=str2num(Keyin), end;
	  if (thresh >=0 & thresh <1)
	    if thresh ==0, Keyin='.00';, end;  % Equivalent to No Mask
	    nm=findstr(Keyin,'.');
	    if(size(Keyin,2) > nm+2), thresh = -1;
	    else
	      if(size(Keyin,2) < nm+2)Keyin(1,nm+2)='0'; end;
	      MaskOpt = ['T',Keyin(1,nm+1),Keyin(1,nm+2)],
	      vp_note(['Threshold= .',Keyin(1,nm+1),Keyin(1,nm+2)]);
	      if strcmp(MaskOpt,'T00'), thresh=0; end;
	    end
	  else
	    Message = ['Invalid Threshold: ',Keyin];
	    vp_message(Message); 
	  end
	end
else MaskOpt=[];
end;

MaskOpt,

meandata=0;
MeanOpt=[];

srpout=0; % PS add SRP output option
prompt=['Save All Resulting SRP and GIS > 16 Output? [Y|N|EXIT <N>]'];
  POpt=vp_opt('X',prompt,'YES','Save Output','NO','Do Not Save SRP and PCs > 16 Output','EXIT','');
  if (strcmp(POpt,'EXIT')), 
	set(FMenu,'Pointer','Arrow');
	vp_note('Program Exited',2); return;
  elseif strcmp(POpt,'NO'), srpout=0,
  elseif strcmp(POpt,'YES'), srpout=1,
  else srpout=0; %default for 'X'
  end;

%h=waitbar(0.3,'Processing SSM PCA');
%set(FMenu,'Pointer','Watch');

for i=1:m
  FName=deblank(NameLst(i,:));
  len=size(FName,2);
  
%  waitbar((i-.8)/m,h);
  if i==1
  	%dirnm=vp_nm('dir',FName);
	%FNameLst=[FName(1:(len-4)),'lst.txt'],
	dirnm=pwd;
	FNameLst=[dirnm,'\', studyname,'lst.txt'],
	fid= fopen(FNameLst,'w');

	if (fid == -1)
	  Message=['Error opening ' P '. Check that you have write permission.'];
	  vp_message(Message);
	  error(Message);
	  return; %PS
	end
	fprintf(fid,'%s\r\n',[FName(1:(len-4)),'.hdr   SUBTRACTION: ',num2str(ncond1)]);
	%fwrite(fid,[FName(1:(len-4)),'.hdr'],'char' );
  end
  %fwrite(fid,[FName(1:(len-4)),'.img'],'char' );
  fprintf(fid,'%s\r\n',[FName(1:(len-4)),'.img']);
end

fclose(fid);

if log_transform, LogOpt='LOG', else, LogOpt='NOLOG'; end;
if rawdata
 OutFile=[dirnm,'\subpcaoutput', studyname,'.txt'],
 if srpout, 
   ret=vp_ssmpca(FNameLst,MaskOpt,LogOpt,'SRPOUT',OutFile);
 else
   ret=vp_ssmpca(FNameLst,MaskOpt,LogOpt,OutFile);
 end;

 %DosCmd = ['ssmpca ',FNameLst,' ',MaskOpt,' ',LogOpt,' > ', OutFile],
else
 OutFile=[dirnm,'\subpcaoutput', studyname,'.txt'],
 if ~ischar(MaskOpt), MaskOpt=''; end;
 if srpout, 
   ret=vp_ssmpca(FNameLst,['SRP ',MaskOpt],'SRPOUT',OutFile);
 else
   ret=vp_ssmpca(FNameLst,['SRP ',MaskOpt],OutFile);
 end
 %DosCmd = ['ssmpca ',FNameLst,' SRP ',MaskOpt,' > ', OutFile],
end

disp('PROCESSING - WAIT FOR RESULTS');

%[ret,w]=vp_dos(DosCmd),
disp(' ');

%waitbar(1,h); pause(1); close(h);

if (ret >=0 ),
 vp_note(['END ssmpca- Check Matlab Window and ', OutFile]);
else,
 %vp_note('PROCESS FAILED, - Check Matlab Window and *output.txt');
 vp_note(['PROCESS FAILED, - Check Matlab Window and ',OutFile]);
 set(FMenu,'Pointer','Arrow');
 out1=POpt;
 return;
end;

% Z transformation, read REF Score file.
ssffile=[dirnm,'\',studyname,'_VScores.txt'];
[mode, ssffile, ngis, ssf, Zssfs, norm1, norm2, vaf] =vp_ssm_scores(-2,ssffile)
[sub, num_gis]=size(ssf),
if (norm1==0 | norm2==0)
% O.K. 
end;

ssfpcafig=figure('name',['PCA SCORES for GIS 1 VAF(',num2str(vaf(1)),'%)']);
bar(ssf(:,1)); xlabel('Subject'); ylabel('SSF SCORE  (for GIS1)');

%yo=max(ssf(:,1))/4;


%PS add Z transformation option
%Normalize SSF Scores (Z-score for each of the (n=num_gis) eigenvectors over subjects)
for x=1:num_gis % for each eigenvector
   Zssfs(:,x)=(ssf(:,x)-mean(ssf(:,x)))./std(ssf(:,x)); 
end

% NO CONTROLS IN THIS VERSION
% Offset calculated for each gis
%if norm2 ~= norm1
%  Zoffset=mean(Zssfs(norm1:norm2,:)),
%else
%  Zoffset=Zssfs(norm1,:),
%end

%for x=1:num_gis
% Zssfs(:,x)=Zssfs(:,x)-Zoffset(x)*ones(sub,1);
%end
    
refname=[dirnm,'\',studyname];
Zssf_name=[refname,'_VZScores.txt'];
vp_write_ssmfile(Zssf_name,num_gis,vaf,NameLst(1:sub,:),Zssfs,5,norm1,norm2);
zscorefig=figure('name',['PCA Z SCORES for GIS 1 VAF(',num2str(vaf(1)),'%)']);
bar(Zssfs(:,1)); xlabel('Subject'); ylabel('Z SCORE  (for GIS1)');

%yo=max(Zssfs(:,1))/4;
%for i=norm1:norm2, text(i,yo,'c','color','c'); end; %Mark controls

pos=get(ssfpcafig,'position');
set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
figure(ssfpcafig);


%  Select Other Gis #
ngis=1; ngis0=1;
Keyin=num2str(ngis);
netnam=['GIS ',num2str(ngis)];

if norm2 > 0
  labelx=['CONTROLS                SUBJECTS'];
else
  labelx=['             SUBJECTS'];
end

labely=['SCATTERPLOT - Z SCORES for ',netnam];
vp_scatterplot(Zssfs(:,ngis),norm2,sub,0,labelx,labely,netnam);

pos1=get(gcf,'position');
gisnam=[pwd,'\',studyname,'_gis',vp_num3str(ngis), '.img'], 
if ~isempty(gisnam), FI=vp_fimg(gisnam,PlaneNo,1,4,0); end;
pos2=get(gcf,'position');
set(gcf,'position',[pos1(1)-pos2(3), pos2(2), pos2(3), pos2(4)]);

while ~(isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  
  if ~strcmp(Keyin, 'Skip'),
    prompt=[' Save GIS Network ' num2str(ngis), ' Scores ? [Y|N|EXIT]'];
    POpt=vp_opt('X',prompt,'YES','Save GIS Scores','NO','SKIP','EXIT','');
    if (POpt=='X' | strcmp(POpt,'EXIT')), 
	Keyin='';
    elseif strcmp(POpt,'NO'),
    else 
        % Save Gis Reference Scores
	vp_write_ssmfile([refname,'_VScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,ssf(:,ngis),4,norm1,norm2)
	vp_write_ssmfile([refname,'_ZScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,Zssfs(:,ngis),5,norm1,norm2)
    end;
  end;

  if ~(isempty(Keyin)),
    if ngis > num_gis -1, ngis= num_gis -1; end;
    if (ngis < 1 ), ngis = 0; end;
    Keyin=num2str(ngis+1);
    prompt=['Display SCORES GIS No.? [1-',num2str(num_gis),'| q]'];
    vp_keyin(prompt,Keyin);

    if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
    else
      ngis=str2num(Keyin);
      if (ngis > num_gis | ngis < 1), vp_note('Error: Number out of range',2);
        ngis= ngis0; Keyin=['Skip'];
      else
        % Display bar graph
	netnam=['GIS ',num2str(ngis)];
        ssfpcafig=figure('name',['PCA SCORES for ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(ssf(:,ngis));
	%yo=max(ssf(:,ngis))/4;

	%%yo=0;
	%%if norm2 > 0
	  %for i=norm1:norm2, text(i,yo,'-','color','r'); end; %Mark controls
	%%  for i=1:norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
	%%  for i=1:norm1:norm2, text(i+.1,yo,'*','color','k'); end; %Mark controls
        %%  xlabel('Subject  (* : control)'); ylabel(['PCA SCORE  -  GIS ',num2str(ngis)]);
	%%else
        %%  xlabel('Subject'); ylabel(['PCA SCORE  -  ',netnam]);
	%%end

	if norm2 ~= 0
	  xlabel('SUBJECT   (Control: __ __ )'); ylabel(['PCA SCORE  -  GIS ',num2str(ngis)]);
	  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
	  text(norm1-.5,ypos,'<','color','b');  %Mark controls
	  text(norm2,ypos,'>','color','b');  %Mark controls
	  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
	else
	  xlabel('SUBJECT '); ylabel(['PCA SCORE  -  ',netnam]);
	end

	% suppress
	%labely=['SCATTERPLOT - SCORES for ',netnam];
	%vp_scatterplot(ssf(:,ngis),norm2,sub,0,labelx,labely,netnam)
     
        zscorefig=figure('name',['PCA Z SCORES for ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(Zssfs(:,ngis));
	%yo=max(Zssfs(:,ngis))/4;

	%%yo=0;
	%%if norm2 > 0
	  %for i=norm1:norm2, text(i,yo,'-','color','c'); end; %Mark controls
	%%  for i=1:norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
	%%  for i=1:norm1:norm2, text(i+.1,yo,'*','color','k'); end; %Mark controls
        %%  xlabel('Subject  (* : control)'); ylabel(['PCA Z SCORE  -  GIS ',num2str(ngis)]);
	%%else
        %%  xlabel('Subject'); ylabel(['PCA Z SCORE - GIS ',num2str(ngis)]);
	%%end

	if norm2 ~= 0
	  xlabel('SUBJECT   (Control: __ __ )'); ylabel(['PCA Z SCORE  -  GIS ',num2str(ngis)]);
	  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
	  text(norm1-.5,ypos,'<','color','b');  %Mark controls
	  text(norm2,ypos,'>','color','b');  %Mark controls
	  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
	else
	  xlabel('SUBJECT '); ylabel(['PCA Z SCORE  -  ',netnam]);
	end


        pos=get(ssfpcafig,'position');
        set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
	
	labely=['SCATTERPLOT - Z SCORES for ',netnam];
	vp_scatterplot(Zssfs(:,ngis),norm2,sub,0,labelx,labely,netnam)
        ngis0=ngis;

	pos1=get(gcf,'position');
        gisnam=[pwd,'\',studyname,'_gis',vp_num3str(ngis), '.img'], 
	if ~isempty(gisnam), FI=vp_fimg(gisnam,PlaneNo,1,4,0); end;
	pos2=get(gcf,'position');
	set(gcf,'position',[pos1(1)-pos2(3), pos2(2), pos2(3), pos2(4)]);

      end
    end 
  
  end %if ~
end


set(FMenu,'Pointer','Arrow');

out1=POpt;

