% vp_ssm_voxOpca.m
% SSM Principal Component Analysis of Orthogonal Components
%
% This special application of SSM is used to remove contributions due to principal
% components that dominate the analysis and allow a second PCA analysis to be performed
% in the reduced, orthogonal space to reveal any underlying significant components
% that may correlate with specific independent measures of interest.
%
% The user  (1) Selects an image component (e.g. a GIS) for evaluation and reduction.
%           (2) Selects a data set of images to perform TPR evaluation of SSF scores 
%               for that GIS and calculate the SRP component contribution.
%               Control specification is optional.
%           (3) Is given an option to remove the TPRd pattern from the data and
%               save the reduced images.
%           (4) Is given the option to perform PCA analysis in the orthogonal space.
%
%               The following options exist for the input data and evaluation method.
%               (A) Input SRP, Calc. GIS Score on SRP - reduce SRP - PCA on new SRP.
%               (B) Input raw data, determine SRP, continue as in (A).
%               (C) Input raw data, determine SRP, GIS Score on SRP - reduce INPUT - PCA.
%               (D) Input raw data, Calc. GIS Score on INPUT NOT SRP - reduce INPUT - PCA.
%
% The new data contains the original data with the TPRd pattern related activity
% removed. The user can then perform further analysis using SSM PCA.
%
% Based in-part on a routine tpr1.m written by Chris Habeck Ph.D. in association with
% Jim Moeller, PhD.Columbia College of Physicians and Surgeons 8/02
% vp_tpr1.m Modified for use with ScAnVP 4.7.8   P. Spetsieris, Ph.D.    9/18/02  
% vp_Opca.m		Rev. ScAnVP 5.1   by  P.S.    7/18/03
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.2   by  P.S.    10/02/03
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.4   by  P.S.    2/02/05
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.6   by  P.S.    11/21/05
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.7.w by  P.S.    11/08/06
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.8   by  P.S.    05/24/07
% vp_ssm_voxOpca.m	Rev. ScAnVP 5.9   by  P.S.    03/24/08
% vp_ssm_voxOpca.m	Rev. ScAnVP 6.2   by  P.S.    02/09/13
% vp_ssm_voxOpca.m	Rev. ScAnVP 6.2.1 by  P.S.    04/29/13
%_____________________________________________________________________________________

global HelpMode FMenu NameLst POpt FNames
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE GENTIM SUBID EXPDATE

if HelpMode, vp_help(mfilename), return ; end;

prompt=['Orthogonal Component PCA - Select Option'];
optA=['(A) Input SRP, Calc. GIS Score on SRP - reduce SRP - perform PCA on new SRP.'];
optB=['(B) Input raw data, determine SRP, continue as in (A).'];
optC=['(C) Input raw data, determine SRP, GIS Score on SRP - reduce input data - PCA.'];
optD=['(D) Input raw data, Calc. GIS Score on input data - reduce input data - PCA.'];
POpt=vp_opt('X',prompt,'A',optA,'B',optB,'C',optC,'D',optD,'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxOpca'); vp_ssm_voxOpca; 
  return; end;
if(strcmp(POpt,'A')),
  vp_note(optA,2);
  opcaopt='A'; end;
if(strcmp(POpt,'B')),
  vp_note(optB,2);
  opcaopt='B'; end;
if(strcmp(POpt,'C')),
  vp_note(optC,2);
  opcaopt='C'; end;
if(strcmp(POpt,'D')),
  vp_note(optD,2);
  opcaopt='D';
end

maskname=[]; masking=0;
log_transform=0; % PS 
im_data=[]; negzero_set=[];
nnorms=0; norm1=0; norm2=0; Zoffset=0;

prompt=['Select PCA GIS component for reduction'];
disp(prompt);
gisfile=get_file(1,'*gis*.img',prompt);
if isempty(gisfile), return; end;

% reshape into column vector and normalize to
% have Euclidean norm=1

%PS
[out1 gisimg]=vp_volfuns('volvect',gisfile);
if (out1 < 0) vp_message('ERROR reading gisfile'); return, end;

normgis=norm(gisimg); % PS save norm
gisimg=gisimg/normgis;

% Derive intrisic mask from gis image
gismask = (gisimg ~= 0);
% Don't count 0's 
% gismaskpos=find(gismask); gismasksum=size(gismaskpos,1),
gismasksum=sum(gismask),
if (gismasksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: GIS has no non-zero elements ']);
   return;
end

% Controls Option
%-----------------------------------------------------------------------
prompt=['Specify Controls? '];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'SPECIFY CONTROLS','','NO CONTROLS','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify Control Subjects Images or Choose No CONTROLS ');
   end
end
  
if (strcmp(POpt,'SPECIFY CONTROLS'))
 nnorms=1; % determine number later
else
 % No Specific Controls
 nnorms=0;
end


if nnorms > 0
  %PS get the image CONTROL filenames {FName} 
  if opcaopt=='A'
    prompt=['Select ONLY CONTROL GROUP Subject Residual Profile (SRP) Images'];
    if vp_filesui(Inf,'*srp*.img',prompt,mfilename) < 0 ,return; end;
  else
    prompt=['Select ONLY CONTROL GROUP Image Data set'];
    if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
  end
  disp(prompt);

  FNames=cellstr(NameLst);
  nnorms= size(NameLst,1);
end

%-----------------------------------------------------------------------

%PS get the image NON-CONTROL filenames {FName} 
if opcaopt=='A'
 if nnorms > 0
  prompt=['Select ONLY NON-CONTROL GROUP Subject Residual Profile (SRP) Images'];
 else
  prompt=['Select Subject Residual Profile (SRP) Images'];
 end
  if vp_filesui(Inf,'*srp*.img',prompt,mfilename) < 0 ,return; end;
else
 if nnorms > 0
  prompt=['Select ONLY NON-CONTROL GROUP Image Data set'];
 else
  prompt=['Select Image Data set'];
 end
 if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
end
disp(prompt);

%-----------------------------------------------------------------------
m= size(NameLst,1);
if nnorms > 0
  NameLst=cellstr(NameLst);
  for i=1:m, FNames(nnorms+i,:)=NameLst(i,:); end; 
else
  FNames=NameLst;
end

FNames=char(FNames),
sub= size(FNames,1),

if vp_checkNIFU(FNames,5), return, end; % Exit for UNIX and .nii files

set(FMenu,'Pointer','Watch');

hO=waitbar(0,'Processing images ...');

im_data=zeros(prod(DIM), sub, 'single');
for i=1:sub
  msg=['reading image ' FNames(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,hO,['Reading image ',num2str(i)]);
  
  [out1 im_data(:,i)]=vp_volfuns('volvect',FNames(i,:));
  if (out1 < 0) vp_message(['ERROR reading im_data file ',num2str(i)]); return; end;
end
%%%% Reading in images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% dim=size(im_data)
% im_data=reshape(im_data,prod(dim(1:3)),sub);

% reshape 3 dimensional images to one-dimensional vectors
waitbar(1,hO); pause(1); close(hO);
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PS Add masking option
  
% Alternative masking options
prompt=['External mask (image size) or GIS Mask'];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'EXTERNAL MASK','','GIS MASK','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify an external [0|1] mask image or use GIS/DATA intrinsic mask to remove insignificant voxels');
   end
end

% Get Data Intrinsic Mask
datmask=im_data(:,1) ~= 0;
for k=1:sub
 datmask=datmask.*(im_data(:,k) ~=0);
end

% Total Intrinsic Mask from GIS and Data
 datmask=datmask.*gismask;

  
if (strcmp(POpt,'EXTERNAL MASK'))
   vp_note('Mask must have the same dimensions as the image files.',5);
   maskname=get_file(1,'*mask*.img','Select Mask');
   if isempty(maskname), return; end;

   if vp_checkNIFU(maskname, 5) % Exit for UNIX and .nii files
	  set(FMenu,'Pointer','Arrow');
   return, end;

   %if ~strcmp(MATYPE,'uchar'),
   %	 vp_note('Mask must be 8bit (TYPE 2)',3); return;
   %end

   [out1 mask]=vp_volfuns('volvect',maskname);
   if (out1 < 0) vp_message('ERROR reading mask');return, end;

   % multiply by intrinsic mask
   mask=mask.*datmask;

else
 % Intrinsic GIS/DATA Mask Only
 mask=datmask;
 % Note, options C and D the maskspec entry is required by PCA but 
 % for intrinsic cases that is equivalent to threshold 0. 
 % For A and B (SRP) the mask is optional. The data is premasked by
 % the combination of any external and the intrinsic mask and the 
 % maskname is entered only for the record. The pca routine may 
 % re-mask with no actual effect. 
 if ((opcaopt == 'C') | (opcaopt == 'D'))
   maskname='T00'; % zero threshold
 end
end

masking=1; % Always Mask
% Note pca will record an input maskname but the data will be premasked

hO=waitbar(0,'Masking images ...');
for k=1:sub
     msg=['Masking Image ',num2str(k)];
     waitbar((i-.8)/sub,hO,msg);
     im_data(:,k)=im_data(:,k).*mask;
end
waitbar(1,hO); pause(1); close(hO);

if opcaopt ~= 'A',
 %PS log_transform=input('Log-transform the data, <=0 values will be set to zero (1==yes/0==no)? ');

 log_transform=1; % PS Force log-transformation
 if log_transform
   hO=waitbar(0,'Processing Log Transformation of Images ...');
   negzero_set=find(im_data<=0);
   im_data(negzero_set)=exp(0); % PS Set all values <=0
   % im_data=log(im_data); % PS perform log transformation
   for i=1:sub, im_data(:,i)=log(im_data(:,i)); %PS do one at a time to save memory
     msg=['Log Image ',num2str(i)];
     waitbar((i-.8)/sub,hO,msg);
   end;
   waitbar(1,hO); pause(1); close(hO);
 end;

 % log transform and tag regions that are not to be included in covariance computation
 % End of data scanning and log-transform 

 %if opcaopt == 'C',  im_D=im_data; end; % PS save log-transformed, masked data
 if opcaopt == 'C',  save im_D.mat im_data; end; % PS save log-transformed, masked data

 % Calculate SRP

 if opcaopt ~= 'D' % 'A' has been previously excluded
  hO=waitbar(0,'Calculating SRP ...');
  waitbar(.2,hO);
  voxno=size(im_data,1);
  sub=size(im_data,2);
  if (sub <= 1)
    set(FMenu,'Pointer','Arrow');
    vp_message(['Error: Number Of Subjects (',num2str(sub),') Must Be > 1']);
    close(hO);
    return;
  end

  %Compute within image mean vector
  if masking
  %PS Don't count 0's in mean
    maskpos=find(mask); masksum=size(maskpos,1),
    if (masksum == 0)
      set(FMenu,'Pointer','Arrow');
      vp_message(['Error: Mask has no positive elements ']);
      close(hO);
      return;
    end
    meanvect=sum(im_data)/masksum;
  else
    meanvect=mean(im_data); % But we always mask so doesn't apply
  end %if masking

  waitbar(.3,hO);

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  %PS Replace because of memory restrictions
  %meanmat=ones(voxno,sub); 
  %for x=1:sub
   % meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
  %end
  
  %devs=im_data-meanmat;
  %clear meanmat;
  
  %PS Save memory
  if masking
    for x=1:sub, im_data(:,x) = (im_data(:,x) - meanvect(1,x)).*mask ; end;
  else
    for x=1:sub, im_data(:,x) = im_data(:,x) - meanvect(1,x) ; end;
  end

  waitbar(.35,hO);

  %Compute mean image of voxels across subjects
  %Needs to be NORMALIZED IMAGES i.e. devs
  
  %PS mean_img=mean(devs');
  % mean_img=mean(im_data'); % Replace to save energy 
   mean_img=mean(im_data, 2); % Forms column vector or row means
   
  %Compute subject deviations from mean image
  % mean_img=mean_img'; % column formed above
  
  waitbar(.4,hO);
  for x=1:sub, %PS
   %devs(:,x)=devs(:,x) -mean_img;
   im_data(:,x)=im_data(:,x) -mean_img;
   waitbar(.4+.6*x/sub,hO);
  end;
  %mean_imgmat=ones(voxno,sub);
  %for x=1:voxno
  %  mean_imgmat(x,:)=mean_img(x,1)*mean_imgmat(x,:);
  %  if x==voxno*(.4), waitbar(.5,hO); end;
  %  if x==voxno*(.8), waitbar(.8,hO); end;
  %end
  %devs=devs-mean_imgmat;
  %im_data=devs ; %srp' 
  
  waitbar(1,hO); pause(1); close(hO);
  
 end % PS END IF NOT OPT D

end %PS END IF NOT OPT A

%PS ssf=im_data'*image/(normgis*normgis);
ssf=im_data'*gisimg,

disp(' ');
disp('Calculated SSF')
ssfopcafig=figure('name','PRE-OPCA (TPR) REF SCORES'); bar(ssf); xlabel('Subject'); ylabel('Subject PRE-OPCA REF SCORE');
%yo=max(ssf)/4; 

%%yo=0;
%%if nnorms > 0
  %for i=1:nnorms, text(i,yo,'-','color','r'); end; %Mark controls
%%  for i=1:nnorms, text(i,yo,'*','color','w'); end; %Mark controls
%%  for i=1:nnorms, text(i+.1,yo,'*','color','k'); end; %Mark controls
%%  xlabel('Subject  (* : control)');
%%else
%%  xlabel('Subject');
%%end;


if nnorms ~= 0
  xlabel('SUBJECT   (Control: __ __ )');
  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
  text(1-.5,ypos,'<','color','b');  %Mark controls
  text(nnorms,ypos,'>','color','b');  %Mark controls
  line([1-.5,nnorms],[ypos,ypos],'LineStyle','--');
else
  xlabel('SUBJECT ');
end



pos=get(ssfopcafig,'position');

% PS add Z transformation option
if sub > 1
    %Zssf=zscore(ssf),
    Zssf=(ssf - mean(ssf))./std(ssf),
    
    Zoffset=0;
    if nnorms > 1 
	Zoffset=mean(Zssf(1:nnorms)),
    elseif nnorms > 0 
	Zoffset=Zssf(1),
    end

    Zssf=Zssf-Zoffset;
    
    zscorefig=figure('name','PRE-OPCA Z REF SCORES','position',[pos(1)+10,pos(2)+20,pos(3),pos(4)]);
    bar(Zssf); xlabel('Subject'); ylabel('Subject PRE-OPCA Z REF SCORE');

    %yo=max(Zssf)/4;

    %%yo=0;
    
    %%if nnorms > 0
      %for i=1:nnorms, text(i,yo,'-','color','c'); end; %Mark controls
    %%  for i=1:nnorms, text(i,yo,'*','color','w'); end; %Mark controls
    %%  for i=1:nnorms, text(i+.1,yo,'*','color','k'); end; %Mark controls
    %%  xlabel('Subject  (* : control)');
    %%else
    %%  xlabel('Subject');
    %%end;
    
    if nnorms ~= 0
      xlabel('SUBJECT   (Control: __ __ )');
      ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
      text(1-.5,ypos,'<','color','b');  %Mark controls
      text(nnorms,ypos,'>','color','b');  %Mark controls
      line([1-.5,nnorms],[ypos,ypos],'LineStyle','--');
    else
      xlabel('SUBJECT ');
    end

    %figure(ssfopcafig); %place non-zscores on top
end

% PS Don't SAVE pre orthogonalization scores - do in TPR

% subtract GIS-related activity
vp_note('Removing TPRd pattern from the data');

%if (opcaopt == 'C') im_data=im_D - gisimg*ssf'; clear im_D;
% clear im_D; dos('del im_D.mat'); 
%else im_data=im_data-gisimg*ssf'; % for 'D' im_data and im_D are the same

if (opcaopt == 'C') load im_D.mat im_data;  dos('del im_D.mat'); 
end;

im_data=im_data - gisimg*ssf';

if masking
   hO=waitbar(0,'Masking new data ...');
   for k=1:sub
     msg=['Masking New Image ',num2str(k)];
     waitbar((k-.8)/sub,hO,msg);
     im_data(:,k)=im_data(:,k).*mask;
   end
   waitbar(1,hO); pause(1); close(hO);
end % masking
vp_note(['Array "im_data" contains the original data with the TPRd pattern related activity removed'],3)

%%%%% Output of reduced images

disp([' '])
disp(['SAVE ORTHOGONAL IMAGES and (optionally) PROCESS PCA ']);
% prompt=['SAVE ORTHOGONAL IMAGES and PROCESS PCA ? [Y|N=CANCEL PROCESSING]'];
% POpt=vp_opt('X',prompt,'YES','Save and process images','NO',['Don''t save images-CANCEL and EXIT']);

% Don't Ask, Assume Answer is YES
POpt='YES';
if POpt=='X', 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'NO'), 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'YES'), 
  
  % PS Reverse log-transformation - except for options A and B (SRP)
  if opcaopt ~= 'B' % option A never transforms and B saves log transformed
   if log_transform
     hO=waitbar(.4,'Processing Inverse Log Transformation of Images ...');
     im_data=exp(im_data); % PS perform exp transformation
     waitbar(.8,hO);

     %PS Set to zero original neg/zero unwanted values
     im_data(negzero_set)=0; % PS 
     waitbar(1,hO); pause(1); close(hO);
   end
  end
  
  clear negzero_set;
  
  dim=DIM
  dirnm=pwd;
  for i=1:length(FNames(:,1))

    vp_hread(FNames(i,:));
    filnm=deblank(FNames(i,:));
    k=max([findstr(filnm,'\') 0]),
    %if k >= 1, dir=filnm(1:k), else dir=[]; end;
    save_imfile=filnm(k+1:length(filnm)), 
    if (opcaopt=='A' | opcaopt=='B')
    	filnm=[dirnm,'\','Osrp_',save_imfile];
	imgstr='SRP image';
    else
    	filnm=[dirnm,'\','O_',save_imfile];
	imgstr='image';
    end
    
    if i==1, 
        if nnorms > 0
	 newlist = [filnm(1:length(filnm)-3),'hdr   CONTROLS: ',num2str(nnorms)];
        else
	 newlist = [filnm(1:length(filnm)-3),'hdr'];
        end
   	hO=waitbar(0,'Writing Reduced Images ...');
    end;

    newlist=strvcat(newlist, filnm);
    TYPE=16; SCALE=1; DESCRIP=['ScAnVP OPCA reduced ' imgstr int2str(i)]; 
    out1=vp_imsave(filnm,reshape(im_data(:,i),dim(1:3)), 1), 
    msg=['writing reduced image ' int2str(i) ' to ' filnm];
    waitbar((i-.8)/sub,hO,msg);
    if (out1 < 0),
      vp_message(['ERROR writing _OPCA file: ' filnm]);
      set(FMenu,'Pointer','Arrow');
      return;
    end;

  end
  waitbar(1,hO); pause(1); close(hO);
  
  clear im_data;
  
%PERFORM PCA ?
  
  prompt=['Images Saved. Perform PCA on Reduced Set? [Y|N=END PROCESSING|X=EXIT]'];
  POpt=vp_opt('X',prompt,'YES','Invoke PCA','NO','EXIT');
  if POpt=='X', 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
  
  elseif strcmp(POpt,'YES'), 

	%studyname = ['VoxOPCA',opcaopt];
	studyname = ['VoxOPCA'];
	vp_keyin('[Assign Orthogonal Output Files Study Name | q]',studyname);
	if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	set(FMenu,'Pointer','Arrow');
	return, else studyname=Keyin; end;
	studyname=[studyname,'_O',opcaopt];

	dirnm=pwd;
	FNameLst=[dirnm,'\',studyname,'lst.txt'],
	fid= fopen(FNameLst,'w');
	if(fid == -1)vp_message(['Error opening FNameLst.txt.']); return; end;
	for i=1:sub+1
	   fprintf(fid,'%s\r\n',newlist(i,:));
	end
	fclose(fid);
	
	opcaoutput=[studyname,'_output.txt'];
	if ((opcaopt == 'A') | (opcaopt == 'B'))
	 %DosCmd = ['ssmpca ',FNameLst,' SRP ',maskname,' > opcaoutput.txt'],
	 %No need to output SRP since it was previously saved
	 
	 ret=vp_ssmpca(FNameLst,['SRP ',maskname],opcaoutput);
	 
	%% else DosCmd = ['ssmpca ',FNameLst,' ',maskname,' > opcaoutput.txt'],
	else ret = vp_ssmpca(FNameLst,maskname,'LOG',opcaoutput);

	% Assumes we always log transform and SRP will NOT be output. If the
	% user wants SRP output, they can run the ssmpca GUI on the ortho. output.
	end; 
	
        hO=waitbar(.4,'Processing Orthogonal PCA ...');
      
	%[s,w]=dos(DosCmd),
	%

	%[ret,w]=vp_dos(DosCmd),
	disp(' ');
	waitbar(1,hO); pause(1); close(hO);
	if (ret >=0 ),
	 vp_note('END ssmpca- Check Matlab Window and opcaoutput.txt');
	else,
	 vp_note('PROCESS FAILED, - Check Matlab Window and opcaoutput.txt');
	end;

  end
  
end


% Z transformation, read REF Score file.
ssffile=[dirnm,'\',studyname,'_VScores.txt'],
[mode, ssffile, ngis, ssf, Zssfs, norm1, norm2, vaf] =vp_ssm_scores(-2,ssffile)
[sub, num_gis]=size(ssf),

% Controls Optional
%if (norm1==0 | norm2==0)
%  set(FMenu,'Pointer','Arrow');
%   vp_message(['Invalid Reference File: ',ssffile,' Control Subjects Not Specified']);
%return; end;

ssfpcafig=figure('name',['OPCA SCORES for GIS 1 VAF(',num2str(vaf(1)),'%)']); bar(ssf(:,1)); ylabel('OPCA SCORE - GIS 1');

%yo=max(ssf(:,1))/4;

%%yo=0;

%%if norm2 > 0
  %for i=norm1:norm2, text(i,yo,'-','color','r'); end; %Mark controls
%%  for i=1:nnorms, text(i,yo,'*','color','w'); end; %Mark controls
%%  for i=1:nnorms, text(i+.1,yo,'*','color','k'); end; %Mark controls
%%  xlabel('Subject  (* : control)');
%%else
%%  xlabel('Subject');
%%end;

%% Is This Correct or should it be nnorms ??
if norm2 ~= 0
  xlabel('SUBJECT   (Control: __ __ )'); 
  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
  text(norm1-.5,ypos,'<','color','b');  %Mark controls
  text(norm2,ypos,'>','color','b');  %Mark controls
  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
else
  xlabel('SUBJECT '); 
end


%PS add Z transformation option
%Normalize SSF Scores (Z-score for each of the (n=num_gis) eigenvectors over subjects)
for x=1:num_gis % for each eigenvector
   Zssfs(:,x)=(ssf(:,x)-mean(ssf(:,x)))./std(ssf(:,x)); 
end

%Zssfs(norm1:norm2,:),
% Offset calculated for each gis

Zoffset(1:num_gis)=0;
if norm2 > 0
  if norm2 ~= norm1
    Zoffset=mean(Zssfs(norm1:norm2,:)),

  else
    Zoffset=Zssfs(norm1,:),
  end
end;

for x=1:num_gis
  Zssfs(:,x)=Zssfs(:,x)-Zoffset(x)*ones(sub,1);
end
    
refname=[dirnm,'\',studyname];
Zssf_name=[refname,'_VZScores.txt'];
vp_write_ssmfile(Zssf_name,num_gis,vaf,FNames,Zssfs,5,norm1,norm2);
zscorefig=figure('name',['OPCA Z SCORES for GIS 1 VAF(',num2str(vaf(1)),'%)']);
bar(Zssfs(:,1)); ylabel('OPCA Z SCORE - GIS 1');

%yo=max(Zssfs(:,1))/4;

%%yo=0;
%%if norm2 > 0
  %for i=norm1:norm2, text(i,yo,'-','color','c'); end; %Mark controls
%%  for i=1:norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
%%  for i=1:norm1:norm2, text(i+.1,yo,'*','color','k'); end; %Mark controls
%%  xlabel('Subject  (* : control)');
%%else
%%  xlabel('Subject ');
%%end

if norm2 ~= 0
  xlabel('SUBJECT   (Control: __ __ )'); 
  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
  text(norm1-.5,ypos,'<','color','b');  %Mark controls
  text(norm2,ypos,'>','color','b');  %Mark controls
  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
else
  xlabel('SUBJECT '); 
end

pos=get(ssfpcafig,'position');
set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
figure(ssfpcafig);


%  Select Other Gis #
ngis=1; ngis0=1;
Keyin=num2str(ngis);

netnam=['GIS ',num2str(ngis)];

if norm2 > 0
  labelx=['CONTROLS                SUBJECTS'];
else
  labelx=['             SUBJECTS'];
end

labely=['SCATTERPLOT -  O Z SCORES for ',netnam];
vp_scatterplot(Zssfs(:,ngis),norm2,sub,0,labelx,labely,netnam);


while ~(isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
  
  if ~strcmp(Keyin, 'Skip'),
    prompt=[' Save GIS Network ' num2str(ngis), ' Scores ? [Y|N|EXIT]'];
    POpt=vp_opt('X',prompt,'YES','Save GIS Scores','NO','SKIP','EXIT','');
    if (POpt=='X' | strcmp(POpt,'EXIT')), 
	Keyin='';
    elseif strcmp(POpt,'NO'),
    else 
        % Save Gis Reference Scores
	vp_write_ssmfile([refname,'_VScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,ssf(:,ngis),4,norm1,norm2)
	vp_write_ssmfile([refname,'_ZScores',num2str(ngis),'.txt'],ngis,vaf(ngis),NameLst,Zssfs(:,ngis),5,norm1,norm2)
    end;
  end;

  if ~(isempty(Keyin)),
    if ngis > num_gis -1, ngis= num_gis -1; end;
    if (ngis < 1 ), ngis = 0; end;
    Keyin=num2str(ngis+1);
    prompt=['Display SCORES ORTHO GIS No.? [1-',num2str(num_gis),'| q]'];
    vp_keyin(prompt,Keyin);

    if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
    else
      ngis=str2num(Keyin);
      if (ngis > num_gis | ngis < 1), vp_note('Error: Number out of range',2);
        ngis= ngis0; Keyin=['Skip'];
      else
        % Display bar graph
	netnam=['GIS ',num2str(ngis)];
        ssfpcafig=figure('name',['OPCA SCORES for ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(ssf(:,ngis));
	%yo=max(ssf(:,ngis))/4;

	%%yo=0;
	%%if norm2 > 0
	  %for i=norm1:norm2, text(i,yo,'-','color','r'); end; %Mark controls
	%%  for i=norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
	%%  for i=norm1:norm2, text(i,yo,'*','color','k'); end; %Mark controls
        %%  xlabel('Subject  (* : control)'); ylabel(['OPCA SCORE  -  GIS ',num2str(ngis)]);
	%%else
        %%  xlabel('Subject'); ylabel(['OPCA SCORE  -  ',netnam]);
	%%end

	if norm2 ~= 0
	  xlabel('SUBJECT   (Control: __ __ )'); ylabel(['OPCA SCORE  -  GIS ',num2str(ngis)]);
	  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
	  text(norm1-.5,ypos,'<','color','b');  %Mark controls
	  text(norm2,ypos,'>','color','b');  %Mark controls
	  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
	else
	  xlabel('SUBJECT '); ylabel(['OPCA SCORE  -  ',netnam]);
	end

	if norm2 > 0
	  labelx=['CONTROLS                SUBJECTS'];
	else
	  labelx=['                SUBJECTS'];
	end

	% suppress
	%labely=['SCATTERPLOT -  O SCORES for ',netnam];
	%vp_scatterplot(ssf(:,ngis),norm2,sub,0,labelx,labely,netnam)
     
        zscorefig=figure('name',['OPCA Z SCORES for ',netnam,' VAF(',num2str(vaf(ngis)),'%)']); bar(Zssfs(:,ngis));
	%yo=max(Zssfs(:,ngis))/4;

	%%yo=0;
	%%if norm2 > 0
	  %for i=norm1:norm2, text(i,yo,'-','color','c'); end; %Mark controls
	%%  for i=norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
	%%  for i=norm1:norm2, text(i,yo,'*','color','k'); end; %Mark controls
        %%  xlabel('Subject  (* : control)'); ylabel(['OPCA Z SCORE  -  GIS ',num2str(ngis)]);
	%%else
        %%  xlabel('Subject'); ylabel(['OPCA Z SCORE - GIS ',num2str(ngis)]);
	%%end

	if norm2 ~= 0
	  xlabel('SUBJECT   (Control: __ __ )'); ylabel(['OPCA Z SCORE  -  GIS ',num2str(ngis)]);
	  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
	  text(norm1-.5,ypos,'<','color','b');  %Mark controls
	  text(norm2,ypos,'>','color','b');  %Mark controls
	  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
	else
	  xlabel('SUBJECT '); ylabel(['OPCA Z SCORE - GIS ',num2str(ngis)]);
	end

        pos=get(ssfpcafig,'position');
        set(ssfpcafig,'position',[pos(1)+40, pos(2)-60, pos(3), pos(4)]);
	
	labely=['SCATTERPLOT -  O Z SCORES for ',netnam];
	vp_scatterplot(Zssfs(:,ngis),norm2,sub,0,labelx,labely,netnam)
        ngis0=ngis;
      end
    end 
  
  end %if ~
end



set(FMenu,'Pointer','Arrow');
