% vp_ssm_voxOtpr.m
% SSM Topographic Profile Rating on Orthogonal Data 
%
% This application of SSM is used to remove the covariance attributed 
% to a given previously derived GIS from a group of one or more subject images 
% and evaluate the TPR scores due to another GIS on the reduced orthogonal
% image data. Both the SRP due to the first GIS and the Orthogonal SRP are saved
% for each subject. 
%
% The user:
%		(1) Selects an image pattern (GIS A) for TPR evaluation of subject 
%		    scores for that GIS and the SRP component contribution.
%		    The intrinsic GIS mask is derived and an optional additional
%		    mask may be input.
%		(2) Selects (a) a large group or (b) a small data set of images and
%		    evaluates the SRP using either:
%		    a) The evaluated group mean for group data evaluation.
%		    b) A predefined input lnmean image for single subject evaluation.
%		    The intrinsic lnmean mask is factored into the composite mask.
%		    Orthogonal SRP images and the masked lnmean image is saved.
%		(3) Is given the option to perform TPR evaluation in the orthogonal space
%		    to obtain subject scores for a second pattern (GIS B).
%
%		The following options exist for the input data: 
%		    (A) Input group raw data, determine SRP using the group mean log image.
%		    (B) Input one or more subject's data and previously determined group
%		    mean log image, determine SRP using input group mean log image.
%		
%		The final steps are the same:
%		GIS A score on SRP, evaluate the component due to GIS A (SRPA) reduce SRP
%		to obtain the orthogonal SRP (OSRP) - TPR of GIS B to derive scores 
%		in orthogonal space.
%
% vp_ssm_voxOtpr.m	ScAnVP 5.7   by  Phoebe Spetsieris    3/14/06
%			Rev. 5.9  by P.S. 3/08/08
%_____________________________________________________________________________________

global HelpMode FMenu NameLst POpt
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE Message GENTIM SUBID EXPDATE

if HelpMode, vp_help(mfilename), return ; end;

prompt=['Orthogonal Space TPR - Select Option'];
optA=['(GROUP) Input group data, determine SRP based on input group mean...'];
optB=['(NEW SUBJECT(S)) Input one or more subjects and determine SRP based on previously derived group mean...'];
POpt=vp_opt('X',prompt,'GROUP',optA,'NEW SUBJECT(S)',optB,'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxOtpr'); vp_ssm_voxOtpr; 
  return; end;
if(strcmp(POpt,'GROUP')),
  vp_note(optA,3);
  otpropt='A';
end;
if(strcmp(POpt,'NEW SUBJECT(S)')),
  vp_note(optB,3);
  otpropt='B'; 
end

maskname=[]; masking=0;
im_data=[]; negzero_set=[];
prompt=['Select GIS A Component for reduction'];
disp(prompt);
gisfile=get_file(1,'*gis*.img',prompt);
if isempty(gisfile), return; end;

[out1 gisimg]=vp_volfuns('volvect',gisfile);
if (out1 < 0) vp_message('ERROR reading gisfile'); return, end;

% reshape into column vector and normalize 
normgis=norm(gisimg); % save norm
gisimg=gisimg/normgis;

% Derive intrinsic mask from gis image
gismask = (gisimg ~= 0);
%PS Don't count 0's 
gismaskpos=find(gismask); gismasksum=size(gismaskpos,1),
if (gismasksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: GIS has no non-zero elements ']);
   return;
end

%-----------------------------------------------------------------------


% get the image filenames 
if otpropt=='A'
  prompt=['Select the GROUP Image Data set'];
  if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
else
  prompt=['Select one or more new subject images'];
  if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
end
disp(prompt);

%-----------------------------------------------------------------------
sub= size(NameLst,1),

set(FMenu,'Pointer','Watch');

h=waitbar(0,'Processing images ...');

for i=1:sub
  msg=['reading image ' NameLst(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,h,['Reading image ',num2str(i)]);
  
  [out1 im_data(:,i)]=vp_volfuns('volvect',NameLst(i,:));
  if (out1 < 0) vp_message(['ERROR reading im_data file ',num2str(i)]); return; end;
  
end

% reshape 3 dimensional images to one-dimensional vectors
waitbar(1,h); pause(1); close(h);
  
% Additional masking option
prompt=['ADDITIONAL Mask'];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'ADD MASK','','INTRINSIC MASK','','EXIT','','Help','');
   if(strcmp(POpt,'X'))return; end;
   if(strcmp(POpt,'EXIT'))return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify an existing [0|1] mask image in addition to GIS intrinsic mask to remove insignificant voxels');
   end
end
  
if (strcmp(POpt,'ADD MASK'))
   maskname=get_file(1,'*mask*.img','Select Mask');
   if isempty(maskname), return; end;

   [out1 mask]=vp_volfuns('volvect',maskname);
   if (out1 < 0) vp_message('ERROR reading mask');return, end;

   masking=1;
end;

if masking % Additional Mask
 mask=gismask.*mask;
else 
 mask=gismask;
end


if otpropt=='B'
  % READ MEAN DEVIATIONS REFERENCE IMAGE
  meanfile=[];
  prompt=['Select SSM mean log reference image'];
  disp(prompt);
  meanfile=get_file(1,'*lnmean*.img',prompt);
  if isempty(meanfile), return; end;

  [out1 mean_img]=vp_volfuns('volvect',meanfile);
  if (out1 < 0) vp_message('ERROR reading mean file');
  return, end;
 
  refvoxno = size(mean_img,1); % PS

  % DERIVE MASK FROM MEAN IMAGE FILE
  meanmask = (mean_img ~= 0);
  %PS Don't count 0's 
  maskpos=find(meanmask); masksum=size(maskpos,1),
  if (masksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: Mean image has no non-zero elements ']);
   return;
  end
  %Create a new composite mask and then mask the lnmean data
  mask=meanmask.*mask;
  mean_img=mean_img.*mask;
  
end %lnmean input
   
h=waitbar(0,'Masking images ...');
for k=1:sub
     msg=['Masking Image ',num2str(k)];
     waitbar((i-.8)/sub,h,msg);
     im_data(:,k)=im_data(:,k).*mask;
end
waitbar(1,h); pause(1); close(h);

log_transform=1; % PS Force log-transformation
if log_transform
   h=waitbar(0,'Processing Log Transformation of Images ...');
   negzero_set=find(im_data<=0);
   im_data(negzero_set)=exp(0); % PS Set all values <=0
   clear negzero_set;
   % im_data=log(im_data); % PS perform log transformation
   for i=1:sub, im_data(:,i)=log(im_data(:,i)); %PS do one at a time to save memory
     msg=['Log Image ',num2str(i)];
     waitbar((i-.8)/sub,h,msg);
   end;
   waitbar(1,h); pause(1); close(h);
end;

% Calculate SRP

h=waitbar(0,'Calculating SRP ...');
waitbar(.2,h);
voxno=size(im_data,1);
sub=size(im_data,2);

%Compute within image mean vector
%PS Don't count 0's in mean
maskpos=find(mask); masksum=size(maskpos,1),
if (masksum == 0)
      set(FMenu,'Pointer','Arrow');
      vp_message(['Error: Mask has no positive elements ']);
      close(h);
      return;
end
meanvect=sum(im_data)/masksum;
% meanvect=mean(im_data);

waitbar(.3,h);

%Compute within-subject deviations from mean of image
%i.e., normalize to whole brain
  
waitbar(.8,h);
  
%Perform in a memory efficient manner
    for x=1:sub, im_data(:,x) = (im_data(:,x) - meanvect(1,x)).*mask ; end;
% for x=1:sub, im_data(:,x) = im_data(:,x) - meanvect(1,x) ; end;

waitbar(.4,h);

dirnm=pwd;
if otpropt=='A'
   %Compute mean image of voxel deviations across subjects
   mean_img=mean(im_data');
   
   % Save mean image for subsequent analysis
   
   %TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mean Image']; 
   TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM Mean Image']; 
   filnm=[dirnm,'\s',num2str(sub), '_lnmean.img'], 
   out1=vp_imsave(filnm,reshape(mean_img,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _lnmean file');
      set(FMenu,'Pointer','Arrow');
      return;
   end;
   mean_img=mean_img';
end;
  
%Save mask mage
   
%TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM Mask Image']; 
TYPE=2; SCALE=1; DESCRIP=['ScAnVP - SSM 0/1 Mask Image']; 
filnm=[dirnm,'\s',num2str(sub), 'mask.img'], 
out1=vp_imsave(filnm,reshape(mask,DIM(1:3)), 1), 
if (out1 < 0),
      vp_message('ERROR writing _mask file');
      set(FMenu,'Pointer','Arrow');
      return;
end;
  
%Compute subject deviations from mean image
  
waitbar(.3,h);
for x=1:sub, %PS
   im_data(:,x)=im_data(:,x) -mean_img;
   waitbar(.3+.6*x/sub,h);
end;
  
waitbar(1,h); pause(1); close(h);

ssf=im_data'*gisimg,

disp(' ');
disp('Calculated SSF')
ssfotprfig=figure('name','PRE-OTPR GIS A REF SCORES'); bar(ssf); xlabel('Subject'); ylabel('Subject PRE-OTPR GIS A REF SCORE');
pos=get(ssfotprfig,'position');
if sub < 5, pos(3)=pos(3)*sub/5; 
	set(ssfotprfig,'position',[pos(1),pos(2),pos(3),pos(4)]);
end;

% subtract GIS-related activity
vp_note('Removing TPRd pattern from the data');

%im_data=im_data-gisimg*ssf'; 
pssf=ssf';
for k=1:sub
    Asrp= gisimg*pssf(k);
    im_data(:,k) = im_data(:,k) - Asrp;
 
    %Save Asrp 
   
    %TYPE=64; SCALE=1; DESCRIP=['ScAnVP - SSM TPRd SRP Image']; 
    TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM TPRd SRP Image']; 
    filnm=deblank(NameLst(k,:));
    p=max([findstr(filnm,'\') 0]),
    save_imfile=filnm(p+1:length(filnm)), 
    filnm=[dirnm,'\','Asrp_',save_imfile]; 
    out1=vp_imsave(filnm,reshape(Asrp,DIM(1:3)), 1), 
    if (out1 < 0),
      vp_message('ERROR writing _Asrp file');
      set(FMenu,'Pointer','Arrow');
      return;
    end;
end

vp_note('Masking data',1);
h=waitbar(0,'Masking new data ...');
for k=1:sub
     msg=['Masking New Image ',num2str(k)];
     waitbar((k-.8)/sub,h,msg);
     im_data(:,k)=im_data(:,k).*mask;
end
waitbar(1,h); pause(1); close(h);

disp([' '])
prompt=['SAVE ORTHOGONAL SRP IMAGES ? [Y|N=CANCEL PROCESSING]'];
POpt=vp_opt('X',prompt,'YES','Save and Continue ','NO',['Don''t save images-CANCEL and EXIT']);
if POpt=='X', 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'NO'), 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
elseif strcmp(POpt,'YES'), 
  
  dim=DIM
  for i=1:length(NameLst(:,1))

    vp_hread(NameLst(i,:));
    filnm=deblank(NameLst(i,:));
    k=max([findstr(filnm,'\') 0]),
    save_imfile=filnm(k+1:length(filnm)), 
    filnm=[dirnm,'\','Osrp_',save_imfile];
    imgstr='SRP image';
    if i==1, 
   	h=waitbar(0,'Writing Reduced Images ...');
    end;

    TYPE=16; SCALE=1; DESCRIP=['ScAnVP OTPR reduced ' imgstr int2str(i)]; 
    out1=vp_imsave(filnm,reshape(im_data(:,i),dim(1:3)), 1), 
    msg=['writing reduced image ' int2str(i) ' to ' filnm];
    waitbar((i-.8)/sub,h,msg);
    if (out1 < 0),
      vp_message('ERROR writing _OTPR file');
      set(FMenu,'Pointer','Arrow');
      return;
    end;


    %a=spm_vol(NameLst(i,:));
    %filnm=deblank(NameLst(i,:));
    %k=max([findstr(filnm,'\') 0]),
    %save_imfile=filnm(k+1:length(filnm)), 
    %a.fname=[dirnm,'\','Osrp_',save_imfile];
    %imgstr='SRP image';
    
    %if i==1, 
    %	h=waitbar(0,'Writing Reduced Images ...');
    %end;
    %a.descrip=[a.descrip ' OTPR reduced ' imgstr int2str(i)];
    %a.dim=[dim(1:3) 16];
    %spm_write_vol(a,reshape(im_data(:,i),dim(1:3)));
    %msg=['writing reduced image ' int2str(i) ' to ' a.fname];
    %waitbar((i-.8)/sub,h,msg);

  end
  waitbar(1,h); pause(1); close(h);
  
  prompt=['Images Saved. Perform TPR on Reduced Image(s)? [Y|N=END PROCESSING|X=EXIT]'];
  POpt=vp_opt('X',prompt,'YES','Invoke TPR','NO','EXIT');
  if POpt=='X', 
	set(FMenu,'Pointer','Arrow'); 
	vp_note('Program Exited',2); return;
  
  elseif strcmp(POpt,'YES'), 

	prompt=['Select GIS B to TPR on Orthogonal SRP'];
	disp(prompt);
	gisfil2=get_file(1,'*gis*.img',prompt);
	if isempty(gisfil2), return; end;

	[out1 gisim2]=vp_volfuns('volvect',gisfil2);
	if (out1 < 0) vp_message('ERROR reading gisfil2'); return, end;

	% reshape into column vector and normalize 
	normgis=norm(gisim2); % save norm
	gisim2=gisim2/normgis;

	%studyname = ['voxOtpr',otpropt];
	studyname = ['voxOtpr'];
	vp_keyin('[Assign Orthogonal TPR Study Name | q]',studyname);
	if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	set(FMenu,'Pointer','Arrow');
	return, else studyname=Keyin; end;
	studyname=[studyname,'_O',otpropt];

	dirnm=pwd;

	ssf=im_data'*gisim2,

	disp(' ');
	disp('Calculated SSF')

	ssfvoxtprfig=figure('name','TPR SCORES GIS B'); bar(ssf); xlabel('Subject'); ylabel('Subject TPR SCORE');
	pos=get(ssfvoxtprfig,'position');
	if sub < 5, pos(3)=pos(3)*sub/5; 
      		set(ssfvoxtprfig,'position',[pos(1),pos(2),pos(3),pos(4)]);
	end;

	for i=1:sub, imls{i}=NameLst(i,:); end; % Create cell array

	% save ssf simple tpr format
	if sub==1
  		ssf_name = strcat(dirnm, ['\',studyname,'_VtprScore.txt']);
	else
  		ssf_name = strcat(dirnm, ['\',studyname,'_VtprScores.txt']);
	end
	vp_write_srpfile(ssf_name,1,imls,ssf)
   
	vp_note(['END ssm_voxOtpr- Check Matlab Window and ',ssf_name]);

  end
  
end


set(FMenu,'Pointer','Arrow');
