% vp_ssm_voxpca  Copyright (C) 2003-2016 Center for Neurosciences, FIMR 
% User interface to invoke voxel based SSM-PCA analysis.
%_______________________________________________________________________
% vp_ssm_voxpca is a GUI interface that allows the user to select a 
% group of ANALYZE format spatially normalized image files interactively
% and invokes the function vp_ssmpca to perform SSM-PCA analysis for
% group pattern (GIS PC files) and subject score derivation (SSF files).
%
% For theoretical information see main ScAnVP menu help and documentation.
%
% The user picks the images to be analyzed and an optional mask file.
% Controls and non-control subjects must be specified separately.
% Data can be raw Talairach normalized, smoothed and realigned images
% or previous (log-transformed) SRP output from a previous PCA analysis.
% Subject image files must be pre-normalized and in consistent orientation
% which may be Neurological (LR) or Radiological(RL). The input file menu 
% setting only affects the orientation of the inter-process display and does
% not affect output. Output image GIS files will have the same orientation
% as the orientation of the input files on disk.
%
% Users can select the default PET grey matter mask ('scanvp\masks\PETmask.img') 
% or another predefined mask. In this version the default mask is 
% symmetrical and compatible with spm5 (it is identical to the label '5' 
% mask which has 69 planes). Alternatevily, a mask file can be created 
% using other ScAnVP options or from raw data or it can be any compatible
% sized file containing zero and non-zero elements. For RAW data
% (spatially normalized and non-SRP), a lower threshold fraction can
% be specified to create a mask based on the input images. The fractional
% threshold is applied to each image's maximum value to determine its lower
% threshold. An overall mask image is created as a composite product of 
% individual 0|1 maps of values above each threshold. A default threshold
% fraction of .35 is provided.  If 0 threshold is entered, the composite default
% intrinsic mask is used. If an external mask is specified, it is multiplied
% by the intrinsic data mask. The final mask used is output as a 0/1 8bit file.
% Non-symmetrical external masks should have the same orientation as the
% input files. Images should be more or less congruent in gray matter
% composition to avoid minimal common output operational space.
%
% The header file of the first input image is used to 
% specify geometric specifications for all images. It is assumed that
% the images have been Talairach normalized but need not have the same
% scalefactors.
% The program prompts for a name to be assigned to output files.
% The routine then calls the MATLAB function to perform the analysis:
% vp_ssmpca(names.lst, [Thresh | mask] , [SRP mask], [SRPOUT], pcaoutput.txt);
%
% Natural log transformation is by default performed in the ssmpca routine
% for raw data. If requested, a 'NOLOG' command line option can be specified.
% Results are saved in the user directory. Results include the evaluated
% PCA's (GIS images), the data mask used by the routine, the group mean image
% of the (log transformed) data (where each subject's image is normalized by 
% subtracting the subject's voxel mean), the optional subject residual profile (srp)
% images (deviations from the group mean), a SSM summary text file (Name_VScores.txt) 
% listing SSF reference (REF) scores for all subjects for the first 16 principal
% components and the "%Variance Accounted For"  for those components.
% The full set of GIS images and scores ( above 16) are only output if the srp
% output option is checked on the interface.
%
% The output SSF REF Scores are then read in by the MATLAB routine and SSF
% Z-Scores over all subjects are calculated for each of the eigenvectors.
% If controls are specified, the Z-Scores are offset by the mean Z-Score over
% controls. Individual GIS REF and Z-Score files can be saved and optionally
% manually copied to a corresponding scanvp\networks\State\VOX\study directory
% for reference by subsequent automated TPR routines.
%
% The scatterplot legend specifies the mean and stdev for each subject set. 
% p-values result from tests of the hypothesis of equal means of the 
% non-control group with respect to the first (control) group using either
% a ttest2 (for samples with equal variance) or tutest2 (unequal variances).
% Equal variance is tested using an ftest. A significance level=0.05
% is assumed.
%
%_______________________________________________________________________
% vp_ssm_pca		4.7.8	P. Spetsieris	10/01/2002
%			Revision 5.1	July 19, 2003
% vp_ssm_voxpca		Revision 5.2	Dec. 18, 2003
% 			Revision 5.3	Jun. 22, 2004
% 			Revision 5.4	Nov. 22, 2004
% 			Revision 5.7.w	Nov. 22, 2006
% 			Revision 5.8.2	July 10, 2007
% 			Revision 5.9	Mar. 10, 2008
% 			Revision 6.2	Jan. 13, 2012
%_______________________________________________________________________
%
% Voxel based SSM implements a version of the SSM Scaled Subprofile Model
% first described for ROIS by J.R.Moeller, et al (1987,1991,1994). The 
% original dos routine (ssmpca.exe) was written by John D. Van Horn Ph.D. (1996)
% and subsequently  modified by Marc Mentis M.D. (1998).
% The routines were substantially modified and adapted for the ScAnVP interface
% by P. Spetsieris Ph.D.. 
%
%  References:
%  J.R. Moeller, S.C. Strother, J.J. Sidtis and D.A. Rottenberg
%  "Scaled Subprofile Model: A statistical approach to the analysis
%  of functional patterns in positron emission tomographic data,"
%  J Cereb Blood Flow Metbol., 1987; vol.7, no.5, pp.649-658.
%
%  Rencher, A.C. (1995) Methods of Multivariate Analysis. New York: John Wiley and Sons.
% 
%  Alexander, G.E. & Moeller, J.R. (1994) Application of the Scaled Subprofile Model
%  to Functional Imaging in Neuropsychiatric Disorders: A Principal Component Approach
%  to Modeling Brain Function in Disease", Hum. Brain Mapp., 1994, 2, 79-94.
%
%  Ma Y, Tang C, Spetsieris P, Dhawan V, Eidelberg D. Abnormal metabolic network
%  activity in Parkinsons disease: Test-retest reproducibility. J Cereb Blood Flow
%  Metab 2007; 27(3):597-605. [NIHMS: 57714]
%
%  D. Eidelberg, Metabolic brain networks in neurodegenerative disorders: a functional
%  imaging approach, Trends in Neurosciences, vol. 32, no. 10, pp. 548-57, 2009.
%
%  Spetsieris P, Eidelberg D. Scaled subprofile modeling of resting state imaging data
%  in Parkinson's disease: Methodological issues, NeuroImage; 54(4):2899-914, 2011.
%
%  Spetsieris P, Ma Y, Peng S, Ko JH, Dhawan V, Tang CC, Eidelberg D. Identification
%  of disease-related spatial covariance patterns using neuroimaging data.
%  J Vis Exp, 2013; 76: e50319  Video: www.jove.com/video/50319/ Jun 25, 2013 - 14 min
%___________________________________________________________________________
% (The 'vp_ssmpca' function can be called using the Matlab interface specified here 
% or in a MATLAB window, using a names.lst file and an optional mask & output specifications.
% Type help vp_ssmpca for further help.)
% (Note, An external mask must have the same dimensions and orientation as the input images.) 

global HelpMode NameLst POpt FMenu FName FNames MaskOpt Keyin
global Ecolor RawData Prompt Filter ANameLst CNameLst LogTrans GraphOut FixedThresh 
global LSrpOut

if HelpMode, vp_help(mfilename), return ; end;

% get the image's filename {FName} 
%-----------------------------------------------------------------------

Fcolor = [.67,.76,.71];
Bcolor = [.88,.90,.88];

pvoxpcagui=findobj('Name','SSM VOXEL PCA');
if ~isempty(pvoxpcagui)
	if(strcmp(get(pvoxpcagui,'Visible'),'on'))
		figure(pvoxpcagui);
	end
	return
end

RawData=1,
LogTrans=1;
LSrpOut=0;
FixedThresh=35;
MaskOpt='';
CNameLst=[];
ANameLst=[];
NameLst=[];
GraphOut=1;


pvoxpcagui = figure('Color',Fcolor,'Name','SSM VOXEL PCA',...
'NumberTitle','off','Position',[100,200,350,260],...
'Units','Pixels');
	
wid=350; hgt=260;
	
uicontrol(pvoxpcagui,'string', 'HELP',...
	'Units','normalized',...
	'position',[275/wid 210/hgt 50/wid 20/hgt],'BackgroundColor',Ecolor,...
	'callback',['vp_help(''vp_ssm_voxpca'')']);

	
uicontrol(pvoxpcagui,'String','QUIT',...
	'Units','normalized',...
	'Position',[278/wid 20/hgt 50/wid 25/hgt],...
	'CallBack',['delete(pvoxpcagui);','set(FMenu,''Pointer'', ''Arrow'');']);
	

uicontrol(pvoxpcagui,'Style','Text','String','SSM VOXEL PCA',...
	'FontName','Times','FontSize',12,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'FontUnits','Normalized',...
	'BackgroundColor',Fcolor,...
	'Units','Normalized',...
	'Position',[.10 .85 .6 .1],'Tag','Empty',...
	'ForegroundColor',Txcolor)


cbtexto= ['delete(findobj(''tag'',''poptext'')); ',...
	'text(0/wid,200/hgt,[''IMAGE TYPE:                      ''],''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''tag'',''poptext'',''FontName'',''Times'',''FontSize'',10,''FontUnits'',''Normalized'',''HorizontalAlignment'',''Left'');'] ;


cbtextNRM= ['delete(findobj(''tag'',''poptext'')); ',...
	'text(0/wid,200/hgt,[''IMAGE TYPE: NORMALIZED RAW DATA ''],''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''tag'',''poptext'',''FontName'',''Times'',''FontSize'',10,''FontUnits'',''Normalized'',''HorizontalAlignment'',''Left'');'] ;


cbtextSRP= ['delete(findobj(''tag'',''poptext''));  ',...
	'text(0/wid,200/hgt,[''IMAGE TYPE: PREPROCESSED SRP    ''],''Units'',''Normalized'',''Fontweight'',''bold'',',...
	'''tag'',''poptext'',''FontName'',''Times'',''FontSize'',10,''FontUnits'',''Normalized'',''HorizontalAlignment'',''Left'');'] ;

eval(cbtextNRM);
axis off;

cbImgTyp = ['val=get(gco,''Value'');',...
	'if val == 1, RawData=1, ',...
	'eval(cbtexto); eval(cbtextNRM);',...
	'else, RawData=0, ',...
	'LogTrans=0; set(findobj(''tag'',''LOG''),''value'',0);',...
	'eval(cbtexto); eval(cbtextSRP);',...
	'end; '];

hImgTyp=uicontrol(pvoxpcagui,'Style','popup','Visible','on',...
	'Units','normalized',...
	'position',[50/wid 205/hgt 220/wid 20/hgt],'BackgroundColor',Bcolor,...
	'String',...
	['Normalized Raw data|SRP residual profile images'],...
	'value',1,...
	'TooltipString',['Select Image Type : ',...
	'Raw Realigned, Smoothed and Talairach Normalized or ',...
	'Subject Residual Profiles of Pre-Processed Data'],...
	'Tag','ImgTyp',...
    	'callback',cbImgTyp);


pcbCImgSet = [' ',...
 'val=get(gco,''value'');',...
 'if (val==0), CNameLst=[]; return; end;',...
 'if (RawData==1)',...
 '  Prompt=[''Select NORMALIZED CONTROL IMAGES-Same Dimensions/Orientation''];',...
 '  Filter=''*.img'';',...
 'else',...
 '  Prompt=[''Select SRP CONTROL IMAGES''];',...
 '  Filter=''*srp*.img'';',...
 'end; ',...
 'CNameLst = get_file(Inf,Filter,Prompt,[]);',...
 'if (isempty(CNameLst) ), set(gco,''value'',0); return, end;'];
 

uicontrol(pvoxpcagui,'string', 'CONTROLS',...
	'TooltipString','Pick Control Image file Set.',...
	'Style','checkbox','value',0,...
	'Units','normalized',...
	'position',[30/wid 153/hgt 125/wid 25/hgt],...
	'BackgroundColor',Bcolor,'callback',pcbCImgSet);

pcbImgSet = [' ',...
 'if (RawData==1)',...
 '  Prompt=[''Select NORMALIZED (Non-Control) IMAGES-Same Dimensions/Orientation''];',...
 '  Filter=''*.img'';',...
 'else',...
 '  Prompt=[''Select SRP (Non-Control) IMAGES''];',...
 '  Filter=''*srp*.img'';',...
 'end; ',...
 'ANameLst = get_file(Inf,Filter,Prompt,[]);',...
 '	if (isempty(ANameLst) ) return, end;'];
 

uicontrol(pvoxpcagui,'string', 'SUBJECT IMAGE FILES',...
	'TooltipString','Pick Image file Set.',...
	'Units','normalized',...
	'position',[30/wid 123/hgt 125/wid 25/hgt],...
	'BackgroundColor',Bcolor,'callback',pcbImgSet);
      

pcbSetMask = ['val=get(gco,''value'');',...
	'if val==1, MaskOpt='''';',...
	'elseif val==2,',...
	'  w=which(''scanvp''); MaskOpt=[w(1:(max(findstr(w,''\''))-4)),''masks\PETMask.img''],',...
	'  if (exist(MaskOpt) ~= 2), vp_note([''Invalid or Non-Existent File: '',MaskOpt]);',...
	'  MaskOpt=''''; set(gco,''value'',1); end;',...
	'elseif val==3,',...
	'  w=which(''scanvp''); MaskOpt=[w(1:(max(findstr(w,''\''))-4)),''masks\PETMask99.img''],',...
	'  if (exist(MaskOpt) ~= 2), vp_note([''Invalid or Non-Existent File: '',MaskOpt]);',...
	'  MaskOpt=''''; set(gco,''value'',1); end;',...
	'elseif val==4,',...
	'  w=which(''scanvp''); MaskOpt=[w(1:(max(findstr(w,''\''))-4)),''masks\PETMask5.img''],',...
	'  if (exist(MaskOpt) ~= 2), vp_note([''Invalid or Non-Existent File: '',MaskOpt]);',...
	'  MaskOpt=''''; set(gco,''value'',1); end;',...
	'elseif val==5,',...
	'  vp_note(''Mask must have the same dimensions as the image files.'',3);',...
	'  MaskOpt = get_file(1,''*mask*.img'',''Select 8bit Mask File Name'',[]);',...
	'  if (isempty(MaskOpt) ), MaskOpt=''''; set(gco,''value'',1); ',...
	'  end;',...
	'else, MaskOpt=''''; ',...
	' if(RawData), vp_thresh; MaskOpt=4; else,',...
	' vp_note(''Threshold Not Valid for SRP data'',3); set(gco,''value'',1); end; end;'];

uicontrol(pvoxpcagui,'string', ['SELECT/RESET MASK or THRESHOLD | SELECT DEFAULT PET MASK |',...
	' PETMask99 | PETMask5 | SELECT OTHER 0/1 MASK | SET THRESHOLD '],...
	'TooltipString',...
	'Select Mask Image File (Must be 8bit 0/1, same dimensions/orientation or Threshold for RAW input)',...
	'Style','popup','value',1,...
	'Units','normalized',...
	'position',[170/wid 123/hgt 165/wid 25/hgt],'BackgroundColor',Bcolor,...
	'callback',pcbSetMask);
      

uicontrol(pvoxpcagui,'string', 'LOG TRANSFORM DATA','tag','LOG',...
	'TooltipString',...
	'Natural Log-transform the data? logs of neg. values will be set to zero',...
	'Style','checkbox','value',1,...
	'Units','normalized',...
	'position',[170/wid 93/hgt 165/wid 25/hgt],'BackgroundColor',Bcolor,...
	'callback',['LogTrans=get(gco,''value''),',...
	' if ~RawData, LogTrans=0; set(gco,''value'',0); vp_note(''Invalid for SRP'',2); end;']);
      

uicontrol(pvoxpcagui,'string', 'SAVE (L)SRP and GIS n>16',...
	'TooltipString',...
	'Save all SRP or LSRP image output and GIS(n) images including for n>16?',...
	'Style','checkbox','value',0,...
	'Units','normalized',...
	'position',[170/wid 63/hgt 165/wid 25/hgt],'BackgroundColor',Bcolor,...
	'callback',['LSrpOut=get(gco,''value''),']);
      
uicontrol(pvoxpcagui,'string', 'GIS SCORE DISPLAY/SAVE',...
	'TooltipString',...
	'Show GIS Score Plots with option to SAVE (REF) files',...
	'Style','checkbox','value',1,...
	'Units','normalized',...
	'position',[105/wid 31/hgt 165/wid 25/hgt],'BackgroundColor',Bcolor,...
	'callback',['GraphOut=get(gco,''value''),']);
      

cbssmvoxpcapro=['statstr=''''; ',...
	' if RawData,  statstr=[statstr,''RAW DATA, ''], else, statstr=[statstr,''SRP DATA, '']; end;',...
	' if and(~isempty(MaskOpt),(MaskOpt~=4)), statstr=[statstr,''MASK: '', MaskOpt, '', ''], else, statstr=[statstr,''MASK OFF, ''];',...
	' if RawData, statstr=[statstr, ''THRESH: '',num2str(FixedThresh),'', '']; end; end; ',...
	' nss=num2str(size(ANameLst,1)),  statstr=[statstr, ''GROUP: '', nss, '' SUBJECTS, ''];',...
	' if ~isempty(CNameLst), ncs=num2str(size(CNameLst,1)); statstr=[statstr,ncs,'' CONTROLS DEFINED, ''],',...
	' else, statstr=[statstr,''NO CONTROLS, '']; end;',...
	' if LogTrans, statstr=[statstr,''LOG Transf., ''], else, statstr=[statstr,'' NO LOG, '']; end;',...
	' POpt=vp_opt(''X'',statstr,''PROCEED'',''Proceed with Processing'',''EDIT'',''Change Parameters'');',...
	' if strcmp(POpt,''PROCEED''),',...
	' set(findobj(''Name'',''SSM VOXEL PCA''),''Pointer'',''watch'');',...
	' vp_ssm_voxpcapro;',...
	' end;'];

uicontrol(pvoxpcagui,'String','PROCESS',...
	'TooltipString','Process SSM VOXEL PCA',...
	'Units','normalized',...
	'Position',[30/wid 20/hgt 60/wid 25/hgt],...
	'CallBack',[ 'if isempty(ANameLst), vp_note([''ERR: File Set Not Defined''],3); return; end; ',...
	'if (isempty(MaskOpt) & RawData), vp_note([''MASK or THRESHOLD Not Defined''],4); return; end; ',...
	'if and(~RawData,(MaskOpt==4)), vp_note(''Threshold Not Valid for SRP data'',3); return; end;',...
	'eval(cbssmvoxpcapro);',...
	' '] );
	
uicontrol(pvoxpcagui,'String',ScAnVPc,'ToolTipString',ScAnVPc,...
	'Units','Normalized',...
	'ForegroundColor','w',...
	'BackgroundColor',Wcolor,...
	'FontName','Times','FontSize',09,'FontAngle','Italic',...
	'HorizontalAlignment','left',...
	'ButtonDownFcn','vp_help(''vp_License'')',...
	'Callback','vp_help(''vp_License'')',...
	'Position',[30/wid 0/hgt 140/wid 015/hgt])
	
