% vp_ssm_voxtpr.m
% Voxel Based SSM topographic Profile Rating (TPR) (Prospective Scoring)
% Provides interface to automated and interactive options to perform
% evaluation of new subject(s) SSM subject scores for a specified 
% predetermined GIS pattern. TPR can be performed on individual scans or
% in group processing. It is assumed that the GIS pattern and subject
% images are similarly oriented (LR or RL). The TPR operation is performed
% in the common non-zero gray space of the subject image and the GIS pattern.
% For group processing the common non-zero space of the group is accessed
% for valid comparison. Thus, images are assumed to be congruent if they
% contain atrophic regions or are limited to right or left hemispheric space. 
%_____________________________________________________________________________________
%
% TPR    (Topographic Profile Rating) New Subject Score Evaluation.
%              
%               I) Automated: Uses predefined methods and file references.
%                  -New group recalibration of pre-defined GIS patterns.
%                  -Computer-aided-diagnosis in some-versions.
%
%                   See specific Help for Automated Options or set
%                   Help Mode ON on Main Menu.
%
%               II) Interactive Options, Reference, Methods, Log. etc.
%
% INTERACTIVE TPR
%           (1) Select a GIS image principal component.
%           (2) Select a data set of images to perform TPR evaluation of SSF scores 
%               for that GIS.
%
%               The following options exist for the input data and evaluation method.
%               (A) Input previously determined SRP. - Calculates GIS Scores on SRP.
%               (B) Input new data and reference group. Determines SRP. - Calc. GIS Scores.
%               (C) Input new data and reference (l)mean (GMP). Determines SRP. - Calc. GIS Scores.
%               (D) Input pre-normalized data. - Calc. GIS Scores directly on input data.
%               (E) Input new data. No GMP centering. - Calc. GIS Scores on row centered data.
%               (G) Input sets of new data. Optional same group. Optional GMP centering.
%
% It is assumed that images have been smoothed and Talairach normalized before analysis.
%
% OPT. A - Single Subjects
% For option A, scores are calculated directly on the input subject(s) SRP. It is
% assumed that SRP has been previously generated with respect to a reference group,
% for example as a result of a PCA analysis.
%
% OPT. A - Can also be used for Interactive Group Recalibration 
% If possible use the 'Automated Group Recalibration' option.
% To use option A to recalibrate the range of scores for a new, complete group of
% subjects that include controls, a 'mock' PCA analysis should be previously performed 
% to generate the SRP, mask and reference mean image (GMP). If a previously derived mask is
% to be used, it should be input into the PCA analysis but the output mask generated
% should be used in the subsequent recalibration steps. The option A reference scores
% and the group mean image can then be saved for subsequent single or multi-subject AUTO-TPR.
% The score, mean and mask files can be renamed to indicate that they are reference
% files and saved in the corresponding network directory as in the example below.
% The output score file must include a header line indicating that it contains REF scores
% similar to that created by the automated method.
% For subjects included in a previous analysis performed with a different set of reference 
% images, there may be a shift in scores because of a difference in GMP offset.
%
%     Network file namimg (EXAMPLE):
%     For example for the network 1 corresponding to hypothetical condition COND,
%     recalibrated using 22 patients and 22 controls, the scanvp directory under
%     networks named COND\VOX\Name22_22\ may contain the following files named
%     as indicated:
%
%       directory:	scanvp\networks\COND\VOX\Name22_22
%       contents:
%           gis file:               Name22_22_gis1.img, Name22_22_gis1.hdr
%           Reference score file:   Name22_22_VScores1.txt
%           log mean file:          Name22_22_lnmean.img, ...hdr
%           mask file:              Name22_22mask##.img, ...hdr, ## optional threshold 
%
% OPT. B,C 
% In B and C, scores are offset from the original or other pregenerated reference data.
% If SRP is obtained from direct PCA analyis involving new data not in the original
% analysis (used in determing the GIS), the new data is incorporated in the
% calculation of the group mean and SRP which may result in a shift in score values
% of subjects that were in the original analysis. To maintain the original scale 
% and add new scores with respect to reference data, options B & C should be used for 
% a single or multiple new subjects. Using options B or C there will be no shift in
% individual score values if the subject data is introduced singly or as part of a
% new group of subjects. To determine SRP in B and C, first the mean value over all
% (log transformed) voxels is evaluated for each subject and used to generate within
% subject voxel deviations from the mean. The reference group mean deviations image
% must be calculated or specified. In option B the input reference group data is 
% used to evaluate this image, whereas in option C the mean image is selected from
% the output of a reference PCA analysis. The mean deviations values are then 
% subtracted from the new subject values to generate the new SRP. For option B a multi
% subject reference group is required and essentially the reference subjects should
% correspond to the subjects including controls of the original study from which the
% GIS was derived. The input GIS corresponding to a single previously derived disease
% specific profile containing the values of the relative (Z-scored) voxel weights is
% used to determine the new SSF subject scores using the following calculation:
% 
% 	          SSF=SRP*GIS/norm(GIS), (options A-C)
%
% where SSF is a vector (nsubs), SRP is a matrix (nsubs,nvoxs) and GIS is a vector (nvoxs).
%
% OPT. D 
% For option D new scores are evaluated directly on the (log) tranformed input data.
% It is assumed that this data has been previously normalized by another user-defined
% method. Mean values are not subtracted and SRP is not calculated. Log and Masking 
% operations are optional. 
%
% 	          SSF=(log)DATA*GIS/norm(GIS), (option D)
%
% An output file containing the new SSF scores for each new subjects is generated
% and plotted. Z scores are generated and offset by the mean Z score of the control
% subjects. Results are saved in the input file directory.
%
% OPT. E
% For option E new scores are evaluated only on row centered data, i.e., only the
% subject mean is subtraced after log transformation. Reference group mean (GMP) is
% not subtracted (no column centering). Only GIS and subject files are input.
% SRP is not evaluated. It is assumed that scores can be compared with similarly
% generated scores, i.e., TPR scores of non-group normalized data. 
%
% OPT. G
% Repeated measures for network expression for a single group of same subjects
% in repeat experiments or for indepenent groups of subjects. Paired t-tests
% or student's independent group mean t-tests are performed. Composite plots. 
%
%
% Written for use with ScAnVP 5.1	P. Spetsieris, Ph.D.    7/21/03  
%			Revision 5.2	P.S. 10/2/03
%			Revision 5.4	P.S. 1/11/05
%			Revision 5.5	P.S. 4/21/05
%			Revision 5.7	P.S. 6/1/06
%			Revision 5.8	P.S. 5/22/07
%			Revision 6.1	P.S. 11/18/10
%			Revision 6.2.1	P.S. 11/29/13
%			Revision 6.3	P.S. 12/5/13
%_______________________________________________________________________

global HelpMode FMenu NameLst POpt
global Message Ecolor Op ImOpFactor Keyin
global hwaitbar AutoTPR

if HelpMode, vp_help(mfilename); return ; end;

out1=0;
ImOpFactor=1;
Message=['MSG:.................'];
Bcolor=[0.8 .85 .95];
Mcolor=[0.56 .65 .75];
AutoTPR=0;

tip1=['Principal Component Analysis'];
tip2=['Topographic Profile Rating (Automatic or Interactive'];
tip3=['Orthogonal Principal Component Analysis'];
tip4=['Select SSM Vox Utility: Display SCORES, Generate MEAN, Thresh. MASK, Exclusive MASK'];


voxssmtprgui=findobj('Name','VOXEL TPR');
if ~isempty(voxssmtprgui)
	if(strcmp(get(voxssmtprgui,'Visible'),'on'))
		figure(voxssmtprgui);
	end
	return
end


voxssmtprgui = figure('Color',Mcolor,'Name','VOXEL TPR','Tag','voxssmtprgui',...
'NumberTitle','off','Position',[100,200,325,350],'UserData',0,...
'Units','Pixels');


uicontrol(voxssmtprgui,'Style','Text','String','Scaled Subprofile Model',...
	'FontName','Times','FontSize',16,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[10/250 75/100 225/250 020/100],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
uicontrol(voxssmtprgui,'Style','Text','String','(Voxel Based TPR)',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[40/250 73/100 125/250 015/100],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
uicontrol(voxssmtprgui,'Style','Text','String','AUTOMATED TPR',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[30/250 67/100 130/250 015/100],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0])
	
titleAU=['USING REFERENCE FILES'];
titleAD=['AUTO-DIAGNOSIS'];
titleAR=['GROUP RECALIBRATION'];
titleRM=['REPEATED MEASURES'];
optAU=['(AU) Input new subject(s) and predefined GIS, determine Scores with respect to reference data.'];
optAR=['(AR) Input new GROUP and predifined GIS, determine new Scores and create reference directory. '];
optAD=['(AD) Input new GROUP and GISes, determine Scores and generate Diagnosis Report.'];
optRM=['(RM) Input GIS and REPEATED IMAGE SETS determine Scores Report and Graph.'];

uicontrol(voxssmtprgui,'string', titleAU,...
	'TooltipString',optAU,...
	'Units','Normalized',...
	'position',[40 215 160 20],...
	'position',[.1231 .6857 .4923 .0571],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_autovoxtpr');
	
uicontrol(voxssmtprgui,'string', titleAR,...
	'TooltipString',optAR,...
	'Units','Normalized',...
	'position',[.1231 .6143 .4923 .0571],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_autovoxrecal');
	
%uicontrol(voxssmtprgui,'string', titleRM,...
%	'TooltipString',optRM,...
%	'Units','Normalized',...
%	'position',[.1231 .5429 .4923 .0571],...
%	'BackgroundColor',Bcolor,'callback','vp_ssm_autovoxrepeat');
	
%uicontrol(voxssmtprgui,'string', titleAD,...
%	'TooltipString',optAD,...
%	'Units','Normalized',...
%	'position',[.1231 .4715 .4923 .0571],...
%	'BackgroundColor',Bcolor,'callback','vp_ssm_autocadgui');
	
%	'Position',[30/250 105/350 130/250 015/100],'Tag','Empty',...
%
uicontrol(voxssmtprgui,'Style','Text','String','INTERACTIVE TPR',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[30/250 130/350 130/250 015/100],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0])
	
titleA=['A - USING SRP, SINGLE or GROUP'];
titleB=['B - USING REFERENCE GROUP     '];
titleC=['C - USING REFERENCE MEAN        '];
titleD=['D - DIRECT - PRE NORMALIZED     '];
titleE=['E - DIRECT-NO GROUP REF MEAN   '];
titleRG=['G - REPEATED TPR     '];

optA=['(A) Input new SRP (subject and group pre-centered data), Calc. GIS Score on SRP. (Group Recalibration Option) '];
optB=['(B) Input new subjects and reference group, determine SRP with respect to ref. group mean, Calculate GIS Scores'];
optC=['(C) Input new subjects and reference mean, determine SRP with respect to mean & GIS Score on SRP.'];
optD=['(D) Input new subjects, Calc. GIS Score on input data directly NO subject or group mean centering.'];
optE=['(E) Input new subjects, Evaluate GIS Score on subject mean centered data. No group centering.'];

uicontrol(voxssmtprgui,'string', titleA,...
	'TooltipString',optA,...
	'Units','Normalized',...
	'position',[.1231 .39 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','POpt=[''A-SRP, GROUP RECAL'']; vp_ssm_voxtprpro');
%	'BackgroundColor',Bcolor,'callback','voxtpropt=''A''; vp_ssm_voxtprpro');
	
uicontrol(voxssmtprgui,'string', titleB,...
	'TooltipString',optB,...
	'Units','Normalized',...
	'position',[40 103 200 20],...
	'position',[.1231 .32 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','POpt=[''B-REF GROUP'']; vp_ssm_voxtprpro');
%	'BackgroundColor',Bcolor,'callback','REFscores=1; voxtpropt=''B''; vp_ssm_voxtprpro');
	
uicontrol(voxssmtprgui,'String',titleC,...
	'TooltipString',optC,...
	'Units','Normalized',...
	'Position',[40 78 200 20],...
	'position',[.1231 .2575 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','POpt=[''C-REF MEAN'']; vp_ssm_voxtprpro');
%	'BackgroundColor',Bcolor,'callback','REFscores=1; readref_mean=1; voxtpropt=''C''; vp_ssm_voxtprpro');
	
uicontrol(voxssmtprgui,'String',titleD,...
	'TooltipString',optD,...
	'Units','Normalized',...
	'Position',[40 53 200 20],...
	'position',[.1231 .1950 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','POpt=[''D-DIRECT'']; vp_ssm_voxtprpro');
%	'BackgroundColor',Bcolor,'callback','voxtpropt=''D''; vp_ssm_voxtprpro');

uicontrol(voxssmtprgui,'String',titleE,...
	'TooltipString',optE,...
	'Units','Normalized',...
	'Position',[40 28 200 20],...
	'position',[.1231 .1325 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','POpt=[''E-DIRECT'']; vp_ssm_voxtprpro');
	
uicontrol(voxssmtprgui,'string', titleRG,...
	'TooltipString',optRM,...
	'Units','Normalized',...
	'position',[.1231 .07 .6154 .0571],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_voxtprrepeat');
	
%'position',[240 250 60 20],'callback',['vp_help(''vp_ssm_voxtpr'');',
uicontrol(voxssmtprgui,'string', 'HELP',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'Units','Normalized',...
	'position',[.75 .6250 .1714 .0571],'callback',['vp_help(''vp_ssm_voxtpr'');',...
	' vp_note(''See HELP window. Invoke Specific Task Help or Set Help Mode ON on Main Menu'');']);
	
%	'Position',[250 5 60 20],
uicontrol(voxssmtprgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Units','Normalized',...
	'Position',[.75 .01 .1714 .0571],...
	'CallBack',['delete(voxssmtprgui);',...
	'set(FMenu,''Pointer'',''Arrow'')';] );





