% vp_ssm_voxtprpro.m
% Voxel Based SSM Interactive Topographic Profile Rating (TPR)
% Performs evaluation of new subject(s) SSM subject scores for a specified GIS
%_____________________________________________________________________________________
%
% The user  (1) Selects a GIS image file.
%           (2) Selects a data set of images to perform TPR evaluation of SSF scores 
%               for that GIS.
%
%               The following options exist for the input data and evaluation method.
%               (A) Input previously determined SRP, Calculate GIS Scores on SRP.
%               (B) Input new data and reference group, determine SRP, Calculate GIS Scores.
%               (C) Input new data and reference mean. Determine SRP. Calculate GIS Scores.
%               (D) Input pre-normalized data, Calc. GIS Scores directly on input data.
%               (E) Input new data. Calc. GIS Scores on subject (not group) normalized data.
%
% It is assumed that images have been smoothed and Talairach normalized before analysis.
% An intrinsic mask is calculated from the specified GIS. A composite mask is formed
% from any additional external and intrinsic masks derived from the input data or mean.
% 
% For option A scores are calculated directly on the input subject(s) SRP. It is
% assumed that SRP has been previously generated with respect to a reference group, for
% example as a result of a PCA analysis.
%
% Option A can be used to recalibrate the range of scores for a new, complete group of
% subjects (and controls). A 'mock' PCA analysis should be previously performed 
% (using the referece GIS intrinsic mask) to generate the SRP, and reference mean images.
% The option A reference scores and the group mean image can be saved for subsequent
% single or multi-subject AUTO-TPR. These operations are best performed in AUTO-TPR.
%
% In B and C scores are offset from pregenerated reference data.
% If SRP is obtained from direct PCA analyis involving new data not in the original
% analysis (used in determing the GIS), the new data is incorporated in the
% calculation of the group mean and SRP which may result in a shift in score values
% of subjects that were in the original analysis. To maintain the original scale 
% and add new scores with respect to reference data, options B & C should be used for 
% a single or multiple new subjects. Using options B or C there will be no shift in
% previous score values if the subject data is introduced singly or as part of a
% new group of 'raw' data. To determine SRP in B and C, first the mean value over all
% (log transformed) voxels is evaluated for each subject and used to generate within
% subject voxel deviations from the mean. The reference group mean deviations image
% must be calculated or specified. In option B the input reference group data is 
% used to evaluate this image, whereas in option C the mean image is selected from
% the output of a reference mean evaluation (e.g. output from a PCA analysis).
% The group mean values are then subtracted from the new subject values to generate
% the new SRP. For option B a multi subject reference group is required and
% essentially the reference subjects should be matched to the subjects (including
% any controls) of the original study from which the GIS was derived. The input GIS
% corresponding to a single previously derived disease specific profile containing
% the values of the relative (Z-scored) voxel weights is used to determine the new
% SSF subject scores using the following calculation:
% 
% 	          SSF=SRP*GIS/norm(GIS), (options A-C)
%
% where SSF is a vector (nsubs), SRP is a matrix (nsubs,nvoxs) and GIS is a vector (nvoxs).
%
% For option D new scores are evaluated directly on the (log) tranformed input data.
% It is assumed that this data has been previously normalized by another user-defined
% method. Mean values are not subtracted and SRP is not calculated. Log and Masking 
% operations are optional.
%
% 	          SSF=(log)DATA*GIS/norm(GIS), (option D)
%
% For option E data is normalized with respect to the subject mean
% but not with respect to the group mean (row centered only). 
%
% 	          SSF=((log)DATA-mean(log)DATA)*GIS/norm(GIS), (option E)
%
% An output file containing the new SSF scores for each new subjects is generated
% and plotted. The Z scores based on the optional reference score mean and std. values
% can also be generated for the new subjects and if controls exist, scores are offset
% by the reference score control mean. The reference file should have been generated
% with the same group mean deviations image (opt. A-C) or none (opt E). 
% Results are saved in the input file directory.
%
% Elements of this routine is based to some extent on the following routines:
% gis2ssf.m, ssm_pca.c     Marc Mentis, M.D.           6/11/1999
% tpr1.m written by Chris Habeck Ph.D.
% Columbia College of Physicians and Surgeons 8/02
%
% Written for use with ScAnVP 5.1	P. Spetsieris, Ph.D.    7/21/03  
%			Revision 5.2	P.S. 10/2/03
%			Revision 5.4	P.S. 1/11/05
%			Revision 5.5	P.S. 4/21/05
%			Revision 5.6	P.S. 11/21/05
%			Revision 5.8	P.S. 07/09/07
%			Revision 6.1	P.S. 11/16/10 Add Option E - No. mean ref image
%			Revision 6.2	P.S. 2/22/12 Optimize mem. and mask code
%			Revision 6.2.1	P.S. 4/29/13 Check for .nii files 
%______________________________________________________________________________________

global HelpMode FMenu NameLst POpt Keyin
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN

if HelpMode, vp_help(mfilename), return ; end;

REFscores=0; readref_mean=0; 
imls=[];
%prompt=['(Interactive) VOXEL SSM TPR- Select Option'];
%optA=['(A) Input new SRP (subject and group pre-centered data), Calc. GIS Score on SRP. (Group Recalibration Option) '];
%optB=['(B) Input new subjects and reference group, determine SRP with respect to ref. group mean, Calculate GIS Scores'];
%optC=['(C) Input new subjects and reference mean, determine SRP with respect to mean & GIS Score on SRP.'];
%optD=['(D) Input new subjects, Calc. GIS Score on input data directly NO subject or group mean centering.'];
%optE=['(E) Input new subjects, Evaluate GIS Score on subject mean centered data. No group centering.'];


if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxtpro'); vp_ssm_voxtpr; 
  return; end;
if(strcmp(POpt,'A-SRP, GROUP RECAL')),
  vp_note(optA,3);
  voxtpropt='A'; end;
if(strcmp(POpt,'B-REF GROUP')),
  vp_note(optB,3);
  REFscores=1;
  voxtpropt='B'; end;
if(strcmp(POpt,'C-REF MEAN')),
  vp_note(optC,3);
  REFscores=1;
  readref_mean=1;
  voxtpropt='C'; end;
if(strcmp(POpt,'D-DIRECT')),
  vp_note(optD,3);
  voxtpropt='D'; end;
if(strcmp(POpt,'E-DIRECT')),
  vp_note(optE,3);
  voxtpropt='E'; end;

maskname=[]; extmasking=0;
im_data=[]; devs=[]; negzero_set=[];
log_transform=0; % PS 
norm1=0; norm2=0;

prompt=['Select PCA GIS component for TPR'];
disp(prompt);
gisfile=get_file(1,'*gis*.img',prompt);
if isempty(gisfile), return; end;

if voxtpropt=='A'
  nm=vp_nm('nm',gisfile);

  %ln=length(nm);
  %ngisch=nm(ln);
  %ngisno=str2num(ngisch);
  %if isempty(ngisno), ngisno=1;
  %else
  % ngisch=nm(ln-1);
  % ngisn1=str2num(ngisch);
  % if ~isempty(ngisn1)
  %  ngisno=ngisno+10*ngisn1; % allow at most 2 digits
  % end
  %end

  [ngisno]=vp_num3str(nm);

end


[out1 gisimg]=vp_volfuns('volvect',gisfile);
if (out1 < 0) vp_message('ERROR reading gisfile'); return, end;


% reshape into column vector and normalize to
% have Euclidean norm=1
normgis=norm(gisimg); % PS save norm
gisimg=gisimg/normgis;

% Extract intrinsic gis mask
mask= (gisimg ~= 0); % default mask

%PS get the image filenames {FName} 
if voxtpropt=='A'
  prompt=['Select The New Subject Residual Profile(s) (SRP)'];
  if vp_filesui(Inf,'*srp*.img',prompt,mfilename) < 0 ,return; end;
else
  if voxtpropt=='D'
    prompt=['Select (Group) Image Data set'];
  else
    prompt=['Select New Image Data'];
  end
  if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
end
disp(prompt);
%-----------------------------------------------------------------------
set(FMenu,'Pointer','Watch');
imfiles=NameLst;
sub= size(NameLst,1);
newsub=sub;
if ( voxtpropt=='B' )
   prompt=['Select Reference Group Image Data set'];
   if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 ,return; end;
   refiles=NameLst;
   refsub= size(NameLst,1);
   if ( (refsub <=1) )
   vp_note('Must include more than one subject for Option B',2); return;
   end
end

pwdd=pwd;
if sub==1
   img1=imfiles(1,:);
   
   nm=vp_nm('nm',img1);
   ssf_name = [nm,'_',voxtpropt,'1'];
else
  ssf_name = [voxtpropt,num2str(sub)];
end

h=waitbar(0,'Processing images ...');

if vp_checkNIFU(imfiles,5), return, end; % Exit for UNIX and .nii files

% allocate space
im_data=zeros(prod(DIM), sub, 'single');

% reshape 3 dimensional images to one-dimensional vectors
for i=1:sub
  msg=['reading image ' imfiles(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,h,['Reading image ',num2str(i)]);
  
  [out1 im_data(:,i)]=vp_volfuns('volvect',imfiles(i,:));
  if (out1 < 0) vp_message(['ERROR reading im_data file ',num2str(i)]); return; end;
  
end
%%%% Reading in images


waitbar(1,h); pause(1); close(h);
  
%Calculate intrinsic data mask
  datmask=im_data(:,1) ~=0;
  for p=1:sub
   datmask=datmask.*(im_data(:,p) ~=0);
  end;
  
  mask=mask.*datmask; % incorporate default mask 

%PS Add External Masking Option

prompt=['External Mask '];
POpt='Help';
while (strcmp(POpt,'Help'))
   POpt=vp_opt('X',prompt,'MASK','','INTRINSIC MASK','','EXIT','','Help','');
   if(strcmp(POpt,'X'))
     set(FMenu,'Pointer','Arrow');
   return; end;
   if(strcmp(POpt,'EXIT'))
     set(FMenu,'Pointer','Arrow');
   return; end;
   if(strcmp(POpt,'Help')),
    vp_note('Specify an existing [0|1] mask image to remove insignificant pixels');
   end
end
  
if (strcmp(POpt,'MASK'))
   maskname=get_file(1,'*mask*.img','Select Mask');
   if isempty(maskname), return; end;

   [out1 extmask]=vp_volfuns('volvect',maskname);
   if (out1 < 0) vp_message('ERROR reading mask');return, end;
   
   extmasking=1;
   mask=extmask.*mask; %Incorporate default mask

end


% ENTER MEAN DEVIATIONS REFERENCE IMAGE FOR OPTION C
%PS 
if voxtpropt == 'C',
  prompt=['Select Mean Deviations Reference Image'];
  disp(prompt);
  meanfile=get_file(1,'*mean*.img',prompt);
  if isempty(meanfile), return; end;
   
  [out1 mean_img]=vp_volfuns('volvect',meanfile);
  if (out1 < 0) vp_message('ERROR reading mean image');return, end;
  meanmask= mean_img ~= 0;

  % Incorporate previous composite mask 
  mask=mask.*meanmask;
  
  mean_img=mean_img';
end % mean image input

% MASK IMAGES
h=waitbar(0,'Masking images with composite intrinsic/external mask...');

for k=1:sub
     msg=['Masking Image ',num2str(k)];
     waitbar((i-.8)/sub,h,msg);
     im_data(:,k)=im_data(:,k).*mask;
end

waitbar(1,h); pause(1); close(h);


% LOG TRANSFORMATION 
if voxtpropt ~= 'A',
 log_transform=1; % PS log-transformation
 
 prompt=['Log-transform the data? neg. values & logs will be set to zero [Y|N|EXIT]'];
 POpt=vp_opt('X',prompt,'YES','Perform Natural Log Transformation','NO','Pre or No Transformation','EXIT','');
 if (POpt=='X' | strcmp(POpt,'EXIT')), 
	set(FMenu,'Pointer','Arrow');
	vp_note('Program Exited',2); return;
 elseif strcmp(POpt,'NO'), log_transform=0,
 else log_transform=1;
 end;

 
 if log_transform
   h=waitbar(.4,'Processing Log Transformation of Images ...');
   %negzero_set=find(im_data <=0);
   for p=1:sub % Save memory
     negzero_set=find(im_data(:,p) <=0);
     im_data(negzero_set,p)=exp(0); % PS Set all values <=0
   end
   %im_data(negzero_set)=exp(0); % PS Set all values <=0
   negzero_set=[]; % free memory
   im_data=log(im_data); % PS perform log transformation
   waitbar(1,h); pause(1); close(h);
 end;

 % End of data scanning and log-transform 

% CALCULATE SRP

 if (voxtpropt ~= 'D') 
  h=waitbar(0,'Calculating Means ...');
  waitbar(.2,h);
  voxno=size(im_data,1);
  sub=size(im_data,2);
  
  %Compute within image mean vector
  %PS Don't count 0's in mean

  masksum=sum(mask);

  if (masksum == 0)
      set(FMenu,'Pointer','Arrow');
      vp_message(['Error: Mask has no positive elements ']);
      close(h);
      return;
  end

  meanvect=sum(im_data)/masksum;

  waitbar(.5,h);

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  %PS Replace because of memory restrictions
  %meanmat=ones(voxno,sub); 
  %for x=1:sub
   % meanmat(:,x)=meanvect(1,x)*meanmat(:,x);
  %end
  waitbar(.8,h);
  
  %devs=im_data-meanmat;
  %meanmat=[]; % free memory
  
  %PS Save memory
  for x=1:sub, im_data(:,x) = ( im_data(:,x)-meanvect(1,x) ).*mask ; end;

  waitbar(1,h); pause(1); close(h);

  if (voxtpropt == 'B')
   % CALCULATE MEAN REFERENCE DEVIATIONS IMAGE
   % reshape 3 dimensional images to one-dimensional vectors

   h=waitbar(0,'Processing Reference Images ...');

   for i=1:refsub
     msg=['reading image ' refiles(i,:)];
     disp(msg);
     waitbar((i-.8)/refsub,h,['Reading reference image ',num2str(i)]);
     
     %ref_dat(:,:,:,i)=spm_read_vols(spm_vol(refiles(i,:)));
     
     [out1 refvect(:,i)]=vp_volfuns('volvect',refiles(i,:));
     if (out1 < 0) vp_message(['ERROR reading reference file ',num2str(i)]); return; end;
     
   end
   
   % READING IN REFERENCE IMAGES
   
   waitbar(1,h); pause(1); close(h);
  
   %Calculate intrinsic data mask
   datmask=refvect(:,1) ~=0;
   for p=1:refsub
    datmask=datmask.*(refvect(:,p) ~=0);
   end;
  
   mask=mask.*datmask; % Incorporate previous composite mask

   % ALWAYS MASK with composite mask 
 
   h=waitbar(0,'Masking Reference Images ...');
   for k=1:refsub
       msg=['Masking Ref. Image ',num2str(k)];
       waitbar((i-.8)/refsub,h,msg);
       refvect(:,k)=refvect(:,k).*mask;
   end
   waitbar(1,h); pause(1); close(h);
 
   if log_transform
     h=waitbar(.4,'Processing Log Transformation of Reference Images ...');
     %negzero_refset=find(refvect <=0);
     for p=1:refsub % Save memory
       negzero_refset=find(refvect(:,p) <=0);
       refvect(negzero_refset,p)=exp(0); % PS Set all values <=0
     end
     %refvect(negzero_refset)=exp(0); % PS Set all values <=0
     negzero_refset=[]; % free memory
     refvect=log(refvect); % PS perform log transformation
     waitbar(1,h); pause(1); close(h);
   end;

 
   h=waitbar(0,'Calculating Reference Means ...');
   waitbar(.2,h);
   voxno=size(refvect,1);
   refsub=size(refvect,2);
   if (refsub <= 1) 
    set(FMenu,'Pointer','Arrow');
    vp_message(['Error: Number Of reference subjects (',num2str(refsub),') Must Be > 1']);
    close(h); return;
   end

   %maskpos=find(mask); masksum=size(maskpos,1),
   masksum=sum(mask);

   %Compute within reference image mean vector
   meanrefvect=sum(refvect)/masksum;

   waitbar(.5,h);

   %Compute within-refsubject deviations from mean of image
   %i.e., normalize to whole brain
   
   %%meanrefmat=ones(voxno,refsub);
   %%for x=1:refsub
    %%meanrefmat(:,x)=meanrefvect(1,x)*meanrefmat(:,x);
   %%end
  
   %%refdevs=refvect-meanrefmat;
   %%refvect=[]; % free memory
   %%meanrefmat=[]; % free memory
   %% waitbar(.8,h);
   %% mean_img=mean(refdevs');
   
   %% mean_img=mean_img';
   %% for x=1:refsub, %PS
    %% refdevs(:,x)=refdevs(:,x) -mean_img;
   %% end;
   %% refssf=refdevs'*gisimg,
   
   %% refdevs=[]; % free memory
   %% mean_img=mean_img';
   
  
  %PS Save memory on Row normalization
   for x=1:refsub, refvect(:,x) = (refvect(:,x) - meanrefvect(1,x)).*mask ; end;
   
   waitbar(.8,h);
   mean_img=mean(refvect,2);
   
   for x=1:refsub, %PS
    refvect(:,x)=refvect(:,x) -mean_img;
   end;

   % Calculate refscores for Z-transformation option later
   gisimg=gisimg.*mask; %Remask 
   normgis=norm(gisimg); % PS save norm
   gisimg=gisimg/normgis;

   refssf=refvect'*gisimg,
   
   refvect=[]; % free memory
   mean_img=mean_img';
   
   waitbar(1,h); pause(1); close(h);
  
  end % if opt=='B'

  if (voxtpropt ~= 'E') 
  
    h=waitbar(.1,'Calculating SRP ...');
  
    %Compute subject deviations from mean image
    mean_img=mean_img';
  
    waitbar(.3,h);
    for x=1:sub, %PS
     %devs(:,x)=devs(:,x) -mean_img;
     im_data(:,x)=im_data(:,x) -mean_img;
     waitbar(.3+.6*x/sub,h);
    end;
  
    %mean_imgmat=ones(voxno,sub);
    %for x=1:voxno
    %  mean_imgmat(x,:)=mean_img(x,1)*mean_imgmat(x,:);
    %  if x==voxno*(.4), waitbar(.5,h); end;
    %  if x==voxno*(.8), waitbar(.8,h); end;
    %end
    %devs=devs-mean_imgmat;
    %im_data=devs ; %srp' 
  
    waitbar(1,h); pause(1); close(h);
  end % PS END IF NOT OPT E
 end % PS END IF NOT OPT D

end %PS END IF NOT OPT A

if (voxtpropt ~= 'D') 
  h=waitbar(0,'Masking SRP ...');
  for k=1:sub
     msg=['Masking SRP Sub. ',num2str(k)];
     waitbar((i-.8)/sub,h,msg);
     im_data(:,k)=im_data(:,k).*mask;
  end
  waitbar(1,h); pause(1); close(h);
end

gisimg=gisimg.*mask; %Remask with composite 
normgis=norm(gisimg); % PS save norm
gisimg=gisimg/normgis;

% CALCULATE SCORES
ssf=im_data'*gisimg,

disp(' ');
disp('Calculated SSF')

ssfvoxtprfig=figure('name','SSF SCORES'); bar(ssf); xlabel('Subject'); ylabel('Subject SSF SCORE');
pos=get(ssfvoxtprfig,'position');
if sub < 5, pos(3)=pos(3)*sub/5; 
      set(ssfvoxtprfig,'position',[pos(1),pos(2),pos(3),pos(4)]);
end;

REFCALformat=0;
if (voxtpropt=='A')
 if sub > 1
    prompt=['Save As Recalibrated REFrence Scores? [Y|N|EXIT]'];
    POpt=vp_opt('X',prompt,'YES','REF-Scores','NO','TPR format','EXIT','');
    if strcmp(POpt,'YES'), 
        REFCALformat=1;
	if sub < 5
	    prompt=['Save Ref. Score? WARNING Limited Subjects = ',num2str(sub),', Continue or Exit and use the B or C REFERENCE options? [Y|N|EXIT]'];
	    POpt=vp_opt('X',prompt,'YES','REF-Scores','NO','TPR format','EXIT','');
	    if strcmp(POpt,'YES'), 
	    else
        	REFCALformat=0;
	    end;
	end
    else
    end;
 end
end

for i=1:sub, imls{i}=imfiles(i,:); end; % Create cell array

%file_name=vp_keyin(['Enter Save Scores Text Filename (q=EXIT) : '],ssf_name);
file_name=vp_keyin(['Enter TPR study name (q=EXIT) : '],ssf_name);
if (isempty(file_name) | file_name=='q'), 
     set(FMenu,'Pointer','Arrow');
     vp_note('Program Exited',2); return;
end;
	
% save ssf simple tpr format for all options
if sub==1
  ssf_name = strcat(pwdd, ['\',file_name,'_VtprScore.txt']);
else
  ssf_name = strcat(pwdd, ['\',file_name,'_VtprScores.txt']);
end
vp_write_srpfile(ssf_name,1,imls,ssf)
vp_dispfile(ssf_name);

if (REFCALformat==1)
  ssf_name = strcat(pwdd, ['\',file_name,'_VScores',num2str(ngisno),'.txt']);
  Zssf_name = strcat(pwdd, ['\',file_name,'_VZScores',num2str(ngisno),'.txt']);
  %save later
end
   
% PS add Z transformation option

getzscores=0;

if (voxtpropt=='A' | voxtpropt=='D')
  if sub == 1
     vp_note(['To derive a Z score for a single subject use Options B or C. ']);
  elseif (sub > 1)
  	%z score option may be misleading for small number of subjects
  	%if recalibrating, get Z scores without asking 
	
	if (sub < 5 & REFCALformat==0)
	    prompt=['Z Score? WARNING Limited Subjects = ',num2str(sub),', Continue or Exit and use the B or C REFERENCE options? [Y|N|EXIT]'];
	    POpt=vp_opt('X',prompt,'YES','Get Z-Scores','NO',['Don''t Z-Score'],'EXIT','');
	    if strcmp(POpt,'YES'), 
	      %getzscores=1;
	      %Zssf=(ssf - mean(ssf))./std(ssf),
	      %zscorefig=figure('name','SSF Z SCORES','position',[pos(1)+50,pos(2)+50,pos(3),pos(4)]);
	      %bar(Zssf); xlabel('Subject'); ylabel('Subject Z SCORE');
	    else
	      set(FMenu,'Pointer','Arrow');
	      vp_message(['END CHECK: ',ssf_name,', ']);
	      return;
	    end;
	    
	end
    
	no=1;
	while no~= 2 % Must enter at least two numbers (can be the same) or enter zero
	 Keyin=['1 - ',num2str(sub)];
	 %Keyin=['1 - '];
	 prompt=['Calculate Z SCORES, Specify 0 Controls or Control Subject Index Range [n1-n2] | 0 | q]   <',Keyin,'>   '];
	 vp_keyin(prompt,Keyin);

	 if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	      set(FMenu,'Pointer','Arrow');
	      vp_message(['END CHECK: ',ssf_name,', ']);
	      return;
	 elseif (strcmp(Keyin,'0') ),
	      norm1=0; norm2=0; no=2;
	 else
      
	      Keyin=strrep(Keyin,' ','');
	      [nn,no]=sscanf(Keyin,'%d-%d'),
	      norm1=nn(1)
	      %if no < 2, norm2=norm1, 
	      %else norm2=nn(2), end;
	      if no ==2  norm2=nn(2) 
	        if (norm1 < 1 | norm1 > sub), vp_note('Error: Number norm1 out of range',2);
	          no=1;
	        elseif (norm2 < 1 | norm2 > sub), vp_note('Error: Number norm2 out of range',2);
	          no=1;
	        elseif (norm1 == 1 & norm2 == sub), vp_note('Error: All subjects cannot be controls',2);
	          no=1;
	        end
	      end;
	 end 
	end % while
    
	if (norm2 > 0)
	  vp_note(['Control Subject Range: ',num2str(norm1),' - ',num2str(norm2)],2);
	end
	
	getzscores=1;
%	Zssf=(ssf - mean(ssf))./std(ssf),

 
	%Normalize SSF Scores (Z-score for gis over ref subjects)
	Zssf=[];
	Zssf=(ssf-mean(ssf))./std(ssf), 

	% Offset calculated for gis

	Zoffset=0;
	if norm2 > 0
	  if norm2 ~= norm1
	      Zoffset=mean(Zssf(norm1:norm2,:)),
	  else
	      Zoffset=Zssf(norm1,:),
	  end
	  Zssf=Zssf-Zoffset;
	end;
    
	zscorefig=figure('name','SSF Z SCORES','position',[pos(1)+50,pos(2)+50,pos(3),pos(4)]);
	bar(Zssf); ylabel('Subject Z SCORE');
	%yo=max(Zssf)/4;

	%%yo=0;
	%%if norm2 > 0
	  %for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
	%%  for i=norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
	%%  for i=norm1:norm2, text(i+.1,yo,'*','color','k'); end; %Mark controls
          %xlabel('Subject  (c: control)');
          xlabel('Subject  ( * : control)');
	%%else
        %%  xlabel('Subject ');
	%%end

	if norm2 ~= 0
	  xlabel('SUBJECT   (Control: __ __ )'); 
	  %ypos=min(Zssf)/2; 
          ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
	  text(norm1-0.5, ypos, '<','color','b');  %Mark controls
	  text(norm2,ypos,'>','color','b'); %Mark controls
	  line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
	else
	  xlabel('SUBJECT '); 
	end

	set(gca,'color',[.9 1.0 .9]);
  end
else % Ref Case Z transform


  prompt=['Z-transform SSF Scores Based on REF Scores? [Y|N|EXIT]'];
  POpt=vp_opt('X',prompt,'YES','Calculate Z Scores','NO','Don''t Z Score','EXIT','');
  if (POpt=='X' | strcmp(POpt,'EXIT') | strcmp(POpt,'NO')), 
          set(FMenu,'Pointer','Arrow');
          vp_message(['END CHECK: ',ssf_name,', ']);
	  vp_note('Program Exited',2); return;
  end
  
  if (voxtpropt=='B')
    ssfscores=refssf ; norm1=0; norm2=0; nosubs=refsub; ngis=1; % get later 
  else   %(voxtpropt=='C')
    ssffile= get_file(1,'*VScores*.txt','Specify REFERENCE Score file',[ ]);  
    [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2] =vp_ssm_scores(-2,ssffile)
    [nosubs, num_gis]=size(ssfscores),
    if (num_gis ~= 1), vp_note('Must specify single gis',2);
      ngis=0;
      while (ngis > num_gis | ngis < 1),
        prompt=['GIS NO.? [1-',num2str(num_gis),'| q]'];
        vp_keyin(prompt,Keyin);
	if (strcmp(Keyin,'q')),return, end;
        ngis=str2num(Keyin);
        if (ngis > num_gis | ngis < 1), vp_note(['Error: ngis= ', num2str(ngis),' Out of Range'],2); end;
      end %while
    end %if
  end %else
      
  if (norm1==0 | norm2==0)
        %if (voxtpropt=='C')
        %  vp_message(['Invalid Reference File: ',ssffile,' Control Subjects Not Specified']);
	%end

	no=1;
	while no~= 2 % Must enter zero or at least two numbers even if the same
	 Keyin=['1 - ',num2str(nosubs)];
	 prompt=['Specify 0 Controls or Control Subject Index Range [n1-n2] | 0 | q]   <',Keyin,'>   '];
	 vp_keyin(prompt,Keyin);

	 if (isempty(Keyin) | strcmp(Keyin,'q') | strcmp(Keyin,'quit')),
	   set(FMenu,'Pointer','Arrow');
	   vp_message(['END CHECK: ',ssf_name,', ']);
	   return;
	 elseif ( strcmp(Keyin,'0') ), norm1=0; norm2=0; no=2;
	 else
      
	      Keyin=strrep(Keyin,' ','');
	      [nn,no]=sscanf(Keyin,'%d-%d');
	      norm1=nn(1) ;
	      if no ==2  norm2=nn(2) 
	        if (norm1 < 1 | norm1 > nosubs), vp_note('Error: Number norm1 out of range',2);
	          no=1;
	        elseif (norm2 < 1 | norm2 > nosubs), vp_note('Error: Number norm2 out of range',2);
	          no=1;
	        elseif (norm1 == 1 & norm2 == nosubs), vp_note('Error: All ref. subjects cannot be controls',2);
	          no=1;
	        end %if norm1
	      end; %if no==2
	 end %if isempty 
	end % while no~=2
	
  end; % if norm1
      
    
  if norm2 > 0
    vp_note(['Control Subject Range: ',num2str(norm1),' - ',num2str(norm2)],2);
    getzscores=1;
  end;
 
  %Normalize SSF Scores (Z-score for gis over ref subjects)
  Zssfs=zeros(size(ssfscores));
  if (ngis <=0), vp_note(['INVALID GIS NO, ngis= ',num2str(ngis)],3); return; end;
  Zssfs(:,ngis)=(ssfscores(:,ngis)-mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 

  % Offset calculated for each gis

  numgis=size(Zssfs,2);
  Zoffset(1:numgis)=0;
  if norm2 > 0
    if norm2 ~= norm1
      Zoffset=mean(Zssfs(norm1:norm2,:)),
    else
      Zoffset=Zssfs(norm1,:),
    end
    Zssfs(:,ngis)=Zssfs(:,ngis)-Zoffset(ngis)*ones(nosubs,1);
  end;
    
 
  %Use the same mean, std and offset calculated for the reference to get the new Zscores.

  %Zssf=zscore(ssf),
  %Zssf=(ssf - mean(ssf))./std(ssf),
  Zssf=(ssf - mean(ssfscores(:,ngis)))./std(ssfscores(:,ngis)); 
  Zssf=Zssf-Zoffset(ngis)*ones(sub,1);;
   
    
  zscorefig=figure('name','TPR Z SCORES'); bar(Zssf); xlabel('New Subject'); ylabel('New Subject Z SCORE');
    
  pos=get(zscorefig,'position');
  pos(3)=max(pos(3)/3, pos(3)*(sub/nosubs));

  set(zscorefig,'position',pos);
   
  %Display Combined Scores (Group + New Subjects)
  ZCssf=[Zssfs(:,ngis); Zssf;];
  ZCscorefig=figure('name','TPR COMBINED Z SCORES'); bar(ZCssf);

  if norm2 > 0
    %xlabel('Reference and New Subject(s) c: control, s: new subject'); ylabel('COMBINED GROUP Z SCORES');
    xlabel('Reference and New Subject(s) * : control, + : new subject'); ylabel('COMBINED GROUP Z SCORES');
  else
    xlabel('Reference and New Subject(s) +: new subject'); ylabel('COMBINED GROUP Z SCORES');
  end
  %yo=max(ZCssf)/4;

  %%yo=0;
  %%if norm2 > 0
    %for i=norm1:norm2, text(i,yo,'c','color','r'); end; %Mark controls
  %%  for i=norm1:norm2, text(i,yo,'*','color','w'); end; %Mark controls
  %%  for i=norm1:norm2, text(i+.1,yo,'*','color','k'); end; %Mark controls
  %%end;
  %for i=nosubs+1:nosubs+sub, text(i,yo,'s','color','m'); end; %Mark new subjects
  %%for i=nosubs+1:nosubs+sub, text(i,yo,'+','color','w'); end; %Mark new subjects
  %%for i=nosubs+1:nosubs+sub, text(i+.1,yo,'+','color','k'); end; %Mark new subjects

  if norm2 > 0
    xlabel('REFERENCE and NEW SUBJECT(S), Controls: __ __, New Subjects: --'); ylabel('COMBINED GROUP Z SCORES');
  else
    xlabel('REFERENCE and NEW SUBJECT(S), New Subjects: --'); ylabel('COMBINED GROUP Z SCORES');
  end

  %ypos=min(ZCssf)/2; 

  ypos=get(gca,'Ytick'); if ypos(2) >=0, ypos=(ypos(1)+ypos(2))/2, else ypos=ypos(2); end;
  if norm2 > 0
    text(norm1-.5,ypos,'<','color','b');  %Mark controls
    text(norm2,ypos,'>','color','b');  %Mark controls
    line([norm1-.5,norm2],[ypos,ypos],'LineStyle','--');
  end

  text((nosubs+0.8),ypos,'<','color','b'); %Mark new subjects
  text((nosubs+sub),ypos,'>','color','b'); %Mark new subjects
  line([nosubs+1,nosubs+sub],[ypos,ypos],'LineStyle',':');
  set(gca,'color',[.9 1.0 .9]);

  pos=get(ZCscorefig,'position');
  pos(1)=pos(1)+ 2*pos(3)/3;
  set(ZCscorefig,'position',pos);
 
end % if voxtpropt


if REFCALformat %Opt A only
    % Save Gis Reference Scores
    %vp_write_ssmfile([refname,'_VScores',num2str(ngis),'.txt'],ngis,vaf(ngis),file_list,ssf(:,ngis),1,norm1,norm2)
    vp_write_ssmfile(ssf_name,ngisno,-1,imls,ssf,4,norm1,norm2)
    if getzscores == 1
	vp_write_ssmfile(Zssf_name,ngisno,-1,imls,Zssf,5,norm1,norm2)
    end
    vp_dispfile(Zssf_name);

else
  if getzscores == 1
     if sub==1
       Zssf_name = strcat(pwdd, ['\',file_name,'_VtprZScore.txt']);
     else
       Zssf_name = strcat(pwdd, ['\',file_name,'_VtprZScores.txt']);
     end
     vp_write_srpfile(Zssf_name,1,imls,Zssf)
     vp_dispfile(Zssf_name);
  end
end

set(FMenu,'Pointer','Arrow');

DiagnOpt=0; % OMIT Diagnostic Report
if ~(voxtpropt=='A' | voxtpropt=='D' | DiagnOpt==0 | norm2==0)
 
 % write diagnostic report
 netnam=vp_nm('dnm',gisfile);

 DIAGNrpt_name = strcat(file_name, '_DIAGNrpt.txt');
 
 Keyin='0'; clear cycle;
 %Min Z for Total TRUE Max
 diagntype=4;
 vp_ssm_prediagnrpt;
 
 %vp_DIAGNrpt(DIAGNrpt_name,netnam,imls,Zssf,'VOXEL based Z Scores',Zssfs(:,ngis),norm1,norm2);
 
 vp_MDiagnRpt(DIAGNrpt_name,netnam,imls,Zssf,'VOXEL based Z Scores',Zssfs(:,ngis),norm1,norm2,nosubs,diagntype,ZRadius,1);
 vp_dispfile(DIAGNrpt_name);
 
 vp_message(['END CHECK: ',ssf_name,' and ',DIAGNrpt_name]);
else

 vp_message(['END CHECK: ',ssf_name]);
end



