% vp_ssm_voxtprrepeat
% Voxel based SSM Automatic Topographic Profile Rating (TPR) evaluated
% for repeated TPR operation of the same or on independent groups.
% For the repeated measures option it performs evaluation of the
% same subject(s) SSM scores for a repeated set of tasks or states
% using a pre-defined GIS pattern. For independent groups the subjects 
% are not the same.
% A scatterplot of scores between the first and each consecutive trial
% is displayed with the option for the same subject cases to connect lines.
% Discriminant p values for t-test comparison of means between different 
% groups and paired t-test p values for repeated same group trials are posted. 
% TPR value files for each trial are output and scatter plots are generated
% between trials. The composite scatter plot of three repeated trials 
% of group data is also displayed.
%_______________________________________________________________________
%
% Input:
% The user selects as input
%	(1) Repeated Measures: A group of one or more spatially registered subject 
%	    imge files for which repeat images were generated for different trials.
%	    Repeated Groups:  A new group for each repeated TPR image evaluation.
%	(2) A predefined coregistered network GIS image for evaluation of subject 
%	    scores for the different trials.
%	    The GIS must have been generated by a previous group PCA analysis
%	    but reference scores files are not considered and derivation group 
%	    mean profiles are optional.
%
% The new subject set need not include subjects from the previous study
% since each subject is evaluated individually on a prospective basis.
% New subjects need not be scanned with the same parameters or tracer as 
% as the derivation subjects but should be scanned and spatially normalized
% similarly for each repeat group or trial that is evaluated.
%
% The GIS definition image file results from an original PCA analysis
% and contains the relative voxel weights for that GIS.
% 
% Method:
% The following procedure is used to derive new subject scores for each trial.
% New subject/group image files and the reference GIS network image file are selected.
% The intrinsic non-zero voxel mask is determined from the reference GIS file.
% New images are masked and natural log transformation is performed.
% Logs of negative data values are set to zero. (Optional non-log
% transformation is not available in this version.)
% Data mean centering normalization is performed on a subject basis:
% The mean value over all non-zero voxels is evaluated for each subject
% for each separate state (condition or trial) and used to generate within subject
% regional deviations from the mean.
% Displayed scores can be direct values, mean centered with respect to the
% pattern reference group or Z-scored with respect to trial 1 or reference.
% Thus, unless reference group normalization is selected, the group log mean 
% image values of the reference data are not subtracted from the corresponding
% new subject values when generating the new residual profile (SRP).
% The new SSF subject scores are determined for each state using the following
% calculation:
% 
% 	          SSF=SRP*GIS/norm(GIS),
%
% where SSF is the score vector (no of new subs.), SRP is the centered subject data 
% matrix (nsubs, nvoxs) and GIS is the pattern vector (nvoxs). 
%
% An output file containing the new SSF scores for each state is generated
% and scatter plots are displayed. P values of (paired) t-test comparison
% are evaluated with respect to the first state (baseline) and are listed
% between other states. Optionally, Z scores are displayed and saved with 
% respect to baseline (trial 1) scores or with respect to the reference 
% pattern derivation group. A composite scatter plot and bar graph plot for 
% all states is also generated. An option for connecting score value points
% on scatter plots between states for paired subjects is provided. 
%_______________________________________________________________________
% vp_ssm_autovoxrepeat    ScAnVP 6.2   P.Spetsieris   3/17/2012
% 	(modeled in part on vp_ssm_autovoxtpr, P.Spetsieris)
% vp_ssm_voxtprrepeat	ScAnVP  6.3	P.Spetsieris   12/13/2013
%_______________________________________________________________________

global POpt FMenu Keyin NameLst
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global HelpMode

if HelpMode, vp_help(mfilename); return ; end;

im_data=[]; imls=[]; ssf=[];

pwdd=pwd;
trial=0;
sub=0;
subn=[];
lastrial=1;

same_group=1; readref_mean=1;
zscore_type=0; % 0: Do not Z-Score; 1: Z-Score to Reference; 2: Z-Score to Trial 1 Scores 
ZTyp='';


prompt=['REPEATED TPR- SELECT GROUP TYPE'];
optA=['Repeated TPR - Score the Same Group in Multiple Trials'];
optB=['Repeated TPR - Score Independent Groups'];
POpt=vp_opt('X',prompt,'REPEATED MEASURES - SAME GROUP',optA,'INDEPENDENT GROUPS TPR',optB,'EXIT','','Help','');
if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxtprrepeat'); 
  return; end;
if(strcmp(POpt,'REPEATED MEASURES - SAME GROUP')),
  vp_note(optA,3);
  same_group=1; end;
if(strcmp(POpt,'INDEPENDENT GROUPS TPR')),
  vp_note(optB,3);
  same_group=0; end;


if trial==0


POpt='STANDARD GIS Network',
prompt=['REPEATED TPR: STANDARD or OTHER REFERENCE GIS?'];
optA=['Specify scanvp\networks Directory GIS to Calculate TPR scores.'];
optB=['Specify Other Directory Reference Network GIS to Calculate TPR scores.'];
POpt=vp_opt('X',prompt,'STANDARD GIS Network',optA,'OTHER REFERENCE GIS Network',optB, 'EXIT','','Help','');

if(strcmp(POpt,'X'))return; end;
if(strcmp(POpt,'EXIT'))return; end;
if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxtprrepeat'); 
  vp_ssm_voxtprrepeat;
  return; end;
if(strcmp(POpt,'STANDARD GIS Network')),
  vp_note(optA,3);
  w=which('scanvp');
  netdir=[w(1:(max(findstr(w,'\'))-4)),'networks'],
elseif(strcmp(POpt,'OTHER REFERENCE GIS Network')),
  netdir=[];
  vp_note(optB,3);
else
  vp_note(['Invalid Option ',POpt],3);
  return;
end

	
 prompt=['REPEATED TPR: Select Reference GIS'];
 disp(prompt);

 gis_name= get_file(1,'*gis*.img',prompt,netdir,[ ]);  
 %gis_name= get_file(1,'*gis*.img',prompt,[ ]);  

 if isempty(gis_name), return; end;
 [dnm,dnm,dir]=vp_nm('dnm',gis_name);
 p=max([findstr(gis_name,'gis') 0]);
 l=length(gis_name);
 condnm=gis_name(1:p-1);
 gisid=gis_name(p+3:l-4);
 if length(gisid) > 1
  ngisid=str2num(gisid);
  if ~isempty(ngisid) % only if numeric
    gisid=num2str(ngisid);
  end
 end

 p=max([findstr(dnm,'networks') 0]);
 l=length(dnm);
 netnam=dnm(p+9:l);

 [out1 gisimg]=vp_volfuns('volvect',char(gis_name));
 if (out1 < 0) vp_message('ERROR reading gisfile');
 return, end;

 % reshape into column vector and normalize to
 % have Euclidean norm=1
 normgis=norm(gisimg); % PS save norm
 gisimg=gisimg/normgis;
 gismask=(gisimg ~= 0);

 num_vox = size(gisimg,1); % PS

 %prompt=['Reference Group Mean Centering ON/OFF'];
 prompt=['Score Output Type: '];
 optA=['Direct Scores, No Group Mean Offset.'];
 optB=['Z Scored With Respect to Trial 1 Scores (Baseline).'];
 optC=['Centered with Respect to Reference Pattern Group Mean.'];
 optD=['Z Scored With Respect to Reference Pattern Group Scores.'];


 POpt=vp_opt('X',prompt,'DIRECT-NO OFFSET', optA, 'Z-SCORED TO TRIAL 1', optB,...
 'REF GROUP MEAN CENTERED',optC,'Z-SCORED TO REF GROUP',optD,'EXIT','','Help','');

 if(strcmp(POpt,'X'))return; end;
 if(strcmp(POpt,'EXIT'))return; end;
 if(strcmp(POpt,'Help')),
  vp_help('vp_ssm_voxtprrepeat'); 
  return; end;
 if(strcmp(POpt,'DIRECT-NO OFFSET')),
  vp_note(optA,3); readref_mean=0; zscore_type=0; ZTyp=''; end;
 if(strcmp(POpt,'Z-SCORED TO TRIAL 1')),
  vp_note(optB,3); readref_mean=0; zscore_type=2; ZTyp='B1-Z'; end;
 if(strcmp(POpt,'REF GROUP MEAN CENTERED')),
  vp_note(optC,3); readref_mean=1; zscore_type=0; ZTyp=''; end;
 if(strcmp(POpt,'Z-SCORED TO REF GROUP')),
  vp_note(optD,3); readref_mean=1; zscore_type=1; ZTyp='Z'; end;

end % if trial ==0

if readref_mean

  % READ MEAN DEVIATIONS REFERENCE IMAGE
  meanfile=[condnm,'lnmean.img'];
  if exist(meanfile) ~=2
    meanfile=[condnm,'lmean.img'];
    if exist(meanfile) ~=2
      vp_message(['Error: Could Not find either ',condnm,'lnmean.img or ',meanfile,' file']);
      return;
    end
  end
 
  [out1 mean_img]=vp_volfuns('volvect',meanfile);
  if (out1 < 0) vp_message('ERROR reading mean file');
  return, end;
 
  refvoxno = size(mean_img,1); % PS
  
  % DERIVE MASK FROM MEAN IMAGE FILE
  mask = (mean_img ~= 0);

  % INCORPORATE Into GIS MASK
  gismask = mask.*gismask;

  masksum=sum(gismask),
  if (masksum == 0)
     set(FMenu,'Pointer','Arrow');
     vp_message(['Error: Composite Mask has no positive elements ']);
     return;
  end

end % readref_mean


if zscore_type==1

  % Z transformation, read REF Score file.
  ssffile=[condnm,'VScores',gisid,'.txt'];
  if exist(ssffile) ~=2
     vp_message(['Error: Could Not find REFerence file ',ssffile,' file']); return;
  end

  [mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2] =vp_ssm_scores(-2,ssffile)
  [nosubs, num_gis]=size(ssfscores),

  %Normalize SSF Scores (Z-score for gis over ref subjects)
  
  ssfmean=mean(ssfscores(:,ngis)); ssfstd=std(ssfscores(:,ngis));
  Zssfs(:,ngis)=(ssfscores(:,ngis)-ssfmean)./ssfstd; 

  % Offset calculated for gis
  Zoffset=0;
  if norm2 > 0
   if norm2 ~= norm1 % Offset Z-Score from Mean Normal Score
        Zoffset=mean(Zssfs(norm1:norm2,ngis)),
   else
        Zoffset=Zssfs(norm1,ngis),
   end
   % REF Z-Score for future use
   REFZssfs=Zssfs(:,ngis)-Zoffset*ones(nosubs,1);
  else
   REFZssfs=Zssfs(:,ngis);
  end
    
end % if zscore_type==1

while trial < lastrial

trial=trial+1;
ssf=[];

if trial==1
 if same_group
  prompt=['REPEATED TPR: Select Subjects Initial State (TRIAL 1) Image Files or CANCEL'];
 else
  prompt=['REPEATED TPR: Select Group 1 Subject Image Files or CANCEL'];
 end
else
 if same_group
  prompt=['REPEATED TPR TRIAL/CONDITION ',num2str(trial),': Select Subjects New State Image Files or CANCEL'];
 else
  prompt=['REPEATED TPR GROUP ',num2str(trial),': Select New Subjects Image Files or CANCEL'];
 end
end
if vp_filesui(Inf,'*.img',prompt,mfilename) < 0 , break; end;
disp(prompt);

imfiles_B=NameLst;
sub= size(NameLst,1);

subn(trial)=sub;

if same_group
 if trial > 1, 
   if sub ~= subn(1),
     set(FMenu,'Pointer','Arrow');
     vp_message('Error: Subjects must be the same in each set'); return;
   end;
 end
end; 

log_transform=1; % PS Force log transformation 

if sub==1
   img1=imfiles_B(1,:);
   outnm=vp_nm('nm',img1);
else
  outnm=['S',num2str(sub),'_',num2str(trial)];
end
 
h=waitbar(0,'Processing images ...');

im_data=zeros(prod(DIM), sub, 'single');

% Reading in images

for i=1:sub
  msg=['reading image ' imfiles_B(i,:)];
  disp(msg);
  waitbar((i-.8)/sub,h,['Reading image ',num2str(i)], ' TRIAL/CONDITION ',num2str(trial));
  [out1 im_data]=vp_volfuns('volvect',imfiles_B(i,:));
  if (out1 < 0) vp_message(['ERROR reading image data file ',num2str(i)]); return, end;

  %Incorporate Intrinsic Data Mask for Incomplete Data Cases
  mask=(im_data ~= 0).*gismask;
  masksum=sum(mask);
  if (masksum == 0)
   set(FMenu,'Pointer','Arrow');
   vp_message(['Error: Mask has no positive elements ']);
   return;
  end
   
  % Masking images
  msg=['Masking Image ',num2str(i)];
  waitbar((i-.6)/sub,h,msg);
  im_data=im_data.*mask;

  if log_transform
   	waitbar((i-.4)/sub,h,'Processing Log Transformation of Image ...');
   	negzero_set=find(im_data<=0);
   	im_data(negzero_set)=exp(0); % PS Set all values <=0
   	negzero_set=[]; % free memory
   	im_data=log(im_data); % PS perform log transformation
  end;
  % End of data scanning and log-transform 

  % CALCULATE SRP
  voxno=size(im_data,1);

  % This requirement may be conservative but necessary for accuracy
  if voxno ~= num_vox, 
 	 set(FMenu,'Pointer','Arrow');
 	 vp_message('Error: Number of GIS voxels do not agree with data'); return;
  end

  %Compute within image mean vector
  %PS Don't count 0's in mean
  meanvect=sum(im_data)/masksum;

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  %for x=1:sub, im_data(:,x) = (im_data(:,x) - meanvect(1,x)).*mask ; end;
  im_data = im_data - meanvect.*mask ; %subtract mask multiplied by constant 

  waitbar(i/sub,h,'Calculating Image SSF...');
  
  if readref_mean
     % Compute subject deviations from mean image 
     im_data=(im_data -mean_img).*mask; 
  else
     %OMIT Compute subject deviations from mean image 
  end

  %generate SSF values for a known GIS pattern based
  %on SRP calculated in the new population.

  % calculate new ssf
  ssf(i)=im_data'*gisimg,
  
end % for subject i

ssf=ssf';

if zscore_type==1 % Z-Score to Reference Scores
  % Use the same mean, std and offset as the reference to calculate new Z scores.
  Zssf=(ssf -ssfmean)./ssfstd; 
  ssf=Zssf-Zoffset*ones(sub,1); %Relabel Zssf ssf
end %if zscore_type==1

if zscore_type==2 % Z-Score to Baseline (Trial 1)
 if trial==1
   ssfmean=mean(ssf); ssfstd=std(ssf);
 end
 ssf=(ssf -ssfmean)./ssfstd; 
end %if zscore_type==2

im_data=[]; %PS free memory
imls=[]; 

for i=1:sub, imls{i}=imfiles_B(i,:); end; % Create cell array

p=max([findstr(dnm,'networks') 0]);
l=length(dnm);
if p~=0, netnam=dnm(p+9:l), else netnam=dnm; end;
nm=vp_nm('nm',netnam); 
p=max([findstr(nm,'_') 3]);
flnm=[nm(1,1:p-1),outnm];
if sub==1
  ssf_name = strcat(pwdd, ['\',flnm,'_S1_Trial',num2str(trial),'_Vtpr',ZTyp,'Score.txt']);
else
  ssf_name = strcat(pwdd, ['\',flnm,'_Trial',num2str(trial),'_Vtpr',ZTyp,'Scores.txt']),
end

%ssf_name, imls, ssf,
vp_write_srpfile(ssf_name,1,imls,ssf)
vp_dispfile(ssf_name);

waitbar(1,h); pause(1); close(h);

disp ('   ***************************************************     ')
disp(' ');
disp('             RESULTS OF GIS2SSM TPR')
disp(' ');
disp(['Newly Calculated ',ZTyp,'SSF'])
ssf

ssftprfig=figure('name',['TPR ',ZTyp,'SCORES TRIAL ', num2str(trial)]);
bar(ssf); xlabel('Subject'); ylabel(['TPR ',ZTyp,'SCORE - TRIAL ', num2str(trial)]);

pos=get(ssftprfig,'position');
%pos(3)=pos(3)*(sub/15); % use 15 as a standard number of subjects
if sub < 15
  pos(3)=max(pos(3)/3, pos(3)*(sub/15)); % use 15 as a standard number of subjects
end
set(ssftprfig,'position',pos);
    
% END GIS2SSF CODE
if trial==1, Basessf=ssf; Sssf=ssf; end;

if trial > 1

  %Display Dual State Scores (Baseline + New State)

  Cssf=[ Basessf; ssf;]; % first and last
  Sssf=[ Sssf; ssf;]; % Sum of all ssfs
  
  Cscorefig=figure('name',['TPR TRIAL 1 vs. TRIAL  ',num2str(trial),' TPR ',ZTyp,'SCORES']); 

  bar(Cssf); ylabel(['TPR ',ZTyp,'SCORES']);
  xlabelbar=['TRIAL 1 vs. TRIAL ',num2str(trial),' SUBJECT INDEX']; 
  xlabel(xlabelbar); 

  xlabelscat=['TRIAL 1 vs. TRIAL ',num2str(trial)]; 

  ypos=min(Cssf)/2; if ypos> 0, ypos=-ypos; end;
  text(1-.5,ypos,'<','color',[0 0 0]);  %Mark baseline start
  text(subn(1),ypos,'>','color',[0 0 0]);  %Mark baseline end
  line([1,subn(1)],[ypos,ypos],'LineStyle','--','color',[0 0 0]);

  text((subn(1)+0.8),ypos,'<','color',[0 0 0]); %Mark new subjects
  text((subn(1)+subn(trial)),ypos,'>','color',[0 0 0]); %Mark new subjects
  line([subn(1)+1,subn(1)+subn(trial)],[ypos,ypos],'LineStyle',':','color',[0 0 0]);
    
  pos=get(Cscorefig,'position');
  pos(1)=pos(1)+2*pos(3)/3;
  set(Cscorefig,'position',pos);
  

% create scatter plots

 if trial >= 2

  labelx=[' TRIAL 1 to ',num2str(trial)];

  %labelx='   ';
  %for i = 1:trial
  %   labelx=[labelx, 'TRIAL ',num2str(i),'               '];
  %end %for 

  labely=['TPR ',ZTyp,'SCORES'];

  if same_group
    disp('paired t-test');
       %vp_scatterpairsplot(Cssf,sub,2*sub,0,xlabelscat,labely,netnam)
	pval=vp_scatternplot(Cssf,subn(1),2,same_group,netnam,xlabelscat,labely);
  else
    disp('student''s t-test');
       %vp_scatterplot(Cssf,subn(1),(subn(1)+subn(trial)),0,xlabelscat,labely,netnam)
       pval=vp_scatternplot(Cssf,[subn(1) subn(trial)],2,same_group,netnam,xlabelscat,labely);
  end

  if trial > 2
    % Display Combined Graph
   
    if same_group
      disp('paired t-test');
	pval=vp_scatternplot(Sssf,subn(1),trial,same_group,netnam,labelx,labely);
    else
      disp('student''s t-test');
      pval=vp_scatternplot(Sssf,subn,trial,same_group,netnam,labelx,labely);
    end
  end; %if trial > 2

 end %if trial > 2

end % if trial > 1

lastrial=lastrial+1; %Repeat for next state
end %while trial

trial=trial-1;

if trial > 2, %display combined bar graph

  Sscorefig=figure('name',['TPR TRIAL 1 TO TRIAL  ',num2str(trial),' TPR SCORES']);
  bar(Sssf);
  ylabel(['TPR ',ZTyp,'SCORES']);

  xlabelscat=['TRIAL 1 TO TRIAL ']; 
  xlabelbar=['TRIAL 1 TO TRIAL ',num2str(trial),' SUBJECT INDEX']; 
  xlabel(xlabelbar); 

  ypos=min(Sssf)/2; if ypos> 0, ypos=-ypos; end;
  
  i=1; j=1;
  text(j-.5,ypos,'<','color',[0 0 0]);  %Mark start
  text(j+subn(i)-1,ypos,'>','color',[0 0 0]);  %Mark end
  line([j, j+subn(i)-1],[ypos,ypos],'LineStyle','--','color',[0 0 0]);

  j=0;
  while i < trial

    i=i+1;
    j=j+subn(i-1);

    text((j+0.8),ypos,'<','color',[0 0 0]); %Mark new subjects
    text((j+subn(i)),ypos,'>','color',[0 0 0]); %Mark new subjects
    line([j+1,j+subn(i)],[ypos,ypos],'LineStyle',':','color',[0 0 0]);

    i=i+1;
    if i > trial, break; end; 
    j=j+subn(i-1);

    text((j+0.8),ypos,'<','color',[0 0 0]); %Mark new subjects
    text((j+subn(i)),ypos,'>','color',[0 0 0]); %Mark new subjects
    line([j+1,j+subn(i)],[ypos,ypos],'LineStyle','--','color',[0 0 0]);

  end % while i

  pos=get(Cscorefig,'position');
  pos(1)=pos(1)+2*pos(3)/3;
  set(Sscorefig,'position',pos);
  

end % if trial

if trial > 1, 
 if zscore_type==1, 
   p_REF=ones(1, trial);
   
   for j=1:trial
    % Independent Groups
    % test equal variances
    ij= sum(subn(1:j-1)); subnj=subn(j);
    [Hf3,Pf3]= ftest_levene(REFZssfs, Sssf(ij+1:ij+subnj), .05); 
    if Hf3 == 0, % equal variances
      ev3='(ev)'; % disp('Equal Variance, ttest2:');
      [H3,Pj,CI3,STATS3]=ttest2(REFZssfs, Sssf(ij+1:ij+subnj), .05);
    else
      ev3='(uev)'; % disp('Unequal Variance, tutest2:');
      [H3,Pj,CI3,STATS3]=tutest2(REFZssfs, Sssf(ij+1:ij+subnj), .05);
    end
    p_REF(1,j)=Pj;
   end %for j

   disp(' ');
   disp(['P values of each trial''s scores with respect to REF scores (t-test for indep. samples)']); 
   p_REF, %display p values

 end % if zscore_type==1
end % if trial > 1

disp(' ');
if same_group, disp('Paired Values t-tests'); else disp('Independent Samples t-tests'); end;

disp(['(rounded) P value matrix of each set i,j of trial''s scores']); 
pval,

[j k]=size(pval);
for i=1:j, for m=1:k,
 disp(['p',num2str(i),num2str(m),'=',num2str(pval(i,m))]);
end; end;


set(FMenu,'Pointer','Arrow');


