% SSM-PCA INFO, ScAnVP (C) 2003-2016
% Center for Neurosciences, Feinstein Institute for Medical Research,
% Northwell Health (Formerly, North Shore-LIJ Health System) 
%		
%
%				SSM-PCA OVERVIEW
% (See also Documentation, References and Task Menu Help)
%
% ScAnVP incorporates an implementation of the Scaled Subprofile Model of
% Principal Component Analysis (SSM-PCA) for multivariate regional and voxel
% based spatial covariance image analysis of single-volume steady-state 
% brain images. SSM-PCA is applied to derive characteristic patterns that
% are disease network biomarkers used to study neurodegenerative disorders.
% Its primary applications have been performed on PET-FDG data to study
% brain metabolism but it has also been applied to study blood flow and other
% parameters using alternative nuclear tracer modalities such as PET-CBF and 
% SPECT as well as time-independent radiological images such as MRI based ASL,
% VBM and DTI and ALFF of BOLD fMRI. Detailed information regarding
% applications is available in a large body of published literature including
% selected references shown below. Here we provide a basic description. 
%
% HISTORY OF IMPLEMENTATION
% SSM was first described by J.R.Moeller, et. al (1987, 1991, 1994). The
% routines incorporated by ScAnVP are based on an early version written by 
% John D. Van Horn Ph.D.(1996) and subsequently modified by Marc Mentis M.D.
% (1998) who also implemented an initial MATLAB version and interface. The
% routines were substantially modified and adapted for the ScAnVP interface
% by P. Spetsieris Ph.D.. They include some contributions for ROI based SSM
% -PCA by Yilong Ma Ph.D. (2003) and voxel based orthogonal PCA by Chris 
% Habeck Ph.D. (2002). Many new interactive and automatic options have since
% been added in the current ScAnVP implementation by P. Spetsieris.
%
% THEORETICAL BASIS
% The SSM-PCA method is a multivariate approach that identifies major
% sources of variation in combined groups of healthy and disease subject
% coregistered steady state data. Using SSM-PCA, the image data are reduced
% to a minimal number of significant orthogonal linearly independent
% principal component covariance patterns while rejecting non-disease related
% sources of variation. These patterns or their linear combination are
% referred to as Group Invariant Subprofiles (GIS networks) that represent
% a unique spatially distributed network of functionally interrelated brain
% regions that is expressed in each subjects to a variable degree. The
% degree of expression in each patient is represented by a simple scalar
% score that can correlate with clinical measures of disease severity.
% Using this method large global mean scalar effects that can obscure
% these smaller disease network-specific contributions in univariate based
% approaches are removed by the inherent logarithmic conversion and mean
% centering of the data performed in the multivariate SSM-PCA. 
% Validated disease patterns can be used to score individual patients with
% respect to a fixed reference sample. These scores can in turn be used to
% assess disease progression or treatment effects or can be entered in
% automated differential diagnosis algorithms that discriminate between
% similar conditions.
%
% BASIC ALGORITHM
% The SSM-PCA algorithm involves several steps performed on the image data
% to determine disease networks or spatially covariant regions or voxels
% that distinguish patients from healthy controls. 
% These include coregistration of images, masking, log transformation and
% centering of the data, principal component analysis (PCA) of the group
% covariance matrix and evaluation of discriminating principal components
% (PCs):
% 
% An equal number of patient and healthy subject single-volume scans are
% entered in the analysis. During preprocessing, images are coregistered
% and normalized into a common stereotaxic space (e.g. Talairach or Montreal
% Neurological Institute [MNI]) using SPM or other external software.
%
% The resulting ANALYZE or nifti format images are then masked or
% thresholded uniformly to limit the analysis to grey matter areas.
% In voxel based analysis the entire brain volume grey matter areas
% are processed whereas in regional analysis user defined planar regional
% or whole volume ATLAS templates are applied to extract ROI mean values.
% Each subjects values are entered into a row of the group data matrix
% where each column corresponds to a specific voxel or region.
% For voxel analysis, operations are performed on common whole brain gray
% matter space determined as the group cross section of individual subject
% space and any external mask. It is assumed that excessive atrophy or
% hemi-image inclusion in group analysis is limited to similar images
% otherwise common masked voxel space is minimal or zero.
% 
% The data is log transformed and each subject dependent row is centered
% by subtracting the row mean of the log values. The data matrix is
% additionally column centered by subtracting the mean row image or
% group mean profile (GMP image).  The resulting subject residual profile
% (SRP) images represent deviations from both subject and group means.
% PCA is applied to the covariance matrix of the double-centered matrix
% to derive orthogonal eigenvectors and associated eigenvalues and subject
% scores.  The spatial eigenvectors are principal component (PC) image
% patterns of the SSM-PCA analysis.  The percent of the total variance
% accounted for (vaf) in the group data by each eigenvector corresponds
% to the relative size of its associated eigenvalue. Subject scores
% represent the degree of expression of the eigenvector in a particular
% subjects image data.
% 
% Disease related PCs, referred to as group invariant subprofiles or (GISs)
% are usually evident as PCs with high vaf (typically  vaf > 5%) whose
% associated subject scores differentiate patients and healthy controls.
% They can be identified by low p-values in Pearsons two-sample t-tests
% (e.g., p < 0.05) or by measures such as Akaike information criteria.
% PCs (typically, one or two) that satisfy the distinctive criteria can
% be vector normalized and linearly combined to yield a single disease-
% related pattern biomarker.  To validate the final pattern as a disease
% biomarker in prospective data, scores are derived in independent groups
% of patients and controls by evaluating each individuals scans expression
% of the pattern using a simple computation of the internal vector product
% of the subject's SRP vector and the pattern vector. Using this single-
% subject score evaluation method referred to as topographic profile rating
% (TPR), the sensitivity and specificity of the GIS can be assessed in
% prospective groups. Scores for validated disease patterns can be used
% to assess disease progression or drug efficacy in individual subjects or
% in group studies. To insure that scores are within the derivation groups
% reference range for comparison, the provided automated TPR procedure
% evaluate SRP in a standardized procedure by applying the same GIS
% intrinsic mask to the new data and subtract the reference GMP from
% the log transformed and subject mean normalized new data. 
%
% GIS patterns represent topographically connected networks of spatially
% covariant regions or voxels associated with a disease where each
% voxel/region has a specific weight attributed to its relative
% contribution. Externally available bootstrapping procedures can be
% applied to the derivation data to identify reliable, highly reproducible
% spatial areas of the GIS for a given level of confidence.
%
% TOOL-BOX MODULES
% 1) VOXEL BASED SSM:
%	Analysis is performed on Talairach normalized Analyze formatted image files.
% 
% 2) ROI BASED SSM:
%	Analysis is performed on ROI data of Talairach normalized, Analyze formatted
%	image files as tabulated in ScAnVP Group ROI Summary reports.
% 
% Both Voxel and ROI based analysis incorporates the following options:
%   PCA (SSM Principal Component Analysis)
%               Derivation of GISs (Group Invariant Subprofiles) PCs
%               and SSF Scores (Subject Scaling Factors).
%   TPR (Topographic Profile Rating)
%               SSF Score Evaluation for new subjects.
%               Highly Automated and Interactive options are available.
%   O-PCA (Principal Component Analysis in Orthogonal Space)
%               Evaluate and optionally remove selected component 
%               covariance from data set and perform PCA in
%               remaining orthogonal space.
%   SCORES      
%               SSM analysis output files are read and bar/scatter graphs of
%               SSF subject scores can be displayed for selected GISs.
%   DISPLAY MAPS 
%               GIS images resulting from the analysis can be visualized using
%               ScAnVP statistical mapping routines including SPM-like results.
%
% See specific task 'Help' and step-by-step instructions under DOCUMENTATION.
% The following references include formal documentation of SSM-PCA
% methodology and results. Extensive additional literature is available on
% the web. See Spetsieris (JOVE, 2013) for a detailed tutorial on ScAnVP
% SSM-PCA routine task use with examples.
%_____________________________________________________________
% REFERENCES
% 
%  J.R. Moeller, S.C. Strother, J.J. Sidtis and D.A. Rottenberg
%  "Scaled Subprofile Model: A statistical approach to the analysis
%  of functional patterns in positron emission tomographic data,"
%  J Cereb Blood Flow Metbol., 1987; vol.7, no.5, pp.649-658.
% 
%  J.R. Moeller, S.C. Strother, "A regional covariance approach
%  to the analysis of functional patterns in positron emission
%  tomographic data," J Cereb Blood Flow Metbol., 1991; vol.11,
%  suppl.1, pp.A121-A135.
% 
%  G.E. Alexander and J.R. Moeller, "Application of the scaled
%  subprofile model to functional imaging in neuropsychiatric
%  disorders: a principal component approach to modeling brain
%  function in disease," Hum. Brain Mapp., 1994, vol. 2, pp.79-94
% 
%  D. Eidelberg, J.R. Moeller, V. Dhawan, P.  Spetsieris,
%  S. Takikawa, T. Ishikawa, T. et al., "The metabolic topography
%  of parkinsonism," J Cereb Blood Flow Metab. 1994, vol. 14,
%  pp.783-801.
%
%  P.G. Spetsieris, Y. Ma, V. Dhawan, J. R. Moeller and D. Eidelberg
% "Highly automated computer-aided diagnosis of neurological disorders
%  using functional brain imaging," In: Reinhardt JM, Pluim JPW, eds.
%  Med. Imaging 2006: Image Processing, Proc. of SPIE, vol. 6144, pp.61445M1-12.
%
%  Ma Y, Tang C, Spetsieris P, Dhawan V, Eidelberg D. Abnormal metabolic network
%  activity in Parkinsons disease: Test-retest reproducibility. J Cereb Blood Flow
%  Metab 2007; 27(3):597-605. [NIHMS: 57714]
%
%  D. Eidelberg, Metabolic brain networks in neurodegenerative disorders: a functional
%  imaging approach, Trends in Neurosciences, vol. 32, no. 10, pp. 548-57, 2009.
%
%  Spetsieris P, Eidelberg D. Scaled subprofile modeling of resting state imaging data
%  in Parkinson's disease: Methodological issues, NeuroImage; 54(4):2899-914, 2011.
% 
%  Peng S, Ma Y, Spetsieris PG, Mattis P, Feigin A, Dhawan V, Eidelberg D.
%  Characterization of disease-related covariance topographies with SSMPCA toolbox:
%  effects of spatial normalization and PET scanners. Hum Brain Mapp, 2013; May 14
%  doi: 10.1002/hbm.22295. www.ncbi.nlm.nih.gov/pubmed/23671030
%
%  Spetsieris P, Ma Y, Peng S, Ko JH, Dhawan V, Tang CC, Eidelberg D. Identification
%  of disease-related spatial covariance patterns using neuroimaging data.
%  J Vis Exp, 2013; 76: e50319  Video: www.jove.com/video/50319/ Jun 25, 2013 - 14 min
%_______________________________________________________________________
% vp_ssminfo.m  7.0           Phoebe Spetsieris, Ph.D.  Dec. 30, 2015


vp_help('vp_ssminfo');
