% vp_ssmpca  Copyright (C) 2003-2016 Center for Neurosciences, FIMR. 
%
% function ret=vp_ssmpca(fnamelst,maskspec,logspec,meanspec,srpspec,outspec)
% log, mean, srp and out specs are optional and not order specific 
%
% This function is called by several application that implement the
% Scaled Sub-Profile Model of principal component analysis (SSM-PCA)
% for voxel-based PET image data sets (see references in the calling module).
% These include direct PCA of the data's spatial covariance matrix to derive
% a set of principal components and corresponding subject score sets for 
% a group of patients and controls as well as subtraction and orthogonal
% data analysis (see corresponding calling modules). 
% Summarizing, the routine assumes ANALYZE format data images, threhold/ 
% masks the voxel space, performs default log transformation of data 
% (negative values in any one image are masked out for the entire set),
% followed by subject and group mean centering (row and column subtraction
% of mean values) and PCA analysis involving SVD of the covariance matrix.
% No assumption is made regarding the orientation of the images other than
% that they are all similarly oriented (e.g. Neurological or Radiological). 
% The output Group Invariant Subprofiles (GISs or PCs) will have the same
% orientation.
%
% To operate the routine a user GUI interface is provided that sends required
% parameters for PCA. Additional routines interface to create a common group mask
% or mean centered image. A default mask is provided.
% The following arguments are formated by the GUI interface:
% The first argument is an input textfile of image names beginning with a 
% header file. Image dimensions must be the same but scale factors need not be.
% The second argument can be a mask image file or threshold entered as T##
% (e.g. if T25 is entered, a mask is creaated from subject values > .25 of max.
% If the threshold is specified as .25 the program exits after creating the mask.
% If the second argument is 'SRP' then PCA is performed on the input files
% directly without any log transformation and with optional GIS masking.
% An argument 'MEAN' can be entered. If it is not followed by a file name,
% the program calculates a mean image and exits.
% An argument 'NOLOG' can be added to suppress default log transformation.
% (In the 'NOLOG' case negative values that are not masked out may be retained).
% An argument 'CONTROLS: #' can be added in the input list to indicate controls.
% An argument 'SUBTRACTION: #' can be added in the input list to indicate Subtraction SSM.
% An argument 'SRPOUT' can be added to save SRP files and PCs above the 16th.
%
%
% Usage: vp_ssmpca(inputlst.txt, [Mask | T## <.35> | 'SRP [Mask]'],
%     ['NOLOG'] ['LOG'] ['MEAN'] ['SRPOUT'], Outfile)
%
% A default mask 'PETmask.img' can be specified.
% The threshold default .35 is provided in the calling interface.
% 
% vp_ssmpca ScAnVP 6.2  P.Spetsieris, Ph.D.     1/30/2012, 7/12/2012
%  (MATLAB version based on scanvp.exe 2003-2009 (ver. 5.7, 10/09) 
%_______________________________________________________________________

function ret=vp_ssmpca(fnamelst,maskspec,logspec,meanspec,srpspec,outspec)

global HelpMode FMenu 
global DIM VOX SCALE TYPE OFFSET ORIGIN DESCRIP GLMAX GLMIN
global EXPTIM DURATION MATYPE Message GENTIM SUBID EXPDATE

%Defaults

ret=0;
outfile=['pcaoutput.txt'];	% default outfilename;
studyname=['studyname'];

masknm=[''];	% mask name
meanfl=[''];	% mean file name
thresh= 0;	% Threshold fraction of max
logdata=1;	% log transform data
srpin=0;	% Perform pca on srp input
srpout=0;	% Output srp files
meaninput=0;	% Mean file entered to calculate srp
maskexit=0;	% Exit after mask creation
meanexit=0;	% Exit after mean file creation

nimgs=0;	% Total number of images (subjects)
nnorms=0;	% Number of controls
nsubs=0;	% Number of subtraction subjects 
dim0=0;		% Common dimensions for data and mask
ori0=0;		% Common origin for data and mask



if HelpMode, vp_help(mfilename), return ; end;

if nargin > 0, fnamelst,
else vp_note('No Name List Specified', 2); ret =-1, return;
end;

if nargin > 1, maskspec, mln=size(maskspec,2), 
  if maskspec(1)=='.', thresh=str2num(maskspec), maskexit=1, outfile='maskoutput.txt';
  elseif and( (maskspec(1)=='T'), (mln==3)), thresh = str2num(maskspec(2:3))/100, 
  elseif and(strcmp('SRP', maskspec(1:3)),(mln>=3)), srpin=1, logdata=0; outfile='srpoutput.txt';
    if mln > 4, masknm=maskspec(5:mln), end; 
  elseif ischar(maskspec), masknm=maskspec,
  else vp_note('Invalid argument maskspec', 2); ret =-2, return;
  end;  
end;

if nargin > 2, logspec, 
 if strcmp(logspec,'NOLOG'), logdata=0,
 elseif strcmp(logspec,'LOG'), logdata=1, 
 elseif strcmp(logspec,'MEAN'), meanexit=1, outfile='meansoutput.txt';
 elseif strcmp(logspec,'SRPOUT'), srpout=1,
 else outfile=logspec; 
 end;
end;

if nargin > 3, meanspec, 
 if strcmp(meanspec,'NOLOG'), logdata=0,
 elseif strcmp(meanspec,'LOG'), logdata=1, 
 elseif strcmp(meanspec,'MEAN'), meanexit=1, outfile='meansoutput.txt';
 elseif strcmp(meanspec,'SRPOUT'), srpout=1,
 else outfile=meanspec; 
 end;
end;

if nargin > 4, srpspec, 
 if strcmp(srpspec,'NOLOG'), logdata=0,
 elseif strcmp(srpspec,'LOG'), logdata=1, 
 elseif strcmp(srpspec,'MEAN'), meanexit=1, outfile='meansoutput.txt';
 elseif strcmp(srpspec,'SRPOUT'), srpout=1,
 else outfile=srpspec; 
 end;
end;

if nargin > 5, outspec, 
 if strcmp(outspec,'NOLOG'), logdata=0,
 elseif strcmp(outspec,'LOG'), logdata=1, 
 elseif strcmp(outspec,'MEAN'), meanexit=1, outfile='meansoutput.txt';
 elseif strcmp(outspec,'SRPOUT'), srpout=1,
 else outfile=outspec; 
 end;
end;


h=waitbar(0,'Processing SSMPCA ...');
set(FMenu,'Pointer','Watch');


fout=fopen(outfile,'wt');

studyname= vp_nm('nm',fnamelst),
nln=size(studyname,2),
if nln > 3,
  studyname(nln-2:nln), studyname(1:nln-3),
  if strcmp(studyname(nln-2:nln),'lst'), studyname=studyname(1:nln-3), end;
end;


fprintf(fout,['STUDY NAME: ',studyname,'\n']);  

fprintf(fout,['Reading No of image files in %s\n\n',fnamelst]);  

flst=fopen(fnamelst,'rt'),

hdr_name='';
img_names='';

[hdr_name, cnt]=fscanf(flst,'%s',1),

nimgs=0; 

while (cnt==1)
 [dummy, cnt]=fscanf(flst,'%s',1),
 if strcmp(dummy,'CONTROLS:')
   [nnorms, cnt]=fscanf(flst,'%d',1),
   fprintf(fout,['No of CONTROLS: %d\n'],nnorms);
 elseif strcmp(dummy,'CONTROLS:')
   [nnorms, cnt]=fscanf(flst,'%d',1),
   fprintf(fout,['No of CONTROLS: %d\n'],nnorms);
 elseif strcmp(dummy,'SUBTRACTION:')
   [nsubs, cnt]=fscanf(flst,'%d',1),
   fprintf(fout,['No of Subtraction Subjects: %d\n'],nsubs);
 else
   if (cnt~=0)
     img_names=strvcat(img_names,dummy);
     nimgs=nimgs+1;
     if and(nimgs==1,nnorms > 0) fprintf(fout,['CONTROLS \n']); end;
     if and(nimgs==nnorms+1,nnorms > 0) fprintf(fout,['\nSUBJECTS \n']); end;
     fprintf(fout,['Image %d:  %s\n'],nimgs,dummy);
   end;
 end
end

img_names,
nimgs,
fclose(flst);

fprintf(fout,['\nInformation from input text file %s : \n'],fnamelst);
fprintf(fout,['Number of Images = %d\n'],nimgs);
fprintf(fout,['Header file name = %s \n'],hdr_name);


%READ HEADER

hdrname=vp_nm('dnm',hdr_name),
vp_hread([hdrname '.hdr']);
dim0=DIM; ori0=ORIGIN;
  

fprintf(fout,['Image Dimensions (DX, DY, DZ): (%d, %d, %d) \n'],DIM);
fprintf(fout,['Total Voxels: (%d) \n'],prod(DIM));
fprintf(fout,['VOX. ORIGIN (X0, Y0, Z0): (%d, %d, %d) \n'],ORIGIN);

%READ IMAGE FILES INTO (nimgs by Total Voxels) DATA MATRIX

fprintf(fout,['\nREAD IMAGE FILES INTO (%d x %d) DATA MATRIX \n'],nimgs,prod(DIM));

imdata=zeros(prod(DIM),nimgs,'single');
xyzdata=zeros(prod(DIM),1,'single');
datamin=zeros(nimgs,1,'single');
datamax=zeros(nimgs,1,'single');

for ni=1:nimgs,

  waitbar((ni)/nimgs,h,['Reading image ',num2str(ni)]);
  fprintf(fout,['Reading IMAGE %d: %s \n'],ni, img_names(ni,:));

  [out1 xyzdata datamin(ni,1) datamax(ni,1)]=vp_volfuns('volvect',img_names(ni,:));
  if (out1 < 0) vp_message('ERROR reading file'); ret=-3; return;
  else,
    if ~isequal(DIM, dim0), vp_message('ERR Header/Data File Dimensions differ');
     ret=-4; return; end;  
    if ~isequal(ORIGIN, ori0), vp_note('WARNING Header/Data File Origins differ',2); end;

    imdata(:,ni)=xyzdata; 
    fprintf(fout,['Data (Max, Min) (%.4f, %.4f) \n'], datamax(ni), datamin(ni) );
  end;
end; % for reading in images

clear xyzdata;


% READ IN OR GENERATE IMAGE MASKS, COUNT NON-ZERO VOXELS. WRITE MASK TO DISK.

workdir=pwd;
%output mask
maskfilnm=[workdir,'\',studyname 'mask.img'], 
maskdat=ones(prod(DIM),1,'single');
masksum0=sum(maskdat),

% read input mask
if ~isempty(masknm),
  % Read image mask data
  [out1 maskdat]=vp_volfuns('volvect',masknm);
  if (out1 <0) vp_message('ERROR reading mask'); ret=-5, return; end;
  % make sure it's 0:1
  maskdat=maskdat ~=0;
  if ~isequal(DIM, dim0), vp_message('ERR Mask/Data File Dimensions differ');
     ret=-6; return; end; 
  if ~isequal(ORIGIN, ori0), vp_note('WARNING Mask/Data File Origins differ'); end;
  
end; 

if or((~isempty(masknm)),(srpin)),
  % Multiply by intrinsic mask
  
  for k=1:nimgs
     maskdat=maskdat.*(imdata(:,k) ~= 0);
  end

  
elseif ~srpin,
  % Create Mask from Threshold
  
   maskfilnm=[workdir,'\',studyname 'mask', num2str(round(thresh*100)), '.img'], 
   fprintf(fout,['\nNumber of THRESHOLD = %.4f voxels in each image\n'], thresh );
  
  for k=1:nimgs
   
   thr_k=thresh *datamax(k,1);
   maskin=( imdata(:,k) >= thr_k) ;
   sum_k=sum(maskin);
   maskdat=maskdat.*maskin ;
   fprintf(fout,['Image %d: Thresh: %.3f Count: %d\n'], k, thr_k, sum_k );
   
  end
  
  
end

if logdata %Remove negative values and zeros before log operation
  for k=1:nimgs
     maskdat=maskdat.*(imdata(:,k) > 0);
  end
end

%mask Data

masksum=sum(maskdat),

if (masksum == 0)
      set(FMenu,'Pointer','Arrow');
      vp_message(['Error: Mask has no positive elements ']);
      close(h);
      ret=-7, return;
end
fprintf(fout,['\nNumber of Non-Zero Mask Voxels; %d\n'], masksum);

%Save Mask Image
   
   TYPE=2; SCALE=1; DESCRIP=['ScAnVP - SSM 0/1 Mask Image']; 
   out1=vp_imsave(maskfilnm,reshape(maskdat,DIM(1:3)), 1), 
   if (out1 < 0),
      vp_message('ERROR writing _mask file');
      close(h);
      ret=-8, return;
   else
     fprintf(fout,['\nSaving Mask file: %s\n'], maskfilnm);
   end;
  
if maskexit

  fprintf(fout,['\nExit on Mask Creation\n']);
  waitbar(1,h); pause(1); close(h);
  set(FMenu,'Pointer','Arrow');

  fclose(fout);
  return;

end;


% MASK, LOG TRANSFORM & CALCULATE MEANS
fprintf(fout,['\nMasking Images, ']);


% MASK IMAGES

  for k=1:nimgs,
     msg=['Masking Image ',num2str(k)];
     waitbar((k-.8)/nimgs,h,msg);
     imdata(:,k)=imdata(:,k).*maskdat;
  end

% LOG TRANSFORMATION 

 if logdata
   fprintf(fout,['Converting to LOG data,\n']); 
   msg=['Processing Log Transformation of Images ...'];
   waitbar(.3,h,msg);
   %negzero_set=find(imdata <=0);
   for p=1:nimgs % Save memory
     negzero_set=find(imdata(:,p) <=0);
     imdata(negzero_set,p)=exp(0); % PS Set all values <=0
   end
   %imdata(negzero_set)=exp(0); % PS Set all values <=0
   negzero_set=[]; % free memory
   imdata=log(imdata); % PS perform log transformation
 end;
 
% End of data scanning and log-transform 

 if nsubs ~= 0
   fprintf(fout,['Subtracting Input 1 from Input 2 \n']); 
   msg=['Subtracting Images ...'];
   waitbar(.35,h,msg);
   for k=1:nsubs
     msg=['Subtracting Sub. ',num2str(k)];
     waitbar((i-.8)/nsubs,h,msg);
     imdata(:,k)=(imdata(:,(k+nsubs))-imdata(:,k));
   end
   imdata=imdata(:,1:nsubs);
   nimgs=nsubs;
 end % Subtraction option
  

% CALCULATE SRP

 if ~srpin

  msg=['Calculating Means ...'];
  waitbar(.4,h,msg);
  nvoxs=size(imdata,1);
  nimgs=size(imdata,2);

  fprintf(fout,[msg,'\n']);

  %Compute within image mean vector
  %PS Don't count 0's in mean
  %maskpos=find(maskdat); masksum=size(maskpos,1),

  %Vector of means for each image
  meanvect=sum(imdata)/masksum;

  waitbar(.5,h);

  %Compute within-subject deviations from mean of image
  %i.e., normalize to whole brain
  
  msg=['Calculating Deviations From Subject Mean ...'];
  waitbar(.6,h,msg);
  fprintf(fout,[msg,'\n']);

  
  %devs=imdata-meanmat;
  %meanmat=[]; % free memory
  
  %PS Subtract one by one to save memory
  for x=1:nimgs, imdata(:,x) = (imdata(:,x) - meanvect(1,x)).*maskdat ; end;


 % CALCULATE MEAN REFERENCE DEVIATIONS IMAGE

  msg=['Calculating Mean Deviations Image ...'];

  waitbar(.7,h,msg);
  fprintf(fout,[msg,'\n']);

  %nvoxs=size(imdata,1);

  %mean_img=mean(imdata'); % (Log) Mean deviations image
  mean_img=mean(imdata,2); % Take mean of each row to form column
   
   
  %Save mean reference image
   

  if logdata
    TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM Log Mean Image']; 
    filnm=[workdir,'\',studyname '_lnmean.img'], 
  else
    TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM Mean Image']; 
    filnm=[workdir,'\',studyname '_mean.img'], 
  end

  out1=vp_imsave(filnm,reshape(mean_img,DIM(1:3)), 1), 
  if (out1 < 0),
      vp_message('ERROR writing _lnmean file');
      set(FMenu,'Pointer','Arrow');
      ret=-9, return;
  else
     fprintf(fout,['\nSaving (Ln)Mean Image file: %s\n'], filnm);
  end;
  
  if meanexit

    fprintf(fout,['\nExit on Mean File Creation\n']);
    waitbar(1,h); pause(1); close(h);
    set(FMenu,'Pointer','Arrow');

    fclose(fout);
    return;

  end;

  msg=['Calculating SRP - Please Wait ...'];
  waitbar(.8,h,msg);
  fprintf(fout,['Calculating SRP ...\n']);

  %Compute subject deviations from mean image
  % mean_img=mean_img'; %Form column
  
  waitbar(.3,h);
  for x=1:nimgs, %PS
   %devs(:,x)=devs(:,x) -mean_img;
   imdata(:,x)=imdata(:,x) -mean_img;
   waitbar(.3+.6*x/nimgs,h);
  end;
  
  %devs = srp' 

 end %~srpin
 waitbar(1,h); pause(1); close(h);

 h=waitbar(0,'Masking SRP ...');
 for k=1:nimgs
     msg=['Masking SRP Sub. ',num2str(k)];
     waitbar((i-.8)/nimgs,h,msg);
     %devs(:,k)=devs(:,k).*maskdat;
     imdata(:,k)=imdata(:,k).*maskdat;
 end
  
 waitbar(1,h); pause(1); close(h);


 % Output SRP images if Flag SRPOUT
 if (srpout)
    h=waitbar(0,'Output SRP ...');
    TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM SRP Image']; 
    if logdata, suffix=['_lsrp']; else suffix=['_srp']; end;
    for k=1:nimgs
       msg=['Output SRP Sub. ',num2str(k)];
       waitbar((i-.8)/nimgs,h,msg);
      % imdata(:,k)=imdata(:,k).*maskdat;
       
       % Pad k sequence no string to avoid alphanumeric sort errors
       filnm=[workdir,'\',studyname,suffix, vp_num3str(k), '.img' ], 

       out1=vp_imsave(filnm,reshape(imdata(:,k),DIM(1:3)), 2), 
       if (out1 < 0),
          vp_message(['ERROR writing _SRP file ',num2str(k)]);
          set(FMenu,'Pointer','Arrow');
          ret=-10, return;
       else
         fprintf(fout,['\nSaving SRP Image file: %s\n'], filnm);
       end;
    end
    waitbar(1,h); pause(1); close(h);
 end;


% PERFORM PCA

% Compute sub-by-sub Covariance matrix
  msg=['Compute Covariance Matrix ...'];
  fprintf(fout,['\nSubject by Subject Covariance Matrix: \n']);
  hwait=waitbar(0.25,msg);

  Tcovar=imdata'*imdata; %devs'*devs
% Tcovar=cov(imdata); Don't use cov because of factor difference
  Tcovar,
  fprintf(fout,['\n']);
  for x=1:nimgs
    for y=1:nimgs
  	fprintf(fout,['%10.3f '],Tcovar(x,y));
    end; 
    fprintf(fout,['\n']);
  end;

  waitbar(.6,hwait);

% Compute Eigenvector (SSF) and Eigenvalue of Covariance matrix
  msg=['Compute SSFs and Eigenvalues ...'];
  fprintf(fout,['\nComputing SSFs and Eigenvalues ...\n']);
  waitbar(0.65,hwait, msg);
  
  [eigenvect eigenval]=eig(Tcovar);
  clear Tcovar;
  
  eigenval=diag(eigenval);

% Convert Eigenvector (SSF) to carry weight of eigenvalue
% (SSF * sqrt(eigenvalue))
  sqrt_eigenval=sqrt(eigenval');
  sqrt_eigenvalmat=[]; %PS initialize
  
  for x=1:nimgs
   sqrt_eigenvalmat(x,:)=sqrt_eigenval(1,:);
  end
  ssf=eigenvect.*sqrt_eigenvalmat;
  clear eigenvect sqrt_eigenvalmat sqrt_eigenval

  waitbar(.7,hwait);

%Compute Percentage Variance Accounted For
  percentvar=eigenval/sum(eigenval);
  percentvar=percentvar';

%gis = fliplr(gismatrix);
  ssf = fliplr(ssf),
  vaf = 100*fliplr(percentvar),
  eigvals=fliplr(eigenval'),
  
%Flip sign of scores if mean control score greater than patients

  if nnorms ~=0
    
    fprintf(fout,['Flipping Neg. Vectors: ']);
    for y=1:nimgs
       meannorm=mean(ssf(1:nnorms,y));
       meanpatient=mean(ssf(nnorms+1:nimgs,y));
       if meannorm > meanpatient,
  	 fprintf(fout,[' %d '],y);
         ssf(:,y)=ssf(:,y)*(-1);
       end
    end;
    fprintf(fout,['\n']);

  end;


  fprintf(fout,['\nSSF Subject Scores by Column: \n']);
  for x=1:nimgs
    for y=1:nimgs
  	fprintf(fout,['%8.3f '],ssf(x,y));
    end; 
    fprintf(fout,['\n']);
  end;

  fprintf(fout,['\nEigenvalues:\n']);
  for x=1:nimgs
  	fprintf(fout,['%.3f   '],eigvals(x));
  end; 
  
  
  fprintf(fout,['\n\nVAF (Percent Variance Accounted For): \n']);
  for x=1:nimgs
  	fprintf(fout,['%.3f   '],vaf(x));
  end; 

% print Scree plot of eigenvalues
  if nimgs > 16, maxeig=16; else maxeig=nimgs; end;
  Xeig=1:maxeig; Yeig=eigvals(1:maxeig);
  vp_plot(Xeig,Yeig,'k-^',2,10,'EIGENVALUE No.','EIGENVALUE LAMBDA','SCREE PLOT');
  
  waitbar(.8,hwait);

nout=nimgs; %if user did not select srp output and there are
%more than 16 images, print only the first 16 images and scores.

if and(~srpout,(nimgs > 16))
    %nout=16; vaf=vaf(1:16); ssf=ssf(:,1:16); 
    nout=16; 
end;

% GENERATE GISs

% Note imdata=deviations (row voxels by column subjects)
% The ssf matrix lists row subjects by column GIS scores
% We multiply imdata by ssf on the right to get a GIS
% matrix of k GISs by column voxels.

% Multiply ssf matrix by imdata to generate gismatrix array

  %gismatrix=imdata*ssf; % imdata=devs, GISs listed by column 
  for x=1:nout
    gismatrix(:,x)=imdata*ssf(:,x); % imdata=devs, GISs listed by column 
  end

% OMIT two steps require more memory
% gismatrix=ssf'*imdata'; % imdata=devs, GISs listed by row
% gismatrix=gismatrix'; % GISs listed by column

  clear imdata;
  
           
% PS non-normalized gismatrix
% Normalize GIS vectors (Z-score voxel weights)
  %for x=1:nimgs
  for x=1:nout
     %mnx=mean(gismatrix(:,x)); stdx=std(gismatrix(:,x));
     %gismatrix(:,x)=(gismatrix(:,x)-mean(gismatrix(:,x)))./std(gismatrix(:,x)); 
     mnx=sum(gismatrix(:,x))./masksum; stdx=std(nonzeros(gismatrix(:,x)));
     gismatrix(:,x)=(gismatrix(:,x)-mnx)./stdx; 
     fprintf(fout,['\nGIS %d Image (mean, std): (%f,%f)\n'], x, mnx, stdx);
  end

% first principal component first

  waitbar(.95,hwait);
  close(hwait);

% write relevant output files

%nout=nimgs; %if user did not select srp output and there are
%%more than 16 images, print only the first 16 images and scores.

%if and(~srpout,(nimgs > 16))
%    %nout=16; vaf=vaf(1:16); ssf=ssf(:,1:16); 
%    nout=16; 
%end;


% Output GIS images 
  hwait=waitbar(0,'Output GISs ...');
  TYPE=16; SCALE=1; DESCRIP=['ScAnVP - SSM GIS Image']; 
  suffix=['_gis'];
  for k=1:nout
       msg=['Output GIS PC',num2str(k)];
       waitbar((i-.8)/nout,hwait,msg);
       gismatrix(:,k)=gismatrix(:,k).*maskdat;
       
       % Pad k sequence no string to avoid alphanumeric sort errors

       %filnm=[workdir,'\',studyname,suffix,num2str(k), '.img'], 
       filnm=[workdir,'\',studyname,suffix, vp_num3str(k), '.img'], 

       out1=vp_imsave(filnm,reshape(gismatrix(:,k),DIM(1:3)), 2), 
       if (out1 < 0),
          vp_message(['ERROR writing _GIS file ',num2str(k)]);
          set(FMenu,'Pointer','Arrow');
          ret=-11, return;
       else
         fprintf(fout,['\nSaving GIS Image file: %s\n'], filnm);
       end;
  end


% Write score file
ssf_name=[workdir,'\',studyname,'_VScores.txt'], 


img_nms='';
for k=1:nimgs
     dummy=vp_nm('nm',img_names(k,:));
     img_nms=strvcat(img_nms,dummy);
end;
     
if vp_write_ssmfile(ssf_name,nout,vaf,img_nms,ssf,4,nnorms) < 0
     fprintf(fout,['\nInvalid Score file: %s\n'], ssf_name);
     vp_message(['ERROR writing _Score file ', ssf_name]);
     ret=-12; 
end

waitbar(1,hwait); pause(1); close(hwait);

%PS free memory
clear mean_img gismatrix 

set(FMenu,'Pointer','Arrow');
fclose(fout);

