% vp_statmap.m
% Create thresholded statistical map
%__________________________________________________________________________
%
% In this routine, the user selects a statistical output Analyze format
% file containing statistical results for each voxel. SSM voxel maps
% as well as output from SSM GIS ROI mapping routine can be input.
% A Mask file determines which voxels are output either inclusively
% or exclusively for all non-zero voxels of the Mask file.
% If the mask file is the same as the statistical file, or background
% file the names must differ. If masking is set off, all statistical
% voxels will be considered. Threshold values can be specified to map 
% only voxels that have values exclusive of the threshold range or 
% inclusive between the threshold values in the statistical file.
% For 'inclusive', the lower threshold value 0 is not included because
% some data files may use that value for background. The results are
% then overlayed on a user selected Background file, usually a standard
% MRI image. The MRI image can be of different dimensions but the origin
% must refer to the same physiological point as that of the origin of the
% statistical input. A nearest point algorithm is used to map the background
% voxels on the output so that if a smoothed background is preferred, the
% background image must be smoothed prior to input.
% All input files (statistical, background and optional mask) are assumed  
% to have the same Neurological (LR) or Radiological orientation on disk.
% Three orthogonal slices are displayed through the origin and
% the user has several options for viewing slices across planes.
% The lower half of the colormap is assigned to the background image and
% the upper half to the statistically significant voxels. The lower threshold
% and the upper threshold are shown on the upper half of the colorbar.
% The resulting display image is saved as a double precision ANALYZE image.
% Various options can be used for removing 2D and cubic clusters and for plots.
% The thresholded statistical data and the composite image including
% background are saved in separate ANALYZE format files (statmapA and statmapB).
% StatmapA can be input into the SPM-like results routine for further options.
% StatmapB can be viewed by "IMAGE VIEW", by selecting a split color map e.g.
% GRAYJET.
% The DATAMAX value used for DISPLAY purposes may not be a found data value but
% is saved in the first voxel of the composite image statmapB so that it can be
% viewed properly at any later time using split colormaps. To view data that
% contains both positive and negative values a triple range map such as
% GRAYBICOL can be used.
% Note, statmapA and statmapB are not valid input names for this routine.
%__________________________________________________________________________
% vp_statmap.m	C.0 	P. Spetsieris	Jun. 8, 1999
% 		rev. 5.8	P. S.	May. 6, 2007
% 		rev. 6.2	P. S.	Apr. 27, 2012
% 		rev. 7.0	P. S.	Oct. 13, 2015

global FI FNames FName mapgui hmaperr PlaneNo0 PlaneNo hplane
global MaxThresh MinThresh 
global hstatname hmname hbgrndname
global GMax1 GMin1 GMax3 GMin3 ProTime
global BgrndFName InputOrient DisplayOrient FMenu
global HelpMode Ecolor mapgui, gPrevDirs
global MaskOn OutOp DatOp MskOp

if HelpMode, vp_help(mfilename), return ; end;

mapgui=findobj('Name','StatMap');
if ~isempty(mapgui)
	if(strcmp(get(mapgui,'Visible'),'on'))
		figure(mapgui);
	end
	return
end

vp_imframe;

FName=[];
maskFName=' ';
statFName=' ';
BgrndFName=' ';

WndNo=0;
WndScl=4;
PlaneNo=1;
Message0=['............................................ '];
Message=Message0;
DefStat=0;
DefBgrnd=0;
DefMask=0;
MaxThresh=0;
MinThresh=0;
GMin1=0;
GMax1=0;
ProTime=0;
OutOp=1;
MskOp=1;
DatOp=1;

Bcolor = [.8,.8,.8];
Kcolor = [.8,1,1];
Fcolor = [.8,.8,1];

MaskOn=0;

mapgui = figure('Color',Fcolor,'Name','StatMap',...
'NumberTitle','off','Position',[100,100,300,275],...
'Resize','on',...
'Units','Pixels');
	
	
uicontrol(mapgui,'string', 'HELP',...
	'position',[230 252 60 20],'BackgroundColor',Ecolor,...
	'callback',['vp_help(''vp_statmap'')']);
	
cbtext2= ['text(11,103,''MaxThreshold'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;

cbtext1= ['text(11,88,''MinThreshold'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',9,''HorizontalAlignment'',''center'');'] ;

cbtext= ['text(15,71,''Range'',''Units'',''pixels'',''Fontweight'',''bold'',',...
	'''FontName'',''Times'',''FontSize'',10,''HorizontalAlignment'',''center'');'] ;

%htxt3=uicontrol(mapgui,'string',['Max=' num2str(GMax1)],'Fontweight','bold',...
htxt3=uicontrol(mapgui,'string',' ','Fontweight','bold',...
	'TooltipString','global statistical file maximum.',...
	'FontName','Times','FontSize',8,...
	'position',[152 124 88 16],'BackgroundColor',Fcolor);

htxt4=uicontrol(mapgui,'string',' ','Fontweight','bold',...
	'TooltipString','global statistical file minimum.',...
	'FontName','Times','FontSize',8,...
	'position',[152 107 88 16],'BackgroundColor',Fcolor);

%cbtext3= ['set(htxt3,''String'',[''Max='' num2str(GMax1)])'];
%cbtext4= ['set(htxt4,''String'',[''Min='' num2str(GMin1)])'];
cbtext3= ['set(htxt3,''String'',[''Max='' num2str(GMax1)]); ',...
	  'set(htxt4,''String'',[''Min='' num2str(GMin1)])'];
	  
%cbtext4= ['del=abs((GMax1-GMin1)/10000); MaxThresh=GMax1+del;',...
%	' set(hthresh2,''String'',num2str(GMax1+del));',...
%	'if GMin1==0, del=0.0001; end; MinThresh = GMin1-del;',...
%	' set(hthresh1,''String'',num2str(GMin1-del));'];

cbtext4= ['MaxThresh=GMax1;',...
	' set(hthresh2,''String'',num2str(GMax1));',...
	' MinThresh = GMin1;',...
	' set(hthresh1,''String'',num2str(GMin1));'];

eval(cbtext);

eval(cbtext1);
eval(cbtext2);

axis off;

cbstatname=['statFName=get(hstatname,''string''); Message=Message0;',...
	'FName=statFName; vp_fimage;',...
	'[GMax1 GMin1]=vp_roifuns(''gminmax'');',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); eval(cbtext3); eval(cbtext4); axis off;',...
	'if isempty(Message), DefStat=1; PlaneNo=max(PlaneNo-1,1); Message=[''STAT :  '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefStat=1; end;',...
	'set(hmaperr,''string'',['' '',Message]);'];

pcbstat=['if InputOrient==1,ortxt='' (Note: Input Radiologically (RL, sw, spm5) Oriented Disk Fie)'';',...
   'else ortxt='' (Note: Input Neurologically (LR, sn, spm99) Oriented Disk File)''; end;',...
   'statFName=get_file(1,''.img'',[''please select statistical file'',ortxt],[]); ',...
	'if ~isempty(statFName), FName=statFName; vp_fimage; set(hstatname,''string'',FName);',...
	'[GMax1 GMin1]=vp_roifuns(''gminmax'');',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); eval(cbtext3); eval(cbtext4); axis off;',...
	'if isempty(Message), DefStat=1; PlaneNo=max(PlaneNo-1,1); Message=[''STAT :  '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefStat=1; end;',...
	'set(hmaperr,''string'',['' '',Message]); end;'];
	
uicontrol(mapgui,'string', 'STAT FILE',...
	'TooltipString','Pick Statistical Image file.',...
	'position',[10 163 75 20],'BackgroundColor',Bcolor,'callback',[pcbstat]);
	
hstatname=uicontrol(mapgui,'style','edit','string',statFName,...
	'TooltipString','Enter Statistical Image file name.',...
	'position',[85 163 205 20],'BackgroundColor',Kcolor,'callback',cbstatname);


cbmaskname=['maskFName=get(hmname,''string''); Message=Message0;',...
	'FName=maskFName; vp_fimage;',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); axis off;',...
	'if isempty(Message), DefMask=1; PlaneNo=max(PlaneNo-1,1); Message=[''MASK : '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefMask=1; end;',...
	'set(hmaperr,''string'',['' '',Message]);'];
	
cbmaskoff=['if MaskOn, MaskOn=0; set(hmoff,''string'',''SET MASK ON''); ',...
	'set(hmfile,''Visible'',''off''); set(hmname,''Visible'',''off'');',...
	'set(hmskmenu,''Visible'',''off'');',...
	'else, MaskOn=1; set(hmfile,''Visible'',''on''); set(hmname,''Visible'',''on'');',...
	'set(hmskmenu,''Visible'',''on'');',...
	'set(hmoff,''string'',''SET MASK OFF''); end;']; 
	
cbOutMenu = ['OutOp=get(houtmenu,''Value'');';];
	
pcbmask=['maskFName=get_file(1,''.img'',''Please Select Mask File'',[]); ',...
	'if ~isempty(maskFName), FName=maskFName; vp_fimage; set(hmname,''string'',FName);',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); axis off;',...
	'if isempty(Message), DefMask=1; PlaneNo=max(PlaneNo-1,1); Message=[''MASK : '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefMask=1; end;',...
	'set(hmaperr,''string'',['' '',Message]); end;'];
	
hmoff=uicontrol(mapgui,'string', 'SET MASK ON',...
	'TooltipString','Set Mask On/Off',...
	'position',[10 226 90 20],'BackgroundColor',Bcolor,...
	'callback',cbmaskoff);
	
hmfile=uicontrol(mapgui,'string', 'MASK FILE',...
	'TooltipString','Pick Mask Image file.',...
    'Visible','off',...
	'position',[10 204 75 20],'BackgroundColor',Bcolor,...
	'callback',pcbmask);
	
hmname=uicontrol(mapgui,'style','edit','string',maskFName,...
	'TooltipString','Enter Mask Image file name.',...
    'Visible','off',...
	'position',[85 204 205 20],'BackgroundColor',Kcolor,'callback',cbmaskname);


cbMskMenu = ['MskOp=get(hmskmenu,''Value'');';];
	
hmskmenu=uicontrol(mapgui,'Style','popup','Visible','off','String',...
	'Inclusive Mask|Exclusive Mask','Tag','MskMenu',...
	'value',MskOp,...
	'TooltipString',['Inclusive: All non-zero voxels of the mask, ',...
	'Exclusive: Only zero voxels of the mask'],...
    	'Position',[105 226 100 20],'callback',cbMskMenu);
    
pcbSetOrient = [' ',...
 'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented Disk Input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

uicontrol(mapgui,'string', 'Check Disk Input File Orientation on ScAnVP Menu',...
	'TooltipString',['Set Common Radiological (Right is Left) or Neurological (Right is Right)',...
    ' Disk Input File Orientation on Main Menu.'],...
	'position',[10 184 280 15],...
	'BackgroundColor',Bcolor,'callback',pcbSetOrient);

cbgrndname=['BgrndFName=get(hstatname,''string''); Message=Message0;',...
	'FName=BgrndFName; vp_fimage;',...
	'[GMax3 GMin3]=vp_roifuns(''gminmax'');',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); axis off;',...
	'if isempty(Message), DefBgrnd=1; PlaneNo=max(PlaneNo-1,1); Message=[''BGRND :  '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefBgrnd=1; end;',...
	'set(hmaperr,''string'',['' '',Message]);'];

pcbgrnd=['if InputOrient==1,ortxt='' (Note: Input Radiologically (RL, sw, spm5) Oriented Disk File)'';',...
    'else ortxt='' (Note: Input Neurologically (LR, sn, spm99) Oriented Disk File)''; end;',...
    'w=which(''scanvp''); mridir=[w(1:(max(findstr(w,''\''))-4)),''mri''],',...
    'BgrndFName=get_file(1,''.img'',[''Please Select Background Image'',ortxt],mridir,[]); ',...
	'if ~isempty(BgrndFName), FName=BgrndFName; vp_fimage;',...
	'set(hbgrndname,''string'',FName); [GMax3 GMin3]=vp_roifuns(''gminmax'');',...
	'figure(mapgui); eval(cbtext); eval(cbtext1); eval(cbtext2); axis off;',...
	'if isempty(Message), DefBgrnd=1; PlaneNo=max(PlaneNo-1,1); Message=[''BGRND :  '',FName]; end;',...
	'if strcmp(Message,''Maximum Plane Reached''), DefBgrnd=1; end;',...
    'nds=size(gPrevDirs,1); gPrevDirs(nds,:)=[];',...
	'set(hmaperr,''string'',['' '',Message]); ',...
    ' end; '];

uicontrol(mapgui,'string', 'BGRND FILE',...
	'TooltipString','Pick Background Image file.',...
	'position',[10 142 75 20],'BackgroundColor',Bcolor,'callback',pcbgrnd);
hbgrndname=uicontrol(mapgui,'style','edit','string',BgrndFName,...
	'TooltipString','Enter Background Image file name.',...
	'position',[85 142 205 20],'BackgroundColor',Kcolor,'callback',cbgrndname);


cbthresh1=['MinThresh=str2num(get(hthresh1,''string'')); Message=Message0;',...
	'set(hmaperr,''string'',[''MSG:'',Message]);'];

cbthresh2=['MaxThresh=str2num(get(hthresh2,''string'')); Message=Message0;',...
	'set(hmaperr,''string'',[''MSG:'',Message]);'];

hthresh1=uicontrol(mapgui,'style','edit','string',num2str(MinThresh),...
	'TooltipString','Lower threshold value',...
	'position',[90 109 60 14],'BackgroundColor',Kcolor,'callback',cbthresh1);

hthresh2=uicontrol(mapgui,'style','edit','string',num2str(MaxThresh),...
	'TooltipString','Upper threshold value',...
	'position',[90 124 60 14],'BackgroundColor',Kcolor,'callback',cbthresh2);


cbOutMenu = ['DatOp=get(houtmenu,''Value''),';];

	
houtmenu=uicontrol(mapgui,'Style','popup','Visible','on','String',...
	'INCLUSIVE RANGE|EXCLUSIVE RANGE','Tag','OutMenu',...
	'value',DatOp,...
	'TooltipString',['Inclusive: All voxels within threshold range except ThMin, ',...
	'Exclusive: Only voxels outside threshold range'],...
    	'Position',[90 82 150 25],'callback',cbOutMenu);
	
pcbplane=['Message=Message0; PlaneNo=PlaneNo+1; vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hmaperr,''string'',[''MSG:'',Message]);'];

uicontrol(mapgui,'string', 'NEXT PLANE',...
	'TooltipString','Display only',...
	'position',[140 60 90 20],'BackgroundColor',Bcolor,'callback',pcbplane);


cbplane=['PlaneNo=str2num(get(hplane,''string'')); Message=Message0;',...
	'vp_fimage;',...
	'if isempty(Message), PlaneNo=max(PlaneNo-1,1); Message=Message0; end;',...
	'set(hplane,''string'',PlaneNo); ',...
	'set(hmaperr,''string'',[''MSG:'',Message]);'];

hplane=uicontrol(mapgui,'style','edit','string',PlaneNo,...
	'TooltipString','Display only',...
	'position',[230 60 60 20],'BackgroundColor',Kcolor,'callback',cbplane);

uicontrol(mapgui,'String','PROCESS',...
	'TooltipString','Create Thresholded Statistical Image Over Background.',...
	'Position',[10 35 60 25],...
	'CallBack',[ 'if ~DefStat, set(hmaperr,''string'',''ERR: Statistical File Not Defined''); return; end; ',...
	'if ~DefMask & MaskOn, set(hmaperr,''string'',''ERR: Mask Not Defined''); return; end; ',...
	'if ~DefBgrnd, set(hmaperr,''string'',''ERR: Background Not Defined''); return; end; ',...
	'if isempty(maskFName) & MaskOn, Message=''Mask Not Defined'',',...
	'set(hmaperr,''string'',[''Error: '',Message]); return; end;',...
	'if isempty(statFName), Message=''Stat File Not Defined'',',...
	'set(hmaperr,''string'',[''Error: '',Message]); return; end;',...
	'if MaxThresh>GMax1, Message=''Threshold > Maximum'',',...
	'set(hmaperr,''string'',[''Error: '',Message]); return; end;',...
	'if MinThresh>MaxThresh, Message=''Minimum > Maximum Threshold'',',...
	'set(hmaperr,''string'',[''Error: '',Message]); return; end;',...
	'if isempty(BgrndFName), Message=''Background File Not Defined'',',...
	'set(hmaperr,''string'',[''Error: '',Message]); return; end;',...
	'set(gcf,''Pointer'',''watch''); if(ProTime==0), set(FI,''name'',''Statistical Map''); ProTime=1; else,',...
	'FI=findobj(''name'',''Statistical Map''); figure(FI); ProTime=ProTime+1; end; vp_statmapro;',...
	'set(hmaperr,''string'',Message);',...
	'set(findobj(''Name'',''StatMap''),''Pointer'',''arrow'');',...
	' '] );
	
uicontrol(mapgui,'String','QUIT',...
	'Position',[230 35 60 25],...
	'CallBack',['delete(mapgui);','delete(FI);']);
	
hmaperr = uicontrol(mapgui,'string', ['MSG : ',Message],...
	'position',[10 10 280 25],...
	'CallBack',['set(hmaperr,''string'',[''MSG: '',Message]);'],...
	'BackgroundColor',Ecolor);

