% vp_statmapro.m
% Create thresholded statistical map processing routine
%_______________________________________________________________________
% vp_statmapro creates a statistical map of values above a certain minimum
% threshold and below a maximum threshold that are either inclusive or
% exclusive. In addition values are either included or excluded by an
% associated image mask. The values are rendered over a background image. 
% Both the statistical data and data over background are saved in separate
% ANALYZE format image. The DATAMAX value used for display purposes may
% not always be found within the data but is saved in the first pixel of the
% composite output image so that the display range is not shifted if the
% saved file is displayed later. 
%__________________________________________________________________________
% vp_statmapro.m	C.0 	P. Spetsieris	Jun. 18, 1999
%			rev. 5.8 	P.S.	May 6, 2007
%			rev. 6.1 	P.S.	May 27, 2011
%			rev. 6.2 	P.S.	Apr 27, 2012
%			rev. 7.0 	P.S.	Oct 13, 2015

global FI FNames FName mapgui hmaperr CM  GCM PlaneNo hplane
global MinThresh MaxThresh maskFName statFName BgrndFName 
global Message GMax1 GMin1 GMax3 GMin3 DATAMIN DATAMAX MskOp DatOp 
global FOREMIN BACKMAX nameimgA VAL_OFFSET

global MaskOn

Message0=['............................................ '];
Message=Message0;

%-----------------------------------------------------------------------
  
%CHECK DIMENSIONS
FName=deblank(statFName);
P=vp_nm('dnm',FName);
[dim1 vox1 scale1 type1 offset1 or1 descrip1 tim1 dur1 mtyp1 gentim1 subid1 expdate1 glmax1 glmin1 valoffset1] = vp_hread([P '.hdr']);
if (valoffset1 ~= 0)vp_note(['Convert to ANALYZE, Nifti file VAL_OFFSET ~=0'],3); FName, valoffset1, exit(-1); end;  
statFNAME=[P '.img'],

FName=deblank(BgrndFName);
P=vp_nm('dnm',FName);
[dim3 vox3 scale3 type3 offset3 or3 descrip3 tim3 dur3 mtyp3 gentim3 subid3 expdate3 glmax3 glmin3 valoffset3] = vp_hread([P '.hdr']);
if (valoffset3 ~= 0)vp_note(['Convert to ANALYZE, NIfti file VAL_OFFSET ~=0'],3); FName, valoffset3, exit(-1); end;  
BgrndFName=[P '.img'],

if ~isequal(dim1,dim3), vp_message('ERROR: Incompatible Dimensions for Background'); return; end;
if ~isequal(vox1,vox3), vp_message('ERROR: Incompatible Voxel Size for Background'); return; end;

if MaskOn,
  FName=deblank(maskFName);
  %P=FName(1:length(FName)-4);
  P=vp_nm('dnm',FName);
  [dim2 vox2 scale2 type2 offset2 or2 descrip2 tim2 dur2 mtyp2 gentim2 subid2 expdate2 glmax2 glmin2 valoffset2] = vp_hread([P '.hdr']);
  maskFNAME=[P '.img'],
  
  if ~isequal(vox1, vox2), 
  	vp_message('ERROR: Incompatible Voxel Size for Mask');
	vox1, vox2,
	return;
  end

  if ~isequal(dim1, dim2)
	dim1, dim2,
  	vp_message('ERROR: Incompatible Dimensions for Mask');
	return;
  end
  
  fidmask=fopen(maskFName,'r');
  
end

P=vp_nm('dir',statFName);
nameimgA=[P 'statmapA' '.img']; %Output Image A name
if strcmp(statFName,nameimgA) 
  	vp_message(['CONFLICT: Invalid STAT file name: ',statFName]);
	return;
end
if strcmp(BgrndFName,nameimgA) 
  	vp_message(['CONFLICT: Invalid BGRND file name: ',BgrndFName]);
	return;
end
if (MaskOn & strcmp(maskFName,nameimgA)) 
  	vp_message(['CONFLICT: Invalid MASK file name: ',maskFName]);
	return;
end

fidresultA=fopen(nameimgA,'w');

nameimg=[P 'statmapB' '.img']; %Output Image B name
if strcmp(statFName,nameimg) 
  	vp_message(['CONFLICT: Invalid STAT file name: ',statFName]);
	return;
end
if strcmp(BgrndFName,nameimg) 
  	vp_message(['CONFLICT: Invalid BGRND file name: ',BgrndFName]);
	return;
end
if (MaskOn & strcmp(maskFName,nameimg)) 
  	vp_message(['CONFLICT: Invalid MASK file name: ',maskFName]);
	return;
end

fidresultB=fopen(nameimg,'w');

ImageSize=dim1(1)*dim1(2);
bgrndSize=dim3(1)*dim3(2);

fidstat=fopen(statFName,'r');
fidbgrnd=fopen(BgrndFName,'r');

figure(FI);
w=which('scanvp');
%cf=[w(1:(max(findstr(w,'\'))-4)),'col\graycols'];
%load(cf); colormap(graycols); GCM=colormap;
cf=[w(1:(max(findstr(w,'\'))-4)),'col\grayhot'];
load(cf); GCM=CM; colormap(GCM);

%Calculate conversion factor to map Forground to upper half of display range
DMinF=GMin1/scale1;
DMaxF=GMax1/scale1;

%if type1 < 16
  % If No Negatives - Normalize Output 
 % if GMin1 > 0 
 %	DMinF=glmin1,
 % 	DMaxF=glmax1,
 % end
%end

%if type1 < 16
  % Check for Negatives 
  %if GMin1 < 0 | GMax1 < 0,
  %	DMinF= -glmax1-1,
  %else
  %	DMinF=glmin1,
  %end
  %DMaxF=glmax1,
%end

if DMaxF-DMinF == 0, 
  	vp_message('ERROR: Zero Forground Range');
	return;
end


if (MaxThresh/scale1 < DMinF | MaxThresh/scale1 > DMaxF), 
  	vp_message('ERROR: Maximum Threshold Out of Range');
	return;
end

if DatOp==1,
 outMinF=MinThresh/scale1; outMaxF=MaxThresh/scale1; 
 if outMinF == outMaxF,
  if outMinF ~= DMinF
  	outMinF=DMinF; 
  else
  % Automatic Shift of Lower Display limit
  	outMinF=outMaxF-(DMaxF-DMinF)/10;
  end
 end
elseif DatOp==2,
 outMinF=DMinF; outMaxF=DMaxF; 
 if MinThresh/scale1 == DMinF,
 	outMinF=MaxThresh/scale1;
 end
end

outscale=1;
%outype=16;
outype=64;

%Calculate conversion factor to map Background to lower half of display range
delta=outMaxF-outMinF;
if delta <= 0, 
  	vp_message(['ERROR: Invalide Display Range: ', num2str(delta)]);
	return;
end


% make correction for even negative/positive range
if outMinF < 0 & outMaxF > 0,
	delta=2*max(outMaxF,-outMinF);
	outMaxF=delta/2;
	% cf=[w(1:(max(findstr(w,'\'))-4)),'col\graybicol']; %
	cf=[w(1:(max(findstr(w,'\'))-4)),'col\grayjet'];
	load(cf); GCM=CM; colormap(GCM);
end;

%outMinB=outMinF-delta; outMaxB=outMinF -delta/128; 
outMinB=outMaxF-2*delta; outMaxB=outMaxF-delta -delta/128;
	
DMinB=GMin3/scale3;
DMaxB=GMax3/scale3;

if DMaxB-DMinB == 0, 
  	vp_message('ERROR: Zero Background Range');
	return;
end
mapfctB=(outMaxB-outMinB)/(DMaxB-DMinB);


DATAMAX=outMaxF*scale1, 
DATAMIN=outMinB*scale1, % For Display only
FOREMIN=outMinF*scale1, % Forground minimum
BACKMAX=outMaxB*scale1, % Background maximum
	
%mnthresh=MinThresh,mxthresh=MaxThresh,

for PlaneNo=1:dim1(3)

	
	statDATA=[];
	statDATA=zeros(dim1(1),dim1(2));
	
      	sz=ImageSize*vp_roifuns('bytes',mtyp1);
  	fseek(fidstat,sz*(PlaneNo-1),'bof');

	outDATA=[];
	outDATA= zeros(dim1(1),dim1(2)) ;
	
	%statDATA= fread(fidstat,[dim1(1),dim1(2)],'uchar');
	statDATA= fread(fidstat,[dim1(1),dim1(2)],mtyp1);
	if DatOp == 1 %Inclusive
	  % Do not include minimum if 0. Data may use 0 as background
	  if (MinThresh==0)
	   [is, js]=find( (MinThresh/scale1) < statDATA & statDATA <= (MaxThresh/scale1) );
	  else
	   [is, js]=find( (MinThresh/scale1) <= statDATA & statDATA <= (MaxThresh/scale1) );
	  end
	elseif DatOp == 2 %Exclusive 
	  %[is, js]=find( (MinThresh/scale1) >= statDATA | statDATA >= (MaxThresh/scale1) );
	  [is, js]=find( (MinThresh/scale1) > statDATA | statDATA > (MaxThresh/scale1) );
	end
	
	if ~isempty(is)
	
          if MaskOn,
	    maskDATA=[];
	    maskDATA=zeros(dim2(1),dim2(2));
	  
      	    sz=ImageSize*vp_roifuns('bytes',mtyp2);
  	    fseek(fidmask,sz*(PlaneNo-1),'bof');
	    maskDATA= fread(fidmask,[dim2(1),dim2(2)],mtyp2);
	  
	    for i=1:length(is)
		if (maskDATA(is(i),js(i)) ~= 0 ) & MskOp ==1 %inclusive
			outDATA(is(i),js(i))=statDATA(is(i),js(i));
		elseif (maskDATA(is(i),js(i)) == 0 ) & MskOp ==2 %exclusive
			outDATA(is(i),js(i))=statDATA(is(i),js(i));
		end
	    end
	  else, 
	    for i=1:length(is)
		outDATA(is(i),js(i))=statDATA(is(i),js(i));
	    end
	  end
	  
	  %maxi=max(max(outDATA));
	  %if maxi > StatMax, StatMax=maxi; end;
	  
	  %outDATA=round(outDATA/2) ; 
	
	end
	
	out1=fwrite(fidresultA,outDATA*scale1,'double');
	
       
      	sz=bgrndSize*vp_roifuns('bytes',mtyp3);
	fseek(fidbgrnd,sz*( (or3(3) +(PlaneNo-or1(3))) -1),'bof');
	bgrndDATA=[];
	%bgrndDATA= fread(fidbgrnd,[dim3(1),dim3(2)],'uchar');
	bgrndDATA= fread(fidbgrnd,[dim3(1),dim3(2)],mtyp3);
	
	%	bgrndDATA= round(bgrndDATA/2);
	
	for i=1:dim1(1)
	  for j=1:dim1(2)
	  	ib = max(1, round(or3(1) + (i-or1(1))* (vox1(1)/vox3(1))) );
		ib = min(ib, dim3(1));
		
	  	jb = max(1, round(or3(2) + (j-or1(2))* (vox1(2)/vox3(2))) );
		jb = min(jb, dim3(2));
		
	  	if outDATA(i,j) == 0 
		  %outDATA(i,j)=bgrndDATA(i-or1(1)+or3(1), j-or1(2)+or3(2));
		  outDATA(i,j)=bgrndDATA(ib,jb);
		  outDATA(i,j)=outMinB + (outDATA(i,j)-DMinB)*mapfctB;
		end
	  end
	end
	
	outDATA=outDATA*scale1;
	
        % Mark with DATAMAX value on the first pixel
	if PlaneNo==1,
	 	outDATA(1,1)=DATAMAX;
	end
	
	out1=fwrite(fidresultB,outDATA,'double');
	
	outDATA=outDATA';
	%outDATA=flipud(outDATA);
	
	%axes(AxI04);
	figure(FI);
	
	%outDATA=outDATA*outscale;
	%imagesc(outDATA,[0 255*outscale]);
	
	%imagesc(outDATA,[0 255]);
	imagesc(outDATA,[DATAMIN DATAMAX]);
	
	axis off image xy
	
	%Correct for aspect ratio
	vp_aspect(1,vox1);
		
	colormap(GCM);
	set(hplane,'string',num2str(PlaneNo));
	drawnow;

end


fclose(fidstat);
if MaskOn, fclose(fidmask); end;
fclose(fidbgrnd);

fclose(fidresultA);
if (vp_hwrite(nameimgA,dim1,vox1,outscale,outype,offset1,or1,...
	['ScAnVP statmap, Thresh=(' num2str(MinThresh),',',num2str(MaxThresh) '), ' descrip1],...
	tim1,dur1,gentim1,subid1,expdate1)) < 0,
	vp_message('hwrite statmapA err'); 
else, 
	vp_message(['Foreground file: ', nameimgA,' saved']); 
end

fclose(fidresultB);

Bcolor = [.8,1,1];

maprogui = figure('Color',Bcolor,'Name','MapName',...
'NumberTitle','off','Position',[100,280,300,60],...
'Units','Pixels');
	

% Save default header before user name read, to keep consistent with image saved.
% Read user name. If not the default name. Warn if only .hdr or .img doesn't exist,
% since if there aren't permissions to overwrite, only the header or image will be
% overwritten and the other will be incorrect.


%cbFName=['staterr=0; namehdr0=[nameimg(1:length(nameimg)-4) ''.hdr''];',...
%	  'FName=deblank(FName); P=FName(1:length(FName)-4); namehdr=[P ''.hdr''];',...
%
cbFName=['staterr=0; namehdr0=[vp_nm(''dnm'',nameimg) ''.hdr''];',...
	'if (vp_hwrite(nameimg,dim1,vox1,outscale,outype,offset1,or1,',...
	'[''ScAnVP PET/MRI map, Thresh=('' num2str(MinThresh),'','',num2str(MaxThresh) ''), '' descrip1],',...
	'tim1,dur1,gentim1,subid1,expdate1)) < 0,',...
	'vp_message(''hwrite err''); staterr=1; ',...
	'else, ',...
	  'FName=get(hsname,''string''),',...
	  'P=vp_nm(''dnm'',FName); namehdr=[P ''.hdr''];',...
	  'FName=[P ''.img''];',...      
	  'if ~strcmp(FName,nameimg),',...
	     'if (exist(FName,''file'') & ~exist(namehdr,''file'')) | ',...
	       ' (~exist(FName,''file'') & exist(namehdr,''file''))  ',...
	       'vp_message([''Remove Invalid File '' FName '' .hdr or .img'']); staterr=3; ',...
	     'else, [s,dosmsg]=dos([''move '' nameimg '' '' FName]), ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	       '[s,dosmsg]=dos([''move '' namehdr0 '' '' namehdr]), ',...
	       'if s ~=0, vp_message(dosmsg); staterr=2; end; ',...
	     'end; ',...
	     'if staterr==0, nameimg=FName, end;',...
	  'end; ',...
	'end;',...
	'if staterr==0,',...
	'set(gco,''string'',[nameimg ''   FILE SAVED'']); ',...
	'disp(''FILE SAVED''); pause(2);',...
	'vp_imz; delete(findobj(''Name'',''MapName'')); end; '];
	
%	'set(hroierr,''string'',[''MSG: '',Message]);'] ;
	
uicontrol(maprogui,'style','text','string','ENTER COMPOSITE MAP NAME: ',...
	'position',[10 35 180 25],'BackgroundColor',Bcolor);
	
hsname=uicontrol(maprogui,'style','edit','string',nameimg,...
	'position',[10 5 280 25],'BackgroundColor','w','callback',cbFName);



disp('END vp_statmapro');



