% vp_transp.m
% Displays transparent superimposed views of PET/MRI ANALYZE images
% in three orthogonal views through the origin.
% The user has the option to display multiple sequential 
% sections with various formating and slice skip options.
% The user selects separate colormaps for forground and background.
% The disk Neurological (LR) or Radiological orientation is specified
% on the main 'scanvp' menu. Foreground and Background files can have
% opposite orientations on disk but if their correct orientation is
% specified both will be displayed in Neurological orientation. An
% unintended application is to enter the same image as forground and
% background with opposite orientation to check for Left-Right symmetry.
% The transparency variable is varied interactively to allow more or less
% of each image to be apparent. To view each change the 'DISPLAY' button
% must be used.
%___________________________________________________________________________
% vp_transp.m	C.4.6	P. Spetsieris  March 28, 2001
%   ScAnVP Rev. 7.0     P. Spetsieris Jan. 7, 2016

global WinView  WinFormat Plane0 MaxPlane MaxDim Opt FName FRName BGName Time Message
global DIM VOX SCALE TYPE OFFSET ORIGIN GCM CB CF SliceDelta
global HelpMode Ecolor HelpMsg
global NumLabels PhyLabels FI ViewLowVal
global XYZDATA FDATA BDATA FDATMIN FDATMAX BDATMIN BDATMAX
global TFact herror hslider
global InputOrient DisplayOrient FMenu

if HelpMode, vp_help(mfilename), return ; end;

FName=[];
FRName=[];
BGName=[];

%Set image user interface controls

bkgcolor = [.8,.8,1];
Opt=1;
Time=0;
Message0=['....'];
Message=Message0;

WinView=1;
WinFormat=4 ;
SliceDelta=1 ;
Plane0=1;
MaxPlane = 1 ;

GCM=colormap;

NumLabels=1;
PhyLabels=1; 
ViewLowVal=1;
       
delete(findobj('Name','transpgui'));

SetF=0; SetB=0;

transpgui = figure('Color',[.6,.6,1],'Name','transpgui',...
'NumberTitle','off','Position',[50,200,250,265],...
'Units','Pixels');

pcbImgOrient = [' ',...
'if (InputOrient==1)',...
  '  vp_note([''Set for Radiologically (RL) oriented disk input files. RESET on main scanvp menu.''], 4);',...
 'else',...
 '  vp_note([''Set for Neurologically (LR) oriented disk input files. RESET on main scanvp menu.''], 4);',...
  'end; figure(FMenu);'];

uicontrol(transpgui,'string', 'Check RL/LR Input File Orientation',...
	'TooltipString',['Check: Radiological (Right is Left) or Neurological ',...
    '(Right is Right) Disk Input File Orientation.'],...
	'position',[10 243 185 15],...
	'BackgroundColor',Ecolor,'callback',pcbImgOrient);

uicontrol(transpgui,'string', 'Help',...
	'TooltipString',HelpMsg,...
	'BackgroundColor',Ecolor,...
	'position',[200 243 40 20],'callback','vp_help(''vp_transp'');');
	

cbview=['WinView=str2num(get(hview,''string''));'];
cbformat=['WinFormat=str2num(get(hform,''string''));'];
cbplane0=['Plane0=str2num(get(hplane0,''string''));'];
cbmaxpln=['MaxPlane=str2num(get(hmaxpln,''string''));'];
cbdelta=['SliceDelta=str2num(get(hdelta,''string''));'];
cbmap=['waitfor(findobj(''name'',''rangegui'')); CM=colormap; N=length(CM); ',...
	' d0=DATAMIN; dN=DATAMAX; ',...
	' [d1,d2]=caxis, c1=((d1-d0)*(N-1)/(dN-d0))+1 ; ',...
	' c2=((d2-d0)*(N-1)/(dN-d0))+1 ; ',...
	' cmin=round(c1), cmin=int16(cmin); cmax=round(c2), cmax=int16(cmax); CN=CM;',...
	' for i=1:cmin, CN(i,:)=CM(1,:); end;',...
	' for i=cmax:N, CN(i,:)=CM(N,:); end;',...
	' if (cmax-cmin) > 1, for i=cmin+1:cmax-1,',...
	' j=(i-cmin)*(N-1)/(cmax-cmin) + 1; CN(i,:)=CM(j,:); end; end;',...
	' CN, colormap(CN); caxis([d0,dN]); '] ;


uicontrol(transpgui,'string', 'VIEW  1=AXI',...
	'TooltipString','Axial View',...
	'position',[10 125 72 20],'callback',...
	['WinView=1; set(hview,''string'',''1'');',...
	'MaxPlane=DIM(3); MaxDim=DIM(3); set(hmaxpln,''string'',num2str(DIM(3)));']);
	
uicontrol(transpgui,'string', '2=SAG',...
	'TooltipString','Sagittal View',...
	'position',[82 125 44 20],'callback',...
	['WinView=2; set(hview,''string'',''2'');',...
	'MaxPlane=DIM(1); MaxDim=DIM(1); set(hmaxpln,''string'',num2str(DIM(1)));']);
	
uicontrol(transpgui,'string', '3=COR',...
	'TooltipString','Coronal View',...
	'position',[126 125 44 20],'callback',...
	['WinView=3; set(hview,''string'',''3'');',...
	'MaxPlane=DIM(2); MaxDim=DIM(2); set(hmaxpln,''string'',num2str(DIM(2)));']);
	
hview=uicontrol(transpgui,'style','edit','string',num2str(WinView),...
	'TooltipString','Enter View Parameter',...
	'position',[200 125 30 20],'BackgroundColor',bkgcolor,'callback',cbview);

	
uicontrol(transpgui,'string', 'FORMAT  1',...
	'TooltipString','Display Format 1=Single or Tile',...
	'position',[10 105 72 20],'callback',['WinFormat=1; set(hform,''string'',''1'');']);
	
uicontrol(transpgui,'string', '2=2x2',...
	'TooltipString','2x2 Display',...
	'position',[82 105 45 20],'callback',['WinFormat=2; set(hform,''string'',''2'');']);
	
uicontrol(transpgui,'string', '3=3x3',...
	'TooltipString','3x3 Display',...
	'position',[127 105 44 20],'callback',['WinFormat=3; set(hform,''string'',''3'');']);
	
hform=uicontrol(transpgui,'style','edit','string',WinFormat,...
	'TooltipString','Enter Display Format',...
	'position',[200 100 30 20],'BackgroundColor',bkgcolor,'callback',cbformat);
	
       
hdelta=uicontrol(transpgui,'style','edit','string',SliceDelta,...
	'TooltipString','Enter number of slices to skip',...
	'position',[200 80 30 20],'BackgroundColor',bkgcolor,'callback',cbdelta);

uicontrol(transpgui,'string', 'SLICE SKIP 1',...
	'TooltipString','Display Consecutive Slices',...
	'position',[10 80 71 20],'callback',['SliceDelta=1; set(hdelta,''string'',''1'');']);
	
uicontrol(transpgui,'string', '2',...
	'TooltipString','Skip 2 slices',...
	'position',[81 80 18 20],'callback',['SliceDelta=2; set(hdelta,''string'',''2'');']);
	
uicontrol(transpgui,'string', '3',...
	'TooltipString','Skip 3 slices',...
	'position',[99 80 18 20],'callback',['SliceDelta=3; set(hdelta,''string'',''3'');']);
	
uicontrol(transpgui,'string', '4',...
	'TooltipString','Skip 4 slices',...
	'position',[117 80 18 20],'callback',['SliceDelta=4; set(hdelta,''string'',''4'');']);
	
uicontrol(transpgui,'string', '5',...
	'TooltipString','Skip 5 slices',...
	'position',[135 80 18 20],'callback',['SliceDelta=5; set(hdelta,''string'',''5'');']);
	
uicontrol(transpgui,'string', '6',...
	'TooltipString','Skip 6 slices',...
	'position',[153 80 18 20],'callback',['SliceDelta=6; set(hdelta,''string'',''6'');']);
	
uicontrol(transpgui,'Style','Check','String','Numerical Labels',...
	'TooltipString','Label Slices Numerically',...
	'Value',NumLabels,'Position',[10 60 100 15],...
	'CallBack',[ 'if ~NumLabels, NumLabels=1; else NumLabels=0, end;']);
	
uicontrol(transpgui,'Style','Check','String','Physical Labels',...
	'TooltipString','Show Slice Position in mm.',...
	'Value',PhyLabels,'Position',[120 60 100 15],...
	'CallBack',[ 'if ~PhyLabels, PhyLabels=1; else PhyLabels=0, end;']);
	
uicontrol(transpgui,'string', 'Initial Plane',...
	'TooltipString','Initial Display Slice Number',...
	'position',[10 35 75 20],'BackgroundColor',bkgcolor);
hplane0=uicontrol(transpgui,'style','edit','string',Plane0,...
	'TooltipString','Initial Display Slice Number',...
	'position',[85 35 35 20],'BackgroundColor',bkgcolor,'callback',cbplane0);

uicontrol(transpgui,'string', 'Final Plane',...
	'TooltipString','Final Display Slice Number',...
	'position',[130 35 75 20],'BackgroundColor',bkgcolor);
hmaxpln=uicontrol(transpgui,'style','edit','string',MaxPlane,...
	'TooltipString','Final Display Slice Number',...
	'position',[205 35 35 20],'BackgroundColor',bkgcolor,'callback',cbmaxpln);

uicontrol(transpgui,'String','CANCEL',...
	'Position',[10 10 60 20],...
	'CallBack',['set(transpgui,''UserData'',-1); delete(transpgui);']);
	
uicontrol(transpgui,'Style','Check','String','View Low Values',...
	'TooltipString','Display low foreground values',...
	'Value',ViewLowVal,'Position',[75 12 100 15],...
	'CallBack',[ 'if ~ViewLowVal, ViewLowVal=1; else ViewLowVal=0, end;']);
	
uicontrol(transpgui,'String','DISPLAY',...
	'Position',[180 10 60 20],...
	'CallBack',['if SetF==0; vp_message(''Foreground Not Specified''); return; end;',...
	'if SetB==0; vp_message(''Background Not Specified''); return; end;',...
	'Opt=3; vp_transpro; set(transpgui,''UserData'',1);']);
	

cbPFRName=['FRName=get_file(1,''.img'',''Please Select Foreground Image'',[]); ',...
	'if ~isempty(FRName),',...
	'Opt=1; Time=Time+1; FName=FRName; vp_transpro;',...
	'set(hrname,''string'',FRName); ',...
	'set(hview,''string'',WinView); ',...
	'set(hform,''string'',WinFormat); ',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'if isempty(Message), Message=Message0; end;',...
	'vp_matcolor; CF=colormap; if ~isempty(CF), SetF=1; FDATA=XYZDATA;',...
	'FDATMIN=DATAMIN; FDATMAX=DATAMAX; end;',...
	'UFI=FI; vp_colrange; eval(cbmap); CF=colormap; ',...
	'end;',...
	];
	
uicontrol(transpgui,'string', 'FOREGRND',...
	'position',[10 217 75 20],'BackgroundColor',bkgcolor,...
	'callback',cbPFRName);
	
%	'if isempty(Message), Message=Message0; end;',...
%	'vp_matcolor; CF=colormap; if ~isempty(CF), SetF=1; FDATA=XYZDATA;',...
%	'FDATMIN=DATAMIN; FDATMAX=DATAMAX; end; end;',...
%
cbFRName=['FRName=get(hrname,''string'');','Message=Message0 ;',...
	'if ~isempty(FRName),',...
	'Opt=1; Time=Time+1; FName=FRName; vp_transpro;',...
	'set(hview,''string'',WinView); ',...
	'set(hform,''string'',WinFormat); ',...
	'set(hplane0,''string'',Plane0); ',...
	'set(hmaxpln,''string'',MaxPlane); ',...
	'vp_matcolor; CF=colormap; if ~isempty(CF), SetF=1; FDATA=XYZDATA;',...
	'FDATMIN=DATAMIN; FDATMAX=DATAMAX; end;',...
	'UFI=FI; vp_colrange; eval(cbmap); CF=colormap;',...
	'end;',...
	];
	
hrname=uicontrol(transpgui,'style','edit','string',FRName,...
	'position',[85 217 155 20],'BackgroundColor',bkgcolor,'callback',cbFRName);

%	'BDATMIN=DATAMIN; BDATMAX=DATAMAX; end; end;',...
%
cbPBGName=['BGName=get_file(1,''.img'',''Please Select Background Image'',[]); ',...
	'if ~isempty(BGName),',...
	'Opt=1; Time=Time+1; FName=BGName; vp_transpro;',...
	'set(hrBGName,''string'',BGName); ',...
	'vp_matcolor; CB=colormap; if ~isempty(CB), SetB=1; BDATA=XYZDATA;',...
	'BDATMIN=DATAMIN; BDATMAX=DATAMAX; end;',...
	'UFI=FI; vp_colrange; eval(cbmap); CB=colormap; ',...
	'end;',...
	];
	
uicontrol(transpgui,'string', 'BACKGRND',...
	'position',[10 198 75 20],'BackgroundColor',bkgcolor,...
	'callback',cbPBGName);
	
%	'BDATMIN=DATAMIN; BDATMAX=DATAMAX; end; end;',...
%
cbBGName=['BGName=get(hrBGName,''string'');','Message=Message0 ;',...
	'if ~isempty(BGName), ',...
	'Opt=1; Time=Time+1; FName=BGName; vp_transpro;',...
	'vp_matcolor; CB=colormap; if ~isempty(CB), SetB=1; BDATA=XYZDATA;',...
	'BDATMIN=DATAMIN; BDATMAX=DATAMAX; end;',...
	'UFI=FI; vp_colrange; eval(cbmap); CB=colormap;',...
	'end;',...
	];
	
hrBGName=uicontrol(transpgui,'style','edit','string',BGName,...
	'position',[85 198 155 20],'BackgroundColor',bkgcolor,'callback',cbBGName);
	
%uicontrol(transpgui,'string', 'TRANSPARENCY',...
%	'position',[75 10 100 25],'BackgroundColor',bkgcolor,'Visible','on',...
%	 'callback',...
%	 ['vp_tfactgui; ']);
	

frmcolor=[0.5 0.7 0.8];

Bcolor = [.8,1,1];

TFact=0.5;

cbTFact=['TFact=str2num(get(hTFact,''string'')); ',...
	['if(TFact >= 0) & (TFact <= 1 ),',...
	'Message=Message0; set(hslider,''Value'',TFact);',...
	'else, vp_message(''Error: Factor Out of Bounds''); end; ']];

cbSliTFact=['TFact=get(gco,''Value''); TFact=(round(TFact*100))/100; set(hTFact,''string'',TFact);',...
	['if(TFact < 0) | (TFact > 1) vp_message(''Error: Factor Out of Bounds'');',...
	'else Message=Message0; end; ']];

uicontrol(transpgui,'Style','text','string', 'MRI (Transparency Factor 0-1) PET',...
	'position',[10 172 180 18],'BackgroundColor',frmcolor);
	
hslider=uicontrol(transpgui,'Style','slider', 'Position', [10 150 180 20], 'Min',0,'Max',1,...
	'Value',TFact,'Callback',cbSliTFact);
	
hTFact=uicontrol(transpgui,'style','edit','string',TFact,...
	'position',[200 150 40 20],'BackgroundColor',Bcolor,'callback',cbTFact);

