% vp_voxssm.m
% Options for Voxel Based SSM Statistical Analysis
% 
% Select similarly oriented files. Operations are performed on common
% non-zero gray matter space. It is assumed that excessive atrophy or hemi-
% image inclusion in group analysis is limited to similar images otherwise
% common masked voxel space will be minimal or zero.
% 
% Options:
%
% A)  PCA    (SSM Principal Component Analysis)
%               Evaluation of GIS Group Invariant Subprofiles
%               and SSF Subject Scaling Factors.
%
% B)  SUB-PCA (SUBTRACTION SSM Principal Component Analysis)
%               Performs SSM PCA analysis on difference data.
%               Input may be images representing two different
%               states for the same subjects.
%
% C)  O_PCA  (Orthogonal Principal Component Analysis)
%               Evaluate and optionally remove component 
%               covariance from an image data set and
%               perform PCA analysis in the orthogonal space.
%
% D)  TPR    (Topographic Profile Rating)
%               New Subject Score Evaluation.
%               I) Interactive Options, Reference, Methods, Log. etc.
%               II) Automatic: Predefined Methods and File References.
%                   ROC Characteristics-AUC, Computer-Aided Diagnosis.
%
% E)  O_TPR  (Orthogonal Topographic Profile Rating)
%               Remove covariance component of a given GIS 1 from the
%               an image data set and perform score evaluation on the
%               reduced SRP (O_SRP) images for a second GIS2.
%
% F)  UTIL   Select SSM based utilities:
%
%               Display SCORES:
%                 Display bar graphs of SSF subject scores for selected
%                 GISs. Values are read from SSF Score output files.
%               ROC CHARACTERISTICS:
%                 Display Scores as in "Display SCORES" and in addition
%		  evaluates and outputs sensitivity vs. specificity,
%                 displays the ROC plot and several other plots that 
%		  evaluates optimum cut-off Z-scores.
%               Log MEAN:
%                 Create Voxel Based Log Mean Deviations Image
%               Threshold MASK:
%                 Create Voxel Based Data Threshold Mask from Group Data
%               Exclusive MASK:
%                 Create Voxel Based Exclusive Mask from Data or GIS File
%               MASK group data:
%                 Mask Group Images using a Pre-Defined Mask File
%               Linear GIS Addition:
%                 Linear Combination of Normalized GIS Vectors
%               Linear GIS Correlation:
%                 Pearson's Correlation of GIS or Other Image Vectors
%
% See Individual Help Routines or set "Help Mode" ON on main menu.
% For References see: help vp_ssm.m
%
% vp_voxssm.m		ScAnVP 5.1		P. Spetsieris	7/15/03
%			rev. ScAnVP 5.2		P.S.	2/04/04
%			rev. ScAnVP 5.3		P.S.	6/15/04
%			rev. ScAnVP 5.4		P.S.	1/28/05
%			rev. ScAnVP 5.5		P.S.	6/01/05
%			rev. ScAnVP 5.6		P.S.	11/10/05
%			rev. ScAnVP 5.7		P.S.	3/14/06
%			rev. ScAnVP 6.1		P.S.	2/03/10
%			rev. ScAnVP 6.2		P.S.	1/20/13	
%_______________________________________________________________________
%

global HelpMode
global POpt
global FMenu Message Ecolor Op ImOpFactor Keyin
global hwaitbar AutoTPR
global Scattering

if HelpMode, vp_help(mfilename); help('vp_ssm_voxpca'); help('vp_ssm_voxtpr'); 
  help('vp_ssm_autovoxtpr'); help('vp_ssm_autovoxrecal');  return ; end;

out1=0;
ImOpFactor=1;
Message=['MSG:.................'];
Bcolor=[0.8 .85 .95];
Bcolor=[0.92 .93 .93];
Mcolor=[0.58 .55 .6];

AutoTPR=0;
FWID=365; FHGT=460;


tip1=['Principal Component Analysis'];
tip1s=['Principal Component Analysis of Subtractio Data'];
tip2=['Topographic Profile Rating (Automatic or Interactive Scoring)'];
tip2D=['Computer-Aided Diagnosis using Auto-TPR'];
tip2s=['Orthogonal Topographic Profile Rating (O-TPR)'];
tip3=['Orthogonal Principal Component Analysis (O-PCA)'];
tip4=['Select SSM Vox Utility: Display SCORES, Generate MEAN, Thresh. MASK, Exclusive MASK, Mask Data'];


voxssmopgui=findobj('Name','VOXEL SSM');
if ~isempty(voxssmopgui)
	if(strcmp(get(voxssmopgui,'Visible'),'on'))
		figure(voxssmopgui);
	end
	return
end


%set(FMenu,'Pointer','Watch');

voxssmopgui = figure('Color',Mcolor,'Name','VOXEL SSM','Tag','voxssmopgui',...
'NumberTitle','off','Position',[100,200,FWID,FHGT],'UserData',0,...
'Units','Pixels');


uicontrol(voxssmopgui,'Style','Text','String','Scaled Subprofile Model',...
	'FontName','Times','FontSize',18,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[.10 .72 .8 .2],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
uicontrol(voxssmopgui,'Style','Text','String','(Voxel Based SSM)',...
	'FontName','Times','FontSize',16,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'Position',[.26 .66 .5 .2],'Tag','Empty',...
	'ForegroundColor',Txcolor)
	
	
uicontrol(voxssmopgui,'Style','Text','String','PRINCIPAL COMPONENT ANALYSIS',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'HorizontalAlignment','Left',...
	'Position',[.07 .62 .9 .15],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0.0])
	

uicontrol(voxssmopgui,'string', 'PCA',...
	'TooltipString',tip1,...
	'Units','Normalized',...
	'position',[25 264 80 20],...
	'position',[.0714 .66 .33 .05],...
	'position',[.0714 .67 .33 .05],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_voxpca');
	
%uicontrol(voxssmopgui,'string', 'PCA_exe',...
%	'TooltipString',tip1,...
%	'Units','Normalized',...
%	'position',[25 264 80 20],...
%	'position',[.0714 .67 .33 .05],...
%	'position',[.0714 .67 .16 .05],...
%	'BackgroundColor',Bcolor,'callback','vp_ssm_voxpca');
	
%uicontrol(voxssmopgui,'string', 'PCA_Mat',...
%	'TooltipString',tip1,...
%	'Units','Normalized',...
%	'position',[25 264 80 20],...
%	'position',[.0714 .67 .33 .05],...
%	'position',[.2314 .67 .16 .05],...
%	'BackgroundColor',Bcolor,'callback','vp_ssm_voxpcaui');
	
uicontrol(voxssmopgui,'string', 'SUBTRACTION-PCA',...
	'TooltipString',tip1s,...
	'Units','Normalized',...
	'position',[.4 .67 .33 .05],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_subvoxpca');
	
uicontrol(voxssmopgui,'string', 'ORTHOGONAL-PCA',...
	'TooltipString',tip3,...
	'Units','Normalized',...
	'position',[25 239 160 20],...
	'position',[.0714 .5975 .66 .05],...
	'position',[.0714 .62 .66 .05],...
	'BackgroundColor',Bcolor,'callback','vp_ssm_voxOpca');
      
uicontrol(voxssmopgui,'Style','Text','String','TOPOGRAPHIC PROFILE RATING',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'HorizontalAlignment','Left',...
	'Units','Normalized',...
	'Position',[.07 .45 .9 .15],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0])
     
uicontrol(voxssmopgui,'String','TPR',...
	'TooltipString',tip2,...
	'Units','Normalized',...
	'Position',[.0714 .4675 .66 .05],...
	'Position',[.0714 .5 .33 .05],...
	'BackgroundColor',Bcolor,...
    	'callback',['if AutoTPR, vp_ssm_autovoxtpr, else, vp_ssm_voxtpr; end;']);
	
uicontrol(voxssmopgui,'String','ORTHOGONAL-TPR',...
	'TooltipString',tip2s,...
	'Units','Normalized',...
	'Position',[.4 .5 .33 .05],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_ssm_voxOtpr; ']);
	
% NOT AVAILABLE IN THIS VERSION
%uicontrol(voxssmopgui,'String','AUTO-DIAGNOSIS',...
%	'TooltipString',tip2D,...
%	'Units','Normalized',...
%	'Position',[.4 .45 .33 .05],...
%	'BackgroundColor',Bcolor,...
%    	'callback',['vp_ssm_autocadgui;']);
	
%uicontrol(voxssmopgui,'Style','Check','String','AUTO',...
%	'TooltipString','ON/OFF Automatic TPR Scoring Based on Previous Results',...
%	'Value',AutoTPR,'Position',[210 55 50 20],...
%	'CallBack',[ 'AutoTPR=get(gco,''value''),']);
	
uicontrol(voxssmopgui,'Style','Text','String','VOXEL UTILITIES',...
	'FontName','Times','FontSize',14,...
	'FontWeight','Bold',...
	'FontAngle','Italic',...
	'BackgroundColor',Mcolor,...
	'Units','Normalized',...
	'HorizontalAlignment','Left',...
	'Position',[.20 .34 .9 .15],'Tag','Empty',...
	'ForegroundColor',[1.0 1.0 0.0])
    
      
% Create a 0|1 mask by assigning 1 to all values not in range [T1,T2]
cbexmask=[  'if HelpMode, vp_note(''Create Mask Excluding Value Range.'',2),',...
  ' else vp_keyin(''Enter Mask Excluded Range [T1,T2| q]'',''0, 0''); ',...
  'if (isempty(Keyin) | strcmp(Keyin,''q'') | strcmp(Keyin,''quit'')),',...
  '	return, else ImOpFactor=str2num(Keyin); end;',...
  'FName = get_file(1,''.img'',''Select image'',[]);',...
  'if ~isempty(FName)',...
   ' FName=deblank(FName); out1=vp_imcvt(FName,''K'',2,ImOpFactor);',...
  'end; end;'];
   
       
%%'callback',['vp_ssm_scores;']);
%%

uicontrol(voxssmopgui,'String','DISPLAY SCORES',...
	'TooltipString','Display SSF File Scores for Selected GIS No.',...
	'Units','Normalized',...
	'Position',[70 162 170 20],...
	'Position',[.2 .40 .4857 .04],...
	'BackgroundColor',Bcolor,...
	'callback',['[mode, ssffile, ngis, ssfscores, Zssfs, norm1, norm2, vaf] =vp_ssm_scores;']);
       
 
uicontrol(voxssmopgui,'string', 'Scatter Points',...
	'TooltipString','Scatter Points or Display Vertically.',...
	'Style','checkbox','value',Scattering,...
	'Units','normalized',...
	'Position',[.7 .40 .24 .04],...
	'BackgroundColor',Bcolor,...
	'callback',['val=get(gco,''value'');',...
	 'if (val==0), Scattering=0; ',...
	 ' else, Scattering=1; ',...
	 'end;']);


uicontrol(voxssmopgui,'String','ROC CHARACTERISTICS',...
	'TooltipString','Display ROC Plot and AUC characteristics.',...
	'Units','Normalized',...
	'Position',[70 137 170 20],...
	'Position',[.2 .35 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_ssm_ROC;']);
       
uicontrol(voxssmopgui,'String','GENERATE MEAN',...
	'TooltipString','Create MEAN LOG DEViation Image',...
	'Units','Normalized',...
	'Position',[70 112 170 20],...
	'Position',[.2 .30 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_ssm_means;']);
       
uicontrol(voxssmopgui,'String','GROUP THRESHOLD MASK',...
	'TooltipString','Create Group Threshold Mask',...
	'Units','Normalized',...
	'Position',[70 87 170 20],...
	'Position',[.2 .25 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_mkTmask;']);
       
uicontrol(voxssmopgui,'String','EXCLUSIVE MASK',...
	'TooltipString','Create Exclusive Range Mask from a Data or GIS Image',...
	'Units','Normalized',...
	'Position',[70 62 170 20],...
	'Position',[.2 .20 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['eval(cbexmask)']);
       
uicontrol(voxssmopgui,'String','MASK GROUP DATA',...
	'TooltipString','Mask Data Images using Pre-Defined Mask',...
	'Units','Normalized',...
	'Position',[70 37 170 20],...
	'Position',[.2 .15 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_maskimgs;']);
       
uicontrol(voxssmopgui,'String','LINEAR GIS ADDITION',...
	'TooltipString','Create A Composite GIS From More Than One Component',...
	'Units','Normalized',...
	'Position',[70 15 170 20],...
	'Position',[.2 .10 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_ssm_GisCombo;']);
       
uicontrol(voxssmopgui,'String','GIS CORRELATION',...
	'TooltipString','Calculate the Pierson''s Correlation of Two or More (GIS) Image Vector''s',...
	'Units','Normalized',...
	'Position',[.2 .05 .4857 .04],...
	'BackgroundColor',Bcolor,...
    	'callback',['vp_giscorr;']);
       
uicontrol(voxssmopgui,'string', 'HELP',...
	'TooltipString','Help displayed in MATLAB window',...
	'BackgroundColor',Ecolor,...
	'Units','Normalized',...
	'position',[.8 .8 .1429 .05],'callback',['vp_help(''vp_voxssm'');',...
	' vp_note(''See HELP window. For Specific Options, Set Help Mode ON on Main Menu'');']);

uicontrol(voxssmopgui,'String','NOTE',...
	'TooltipString','Note on operational space',...
	'Units','Normalized',...
    'BackgroundColor',Bcolor,...
	'Position',[.80 .5 .1714 .05],...
	'CallBack',['vp_note(''Group PCA or TPR Operations are limited to common, intersecting, non-zero space. Select congruent images.'',4);',...
	'set(FMenu,''Pointer'',''Arrow'')';] );	

uicontrol(voxssmopgui,'String','QUIT',...
	'TooltipString','Quit and exit.',...
	'Units','Normalized',...
	'Position',[280 5 60 20],...
	'Position',[.80 .0125 .1714 .05],...
	'CallBack',['delete(voxssmopgui);',...
	'set(FMenu,''Pointer'',''Arrow'')';] );


